package gov.nih.nci.protegex.edit;

import java.io.*;
import java.text.*;
import java.sql.*;
import java.lang.Math.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.net.InetAddress;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.model.Task;

import edu.stanford.smi.protegex.owl.ui.widget.OWLUI;

import edu.stanford.smi.protege.util.Log;
import java.util.logging.Level;


/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class BatchEditTask extends BatchTask {

    /**
     * Requests that the task be cancelled. This
     * method will only be called if the task
     * can be cancelled.
     */

    NCIEditTab tab = null;
    OWLModel owlModel;

    boolean done = false;

    int total = 0;

    String message;
    int max = 10000;
    int min = 0;
    boolean cancelled = false;
    boolean canCancel = true;
    String title = null;

    String infile = null;
    String outfile = null;
    PrintWriter pw = null;

    Vector data_vec = null;
    OWLWrapper wrapper = null;


    public BatchEditTask(NCIEditTab tab, OWLModel kb)
    {
		super(tab, kb);
		this.owlModel = kb;
		this.wrapper = tab.getWrapper();
		initialize();
	}

    public BatchEditTask(NCIEditTab tab, OWLModel kb,
                         String infile, String outfile)
    {
		super(tab, kb);
		this.infile = infile;
		this.outfile = outfile;
		this.owlModel = kb;
		this.wrapper = tab.getWrapper();
		data_vec = getData(infile);
		setMax(data_vec.size());
	}

	public void closePrintWriter()
	{
		super.closePrintWriter();
	}

    private void setMax(int max)
    {
		this.max = max;
	}


    public void initialize()
    {
		cancelled = false;
		String title = "Batch Edit Processing";
		setTitle(title);
		setMessage("Batch editing in progress, please wait ...");
	}

    public void openPrintWriter()
    {
        if (outfile != null && outfile.compareTo("") != 0)
        {
			this.pw = super.openPrintWriter(outfile);
			super.setPrintWriter(this.pw);
		}
	}


    public void cancelTask()
    {
        cancelled = true;
        closePrintWriter();
	}


    /**
     * Gets the title for this task.
     */

    public void setTitle(String title)
    {
		this.title = title;
	}


    public String getTitle()
    {
		return title;
	}

    public void setMessage(String message)
    {
		this.message = message;
	}

    public String getMessage()
    {
		return message;
	}

    /**
     * Gets the minimum progress value for
     * this task.
     */
    public int getProgressMin()
    {
		return min;
	}


    /**
     * Gets the maximum progress value for
     * this task.
     */
    public int getProgressMax()
    {
		return max;
	}


    /**
     * Checks whether this Task has been cancelled.
     * Unless either method is overloaded, this will return true after cancelTask
     * has been called (e.g., via the cancel button).
     *
     * @return true  if this has been cancelled
     */
    public boolean isCancelled()
    {
		return cancelled;
	}

    public void setCanCancel(boolean b)
    {
		canCancel = b;
	}

    /**
     * Determines if the task can be cancelled
     *
     * @return <code>true</code> if the task can
     *         be cancelled, or <code>false</code> if the
     *         task cannot be cancelled.
     */
    public boolean isPossibleToCancel()
	{
		return canCancel;
	}

/*
1	concept name
2	edit|new|delete|del-all
3	property|role|parent|association|propertyqualifier|associationqualifier
4	property name|role name|parent name|qualifier name|association name
5	property value|role value|qualifier value|association value
6	new property value|new role value|new association value|new qualifier value
7	qualified property name|qualified association name|role modifier|bar-delimited property qualifier names
8	qualified property value|qualified association value|bar-delimited property qualifier values
9	new bar-delimited property qualifier values (this field is for edit property with qualifiers only)
*/

    public boolean processTask(int taskId)
    {

		String s = (String) data_vec.elementAt(taskId-1);

		System.out.println(s);
		super.print(s);

		try {
			owlModel.beginTransaction("BatchEdit. Processing " + s);

			Vector w = getTokenStr(s, 9);
			String name = (String) w.elementAt(0);

System.out.println("NAME: " + name);

			String edit = (String) w.elementAt(1);

System.out.println("EDIT TYPE: " + edit);

			String attribute = (String) w.elementAt(2);

System.out.println("attribute: " + attribute);

			String attributeName = (String) w.elementAt(3);

System.out.println("attributeName: " + attributeName);

			String attributeValue = (String) w.elementAt(4);

System.out.println("attributeValue: " + attributeValue);

			String newAttributeValue = (String) w.elementAt(5);

System.out.println("newattributeValue: " + newAttributeValue);


            boolean retval = true;
            if (edit.compareToIgnoreCase("new") == 0)
            {
				if (attribute.compareToIgnoreCase("property") == 0)
				{
					retval = wrapper.addAnnotationProperty(name, attributeName, attributeValue);
				}
				else if (attribute.compareToIgnoreCase("parent") == 0)
				{
					OWLNamedClass hostClass = wrapper.getOWLCls(name);
					OWLNamedClass targetClass = wrapper.getOWLCls(attributeName);
					retval = wrapper.addDirectSuperclass(hostClass, targetClass);
				}

				else if (attribute.compareToIgnoreCase("association") == 0)
				{
					OWLNamedClass hostClass = wrapper.getOWLCls(name);
					retval = wrapper.addObjectProperty(hostClass, attributeName, attributeValue);
				}

				else if (attribute.compareToIgnoreCase("role") == 0)
				{
					int pos = attributeValue.indexOf('|');
					String modifier = attributeValue.substring(0, pos);
					String value = attributeValue.substring(pos+1);
					retval = wrapper.addRestriction(name, attributeName, value, modifier);
				}
			}
            else if (edit.compareToIgnoreCase("delete") == 0)
            {
				if (attribute.compareToIgnoreCase("property") == 0)
				{
					retval = wrapper.removeAnnotationProperty(name, attributeName, attributeValue);
				}
				else if (attribute.compareToIgnoreCase("parent") == 0)
				{
					OWLNamedClass hostClass = wrapper.getOWLCls(name);
					OWLNamedClass targetClass = wrapper.getOWLCls(attributeName);
					retval = wrapper.removeDirectSuperclass(hostClass, targetClass);
				}

				else if (attribute.compareToIgnoreCase("association") == 0)
				{
					OWLNamedClass hostClass = wrapper.getOWLCls(name);
					retval = wrapper.removeObjectProperty(hostClass, attributeName, attributeValue);
				}

				else if (attribute.compareToIgnoreCase("role") == 0)
				{
					int pos = attributeValue.indexOf('|');
					String modifier = attributeValue.substring(0, pos);
					String value = attributeValue.substring(pos+1);
					retval = wrapper.removeRestriction(name, attributeName, value, modifier);
				}
			}
            else if (edit.compareToIgnoreCase("edit") == 0)
            {
				if (attribute.compareToIgnoreCase("property") == 0)
				{
					retval = wrapper.modifyAnnotationProperty(name, attributeName, attributeValue, newAttributeValue);
				}
				else if (attribute.compareToIgnoreCase("role") == 0)
				{
					int pos = attributeValue.indexOf('|');
					String modifier = attributeValue.substring(0, pos);
					String value = attributeValue.substring(pos+1);

					pos = newAttributeValue.indexOf('|');
					String newmodifier = newAttributeValue.substring(0, pos);
					String newvalue = newAttributeValue.substring(pos+1);

					retval = wrapper.modifyRestriction(name, attributeName, value, modifier,
					                                           newvalue, newmodifier);
				}
			}

			// to be implemented
			/*
			else if (edit.compareToIgnoreCase("delete-all") == 0)
			{

			}
			*/


			if (retval)
			{
				super.print("\t Done.");
			}
			else
			{
				super.print("\t Failed.");
			}
			owlModel.commitTransaction();

		} catch (Exception ex) {

			owlModel.rollbackTransaction();
			//Log.getLogger().warn("Error occurred", ex);
			OWLUI.handleError(owlModel, ex);
			super.print(" Failed.");
			return false;
		}

		return true;
	}


	public int getlengthOfTask(String filename)
	{
		if (filename == null) return 0;
		String s;
		int knt = 0;
		try
		{
			BufferedReader inFile = new BufferedReader(new FileReader(filename));
			while ((s = inFile.readLine()) != null)
			{
				knt++;
			}
			inFile.close();
		}
		catch(Exception e)
		{
			System.err.println(e);
		}

		//if (knt > 6) return knt-6;
		return knt;
    }

    // to be implemented
    public void setDefaultValues()
    {

	}


	public Vector getData(String filename)
	{
		Vector v = new Vector();
		if (filename == null) return v;
		String s;
		int knt = 0;
		BufferedReader inFile = null;
		try
		{
			inFile = new BufferedReader(new FileReader(filename));
			while ((s = inFile.readLine()) != null)
			{
				s = s.trim();
				if (s.length() > 0)
				{
					v.add(s);
					knt++;
			    }
			}
			inFile.close();
		}
		catch(Exception e)
		{
			System.err.println(e);
		}

		//System.out.println("Batch file " + filename + " loaded.");
		return v;
    }


	private Vector getTokenStr(String value, int length)
	{
		Vector tokenValues = new Vector();
		String s = value + "\t\t\t\t\t\t\t\t\t";
		int n = s.indexOf("\t");
		int i = 0;
		while (n != -1 && i < length)
		{
			String t = s.substring(0, n);
			tokenValues.add(t);
			s = s.substring(n+1);
			i++;
			n = s.indexOf("\t");
		}
		for (i=0; i<length; i++)
		{
			s = (String) tokenValues.elementAt(i);
			if (s.compareTo("") == 0)
			{
				tokenValues.setElementAt("NA", i);
			}
		}
		return tokenValues;
   	}

	String removeTabs(String s)
	{
		String rets=s;
		while (rets.startsWith("\t"))
		{
			rets = rets.substring(1);
		}
		return rets;
	}


	public String removeRtnChar(String line)
	{
		String s = line;
		char c = s.charAt(s.length()-1);
		if (c == '\n')
		{
			s = s.substring(0, s.length()-1);
		}
		return s;
	}


	public String getStringName(String line)
	{
		String s = line;
		int pos = s.indexOf(":");
		if (pos == -1) return "";
		return s.substring(0, pos);
	}

	public String getStringValue(String line)
	{
		String s = line;
		int pos = s.indexOf(":");
		String t1 = s.substring(pos+1);
       	 	return (removeTabs(t1));
	}

	private String readALine(Reader inFile){
		StringBuffer sb = new StringBuffer();
		try{
			int ch=inFile.read();
			int pre_ch=0;

			while (ch>-1){
				sb.append((char)ch);
				pre_ch = ch;
				ch = inFile.read();
				if (pre_ch==13 && ch==10){
					sb.deleteCharAt(sb.length()-1);
					return sb.toString();
				}
				if (ch==-1) return sb.toString();
			}
			return null;
		}catch(Exception e){
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
			return null;
		}
	}

    public String getToday()
    {
		Calendar cal = Calendar.getInstance();
		SimpleDateFormat  sdf = new SimpleDateFormat("MMM d, yyyy");
		String today = sdf.format(cal.getTime());
		return today;
    }



    public Vector validateData(Vector v)
    {
		Vector supportedRoles = wrapper.getSupportedRoles();
		Vector supportedProperties = wrapper.getSupportedAnnotationProperties();
		Vector supportedAssociations = wrapper.getSupportedAssociations();

		HashSet newProp_hash = new HashSet();
        HashSet newRole_hash = new HashSet();
        HashSet newParent_hash = new HashSet();

		Vector w = new Vector();

		for (int i=0; i<v.size(); i++)
		{
			try {
				int j = i+1;
				String s = (String) v.elementAt(i);

				Vector u = getTokenStr(s, 6);
				String cls_name = (String) u.elementAt(0);
				String action = (String) u.elementAt(1);
				String attribute = (String) u.elementAt(2);
				String attributename = (String) u.elementAt(3);
				String attributevalue_1 = (String) u.elementAt(4);
				String attributevalue_2 = (String) u.elementAt(5);


	System.out.println("NAME: " + cls_name);
	System.out.println("EDIT TYPE: " + action);
	System.out.println("attribute: " + attribute);
	System.out.println("attributeName: " + attributename);
	System.out.println("attributevalue_1: " + attributevalue_1);
	System.out.println("attributevalue_2: " + attributevalue_2);


				OWLNamedClass hostClass  = wrapper.getOWLCls(cls_name);

				Vector superclasses = new Vector();
				if (hostClass == null)
				{
					String error_msg = "Error found on record " + j + " -- concept " + cls_name + " does not exist.";
					w.add(error_msg);
					System.out.println(error_msg);
				}
				else
				{
					superclasses = wrapper.getDirectSuperclassNames(hostClass);
				}

				if (action.compareToIgnoreCase("new") != 0 &&
					action.compareToIgnoreCase("edit") != 0 &&
					action.compareToIgnoreCase("delete") != 0)
				{
					String error_msg = "Error found on record " + j + " -- action " + action + " is not supported.";
					w.add(error_msg);
					System.out.println(error_msg);
				}

				if (attribute.compareToIgnoreCase("parent") != 0 &&
					attribute.compareToIgnoreCase("role") != 0 &&
					attribute.compareToIgnoreCase("property") != 0 &&
					attribute.compareToIgnoreCase("association") != 0)
				{
					String error_msg = "Error found on record " + j + " -- attribute " + attribute + " is not supported.";
					w.add(error_msg);
					System.out.println(error_msg);
				}

				if (action.compareToIgnoreCase("new") == 0)
				{
					if (hostClass != null)
					{
						if (attribute.compareToIgnoreCase("role") == 0)
						{
							if (!supportedRoles.contains(attributename))
							{
								String error_msg = "Error found on record " + j + " -- role " + attributename + " is not identificable.";
								w.add(error_msg);
								System.out.println(error_msg);
							}
							else
							{
								int pos = attributevalue_1.indexOf("|");
								if (pos == -1)
								{
									String error_msg = "Error found on record " + j + " -- missing modifier or filler.";
									w.add(error_msg);
									System.out.println(error_msg);
								}
								else
								{
									String modifier = attributevalue_1.substring(0, pos);
									String filler = attributevalue_1.substring(pos+1, attributevalue_1.length());

									OWLNamedClass targetClass  = (OWLNamedClass) owlModel.getCls(filler);
									if (targetClass == null)
									{
										String error_msg = "Error found on record " + j + " -- concept " + filler + " does not exist.";
										w.add(error_msg);
										System.out.println(error_msg);
									}
									else
									{
										if (wrapper.hasRole(hostClass, attributename, filler))
										{
											String error_msg = "Error found on record " + j + " -- role already exists.";
											w.add(error_msg);
											System.out.println(error_msg);
										}
										else
										{
											String hashValue = cls_name + "|" + attributename + "|" + attributevalue_1;
											if (!newRole_hash.contains(hashValue))
											{
												newRole_hash.add(hashValue);
											}

										}

									}
								}
							}
						}

						else if (attribute.compareToIgnoreCase("parent") == 0)
						{
							try {
								OWLNamedClass superClass  = (OWLNamedClass) owlModel.getCls(attributename);
								if (superClass == null)
								{
									String error_msg = "Error found on record " + j + " -- superconcept " + attributename + " does not exist.";
									w.add(error_msg);
									System.out.println(error_msg);
								}

								if (superclasses.contains(superClass.getBrowserText()))
								{
									String error_msg = "Error found on record " + j + " -- superconcept already exists.";
									w.add(error_msg);
									System.out.println(error_msg);
								}

								else
								{
									String hashValue = cls_name + "|" + attribute + "|" + attributename;
									if (!newParent_hash.contains(hashValue))
									{
										newParent_hash.add(hashValue);
									}
								}

						    } catch (Exception e) {
								String error_msg = "Error found on record " + j + " -- superconcept " + attributename + " does not exist.";
								w.add(error_msg);
								System.out.println(error_msg);
							}



						}

						else if (attribute.compareToIgnoreCase("property") == 0)
						{
							if (!supportedProperties.contains(attributename))
							{
								String error_msg = "Error found on record " + j + " -- property " + attributename + " is not identificable.";
								w.add(error_msg);
								System.out.println(error_msg);
							}
							else
							{
								if (wrapper.hasProperty(hostClass, attributename, attributevalue_1))
								{
									String error_msg = "Error found on record " + j + " -- property already exists.";
									w.add(error_msg);
									System.out.println(error_msg);
								}

								else
								{
									String hashValue = cls_name + "|" + attributename + "|" + attributevalue_1;
									if (!newProp_hash.contains(hashValue))
									{
										newProp_hash.add(hashValue);
									}
								}
							}
						}
						else if (attribute.compareToIgnoreCase("association") == 0)
						{
							if (!supportedAssociations.contains(attributename))
							{
								String error_msg = "Error found on record " + j + " -- association " + attributename + " is not identificable.";
								w.add(error_msg);
								System.out.println(error_msg);
							}
							else
							{
								OWLNamedClass targetClass  = (OWLNamedClass) owlModel.getCls(attributevalue_1);
								if (targetClass == null)
								{
									String error_msg = "Error found on record " + j + " -- concept " + attributevalue_1 + " does not exist.";
									w.add(error_msg);
									System.out.println(error_msg);
								}
								else
								{
									if (wrapper.hasAssociation(hostClass, attributename, attributevalue_1))
									{
										String error_msg = "Error found on record " + j + " -- association already exists.";
										w.add(error_msg);
										System.out.println(error_msg);
									}
								}
							}
						}
					}
				}


				else if (action.compareToIgnoreCase("edit") == 0 || action.compareToIgnoreCase("delete") == 0)
				{
					if (hostClass != null)
					{
						if (attribute.compareToIgnoreCase("parent") == 0)
						{
							if (action.compareToIgnoreCase("delete") == 0)
							{
								OWLNamedClass superClass  = (OWLNamedClass) owlModel.getCls(attributename);
								if (superClass == null)
								{
									String error_msg = "Error found on record " + j + " -- superconcept " + attributename + " does not exist.";
									w.add(error_msg);
									System.out.println(error_msg);
								}


								if (!superclasses.contains(superClass.getBrowserText()))
								{
									String error_msg = "Error found on record " + j + " -- superconcept does not exist.";
									w.add(error_msg);
									System.out.println(error_msg);
								}

								else
								{
									String hashValue = cls_name + "|" + attribute + "|" + attributename;
									if (newParent_hash.contains(hashValue))
									{
										newParent_hash.remove(hashValue);
									}
								}
							}
							else
							{
								String error_msg = "Error found on record " + j + " -- edit parent action is not supported. Use delete and add actions instead.";
								w.add(error_msg);
								System.out.println(error_msg);
							}
						}

						else if (attribute.compareTo("role") == 0)
						{
							if (!supportedRoles.contains(attributename))
							{
								String error_msg = "Error found on record " + j + " -- role " + attributename + " is not identificable.";
								w.add(error_msg);
								System.out.println(error_msg);
							}
							else
							{
								int pos = attributevalue_1.indexOf("|");
								if (pos == -1)
								{
									String error_msg = "Error found on record " + j + " -- missing modifier or filler.";
									w.add(error_msg);
									System.out.println(error_msg);
								}
								else
								{
									String modifier = attributevalue_1.substring(0, pos);
									String filler = attributevalue_1.substring(pos+1, attributevalue_1.length());

									OWLNamedClass targetClass  = (OWLNamedClass) owlModel.getCls(filler);
									if (targetClass == null)
									{
										String error_msg = "Error found on record " + j + " -- concept " + filler + " does not exist.";
										w.add(error_msg);
										System.out.println(error_msg);
									}
									else
									{
										if (!wrapper.hasRole(hostClass, attributename, filler))
										{
											String hashValue = cls_name + "|" + attributename + "|" + attributevalue_1;
											if (!newRole_hash.contains(hashValue))
											{
												String error_msg = "Error found on record " + j + " -- role does not exist.";
												w.add(error_msg);
												System.out.println(error_msg);
											}
										}

										if (action.compareTo("edit") == 0)
										{
											pos = attributevalue_2.indexOf("|");
											if (pos == -1)
											{
												String error_msg = "Error found on record " + j + " -- missing modifier or filler.";
												w.add(error_msg);
												System.out.println(error_msg);
											}
											else
											{
												modifier = attributevalue_2.substring(0, pos);
												filler = attributevalue_2.substring(pos+1, attributevalue_2.length());

												targetClass  = (OWLNamedClass) owlModel.getCls(filler);
												if (targetClass == null)
												{
													String error_msg = "Error found on record " + j + " -- concept " + filler + " does not exist.";
													w.add(error_msg);
													System.out.println(error_msg);
												}
											}
										}
									}
								}
							}
						}
						else if (attribute.compareToIgnoreCase("property") == 0)
						{
							if (!supportedProperties.contains(attributename))
							{
								String error_msg = "Error found on record " + j + " -- property " + attributename + " is not identificable.";
								w.add(error_msg);
								System.out.println(error_msg);
							}
							else
							{
								Boolean editable = wrapper.isReadOnly(attributename);
								if (editable.equals(Boolean.TRUE))
								{
									String error_msg = "Error found on record " + j + " -- property " + attributename + ", it is read-only.";
									w.add(error_msg);
									System.out.println(error_msg);
								}

								if (!wrapper.hasProperty(hostClass, attributename, attributevalue_1))
								{
									String hashValue = cls_name + "|" + attributename + "|" + attributevalue_1;
									if (!newProp_hash.contains(hashValue))
									{
										String error_msg = "Error found on record " + j + " -- property " + "("
										   + attributename + ", " + attributevalue_1 + ") does not exist.";
										w.add(error_msg);
										System.out.println(error_msg);
									}
								}

								if (action.compareToIgnoreCase("edit") == 0)
								{
									String hashValue = cls_name + "|" + attributename + "|" + attributevalue_2;
									if (wrapper.hasProperty(hostClass, attributename, attributevalue_2) ||
										newProp_hash.contains(hashValue))
									{
										String error_msg = "Error found on record " + j + " -- property " + "("
										   + attributename + ", " + attributevalue_2 + ") already exists.";
										w.add(error_msg);
										System.out.println(error_msg);
									}
								}
							}
						}
						else if (attribute.compareTo("association") == 0)
						{
							if (!supportedAssociations.contains(attributename))
							{
								String error_msg = "Error found on record " + j + " -- association " + attributename + " is not identificable.";
								w.add(error_msg);
								System.out.println(error_msg);
							}

							if (action.compareToIgnoreCase("delete") == 0)
							{
								OWLNamedClass targetClass  = (OWLNamedClass) owlModel.getCls(attributevalue_1);
								if (targetClass == null)
								{
									String error_msg = "Error found on record " + j + " -- concept " + attributevalue_1 + " does not exist.";
									w.add(error_msg);
									System.out.println(error_msg);
								}
								else
								{
									if (!wrapper.hasAssociation(hostClass, attributename, attributevalue_1))
									{
										String error_msg = "Error found on record " + j + " -- association does not exist.";
										w.add(error_msg);
										System.out.println(error_msg);
									}
								}
							}
							else
							{
								String error_msg = "Error found on record " + j + " -- edit association action is not supported. Use delete and add actions instead.";
								w.add(error_msg);
								System.out.println(error_msg);
							}
						}
					}
				}

			} catch (Exception e) {
				Log.getLogger().log(Level.WARNING, "Exception caught", e);
				//return null;
			}
		}
		return w;
	}
}

/*
TDE batch modifications include

	1) editing of an existing property,
	2) deletion of an existing property,
	3) addition of a new property,
	4) editing of an existing role,
	5) deletion of an existing role
	6) addition of a new role
	7) addition of a new parent
	8) deletion of an existing parent
	9) editing of an existing property/association qualifier
	10 deletion of an existing property/association qualifier
	11) addition of a new property/association qualifier

Role descriptions include the role modifier, the default role modifier in all operations is �some� (allowed values:  some, all, poss)

File format is tab-delimited, the column fields are

1	concept name
2	edit|new|delete|del-all
3	property|role|parent|association|propertyqualifier|associationqualifier
4	property name|role name|parent name|qualifier name|association name
5	property value|role value|qualifier value|association value
6	new property value|new role value|new association value|new qualifier value
7	qualified property name|qualified association name|role modifier|bar-delimited property qualifier names
8	qualified property value|qualified association value|bar-delimited property qualifier values
9	new bar-delimited property qualifier values (this field is for edit property with qualifiers only)


conceptName\tedit\tproperty\tproperty name\tproperty value\tnew property value\tqualified property name\tqualified property value\tnew bar-delimited property qualifier values

Olfactory_Cistern\tnew\tproperty\tSynonym\t\tOlfactory_Cistern_Synonym_1\t\t\t
Olfactory_Cistern\tnew\tproperty\tSynonym\t\tOlfactory_Cistern_Synonym_2\t\t\t
Olfactory_Cistern\tnew\tproperty\tSynonym\t\tOlfactory_Cistern_Synonym_3\t\t\t
Olfactory_Cistern\tnew\tproperty\tSynonym\t\tOlfactory_Cistern_Synonym_4\t\t\t
Olfactory_Cistern\tnew\tproperty\tSynonym\t\tOlfactory_Cistern_Synonym_5\t\t\t
Olfactory_Cistern\tnew\tproperty\tSynonym\t\tOlfactory_Cistern_Synonym_6\t\t\t

*/