package gov.nih.nci.protegex.edit;

import java.io.*;
import java.text.*;
import java.sql.*;
import java.lang.Math.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.net.InetAddress;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.model.Task;

import edu.stanford.smi.protegex.owl.ui.widget.OWLUI;

import edu.stanford.smi.protege.util.Log;
import java.util.logging.Level;


/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class BatchLoadTask extends BatchTask {

    /**
     * Requests that the task be cancelled. This
     * method will only be called if the task
     * can be cancelled.
     */

    NCIEditTab tab = null;
    OWLModel owlModel;

    boolean done = false;

    int total = 0;

    String message;
    int max = 10000;
    int min = 0;
    boolean cancelled = false;
    boolean canCancel = true;
    String title = null;

    String infile = null;
    String outfile = null;

    PrintWriter pw = null;

    Vector data_vec = null;
    OWLWrapper wrapper = null;

    public BatchLoadTask(NCIEditTab tab, KnowledgeBase kb,
                         String infile, String outfile)
    {
		super(tab, kb);
		this.infile = infile;
		this.outfile = outfile;
		this.wrapper = tab.getWrapper();
		this.owlModel = tab.getOWLModel();
		//initialize();
		data_vec = getData(infile);
		setMax(data_vec.size());
	}

    private void setMax(int max)
    {
		this.max = max;
	}


    public void initialize()
    {
		cancelled = false;
		String title = "Batch Load Processing";
		setTitle(title);
		setMessage("Batch load processing in progress, please wait ...");
	}

    public void openPrintWriter()
    {
        if (outfile != null && outfile.compareTo("") != 0)
        {
			this.pw = super.openPrintWriter(outfile);
			super.setPrintWriter(this.pw);
		}
	}


    public PrintWriter getPrintWriter()
    {
		return pw;
	}

	public void closePrintWriter()
	{
		super.closePrintWriter();
	}


    public void cancelTask()
    {
		super.cancelTask();
        cancelled = true;
	}


    /**
     * Gets the title for this task.
     */

    public void setTitle(String title)
    {
		this.title = title;
	}


    public String getTitle()
    {
		return title;
	}

    public void setMessage(String message)
    {
		this.message = message;
	}

    public String getMessage()
    {
		return message;
	}

    /**
     * Gets the minimum progress value for
     * this task.
     */
    public int getProgressMin()
    {
		return min;
	}


    /**
     * Gets the maximum progress value for
     * this task.
     */
    public int getProgressMax()
    {
		return max;
	}


    /**
     * Checks whether this Task has been cancelled.
     * Unless either method is overloaded, this will return true after cancelTask
     * has been called (e.g., via the cancel button).
     *
     * @return true  if this has been cancelled
     */
    public boolean isCancelled()
    {
		return cancelled;
	}

    public void setCanCancel(boolean b)
    {
		canCancel = b;
	}

    /**
     * Determines if the task can be cancelled
     *
     * @return <code>true</code> if the task can
     *         be cancelled, or <code>false</code> if the
     *         task cannot be cancelled.
     */
    public boolean isPossibleToCancel()
	{
		return canCancel;
	}

    public boolean processTask(int taskId)
    {
		try {
			String s = (String) data_vec.elementAt(taskId-1);
			Vector w = getTokenStr(s, 3);
			String name = (String) w.elementAt(0);
			String pt = (String) w.elementAt(1);
			String sup = (String) w.elementAt(2);

			if (owlModel == null)
			{
				System.out.println("WARNING: owlModel is null...");
				return false;
			}

			//owlModel.beginTransaction("BatchLoad. Creating " + name);
			super.print("Creating " + name);
			Cls cls = wrapper.createCls(name, pt, sup);

			if (cls != null)
			{
				super.print("\t" + name + " created. \n");
			}
			else
			{
				super.print("\t" + "Unable to create class " + name + "\n");
				return false;
			}

/*
			try {
				owlModel.beginTransaction("BatchLoad. Creating " + name);
				super.print("Creating " + name);
				Cls cls = wrapper.createCls(name, pt, sup);

				if (cls != null)
				{
					super.print("\t" + name + " created. \n");
				}
				else
				{
					super.print("\t" + "Unable to create class " + name + "\n");
				}

				owlModel.commitTransaction();

			} catch (Exception ex) {
				System.out.println("Unable to create class " + name);
				super.print("Unable to create " + name);
				owlModel.rollbackTransaction();
				OWLUI.handleError(owlModel, ex);
				return false;
			}
*/


		} catch (Exception e) {
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
			return false;
		}

		return true;
	}


	public int getlengthOfTask(String filename)
	{
		if (filename == null) return 0;
		String s;
		int knt = 0;
		try
		{
			BufferedReader inFile = new BufferedReader(new FileReader(filename));
			while ((s = inFile.readLine()) != null)
			{
				if (s != null && s.compareTo("\n") != 0)
				{
					knt++;
			    }
			    else
			    {
					System.out.println("Skipped an empty line.");
				}
			}
			inFile.close();
		}
		catch(Exception e)
		{
			System.err.println(e);
		}

		return knt;
    }

/*
Olfactory_Cistern_sub_1	Olfactory_Cistern_pt_1	Olfactory_Cistern
Olfactory_Cistern_sub_2	Olfactory_Cistern_pt_2	Olfactory_Cistern
Olfactory_Cistern_sub_3	Olfactory_Cistern_pt_3	Olfactory_Cistern
Olfactory_Cistern_sub_4	Olfactory_Cistern_pt_4	Olfactory_Cistern
Olfactory_Cistern_sub_5	Olfactory_Cistern_pt_5	Olfactory_Cistern
Olfactory_Cistern_sub_6	Olfactory_Cistern_pt_6	Olfactory_Cistern
Olfactory_Cistern_sub_7	Olfactory_Cistern_pt_7	Olfactory_Cistern
Olfactory_Cistern_sub_8	Olfactory_Cistern_pt_8	Olfactory_Cistern
*/

    public Vector validateData(Vector v)
    {
		Vector w = new Vector();
		try {
			for (int i=0; i<v.size(); i++)
			{
				int j = i+1;
				String s = (String) v.elementAt(i);
				Vector u = getTokenStr(s, 3);
				String name = (String) u.elementAt(0);
				String pt = (String) u.elementAt(1);
				String sup = (String) u.elementAt(2);

				if (!NCIEditFilter.checkXMLNCNameCompliance(name))
				{
					String error_msg = "Error found on record " + j + " -- concept name " + name + " is invalid.";
					w.add(error_msg);
					System.out.println(error_msg);
				}

				if (owlModel.getCls(name) != null)
				{
					String error_msg = "Error found on record " + j + " -- concept " + name + " already exists.";
					w.add(error_msg);
					System.out.println(error_msg);
				}

				if (owlModel.getCls(sup) == null)
				{
					String error_msg = "Error found on record " + j + " -- superconcept does not exist.";
					w.add(error_msg);
					System.out.println(error_msg);
				}
		    }

		} catch (Exception e) {
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
			return null;
		}

		return w;

	}


	public Vector getData(String filename)
	{
		Vector v = new Vector();
		if (filename == null) return v;
		String s;
		int knt = 0;
		BufferedReader inFile = null;
		try
		{
			inFile = new BufferedReader(new FileReader(filename));
			while ((s = inFile.readLine()) != null)
			{
				s = s.trim();
				if (s.length() > 0)
				{
					v.add(s);
					knt++;
			    }
			}
			inFile.close();
		}
		catch(Exception e)
		{
			System.err.println(e);
		}

		//System.out.println("Batch file " + filename + " loaded.");
		return v;
    }

	private Vector getTokenStr(String value, int length)
	{
		Vector tokenValues = new Vector();
		String s = value + "\t\t\t\t\t\t\t\t\t";
		int n = s.indexOf("\t");
		int i = 0;
		while (n != -1 && i < length)
		{
			String t = s.substring(0, n);
			tokenValues.add(t);
			s = s.substring(n+1);
			i++;
			n = s.indexOf("\t");
		}
		for (i=0; i<length; i++)
		{
			s = (String) tokenValues.elementAt(i);
			if (s.compareTo("") == 0)
			{
				tokenValues.setElementAt("NA", i);
			}
		}
		return tokenValues;
   	}

	public String removeTabs(String s)
	{
		String rets=s;
		while (rets.startsWith("\t"))
		{
			rets = rets.substring(1);
		}
		return rets;
	}


	public String removeRtnChar(String line)
	{
		String s = line;
		char c = s.charAt(s.length()-1);
		if (c == '\n')
		{
			s = s.substring(0, s.length()-1);
		}
		return s;
	}


	public String getStringName(String line)
	{
		String s = line;
		int pos = s.indexOf(":");
		if (pos == -1) return "";
		return s.substring(0, pos);
	}

	public String getStringValue(String line)
	{
		String s = line;
		int pos = s.indexOf(":");
		String t1 = s.substring(pos+1);
       	 	return (removeTabs(t1));
	}

	private String readALine(Reader inFile){
		StringBuffer sb = new StringBuffer();
		try{
			int ch=inFile.read();
			int pre_ch=0;

			while (ch>-1){
				sb.append((char)ch);
				pre_ch = ch;
				ch = inFile.read();
				if (pre_ch==13 && ch==10){
					sb.deleteCharAt(sb.length()-1);
					return sb.toString();
				}
				if (ch==-1) return sb.toString();
			}
			return null;
		}catch(Exception e){
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
			return null;
		}
	}


    public String getToday()
    {
		Calendar cal = Calendar.getInstance();
		SimpleDateFormat  sdf = new SimpleDateFormat("MMM d, yyyy");
		String today = sdf.format(cal.getTime());
		return today;
    }

}

