package gov.nih.nci.protegex.edit;

import edu.stanford.smi.protege.widget.*;
import edu.stanford.smi.protege.resource.*;
import edu.stanford.smi.protege.ui.*;


import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.ui.InstanceDisplay;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protege.widget.ClsWidget;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.ui.cls.*;
import edu.stanford.smi.protegex.owl.ui.clsdesc.DisjointClassesWidget;
import edu.stanford.smi.protegex.owl.ui.conditions.ConditionsWidget;
import edu.stanford.smi.protegex.owl.ui.icons.OWLIcons;
import edu.stanford.smi.protegex.owl.ui.resourcedisplay.ResourceDisplay;
import edu.stanford.smi.protegex.owl.ui.subsumption.InferredSubsumptionTreePanel;
import edu.stanford.smi.protegex.owl.ui.widget.WidgetUtilities;

import java.io.*;
import java.sql.Connection;
import java.util.*;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import edu.stanford.smi.protegex.owl.model.RDFSClass;

import edu.stanford.smi.protegex.owl.model.OWLProperty;
import edu.stanford.smi.protege.event.*;

import edu.stanford.smi.protegex.owl.ui.ProtegeUI;
import edu.stanford.smi.protegex.owl.ui.dialogs.ModalDialogFactory;
import edu.stanford.smi.protege.util.LabeledComponent;

import edu.stanford.smi.protege.util.ComponentFactory;
import edu.stanford.smi.protegex.owl.model.impl.DefaultRDFSDatatype;

import edu.stanford.smi.protege.util.Log;
import java.util.logging.Level;

/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class CloneTreePanel extends TreePanel implements ClsListener, KnowledgeBaseListener
{
    private boolean canDrop;
    private static final String SUBCLASSOF = "rdfs:subClassOf";

    public CloneTreePanel(NCIEditTab tab, Instance selectedInstance,
    	TreeNode dmtn,  String code, Vector treeitems)
    {
		super(tab, selectedInstance, dmtn, code, treeitems);
    }

    public CloneTreePanel(NCIEditTab tab, Instance selectedInstance, OWLModel kb)
    {
		super(tab, selectedInstance, kb);
		type=TreePanel.TYPE_CLONE;
    }


	public boolean containsCls(){
		if (droppedCls==null)
			return false;
		return true;
	}



	void addPopup(final JTree edit_tree)
	{
		model = (DefaultTreeModel) edit_tree.getModel();

		m_popup = new JPopupMenu();
		m_action = new AbstractAction()
		{
			public void actionPerformed(ActionEvent e)
			{
				if (m_clickedPath==null)
					return;
				if (edit_tree.isExpanded(m_clickedPath))
					edit_tree.collapsePath(m_clickedPath);
				else
					edit_tree.expandPath(m_clickedPath);
			}
		};
		m_popup.add(m_action);
		m_popup.addSeparator();

/*
		Action a1 = new AbstractAction("Delete")
		{
			public void actionPerformed(ActionEvent e)
			{
        		TreeNode selectedNode
            		= (TreeNode) edit_tree.getLastSelectedPathComponent();

				TreeItem item = (TreeItem)selectedNode.getUserObject();

				if (item.getType() == TreeItem.TYPE_PROPERTY)
				{
					String prop_name = item.getName();
					String prop_value = item.getValue();

					if (wrapper == null)
					{
						wrapper = tab.getOWLWrapper();
					}

					Boolean value = wrapper.isReadOnly(prop_name);

					if (Boolean.TRUE.equals(value))
					{
						ModalDialog.showMessageDialog(null, "Cannot delete " + prop_name + "; it is read only.");
						System.out.println("WARNING: " + prop_name + " is read only.");
						return;
					}
			    }


				if (selectedNode.getParent() != null)
				{
					System.out.println("Removing selected node");
					model.removeNodeFromParent(selectedNode);
					model.reload();
					edit_tree.repaint();
				}
				else
				{
					System.out.println("WARNING: selectedNode.getParent() == null");
				}
			}
		};
*/

		Action a1 = new AbstractAction("Delete")
		{
			public void actionPerformed(ActionEvent e)
			{
        		TreeNode selectedNode
            		= (TreeNode) edit_tree.getLastSelectedPathComponent();

				TreeItem item = (TreeItem)selectedNode.getUserObject();

				if (item.getType() == TreeItem.TYPE_PROPERTY)
				{
					String prop_name = item.getName();

					// 042507 (Cannot delete preferred name)
					String pt_name = wrapper.getSlotName(tab.PREFERRED_NAME);
					if (prop_name.compareTo(pt_name) == 0)
					{
						ModalDialog.showMessageDialog(null, "Cannot delete " + prop_name + ".");
						System.out.println("WARNING: Cannot delete " + prop_name +".");
						return;
					}

					String prop_value = item.getValue();

					if (wrapper == null)
					{
						wrapper = tab.getOWLWrapper();
					}


					Boolean value = wrapper.isReadOnly(prop_name);
					if (Boolean.TRUE.equals(value))
					{
						ModalDialog.showMessageDialog(null, "Cannot delete " + prop_name + "; it is read only.");
						System.out.println("WARNING: " + prop_name + " is read only.");
						return;
					}
			    }
			    // 041707
			    else if (item.getType() == TreeItem.TYPE_PARENT)
			    {
					int n = getSuperclassCount();
					if (n == 1)
					{
						ModalDialog.showMessageDialog(null, "Cannot delete the last named superclass.");
						return;
					}
				}


				if (selectedNode.getParent() != null)
				{
					model.removeNodeFromParent(selectedNode);
					model.reload();
					edit_tree.repaint();
				}
				else
				{
					System.out.println("WARNING: selectedNode.getParent() == null");
				}
			}
		};
		m_popup.add(a1);
		m_popup.addSeparator();

		Action a2 = new AbstractAction("Modify Property")
		{
			public void actionPerformed(ActionEvent e)
			{

        		TreeNode selectedNode
            		= (TreeNode) edit_tree.getLastSelectedPathComponent();

				TreeItem item = (TreeItem)selectedNode.getUserObject();

				if (item.getType() != TreeItem.TYPE_PROPERTY)
				{
					return;
				}

                String prop_name = item.getName();
                String prop_value = item.getValue();

				if (wrapper == null)
				{
					wrapper = tab.getOWLWrapper();
				}

                Boolean value = wrapper.isReadOnly(prop_name);
                if (Boolean.TRUE.equals(value))
                {
					ModalDialog.showMessageDialog(null, "Cannot modify " + prop_name + "; it is read only.");
					System.out.println("WARNING: " + prop_name + " is read only.");
					return;
				}

				if (isNonEditable(prop_name))
				{
					ModalDialog.showMessageDialog(null, "Cannot add " + prop_name + "; it is created by computer.");
					System.out.println("WARNING: " + prop_name + " is assigned by computer.");
					return;
				}

                boolean hasAuthorization = tab.getFilter().canModify(item, initialState);
                if (!hasAuthorization)
                {
					String user = tab.getUserName();
					System.out.println("User " + user + " does not have privilege to modify this property.");
					String msg = "User " + user + " does not have privilege to modify this property.";

					ModalDialog.showMessageDialog(null, msg);
					return;
				}

/*
				if (prop_name.equals(tab.getSlotName(NCIEditTab.FULL_SYN)) ||
				    prop_name.equals(tab.getSlotName(NCIEditTab.DEFINITION)) ||
					prop_name.equals(tab.getSlotName(NCIEditTab.GO_ANNOTATION)) ||
					prop_name.equals(tab.getSlotName(NCIEditTab.LONG_DEFINITION)) ||
					prop_name.equals(tab.getSlotName(NCIEditTab.ALT_LONG_DEFINITION)) ||
					prop_name.equals(tab.getSlotName(NCIEditTab.ALT_DEFINITION))) {

				//if (prop_name.equals(tab.getSlotName(NCIEditTab.FULL_SYN)) || prop_name.equals(tab.getSlotName(NCIEditTab.DEFINITION))){
*/
				if (
				    prop_name.equals(tab.getSlotName(NCIEditTab.DEFINITION)) ||
					prop_name.equals(tab.getSlotName(NCIEditTab.GO_ANNOTATION)) ||
					prop_name.equals(tab.getSlotName(NCIEditTab.LONG_DEFINITION)) ||
					prop_name.equals(tab.getSlotName(NCIEditTab.ALT_LONG_DEFINITION)) ||
					prop_name.equals(tab.getSlotName(NCIEditTab.ALT_DEFINITION))) {

				//if (prop_name.equals(tab.getSlotName(NCIEditTab.FULL_SYN)) || prop_name.equals(tab.getSlotName(NCIEditTab.DEFINITION))){

					CustomizedAnnotationDialog dialog
						= new CustomizedAnnotationDialog(tab.getEditPanel(), prop_name, prop_value, -1, new NCIAnnotationsTableModel(kb));

                    if (!dialog.cancelBtnPressed)
                    {
						String newValue = dialog.getValue();
						item.setValue(newValue);
						item.setNameValue(prop_name + ": " + item.getValue());
						selectedNode.setUserObject((Object) item);
						model.reload();
				    }
				}

				else if (prop_name.equals(tab.getSlotName(NCIEditTab.FULL_SYN)))
				{

				//if (prop_name.equals(tab.getSlotName(NCIEditTab.FULL_SYN)) || prop_name.equals(tab.getSlotName(NCIEditTab.DEFINITION))){

					CustomizedAnnotationDialog dialog =
						new CustomizedAnnotationDialog(tab.getEditPanel(), prop_name, prop_value, -1, new NCIFULLSYNTableModel(tab.getEditPanel(), kb));

                    if (!dialog.cancelBtnPressed)
                    {
						String newValue = dialog.getValue();
						item.setValue(newValue);
						item.setNameValue(prop_name + ": " + item.getValue());

						selectedNode.setUserObject((Object) item);
						model.reload();
				    }

				} else{


					Slot slot = kb.getSlot(prop_name);
					RDFProperty property = (RDFProperty) slot;
					String[] allowedvalues = getAllowableValues(property);
					if (allowedvalues != null)
					{
						JList valueList;
						valueList = ComponentFactory.createList(null);
						valueList.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

						Arrays.sort(allowedvalues);
						java.util.List list = Arrays.asList(allowedvalues);
						valueList.setListData(list.toArray());
						//valueList.setSelectedIndex(0);
						JScrollPane comp = new JScrollPane(valueList);

						valueList.setSelectedValue(prop_value, true);

						String label = property.getBrowserText();
						LabeledComponent lc = new LabeledComponent(label, comp);

						lc.setPreferredSize(new Dimension(400, 120));
						int r = ProtegeUI.getModalDialogFactory().showDialog(tab, lc, "Add Annotation", ModalDialogFactory.MODE_OK_CANCEL);
						if (r == ModalDialogFactory.OPTION_OK) {

							String newValue = (String) valueList.getSelectedValue();

							if (newValue.compareTo("") == 0)
							{
								Component warning_label = new JLabel("Property value is mandatory.");
								LabeledComponent lc2 = new LabeledComponent("", warning_label);
								int k = ProtegeUI.getModalDialogFactory().showDialog(tab.getEditPanel(), lc2, "WARNING", ModalDialogFactory.MODE_CLOSE);
							}
							else
							{
								item.setValue(newValue);
								item.setNameValue(prop_name + ": " + item.getValue());
								selectedNode.setUserObject((Object) item);
								model.reload();

							}
						}
					}

					else
					{

						JTextArea textArea = new JTextArea(value.toString());
						textArea.setLineWrap(true);
						textArea.setWrapStyleWord(true);
						textArea.setText(prop_value);

						Component comp = new JScrollPane(textArea);
						LabeledComponent lc = new LabeledComponent(prop_name, comp);

						lc.setPreferredSize(new Dimension(400, 120));
						int r = ProtegeUI.getModalDialogFactory().showDialog(tab.getEditPanel(), lc, "Edit Annotation", ModalDialogFactory.MODE_OK_CANCEL);
						if (r == ModalDialogFactory.OPTION_OK) {
							String newValue = textArea.getText();

							if (newValue.compareTo("") == 0)
							{
								Component warning_label = new JLabel("Property value is mandatory.");
								LabeledComponent lc2 = new LabeledComponent("", warning_label);
								int k = ProtegeUI.getModalDialogFactory().showDialog(tab.getEditPanel(), lc2, "WARNING", ModalDialogFactory.MODE_CLOSE);
							}
							else
							{
								item.setValue(newValue);
								item.setNameValue(prop_name + ": " + item.getValue());
								selectedNode.setUserObject((Object) item);
								model.reload();

							}
						}
				    }
				}

            }

		};

		m_popup.add(a2);
		m_popup.addSeparator();

//		Action a3 = new AbstractAction("Modify Restriction")
//		{
		Action a3 = new AbstractAction("Modify a restriction ...",
					NCIOWLIcons.getImageIcon("edit")) {

			public void actionPerformed(ActionEvent e) {

				TreeNode selectedNode
					= (TreeNode) edit_tree.getLastSelectedPathComponent();

				TreeItem item = (TreeItem) selectedNode.getUserObject();

				if (item.getType() != TreeItem.TYPE_RESTRICTION)
				{
					return;
				}

				boolean retval = modifyRestriction(item);//, item.getCls(), item.getIsDefining());
				if (retval)
				{
					selectedNode.setUserObject((Object) item);
					model.reload();
					edit_tree.repaint();
				}
				//edit_tree.repaint();

			}
		};

		m_popup.add(a3);
		m_popup.addSeparator();

		Action a4 = new AbstractAction("Add Property")
		{
			public void actionPerformed(ActionEvent e)
			{
        		TreeNode selectedNode
            		= (TreeNode) edit_tree.getLastSelectedPathComponent();

                if (selectedNode != null)
                {
					String currValue = (String)selectedNode.getUserObject().toString();
					CreatePropertyDialog dlg = new CreatePropertyDialog(tab, "", "");

					String prop_name = dlg.getPropertyName();
/*
					if (isNonEditable(prop_name))
					{
						ModalDialog.showMessageDialog(null, "Cannot add " + prop_name + "; it is created by computer.");
						System.out.println("WARNING: " + prop_name + " is assigned by computer.");
						return;
					}
*/
					String prop_value = dlg.getPropertyValue();
					if (prop_name.compareTo("") != 0 && prop_value.compareTo("") != 0)
					{
						TreeNode child_node = addProperty(prop_name, prop_value, null);
						edit_tree.repaint();
					}

			    }
			}
		};
		m_popup.add(a4);
		m_popup.addSeparator();

		Action a5 =
				new AbstractAction("Create a restriction ...",
						OWLIcons.getCreateIcon(OWLIcons.OWL_RESTRICTION))
		{
			String role_name = "";
			String role_value = "";
			String role_modifier = "";
			boolean isDefining = false;
			OWLRestriction restriction = null;

			public void actionPerformed(ActionEvent e)
			{
				boolean cancelBtnPressed = false;
				boolean dataEntryComplete = false;

				while (!cancelBtnPressed || !dataEntryComplete)
				{
					//final NCIRoleGroupEditorPanel rolegroupPanel = new NCIRoleGroupEditorPanel
					//   (tab, kb, cond_model_1, -1, null);

					final NCIRoleGroupEditorPanel rolegroupPanel = new NCIRoleGroupEditorPanel
					   (tab, kb);//, cond_model_1, -1, null);


					String label = "Roles (Simple Restrictions)";
					LabeledComponent lc = new LabeledComponent(label, rolegroupPanel);

					final String create_label = "Create a Restriction";
					Action createRestrictionAction_1 =
							new AbstractAction("Create a role (simple restriction)...",
									OWLIcons.getCreateIcon(OWLIcons.OWL_RESTRICTION)) {

						public void actionPerformed(ActionEvent e)
						{
							OWLRestriction r = null;
							boolean completed = false;

							while (!completed)
							{
								NCICreateRolePanel dlg = new NCICreateRolePanel(rolegroupPanel, kb,
									kb.getCls(OWLNames.Cls.SOME_VALUES_FROM_RESTRICTION),
									null, null, null);

								int ans = ModalDialog.showDialog(null, dlg, create_label, ModalDialog.MODE_OK_CANCEL);
								if (ans == ModalDialog.OPTION_OK)
								{
									r = dlg.getResult();
									if (r != null)
									{
										rolegroupPanel.getRoleGroupTableModel().addRow(r);
										completed = true;
										break;
									}
									else
									{
										try {
											Object[] options = {"OK"};
											int ans2 = JOptionPane.showOptionDialog(tab, "Incomplete data entry.", "Confirmation",
											JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
											null, options, options[0]);
											completed = false;
										}
										catch(Exception ex){
											Log.getLogger().log(Level.WARNING, "Exception caught", ex);
										}
									}
								}
								else
								{
									completed = true;
								}
							}
						}
					};

					//lc.addHeaderSeparator();
					JButton hb_add_6 = lc.addHeaderButton(createRestrictionAction_1);


					//////////////////////////////////////////////////////////////////////////////////////////////////

				   final String modify_label = "Modify a Restriction";
					Action modifyRestrictionAction_1 =
							new AbstractAction("Modify a role (simple restriction)...",
									NCIOWLIcons.getImageIcon("edit")) {

						public void actionPerformed(ActionEvent e)
						{
							OWLRestriction r = null;
							boolean completed = false;


							int selIndex = rolegroupPanel.getRoleGroupTable().getSelectedRow();
							if (selIndex == -1) return;

							r = rolegroupPanel.getRoleGroupTableModel().getRole(selIndex);

							RDFProperty p = r.getOnProperty();
							String prop_name = p.getName();

							String fillerText = r.getFillerText();
							String prop_modifier = wrapper.getRestrictionType(r);

							String label = wrapper.restrictionType2Label(prop_modifier);
							edu.stanford.smi.protege.model.Cls metaCls = wrapper.getMetaClsByName(label);

							int group_number = rolegroupPanel.getRoleGroupTableModel().getRoleGroupNumber(selIndex);

							while (!completed)
							{
								NCICreateRolePanel dlg = new NCICreateRolePanel(rolegroupPanel, kb,
									metaCls,
									p, fillerText, null);


								int ans = ModalDialog.showDialog(null, dlg, modify_label, ModalDialog.MODE_OK_CANCEL);
								if (ans == ModalDialog.OPTION_OK)
								{
									r = dlg.getResult();
									if (r != null)
									{
										System.out.println(r.getBrowserText());
										//rolegroupPanel.getRoleGroupTableModel().addRow(r);

										rolegroupPanel.getRoleGroupTableModel().updateRestriction(r, selIndex);

										completed = true;
										break;
									}
									else
									{
										try {
											Object[] options = {"OK"};
											int ans2 = JOptionPane.showOptionDialog(tab, "Incomplete data entry.", "Confirmation",
											JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
											null, options, options[0]);
											completed = false;
										}
										catch(Exception ex){
											Log.getLogger().log(Level.WARNING, "Exception caught", ex);
										}
									}
								}
								else
								{
									completed = true;
								}
							}
						}
					};

					lc.addHeaderSeparator();
					JButton hb_modify_6 = lc.addHeaderButton(modifyRestrictionAction_1);

					//////////////////////////////////////////////////////////////////////////////////////////////////

					lc.addHeaderSeparator();
					JButton hb_delete_6 = lc.addHeaderButton(new NCIDeleteRoleAction(rolegroupPanel, tab, rolegroupPanel.getRoleGroupTable()));
					//String label = "Create Restriction";

					int result = ModalDialog.showDialog(null, lc, create_label, ModalDialog.MODE_OK_CANCEL);

					if (result == ModalDialog.OPTION_OK)
					{
						RDFSClass restriction = rolegroupPanel.getRolegroups();
						if (restriction == null)
						{
							try {
								Object[] options = {"OK"};
								int ans = JOptionPane.showOptionDialog(tab, "Incomplete data entry.", "Confirmation",
								JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
								null, options, options[0]);
								dataEntryComplete = false;
							}
							catch(Exception ex){
								Log.getLogger().log(Level.WARNING, "Exception caught", ex);
							}
						}
						else
						{
							// Process data
							HashMap hmap = rolegroupPanel.getRoleGroupTableModel().getRolegroups();
							RDFSClass aClass = rolegroupPanel.roleGroupHash2RDFSClass(hmap);
							boolean isDedefining = rolegroupPanel.getIsDefining();

						    TreeNode child_node = addRestriction(aClass, isDedefining);
						    edit_tree.repaint();
						    break;
						}
					}
					else
					{
						break;
					}
				}
			}
		};

		m_popup.add(a5);
		m_popup.addSeparator();

		Action a6 = new AbstractAction("Add Association")
		{
			public void actionPerformed(ActionEvent e)
			{
				CreateObjPropertyDialog dlg = new CreateObjPropertyDialog(tab);
				if (!dlg.cancelBtnPressed)
				{
					String association_name = dlg.getPropertyName();
					String association_value = dlg.getPropertyValue();

					Slot slot = dlg.getSelectedSlot();
					Cls  cls = dlg.getSelectedCls();
/*
					if (association_value.compareTo(tab.getClsName(NCIEditTab.RETIRED_CONCEPTS)) == 0 ||
						association_value.compareTo(tab.getClsName(NCIEditTab.PREMERGED_CONCEPTS)) == 0 ||
						association_value.compareTo(tab.getClsName(NCIEditTab.PRERETIRED_CONCEPTS)) == 0)
					{
						ProtegeUI.getModalDialogFactory().showErrorMessageDialog(tab, "Cannot select " +  association_value + ".");
						return;
					}
*/
					//010407
					boolean invaldSelection = wrapper.isPremerged((OWLNamedClass) cls);
					if (invaldSelection)
					{
						ProtegeUI.getModalDialogFactory().showErrorMessageDialog(tab, "Cannot select a pre-merged concept.");
						return;
					}
					invaldSelection = wrapper.isPreretired((OWLNamedClass) cls);
					if (invaldSelection)
					{
						ProtegeUI.getModalDialogFactory().showErrorMessageDialog(tab, "Cannot select a pre-retired concept.");
						return;
					}
					invaldSelection = wrapper.isRetired((OWLNamedClass) cls);
					if (invaldSelection)
					{
						ProtegeUI.getModalDialogFactory().showErrorMessageDialog(tab, "Cannot select a retired concept.");
						return;
					}

					if (association_name.compareTo("") != 0 &&
						association_value.compareTo("") != 0)
					{
//							System.out.println("Add association: " +  association_name
//				   + " " + association_value );

						TreeNode child_node = addAssociation(slot, cls, association_name, association_value);
						edit_tree.repaint();
					}
				}
			 }



		};
		m_popup.add(a6);

		m_popup.addSeparator();

		Action a7 = new AbstractAction("Add Parent")
		{
			public void actionPerformed(ActionEvent e)
			{
				java.awt.TextField textfield = new TextField();
				String label = "Select a class.";

				Collection clses = kb.getRootClses();
				Cls cls = DisplayUtilities.pickConcreteCls(textfield, kb, clses);
				if (cls == null)
				{
					System.out.println("cls is null");
				}
				else
				{
					//String prop_value = cls.getBrowserText();
					/*
					String prop_value = tab.getOWLWrapper().getInternalName(cls);

					if (prop_value.compareTo(tab.getClsName(NCIEditTab.RETIRED_CONCEPTS)) == 0 ||
						prop_value.compareTo(tab.getClsName(NCIEditTab.PRERETIRED_CONCEPTS)) == 0 ||
						prop_value.compareTo(tab.getClsName(NCIEditTab.PREMERGED_CONCEPTS)) == 0)
					{
						ProtegeUI.getModalDialogFactory().showErrorMessageDialog(tab, "Cannot select " +  prop_value + ".");
						return;
					}

					//System.out.println("Add parent: " + cls.getBrowserText());
					//String prop_name = SUBCLASSOF;//"rdfs:subClassOf";

					//TreeNode child_node = addParent(prop_value);
					*/
					//010407
					boolean invaldSelection = wrapper.isPremerged((OWLNamedClass) cls);
					if (invaldSelection)
					{
						ProtegeUI.getModalDialogFactory().showErrorMessageDialog(tab, "Cannot select a pre-merged concept.");
						return;
					}
					invaldSelection = wrapper.isPreretired((OWLNamedClass) cls);
					if (invaldSelection)
					{
						ProtegeUI.getModalDialogFactory().showErrorMessageDialog(tab, "Cannot select a pre-retired concept.");
						return;
					}
					invaldSelection = wrapper.isRetired((OWLNamedClass) cls);
					if (invaldSelection)
					{
						ProtegeUI.getModalDialogFactory().showErrorMessageDialog(tab, "Cannot select a retired concept.");
						return;
					}


					TreeNode child_node = addParent(cls);
				}

				edit_tree.repaint();
			}
		};
		m_popup.add(a7);

		m_popup.addSeparator();

		Action a8 = new AbstractAction("Copy")
		{
			public void actionPerformed(ActionEvent e)
			{
        		TreeNode selectedNode
            		= (TreeNode) edit_tree.getLastSelectedPathComponent();

				TreeItem item = (TreeItem)selectedNode.getUserObject();
                //[#6179] User is not prevented from duplicating rdfs:label in the Copy Tab
				if (item.getType() == TreeItem.TYPE_PROPERTY)
				{
					String prop_name = item.getName();
					if (prop_name.compareTo("rdfs:label") == 0)
					{
						ProtegeUI.getModalDialogFactory().showErrorMessageDialog(tab, "Cannot copy " + prop_name + ".");
						return;
					}

					Boolean value = wrapper.isReadOnly(prop_name);
					if (Boolean.TRUE.equals(value))
					{
						ProtegeUI.getModalDialogFactory().showErrorMessageDialog(tab, "Cannot copy " + prop_name + ".");
						return;
					}
 				}
				tab.setCopyItem(item);
			}
		};
		m_popup.add(a8);
		m_popup.addSeparator();

		Action a9 = new AbstractAction("Paste")
		{
			public void actionPerformed(ActionEvent e)
			{
				TreeItem copied_item = tab.getCopyItem();
				addTreeItem(copied_item);
			}
		};
		m_popup.add(a9);

		edit_tree.add(m_popup);
		edit_tree.addMouseListener(new PopupTrigger());
	}




	private void addTreeItem(TreeItem item)
	{
		if (item.getType() == TreeItem.TYPE_PROPERTY)
			addProperty(item.getName(), item.getValue(), null);
		else if (item.getType() == TreeItem.TYPE_RESTRICTION)
		{
			//addRestriction(item.getName(), item.getValue(), item.getModifier());
			addRestriction(item.getCls(), item.getIsDefining());//Name(), item.getValue(), item.getModifier());
		}
		if (item.getType() == TreeItem.TYPE_PARENT)
			//addParent(item.getValue());

			addParent((Cls) item.getCls());

		edit_tree.repaint();
	}


	private boolean modifyRestriction(TreeItem item) //RDFSClass aClass, boolean defining)
	{
			OWLClass aClass_old = (OWLClass) item.getCls();//aClass;

			boolean defining = item.getIsDefining();

            boolean cancelBtnPressed = false;
            boolean dataEntryComplete = false;

			while (!cancelBtnPressed || !dataEntryComplete)
			{
			    final NCIRoleGroupEditorPanel rolegroupPanel = new NCIRoleGroupEditorPanel(tab, kb,
			       //(OWLNamedClass) getDroppedCls(), aClass_old, defining);

			       (OWLNamedClass) selectedInstance, aClass_old, defining);


                String label = "Roles (Simple Restrictions)";
				LabeledComponent lc = new LabeledComponent(label, rolegroupPanel);

				Action createRestrictionAction_1 =
						new AbstractAction("Create a Role (simple restriction)...",
								OWLIcons.getCreateIcon(OWLIcons.OWL_RESTRICTION)) {

					public void actionPerformed(ActionEvent e)
					{
						NCICreateRolePanel dlg = new NCICreateRolePanel(rolegroupPanel, kb,
							kb.getCls(OWLNames.Cls.SOME_VALUES_FROM_RESTRICTION),
							null, null, null);

						int ans = ModalDialog.showDialog(null, dlg, "Create a Role (simple restriction)", ModalDialog.MODE_OK_CANCEL);
						if (ans == ModalDialog.OPTION_OK)
						{
							OWLRestriction r = dlg.getResult();
							rolegroupPanel.getRoleGroupTableModel().addRow(r);
						}
					}
				};

				//lc.addHeaderSeparator();
				JButton hb_add_6 = lc.addHeaderButton(createRestrictionAction_1);
				//////////////////////////////////////////////////////////////////////////////////////////////////

			   final String modify_label = "Modify a Restriction";
				Action modifyRestrictionAction_1 =
						new AbstractAction("Modify a role (simple restriction)...",
								NCIOWLIcons.getImageIcon("edit")) {

					public void actionPerformed(ActionEvent e)
					{
						OWLRestriction r = null;
						boolean completed = false;


						int selIndex = rolegroupPanel.getRoleGroupTable().getSelectedRow();
						if (selIndex == -1) return;
                        r = rolegroupPanel.getRoleGroupTableModel().getRole(selIndex);

						RDFProperty p = r.getOnProperty();
						String prop_name = p.getName();

						String fillerText = r.getFillerText();
						String prop_modifier = wrapper.getRestrictionType(r);

						String label = wrapper.restrictionType2Label(prop_modifier);
						edu.stanford.smi.protege.model.Cls metaCls = wrapper.getMetaClsByName(label);

                        int group_number = rolegroupPanel.getRoleGroupTableModel().getRoleGroupNumber(selIndex);

						while (!completed)
						{
							NCICreateRolePanel dlg = new NCICreateRolePanel(rolegroupPanel, kb,
								metaCls,
								p, fillerText, null);


							int ans = ModalDialog.showDialog(null, dlg, modify_label, ModalDialog.MODE_OK_CANCEL);
							if (ans == ModalDialog.OPTION_OK)
							{
								r = dlg.getResult();
								if (r != null)
								{
									System.out.println(r.getBrowserText());
									//rolegroupPanel.getRoleGroupTableModel().addRow(r);

									rolegroupPanel.getRoleGroupTableModel().updateRestriction(r, selIndex);

									completed = true;
									break;
								}
								else
								{
									try {
										Object[] options = {"OK"};
										int ans2 = JOptionPane.showOptionDialog(tab, "Incomplete data entry.", "Confirmation",
										JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
										null, options, options[0]);
										completed = false;
									}
									catch(Exception ex){
										Log.getLogger().log(Level.WARNING, "Exception caught", ex);
									}
								}
							}
							else
							{
								completed = true;
							}
						}
					}
				};

				lc.addHeaderSeparator();
				JButton hb_modify_6 = lc.addHeaderButton(modifyRestrictionAction_1);

//////////////////////////////////////////////////////////////////////////////////////////////////


				JButton hb_delete_6 = lc.addHeaderButton(new NCIDeleteRoleAction(rolegroupPanel, tab, rolegroupPanel.getRoleGroupTable()));

				int result = ModalDialog.showDialog(null, lc, "Edit a Restriction", ModalDialog.MODE_OK_CANCEL);

				if (result == ModalDialog.OPTION_OK)
				{
					RDFSClass restriction = rolegroupPanel.getRolegroups();
					if (restriction == null)
					{
						try {
							Object[] options = {"OK"};
							int ans = JOptionPane.showOptionDialog(tab, "Incomplete data entry.", "Confirmation",
							JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
							null, options, options[0]);
							dataEntryComplete = false;
						}
						catch(Exception ex){
							Log.getLogger().log(Level.WARNING, "Exception caught", ex);
						}
					}
					else
					{
						// Process data
						HashMap hmap = rolegroupPanel.getRoleGroupTableModel().getRolegroups();
						RDFSClass aClass = rolegroupPanel.roleGroupHash2RDFSClass(hmap);
						boolean isDefining = rolegroupPanel.getIsDefining();
						item.setCls(aClass);
						item.setNameValue(aClass.getBrowserText());
						item.setIsDefining(isDefining);

					    //model.modifyRow(selIndex, (RDFSClass) aClass, rolegroupPanel.getIsDefining());

						//enableSaveButton(true);
						return true;
					}
				}
				else
				{
					return false;
				}
			}
			return false;
	}



   public boolean deleteRestriction(RDFSClass r)
    {
		Enumeration enumeration = root_node.children();
		while (enumeration.hasMoreElements())
		{
			TreeNode dmtn = (TreeNode) enumeration.nextElement();
			TreeItem item = (TreeItem) dmtn.getUserObject();
			if (item.getType() == TreeItem.TYPE_RESTRICTION)
			{
				if (item.toString().compareTo(r.getBrowserText()) == 0)
				{
					model.removeNodeFromParent(dmtn);
					model.reload();
				}
		    }
		}
		return true;
    }

    private String[] getAllowableValues(RDFProperty property)
    {
		boolean rangeDefined = property.isRangeDefined();
		if (!rangeDefined) return null;
		boolean includingSuperproperties = false;
		Vector v = new Vector();
		Collection c = property.getRanges(includingSuperproperties);
		if (c == null) return null;
		for(Iterator iterator = c.iterator(); iterator.hasNext();){
			Object value = (Object) iterator.next();

			if (value instanceof DefaultRDFSDatatype)
			{
				return null;
			}
			else
			{
				Collection values = ((OWLDataRange) value).getOneOf().getValues();
				if (values == null) return null;
				if (values.size() > 0)
				{
					for(Iterator iterator2 = values.iterator(); iterator2.hasNext();){
						Object value2 = (Object) iterator2.next();
						v.add(value2.toString());
					}
				}
		    }
		}
		Object[] objs = v.toArray();
		String[] allowedvalues = new String[objs.length];

		for (int i=0; i<objs.length; i++)
		{
			allowedvalues[i] = (String) objs[i];
		}
		return allowedvalues;
	}

    private String getTreeItemKey(TreeItem item)
    {
		int type = item.getType();
		if (item.getName().compareTo("rdfs:label") == 0 ||
		    item.getName().compareTo("code") == 0 ||
		    item.getName().compareTo("hasType") == 0)
		{
			type = 0;
		}

		String key = "" + type + item.toString();
		return key.toUpperCase();
	}

    private TreeItem[] vector2Array(Vector v)
    {
		TreeItem[] a = new TreeItem[v.size()];
		for (int i=0; i<a.length; i++)
		{
			TreeItem item = (TreeItem) v.elementAt(i);
			a[i] = item;
		}
		return a;
	}


    private TreeItem[] sortTreeItems(TreeItem[] a)
    {
        boolean done = false;
        if (a == null || a.length == 0) {
            return a;
        }
        while (!done) {
            done = true;
            for (int i = 0;i < a.length-1;i++) {
			    TreeItem item1 = (TreeItem) a[i];
			    TreeItem item2 = (TreeItem) a[i+1];

				String key_1 = getTreeItemKey(item1);
				String key_2 = getTreeItemKey(item2);

                if (key_1.compareTo(key_2) <= 0) {
                } else {
                    done = false;
                    TreeItem tmp = a[i];
                    a[i] = a[i+1];
                    a[i+1] = tmp;
                }
            }
        }
        return a;
	}


    private int findNodePosition(TreeNode parent, TreeItem child)
    {
		if (parent.getChildCount() == 0)
		{
			return 0;
		}

        String key_1 = getTreeItemKey(child);

		Vector v = new Vector();
        Enumeration enumeration = parent.children();
		while (enumeration.hasMoreElements())
		{
			TreeNode dmtn = (TreeNode) enumeration.nextElement();
			TreeItem item = (TreeItem) dmtn.getUserObject();
			v.add(item);
		}
		//042506
		//v.add(child);
		TreeItem[] a = vector2Array(v);
		TreeItem[] b = sortTreeItems(a);

		for (int i=0; i<b.length; i++)
		{
			TreeItem item = (TreeItem) b[i];
			String key_2 = getTreeItemKey(item);

			//042506
			//if (key_2.compareTo(key_1) == 0)
			if (key_2.compareTo(key_1) > 0)
			{
				return i;
			}
		}
		return parent.getChildCount();
	}

	private TreeNode addObject(TreeNode parent,
							   Object child,
							   boolean shouldBeVisible) {


		if (parent == null)
		{
			System.out.println("WARNING addObject -- parent = null.");
		}

		//061306
		//TreeNode childNode = new TreeNode(child);
		TreeNode childNode = new TreeNode(child, ((TreeItem)child).getType());

		model.insertNodeInto(childNode,
							 parent,
							 //parent.getChildCount());
		                     findNodePosition(parent, (TreeItem) child));

		model.reload();

		if (shouldBeVisible) {
			edit_tree.scrollPathToVisible(new TreePath(childNode.getPath()));
		}
		return childNode;
	}


	public TreeNode addAssociation(Slot slot, Cls cls, String name, String value)
	{
		if (value.compareTo("owl:Class") == 0) return null;

		TreeItem item = new TreeItem();
		item.setType(TreeItem.TYPE_ASSOCIATION);
		item.setName(name);

		RDFProperty aProperty = (RDFProperty) slot;
		item.setProperty(aProperty);

		RDFSClass aClass = (RDFSClass) cls;
		item.setCls(aClass);

		item.setValue(value);
		item.setNameValue(name + ": " + value);

		return addObject(root_node, (Object) item, true);
	}

    private int getSuperclassCount()
    {
		int n = 0;
		Enumeration enumeration = root_node.children();
		while (enumeration.hasMoreElements())
		{
			TreeNode dmtn = (TreeNode) enumeration.nextElement();
			TreeItem item = (TreeItem) dmtn.getUserObject();
            if (item.getType() == TreeItem.TYPE_PARENT)
			{
			    n++;
		    }
		}
		return n;
    }

	public void 	directInstanceAdded(ClsEvent event){}
	public void 	directInstanceRemoved(ClsEvent event){}
 	public void 	directSubclassAdded(ClsEvent event){}
 	public void 	directSubclassMoved(ClsEvent event){}
 	public void 	directSubclassRemoved(ClsEvent event){
	}
 	public void 	directSuperclassAdded(ClsEvent event){}
 	public void 	directSuperclassRemoved(ClsEvent event){}
 	public void 	templateFacetAdded(ClsEvent event){}
 	public void 	templateFacetRemoved(ClsEvent event){}
 	public void 	templateFacetValueChanged(ClsEvent event){}
 	public void 	templateSlotAdded(ClsEvent event){}
 	public void 	templateSlotRemoved(ClsEvent event){}
 	public void 	templateSlotValueChanged(ClsEvent event){}


 	////////////////////////////////////////////////////////
 	public void 	clsCreated(KnowledgeBaseEvent event){}
	public void 	clsDeleted(KnowledgeBaseEvent event){
		Cls cls = event.getCls();
		System.out.println(cls.isDeleted()+"|"+cls.isBeingDeleted());
		System.out.println(cls.getName()+"|"+cls.getBrowserText()+"|"+cls.getFrameID()+
			"|"+cls.isValid());
	}
	public void 	defaultClsMetaClsChanged(KnowledgeBaseEvent event){}
	public void 	defaultFacetMetaClsChanged(KnowledgeBaseEvent event){}
	public void 	defaultSlotMetaClsChanged(KnowledgeBaseEvent event){}
	public void 	facetCreated(KnowledgeBaseEvent event){}
	public void 	facetDeleted(KnowledgeBaseEvent event){}
	public void 	frameNameChanged(KnowledgeBaseEvent event){}
	public void 	instanceCreated(KnowledgeBaseEvent event){}
	public void 	instanceDeleted(KnowledgeBaseEvent event){}
	public void 	slotCreated(KnowledgeBaseEvent event){}
 	public void 	slotDeleted(KnowledgeBaseEvent event){}
}
