package gov.nih.nci.protegex.edit;

import java.io.*;
import java.util.*;
import org.w3c.dom.*;
import org.apache.xerces.parsers.*;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;
import org.xml.sax.InputSource;

/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class ComplexPropertyParser extends DefaultHandler
{
    public final static int TERM_NAME = 0;
    public final static int TERM_GROUP = 1;
    public final static int TERM_SOURCE = 2;
    public final static int TERM_SOURCE_CODE = 3;

	private static Vector properties;

    public static Vector parse(String complex_property)
    {
		try
		{
			complex_property="<root>"+complex_property+"</root>";
			DOMParser parser = new DOMParser();
			BufferedReader reader = new BufferedReader(new StringReader(complex_property));
			InputSource inputsource = new InputSource(reader);
			parser.parse(inputsource);
			Document document = parser.getDocument();

			properties = new Vector();
			traverse (document);
			reader.close();
		} catch (Exception e) {
			//System.out.println("ComplexPropertyParser "+e);
		}
		return properties;
	}

    private static void traverse (Node node)
    {
        int nodetype = node.getNodeType();
        if (nodetype == Node.ELEMENT_NODE)
        {
			NodeList children = node.getChildNodes();
			String nodevalue = children.item(0).getNodeValue();
			if (children != null && nodevalue!=null){
				properties.add(nodevalue);
			}
		}

		NodeList children = node.getChildNodes();
        if (children != null) {
            for (int i=0; i< children.getLength(); i++)
                traverse (children.item(i));
        }
    }


    public static HashMap parseXML(String complex_property)
    {
		HashMap name2val_hashmap = new HashMap();
		try
		{
			complex_property="<root>"+complex_property+"</root>";
			DOMParser parser = new DOMParser();
			BufferedReader reader = new BufferedReader(new StringReader(complex_property));
			InputSource inputsource = new InputSource(reader);
			parser.parse(inputsource);
			Document document = parser.getDocument();

			//properties = new Vector();
			traverse (document, name2val_hashmap);
			reader.close();
		} catch (Exception e) {
			System.out.println("ComplexPropertyParser "+e);
		}
		return name2val_hashmap;
	}


    private static void traverse (Node node, HashMap name2val_hashmap)
    {
        int type = node.getNodeType();
        if (type == Node.ELEMENT_NODE)
        {
            String nodename = node.getNodeName();
			NodeList children = node.getChildNodes();
			if (children!=null && children.item(0)!=null){
				String nodevalue = children.item(0).getNodeValue();
				if (nodevalue!=null)
					name2val_hashmap.put(nodename, nodevalue);
			}
		}
		NodeList children = node.getChildNodes();
        if (children != null) {
            for (int i=0; i< children.getLength(); i++)
                traverse (children.item(i), name2val_hashmap);
        }
    }



	public static String getText(String complex_property)
	{
		HashMap map = ComplexPropertyParser.parseXML(complex_property);
        if (map == null) return complex_property;
        if (map.isEmpty()) return complex_property;
        if (map.keySet().isEmpty()) return complex_property;
        String key = "def-definition";
        if (map.containsKey(key))
        {
			return (String) map.get(key);
		}
        key = "go-term";
        if (map.containsKey(key))
        {
			return (String) map.get(key);
		}
		return complex_property;
	}



	public static Vector parseFullSyn(String value){
		Vector v = new Vector();
		StringTokenizer st = null;

		if (value != null)
			st = new StringTokenizer(value, "|");

		int num_tokens = st.countTokens();
		if (num_tokens>=3){
			while (st.hasMoreTokens())
				v.add(st.nextToken());
		}
		return v;
	}


	public static String pipeDelim2XML(String value)
	{
		if (value.indexOf("term-name") != -1)
		{
			return value;
		}
		Vector v = parseFullSyn(value);

		if (v.size() == 0) return value;

		String t = "";
		if (v.size() > 0)
		{
			String s = (String) v.elementAt(0);
			t = t + "<term-name>" + s + "</term-name>";
		}
		if (v.size() > 1)
		{
			String s = (String) v.elementAt(1);
			t = t + "<term-group>" + s + "</term-group>";
		}
		if (v.size() > 2)
		{
			String s = (String) v.elementAt(2);
			t = t + "<term-source>" + s + "</term-source>";
		}
		if (v.size() > 3)
		{
			String s = (String) v.elementAt(3);
			t = t + "<source-code>" + s + "</source-code>";
		}
        return t;
	}


    public static void main(String[] args){
		Vector v = parse("<term-name>11</term-name><term-group>22</term-group>"+
				"<term-source>33</term-source><source-code>44</source-code>");
		for (int i=0; i<v.size(); i++)
			System.out.println(v.elementAt(i));
	}
}



