package gov.nih.nci.protegex.edit;

import edu.stanford.smi.protege.widget.*;
import edu.stanford.smi.protege.resource.*;
import edu.stanford.smi.protege.ui.*;


import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.ui.InstanceDisplay;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protege.widget.ClsWidget;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.ui.cls.*;
import edu.stanford.smi.protegex.owl.ui.clsdesc.DisjointClassesWidget;
import edu.stanford.smi.protegex.owl.ui.conditions.ConditionsWidget;
import edu.stanford.smi.protegex.owl.ui.icons.OWLIcons;
import edu.stanford.smi.protegex.owl.ui.resourcedisplay.ResourceDisplay;
import edu.stanford.smi.protegex.owl.ui.subsumption.InferredSubsumptionTreePanel;
import edu.stanford.smi.protegex.owl.ui.widget.WidgetUtilities;
import edu.stanford.smi.protegex.owl.model.impl.DefaultOWLObjectProperty;

import java.io.*;
import java.sql.Connection;
import java.util.*;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import edu.stanford.smi.protegex.owl.model.RDFSClass;

import edu.stanford.smi.protegex.owl.model.OWLProperty;
import edu.stanford.smi.protegex.owl.ui.widget.OWLUI;

/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class ConceptState
{
    static final int TYPE_UNKNOWN = -1;
	static final int TYPE_SPLIT = 0;
	static final int TYPE_PREMERGE = 1;
	static final int TYPE_MERGE = 2;
	static final int TYPE_PRERETIRE = 3;
	static final int TYPE_RETIRE = 4;
	static final int TYPE_CLONE = 5;

    private static final String SUBCLASSOF = "rdfs:subClassOf";

//////////////////////////////////////////////////////
    private static OWLWrapper wrapper = null;
    private OWLNamedClass cls = null;
    private QuickSortVecStrings sort = null;

	private	Vector old_child_vec = null;
	private	Vector old_source_role_vec = null;
    private Vector inboundRoles = null;
    private Vector oldSubclasses = null;

    Vector state_vec = null;
    OWLModel kb = null;
    HashSet hset = null;

    private Vector associationNames = null;

    public ConceptState(OWLWrapper wrapper, OWLNamedClass cls)
    {
		this.kb = cls.getOWLModel();
		this.cls = cls;
		this.sort = new QuickSortVecStrings();
    	this.state_vec = new Vector();

    	associationNames = new Vector();
        associationNames.add(RDFSNames.Slot.IS_DEFINED_BY);
        associationNames.add(RDFSNames.Slot.SEE_ALSO);
        associationNames.add(OWLNames.Slot.SAME_AS);
        associationNames.add(kb.getOWLDifferentFromProperty().getBrowserText());
        associationNames.add(kb.getOWLEquivalentPropertyProperty().getBrowserText());
        associationNames.add(kb.getOWLDisjointWithProperty().getBrowserText());

    	this.wrapper = wrapper;

		old_child_vec = new Vector();
		old_source_role_vec = new Vector();

		initialize(cls);

    }

    private void initialize(OWLNamedClass cls)
    {

		String name = wrapper.getInternalName(cls);
		String value = "";
		String restriction = "";
		String modifier = "";
		int cardinality = -1;

		TreeItem item = new TreeItem(TreeItem.TYPE_CONCEPT,
				name,
				value,
				restriction,
				modifier,
				cardinality);

		item.setNameValue(name);
        state_vec=new Vector();
        state_vec.add(item);
		try{
			getSlots(cls);
			//getParents((OWLNamedClass) cls);
			getRoles((OWLNamedClass) cls);
			getAssociations((OWLNamedClass) cls);
		}
		catch(Exception e){
			OWLUI.handleError(kb, e);
		}
    }


    public Vector get_state_vec()
    {
		return state_vec;
	}

    public void outputState(Vector v)
    {
		for (int i=0; i<v.size(); i++)
		{
			TreeItem item = (TreeItem) v.elementAt(i);
			System.out.println("Type: " + item.getType());
			System.out.println("Name: " + item.getName());
			System.out.println("Value: " + item.getValue());
			if (item.getModifier().compareTo("") != 0)
			{
				System.out.println("Modifier: " + item.getModifier());
		    }
			System.out.println("");
		}
	}

/*
	private void getParents(OWLNamedClass cls)
	{
		Collection parent_vec = wrapper.getDirectSuperclasses((RDFSClass) cls);

		if (parent_vec == null || parent_vec.isEmpty())
		{
			return;
		}

		if (parent_vec != null) {
			Iterator j = parent_vec.iterator();
			while(j.hasNext()){
				String value = (String)j.next();
				try {
                    TreeItem item = new TreeItem();
					item.setType(TreeItem.TYPE_PARENT);
					item.setName(SUBCLASSOF);
					item.setValue(value);
					item.setNameValue(SUBCLASSOF + ": " + value);

					item.setCls((RDFSClass) );

					state_vec.add(item);
				}
				catch(Exception e){
					OWLUI.handleError(owlModel, e);
				}

			}
		}
	}
*/

//082706
	private void getParents(OWLNamedClass cls)
	{
		Collection parent_vec = cls.getDirectSuperclasses();//cls.getDirectSuperclasses();

		if (parent_vec == null || parent_vec.isEmpty())
		{
			return;
		}

		if (parent_vec != null) {
			Iterator j = parent_vec.iterator();
			while(j.hasNext())
			{
				Object obj = j.next();
				if (obj instanceof OWLNamedClass)
				{
					Cls owl_cls = (Cls) obj;
					try {
						TreeItem item = new TreeItem();
						item.setType(TreeItem.TYPE_PARENT);
						item.setName(SUBCLASSOF);
						String cls_name = wrapper.getInternalName(owl_cls);
						item.setValue(cls_name);
						item.setNameValue(SUBCLASSOF + ": " + cls_name);

						item.setCls((RDFSClass) owl_cls);

						boolean retval = cls.hasEquivalentClass((RDFSClass) owl_cls);
						item.setIsDefining(false);
						if (retval)
						{
							item.setIsDefining(true);
						}

						state_vec.add(item);

					}
					catch(Exception e){
						OWLUI.handleError(kb, e);
					}

			    }


			}
		}
    }


	private void getSlots(Cls cls)
	{
		Collection ownslots = cls.getOwnSlots();
		if (ownslots != null) {
			Iterator j = ownslots.iterator();
			while(j.hasNext()){
				Slot slot = (Slot)j.next();
				Collection slotColl = cls.getOwnSlotValues(slot);
				if(slotColl == null || slotColl.isEmpty())
					continue;

				String slotName = slot.getName();

				if (slotName.equals(Model.Slot.NAME)) {
					continue;
				}
				else if (slotName.equals(Model.Slot.DIRECT_SUBCLASSES))
					continue;
				else if (slotName.equals(Model.Slot.DIRECT_INSTANCES))
					continue;
				else {
					getOwnslot(slotName, slotColl);
				}
			}
		}
	}

	private String getRestrictionString(OWLRestriction r)
	{
	    String prop = "";
		if (!(r instanceof OWLNamedClass))
        {
			String prop_modifier = wrapper.getRestrictionType(r);
			RDFProperty p = r.getOnProperty();
            String prop_name = p.getName();
		    String prop_value = r.getFillerText();
            prop = prop_name + ": " + prop_modifier + " " + prop_value;
		}
		return prop;
	}

/*
	private void getRoles(OWLNamedClass cls)
	{
		boolean includingSuperclassRestrictions = false; // no inherited ones
		java.util.Collection col = cls.getRestrictions(includingSuperclassRestrictions);
		for(Iterator iterator = col.iterator(); iterator.hasNext();)
		{
			OWLRestriction r = (OWLRestriction) iterator.next();
			if (!(r instanceof OWLNamedClass))
            {
				try {

                    TreeItem item = new TreeItem();

					RDFProperty p = r.getOnProperty();
					String prop_name = p.getName();
					String prop_value = r.getFillerText();

					String prop_modifier = wrapper.getRestrictionType(r);

					item.setNameValue(prop_name + ": " + prop_modifier + " " + prop_value);
                    item.setType(TreeItem.TYPE_RESTRICTION);
                    item.setName(prop_name);
                    item.setValue(prop_value);
                    item.setModifier(prop_modifier);

                    item.setNameValue(getRestrictionString(r));
                    item.setCls((RDFSClass) r);

                    state_vec.add(item);
				}
				catch(Exception e){
					OWLUI.handleError(owlModel, e);
				}
			}
		}
	}
*/
	private HashSet getOWLAnnotationProperties()
	{
		HashSet hset = new HashSet();
		Collection annotationSlots = kb.getOWLAnnotationProperties();
		for(Iterator iterator = annotationSlots.iterator(); iterator.hasNext();) {
			Slot slot = (Slot) iterator.next();
			String slotName = slot.getName();
			hset.add(slotName);
		}
		return hset;
	}


    private boolean isUserDefinedRDFProperty(String name)
    {
		Collection allProperties = kb.getVisibleUserDefinedRDFProperties();
		Collection selectableProperties = new ArrayList();
		for (Iterator it = allProperties.iterator(); it.hasNext();) {
			RDFProperty property = (RDFProperty) it.next();
			if (property.isAnnotationProperty()
				&& property.hasObjectRange()
				&& property instanceof DefaultOWLObjectProperty)
			{
				//if (property.getBrowserText().compareTo(name) == 0)
				if (property.getName().compareTo(name) == 0)
				{
					return true;
				}
			}
		}
		return false;
	}

    private String getStringLanguage(Object obj)
    {
		if (obj instanceof RDFSLiteral)
		{
			RDFSLiteral literal = (RDFSLiteral) obj;
            return literal.getLanguage();
		}
		//guoi
		/*else
		{
			return (String) obj;
		}*/
		return "";
	}

    private void getOwnslot(String slotname, Collection values)
    {
		if (hset == null)
		{
//			System.out.println("WARNING: hset == null in getOwnslot");
			hset = getOWLAnnotationProperties();
			if (hset == null)
			{
				return;
			}
		}
		if (!hset.contains(slotname))
		{
             if (slotname.compareTo(":DIRECT-SUPERCLASSES") != 0 &&
                 slotname.compareTo(":DIRECT-TYPE") != 0 &&
                 slotname.compareTo(":ROLE") != 0 &&
                 slotname.compareTo(SUBCLASSOF) != 0)
			 {
				 if (values == null) return;
				 Iterator i = values.iterator();
				 while(i.hasNext()) {
					 Object obj = i.next();
					 TreeItem item = new TreeItem();

					 if (obj instanceof OWLNamedClass)
					 {
						 OWLNamedClass cls = (OWLNamedClass) obj;
						 String internalName = wrapper.getInternalName(cls);

						 //if (cls.getBrowserText().compareTo("owl:Class") != 0)
						 if (internalName.compareTo("owl:Class") != 0)
						 {
							 item.setType(TreeItem.TYPE_ASSOCIATION);
							 item.setName(slotname);
							 item.setValue(internalName);
							 item.setNameValue(slotname + ": " + internalName);
							 state_vec.add(item);
							 //item_vec.add(item);
					     }
					 }
				 }
				 return;
			 }
		}


		if (values != null) {
			Iterator i = values.iterator();
			while(i.hasNext()) {
				Object obj = i.next();
                if (hset.contains(slotname))
				{
					ValueType type = getObjectValueType(obj);

					String entry = convertObjecttoString(obj, type);

                    String stringType = type.toString();

					try {
						TreeItem item = new TreeItem();
						if (slotname.compareTo(SUBCLASSOF) == 0)
						{
							item.setType(TreeItem.TYPE_PARENT);

							item.setCls((RDFSClass) obj);
						}

						else if (associationNames.contains(slotname)
						    || isUserDefinedRDFProperty(slotname))
						{
							item.setType(TreeItem.TYPE_ASSOCIATION);

//111706

							item.setCls((RDFSClass) obj);
							String targetname = wrapper.getInternalName((OWLNamedClass) obj);
							item.setValue(targetname);
							item.setNameValue(slotname, targetname);

						}

						else
						{
							item.setType(TreeItem.TYPE_PROPERTY);
						}

						item.setName(slotname);

/*
						if (slotname.compareToIgnoreCase(wrapper.getSlotName(NCIEditTab.CODE)) == 0)
						{
							if (code == null)
							{
								item.setValue(entry);
								item.setNameValue(slotname + ": " + entry);

								code = entry;
							}
							else
							{
								item.setValue(code);
								item.setNameValue(slotname + ": " + code);
							}
						}

//111706
						else

*/
						if (item.getType() == TreeItem.TYPE_PROPERTY)
						{
							// properties

							item.setValue(obj);
							item.setLanguage(getStringLanguage(obj));
							//item.setNameValue(slotname + ": " + entry);
							item.setNameValue(slotname, obj);

						}

						state_vec.add(item);
						//item_vec.add(item);

					}
					catch(Exception e){
						OWLUI.handleError(kb, e);
					}

			    }

			}
		}
	}


	/*
    private void getOwnslot(String slotname, Collection values)
    {
		hset = getOWLAnnotationProperties();
		if (hset == null)
		{
			return;
		}

		if (!hset.contains(slotname))
		{
             if (slotname.compareTo(":DIRECT-SUPERCLASSES") != 0 &&
                 slotname.compareTo(":DIRECT-TYPE") != 0 &&
                 slotname.compareTo(":ROLE") != 0 &&
                 slotname.compareTo(SUBCLASSOF) != 0)
			 {
				 if (values == null) return;
				 Iterator i = values.iterator();
				 while(i.hasNext()) {
					 Object obj = i.next();
					 TreeItem item = new TreeItem();

					 if (obj instanceof OWLNamedClass)
					 {
						 OWLNamedClass cls = (OWLNamedClass) obj;
						 if (cls.getBrowserText().compareTo("owl:Class") != 0)
						 {
							 item.setType(TreeItem.TYPE_ASSOCIATION);
							 item.setName(slotname);

							 String internalName = wrapper.getInternalName(cls);

							 //item.setValue(cls.getBrowserText());
							 //item.setNameValue(slotname + ": " + cls.getBrowserText());

							 item.setValue(internalName);
							 item.setNameValue(slotname + ": " + internalName);

							 state_vec.add(item);
					     }
					 }
				 }
				 return;

			 }

		}

		if (values != null) {
			Iterator i = values.iterator();
			while(i.hasNext()) {
				Object obj = i.next();

                if (hset.contains(slotname))
				{
					ValueType type = getObjectValueType(obj);

					String entry = convertObjecttoString(obj, type);

					String stringType = type.toString();

					try {
						TreeItem item = new TreeItem();

						if (slotname.compareTo(SUBCLASSOF) == 0)
						{
							item.setType(TreeItem.TYPE_PARENT);
						}
						else if (associationNames.contains(slotname)
						    || isUserDefinedRDFProperty(slotname))
						{
							item.setType(TreeItem.TYPE_ASSOCIATION);
						}
						else
						{
							item.setType(TreeItem.TYPE_PROPERTY);
						}

                        item.setName(slotname);
						item.setValue(obj);
						item.setNameValue(slotname, obj);

						state_vec.add(item);
					}
					catch(Exception e){
						OWLUI.handleError(owlModel, e);
					}

			    }

			}
		}
	}
	*/

    protected String convertObjecttoString(Object obj, ValueType type) {
		if(type.equals(ValueType.STRING))
			return obj.toString();
		else if(type.equals(ValueType.FLOAT))
			return obj.toString();
		else if(type.equals(ValueType.INTEGER))
			return obj.toString();
		else if(type.equals(ValueType.ANY))
			return obj.toString();
		else if(type.equals(ValueType.BOOLEAN))
			return obj.toString();
		else if(type.equals(ValueType.CLS)) {
			Cls cls = (Cls) obj;
			return cls.getName();
		}
		else if(type.equals(ValueType.INSTANCE)) {
			Instance inst = (Instance) obj;
			return inst.getName();
		}
		else if(type.equals(ValueType.SYMBOL))
			return obj.toString();
		else return "Unknown";
    }


    protected ValueType getObjectValueType(Object obj) {
		if (obj instanceof Boolean)
			return ValueType.BOOLEAN;
        else if (obj instanceof Cls)
			return ValueType.CLS;
        else if (obj instanceof Float)
			return ValueType.FLOAT;
        else if (obj instanceof Instance)
			return ValueType.INSTANCE;
        else if (obj instanceof Integer)
			return ValueType.INTEGER;
        else if (obj instanceof String)
			return ValueType.STRING;
        else
			return ValueType.ANY;
    }



    private TreeItem[] vector2Array(Vector v)
    {
		TreeItem[] a = new TreeItem[v.size()];
		for (int i=0; i<a.length; i++)
		{
			TreeItem item = (TreeItem) v.elementAt(i);
			a[i] = item;
		}
		return a;
	}


    private String getTreeItemKey(TreeItem item)
    {
		int type = item.getType();
		if (item.getName().compareTo("rdfs:label") == 0 ||
		    item.getName().compareTo("code") == 0 ||
		    item.getName().compareTo("hasType") == 0)
		{
			type = 0;
		}

		String key = "" + type + item.toString();
		return key.toUpperCase();
	}


    private TreeItem[] sortTreeItems(TreeItem[] a)
    {
        boolean done = false;
        if (a == null || a.length == 0) {
            return a;
        }
        while (!done) {
            done = true;
            for (int i = 0;i < a.length-1;i++) {
			    TreeItem item1 = (TreeItem) a[i];
			    TreeItem item2 = (TreeItem) a[i+1];

				String key_1 = getTreeItemKey(item1);
				String key_2 = getTreeItemKey(item2);

                if (key_1.compareTo(key_2) <= 0) {
                } else {
                    done = false;
                    TreeItem tmp = a[i];
                    a[i] = a[i+1];
                    a[i+1] = tmp;
                }
            }
        }
        return a;
	}


	public Vector getInboundRoles()
	{
		return inboundRoles;
	}

	public Vector getOldSubclasses()
	{
		return oldSubclasses;
	}


/*

	private void getRoles(OWLNamedClass cls)
	{
		//NCIConditionsTableModel.SET_SUPERCLASS
		//NCIConditionsTableModel.SET_RESTRICTION

        HashSet hset = new HashSet();


        Vector v = wrapper.getDefinitionItems(cls, NCIConditionsTableModel.SET_SUPERCLASS);
        for (int i=0; i<v.size(); i++)
        {
			TreeItem item = (TreeItem) v.elementAt(i);
			String value = SUBCLASSOF + ": " + wrapper.getInternalName(item.getCls());
			if (!hset.contains(value))
			{
				item.setNameValue(value);
                state_vec.add(item);
			    hset.add(value);
			}
		}
        v = wrapper.getDefinitionItems(cls, NCIConditionsTableModel.SET_RESTRICTION);
        for (int i=0; i<v.size(); i++)
        {
			TreeItem item = (TreeItem) v.elementAt(i);
			String value = item.getCls().getBrowserText();
 			if (!hset.contains(value))
			{
				item.setNameValue(value);
                state_vec.add(item);
			    hset.add(value);
			}
		}


        v = wrapper.getDirectSuperclassItems(cls, NCIConditionsTableModel.SET_SUPERCLASS);
        for (int i=0; i<v.size(); i++)
        {
			TreeItem item = (TreeItem) v.elementAt(i);
			String value = SUBCLASSOF + ": " + wrapper.getInternalName(item.getCls());
			if (!hset.contains(value))
			{
				item.setNameValue(value);
                state_vec.add(item);
			    hset.add(value);
			}
		}
        v = wrapper.getDirectSuperclassItems(cls, NCIConditionsTableModel.SET_RESTRICTION);
        for (int i=0; i<v.size(); i++)
        {
			TreeItem item = (TreeItem) v.elementAt(i);
			String value = item.getCls().getBrowserText();
			if (!hset.contains(value))
			{
				item.setNameValue(value);
                state_vec.add(item);
			    hset.add(value);
			}
		}

	}
*/



	private void getRoles(OWLNamedClass cls)
	{
        HashSet hset = new HashSet();
        RDFSClass[] a = getEquivalentNamedSuperClasses(cls);
        if (a != null)
        {
			for (int i=0; i<a.length; i++)
			{
				RDFSClass aClass = (RDFSClass) a[i];
				OWLNamedClass clse = (OWLNamedClass) aClass;
				String value = SUBCLASSOF + ": " + wrapper.getInternalName(clse) + " [defined]";
				if (!hset.contains(value))
				{
					TreeItem item = new TreeItem();
					item.setType(TreeItem.TYPE_PARENT);
					item.setName(SUBCLASSOF);
					item.setValue(wrapper.getInternalName(clse));
					item.setCls(aClass);
					item.setNameValue(value);
					item.setIsDefining(true);
					state_vec.add(item);
					hset.add(value);
				}
			}
	    }

        a = getEquivalentAnonymousSuperClasses(cls);
        if (a != null)
        {
			for (int i=0; i<a.length; i++)
			{
				RDFSClass aClass = (RDFSClass) a[i];
				String value = aClass.getBrowserText() + " [defined]";
				if (!hset.contains(value))
				{
					TreeItem item = new TreeItem();
					item.setType(TreeItem.TYPE_RESTRICTION);
					//item.setName(SUBCLASSOF);
					item.setValue(value);
					item.setCls(aClass);
					item.setNameValue(value);
					item.setIsDefining(true);
					state_vec.add(item);
					hset.add(value);
				}
			}
	    }

        a = getPureNamedSuperClasses(cls);
        if (a != null)
        {
			for (int i=0; i<a.length; i++)
			{
				RDFSClass aClass = (RDFSClass) a[i];
				OWLNamedClass clse = (OWLNamedClass) aClass;
				String value = SUBCLASSOF + ": " + wrapper.getInternalName(clse);
				if (!hset.contains(value))
				{
					TreeItem item = new TreeItem();
					item.setType(TreeItem.TYPE_PARENT);
					item.setName(SUBCLASSOF);
					item.setValue(wrapper.getInternalName(clse));
					item.setCls(aClass);
					item.setNameValue(value);
					item.setIsDefining(false);
					state_vec.add(item);
					hset.add(value);
				}
			}
	    }


        a = getPureAnonymousSuperClasses(cls);
        if (a != null)
        {
			for (int i=0; i<a.length; i++)
			{
				RDFSClass aClass = (RDFSClass) a[i];
				String value = aClass.getBrowserText();
				if (!hset.contains(value))
				{
					TreeItem item = new TreeItem();
					item.setType(TreeItem.TYPE_RESTRICTION);
					//item.setName(SUBCLASSOF);
					item.setValue(value);
					item.setCls(aClass);
					item.setNameValue(value);
					item.setIsDefining(false);
					state_vec.add(item);
					hset.add(value);
				}
			}
	    }
	}


    private void getAssociations(OWLNamedClass cls)
    {
		Vector v = new Vector();
		Collection allProperties = kb.getVisibleUserDefinedRDFProperties();
		for (Iterator it = allProperties.iterator(); it.hasNext();) {
			RDFProperty property = (RDFProperty) it.next();
			if (property.isAnnotationProperty()
				&& property.hasObjectRange()
				&& property instanceof DefaultOWLObjectProperty)
			{
				v.add(property);
			}
		}

		v.add(kb.getOWLDifferentFromProperty());
		v.add(kb.getOWLDisjointWithProperty());
		v.add(kb.getOWLEquivalentPropertyProperty());
		v.add(kb.getOWLSameAsProperty());
		v.add(kb.getRDFSIsDefinedByProperty());

		RDFProperty seeAlsoSlot = kb.getRDFProperty(RDFSNames.Slot.SEE_ALSO);
		v.add(seeAlsoSlot);

		for (int i=0; i<v.size(); i++)
		{
			RDFProperty property = (RDFProperty) v.elementAt(i);
			if(property.isVisible()) {
				Collection c = cls.getPropertyValues(property);
				for (Iterator it = c.iterator(); it.hasNext();) {
					Object value = it.next();
					OWLNamedClass aClass = (OWLNamedClass) value;

					TreeItem item = new TreeItem();
					item.setType(TreeItem.TYPE_ASSOCIATION);
					item.setProperty(property);

					item.setName(property.getBrowserText());
					item.setValue(aClass.getBrowserText());

					item.setProperty(property);

					item.setCls(aClass);
					String nameValue = property.getName() + ": " + aClass.getBrowserText();

					item.setNameValue(nameValue);
					item.setIsDefining(false);
					state_vec.add(item);
				}
			}
		}

		v.clear();
	}


	/*

    private void getAssociations(OWLNamedClass cls)
    {
		Vector v = new Vector();
		Collection allProperties = kb.getVisibleUserDefinedRDFProperties();
		//Collection selectableProperties = new ArrayList();
		for (Iterator it = allProperties.iterator(); it.hasNext();) {
			RDFProperty property = (RDFProperty) it.next();
			if (property.isAnnotationProperty()
				&& property.hasObjectRange()
				&& property instanceof DefaultOWLObjectProperty)
			{
				v.add(property);
			}
		}

		v.add(kb.getOWLDifferentFromProperty());
		v.add(kb.getOWLDisjointWithProperty());
		v.add(kb.getOWLEquivalentPropertyProperty());
		v.add(kb.getOWLSameAsProperty());
		v.add(kb.getRDFSIsDefinedByProperty());

		RDFProperty seeAlsoSlot = kb.getRDFProperty(RDFSNames.Slot.SEE_ALSO);
		v.add(seeAlsoSlot);

		for (int i=0; i<v.size(); i++)
		{
			RDFProperty property = (RDFProperty) v.elementAt(i);
			if(property.isVisible()) {
				Collection c = cls.getPropertyValues(property);
				for (Iterator it = c.iterator(); it.hasNext();) {
					Object value = it.next();

					TreeItem item = (TreeItem) v.elementAt(i);
					item.setProperty(property);
					OWLNamedClass aClass = (OWLNamedClass) value;

					item.setCls(aClass);
					String nameValue = property.getName() + ": " + aClass.getBrowserText();

					item.setNameValue(nameValue);
					item.setIsDefining(false);
					state_vec.add(item);
				}
			}
		}
	}

    */

    public static RDFSClass[] getPureNamedSuperClasses(OWLNamedClass hostClass)
    {
		Collection c = hostClass.getPureSuperclasses();
		if (c.size() == 0) return null;
		Vector v = new Vector();
		RDFSClass[] sups = new RDFSClass[c.size()];
		for (Iterator it = c.iterator(); it.hasNext();) {
			Object value = it.next();
			RDFSClass cls = (RDFSClass) value;
			if (cls instanceof OWLNamedClass)
			{
				v.add(cls);
			}
		}
		if (v.size() == 0) return null;
		RDFSClass[] clses = new RDFSClass[v.size()];
		for (int i=0; i<v.size(); i++) {
			clses[i] = (RDFSClass) v.elementAt(i);
		}
		return clses;
	}

    public static RDFSClass[] getPureAnonymousSuperClasses(OWLNamedClass hostClass)
    {
		Collection c = hostClass.getPureSuperclasses();
		if (c.size() == 0) return null;
		Vector v = new Vector();
		RDFSClass[] sups = new RDFSClass[c.size()];
		for (Iterator it = c.iterator(); it.hasNext();) {
			Object value = it.next();
			RDFSClass cls = (RDFSClass) value;
			if (cls instanceof OWLAnonymousClass)
			{
				v.add(cls);
			}
		}
		if (v.size() == 0) return null;
		RDFSClass[] clses = new RDFSClass[v.size()];
		for (int i=0; i<v.size(); i++) {
			clses[i] = (RDFSClass) v.elementAt(i);
		}
		return clses;
	}

    public static RDFSClass getDefinition(OWLNamedClass hostClass)
    {
		return hostClass.getDefinition();
	}



    public static RDFSClass[] getEquivalentNamedSuperClasses(OWLNamedClass hostClass)
    {
		HashSet hset = new HashSet();
		RDFSClass[] sups = getPureNamedSuperClasses(hostClass);
		if (sups != null) {
			for (int i=0; i<sups.length; i++) {
				RDFSClass cls = sups[i];
				if (cls instanceof OWLNamedClass)
				{
					hset.add(cls.getBrowserText());
				}
			}
     	}
		Vector v = new Vector();
		Collection c = hostClass.getSuperclasses(false);
		for (Iterator it = c.iterator(); it.hasNext();) {
			Object value = it.next();
			RDFSClass cls = (RDFSClass) value;
			String browserText = cls.getBrowserText();
			if (cls instanceof OWLNamedClass && !hset.contains(browserText))
			{
				hset.add(browserText);
				v.add(cls);
			}
		}
		hset.clear();
		if (v.size() == 0) return null;
		RDFSClass[] clses = new RDFSClass[v.size()];
		for (int i=0; i<v.size(); i++) {
			clses[i] = (RDFSClass) v.elementAt(i);
		}
		return clses;
	}

/*
    public static RDFSClass[] getEquivalentAnonymousSuperClasses(OWLNamedClass hostClass)
    {
		HashSet hset = new HashSet();
		RDFSClass[] sups = getPureAnonymousSuperClasses(hostClass);
		if (sups != null) {
			for (int i=0; i<sups.length; i++) {
				RDFSClass cls = sups[i];
				if (cls instanceof OWLAnonymousClass)
				{
					hset.add(cls.getBrowserText());
				}
			}
     	}
		Vector v = new Vector();
		Collection c = hostClass.getSuperclasses(false);
		for (Iterator it = c.iterator(); it.hasNext();) {
			Object value = it.next();
			RDFSClass cls = (RDFSClass) value;
			String browserText = cls.getBrowserText();
			if (cls instanceof OWLAnonymousClass && !hset.contains(browserText))
			{
				hset.add(browserText);
				v.add(cls);
			}
		}
		hset.clear();
		if (v.size() == 0) return null;
		RDFSClass[] clses = new RDFSClass[v.size()];
		for (int i=0; i<v.size(); i++) {
			clses[i] = (RDFSClass) v.elementAt(i);
		}
		return clses;
	}
*/


    public static RDFSClass[] getEquivalentAnonymousSuperClasses(OWLNamedClass hostClass)
    {
		RDFSClass equivalentClass = getDefinition(hostClass);
		if (equivalentClass == null) return null;

		Vector v = new Vector();
		if (equivalentClass instanceof OWLIntersectionClass) {
			OWLIntersectionClass intersectionCls = (OWLIntersectionClass) equivalentClass;

			// 121406
			if (wrapper.containsSimpleRestrictionsOnly(equivalentClass))
			{
				v.add((RDFSClass) equivalentClass);
			}
			else
			{
				Collection operands = ((OWLIntersectionClass) equivalentClass).getOperands();
				for (Iterator oit = operands.iterator(); oit.hasNext();) {
					RDFSClass operand = (RDFSClass) oit.next();
					if (operand instanceof OWLAnonymousClass)
					{
						v.add((RDFSClass) operand);
					}
				}
		    }
		}
		else {

			if (equivalentClass instanceof OWLAnonymousClass)
			{
				v.add((RDFSClass) equivalentClass);
			}
		}
		if (v.size() == 0) return null;
		RDFSClass[] clses = new RDFSClass[v.size()];
		for (int i=0; i<v.size(); i++) {
			clses[i] = (RDFSClass) v.elementAt(i);
		}
		return clses;
	}





	public static void dumpRelatedClasses(OWLNamedClass hostClass)
	{
		if (hostClass == null)
		{
			System.out.println(	"dumpRelatedClasses hostClass == NULL -- nothing done.");
			return;
		}
		System.out.println("===========================================================================\n");
		System.out.println("hostClass: " + hostClass.getBrowserText());
        RDFSClass[] sups = getPureNamedSuperClasses(hostClass);
        if (sups != null)
        {
			for (int i=0; i<sups.length; i++)
			{
				RDFSClass sup = sups[i];
				System.out.println("getPureNamedSuperClasses: " + sup.getBrowserText());
			}
		}
		else
		{
			System.out.println("getPureNamedSuperClasses: NONE");
		}
		System.out.println("\n");
        sups = getPureAnonymousSuperClasses(hostClass);
        if (sups != null)
        {
			for (int i=0; i<sups.length; i++)
			{
				RDFSClass sup = sups[i];
				System.out.println("getPureAnonymousSuperClasses: " + sup.getBrowserText());
			}
		}
		else
		{
			System.out.println("getPureAnonymousSuperClasses: NONE");
		}

		System.out.println("\n");
        sups = getEquivalentNamedSuperClasses(hostClass);
        if (sups != null)
        {
			for (int i=0; i<sups.length; i++)
			{
				RDFSClass sup = sups[i];
				System.out.println("getEquivalentNamedSuperClasses: " + sup.getBrowserText());
			}
		}
		else
		{
			System.out.println("getEquivalentNamedSuperClasses: NONE");
		}

		System.out.println("\n");
        sups = getEquivalentAnonymousSuperClasses(hostClass);
        if (sups != null)
        {
			for (int i=0; i<sups.length; i++)
			{
				RDFSClass sup = sups[i];
				System.out.println("getEquivalentAnonymousSuperClasses: " + sup.getBrowserText());
			}
		}
		else
		{
			System.out.println("getEquivalentAnonymousSuperClasses: NONE");
		}

		RDFSClass definition = getDefinition(hostClass);
		if (definition != null)
		{
			System.out.println("getDefinition: " + definition.getBrowserText());
		}

        System.out.println("===========================================================================\n");
	}

}
