package gov.nih.nci.protegex.edit;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;

import java.text.SimpleDateFormat;


/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class CustomizedAnnotationData
{
    String title;
    String key;
    String annotation_name;

	String[] component_names;
	String[] component_types;

	HashMap  combobox_values_hmap;
	HashMap  default_values_hmap;


	public CustomizedAnnotationData(String annotation_name,
	                                String tagname_data,
	                                String component_type_data,
	                                String combobox_data,
	                                String default_value_data)
	{
		this.title = "";
		this.key = "";

		component_names = vector2Array(parseData(tagname_data));
		//dumpArray(component_names);

		component_types = vector2Array(parseData(component_type_data));
		//dumpArray(component_types);

		combobox_values_hmap = toHashMap(parseData(combobox_data), true);
		//dumpHashMap(combobox_values_hmap, true);

		default_values_hmap = toHashMap(parseData(default_value_data));
		//dumpHashMap(default_values_hmap);

	}


    private void dumpArray(String[] a)
    {
        for (int i=0; i<a.length; i++)
        {
			String s = (String) a[i];
			System.out.println(s);
		}
	}

    private void dumpHashMap(HashMap hmap)
    {
		Iterator it = hmap.keySet().iterator();
		while (it.hasNext())
		{
			String key = (String) it.next();
			System.out.println("Key: " + key);
			String value = (String) hmap.get(key);
			System.out.println("value: " + value);
		}
	}

    private void dumpHashMap(HashMap hmap, boolean delimited)
    {
		Iterator it = hmap.keySet().iterator();
		while (it.hasNext())
		{
			String key = (String) it.next();
			System.out.println("Key: " + key);
			if (delimited)
			{
				String[] a = (String[]) hmap.get(key);
				for (int i=0; i<a.length; i++)
				{
					String t = (String) a[i];
					System.out.println("\t" + t);
				}

			}
			else
			{
				String value = (String) hmap.get(key);
				System.out.println("value: " + value);
			}
		}
	}


    private HashMap toHashMap(Vector v)
    {
		return toHashMap(v, false);
	}

    private HashMap toHashMap(Vector v, boolean delimited)
    {
        HashMap hmap = new HashMap();
        for (int i=0; i<v.size(); i++)
        {
			String s = (String) v.elementAt(i);
			if (s.compareTo("") != 0)
			{
				if (delimited)
				{
                    String[] a = parseData(s, '|');
                    hmap.put(component_names[i], a);
				}
			    else
			    {
					hmap.put(component_names[i], s);
			    }
			}
		}
        return hmap;
	}

    private String[] vector2Array(Vector v)
    {
        String[] a = new String[v.size()];
        for (int i=0; i<v.size(); i++)
        {
			a[i] = (String) v.elementAt(i);
		}
		return a;
	}

	private Vector parseData(String data)
	{
		Vector v = new Vector();

		String t = data;
		int pos = t.indexOf("$");

		while (pos != -1)
		{
			String s = t.substring(0, pos);
			v.add(s);
			t = t.substring(pos+1);
			pos = t.indexOf("$");
		}
		v.add(t);
		return v;
	}

	private String[] parseData(String data, char ch)
	{
		Vector v = new Vector();

		String t = data;
		int pos = t.indexOf(ch);

		while (pos != -1)
		{
			String s = t.substring(0, pos);
			v.add(s);
			t = t.substring(pos+1);
			pos = t.indexOf(ch);
		}
		v.add(t);
		return vector2Array(v);
	}



	public String getAnnotationValue()
	{
		return "";
	}

	public String[] get_component_names()
	{
		return component_names;
	}

	public String[] get_component_types()
	{
		return component_types;
	}

	public HashMap get_combobox_values_hmap()
	{
		return combobox_values_hmap;
	}

	public HashMap get_default_values_hmap()
	{
		return default_values_hmap;
	}

	public void setAnnotationName(String annotation_name)
	{
		this.annotation_name = annotation_name;
	}


	public void setTitle(String title)
	{
		this.title = title;
	}

	public void setKey(String key)
	{
		this.key = key;
	}




    public static void main(String[] args) {

		String annotation_name = "DEFINITION";
		String tagname_data = "def-definition$Definition_Review_Date$def-source$Definition_Reviewer_Name$attr";
		String component_type_data = "JTextArea$Date: 2007/01/26 19:13:18 $JComboBox$User$JTextField";
		String combobox_data = "$$CADSR|CTCAE|FDA|NCI|NCI-GLOSS$$";
		String default_value_data = "$$NCI$$";

		CustomizedAnnotationData test = new CustomizedAnnotationData(annotation_name,
		         tagname_data, component_type_data, combobox_data, default_value_data);


		annotation_name = "FULL_SYN";
		tagname_data = "term-name$term-group$term-source$source-code";
		component_type_data = "JTextArea$JComboBox$JComboBox$JTextField";
		combobox_data = "$BR|CA2|CA3|CN|CNU|CS|DN|FB|HD|PT|SN|SY$BioCarta|CADSR|CDC|CDISC|CTCAE|CTRM|DCP|DTP|FDA|KEGG|MTH|NCI|NCI-GLOSS|SEER$";
		default_value_data = "$SY$NCI$";

		test = new CustomizedAnnotationData(annotation_name,
		         tagname_data, component_type_data, combobox_data, default_value_data);


		annotation_name = "GO_ANNOTATION";
		tagname_data = "go-term$go-source$source-date$go-evi$go-id";
		component_type_data = "JTextArea$JComboBox$Date: 2007/01/26 19:13:18 $JTextField$JTextField";
		combobox_data = "$CADSR|CTCAE|FDA|NCI|NCI-GLOSS$$$";
		default_value_data = "$NCI$$";

		test = new CustomizedAnnotationData(annotation_name,
		         tagname_data, component_type_data, combobox_data, default_value_data);


	}
}

