package gov.nih.nci.protegex.edit;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.util.Log;
import edu.stanford.smi.protege.util.ModalDialog;
import edu.stanford.smi.protegex.owl.model.OWLIntersectionClass;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.OWLNamedClass;
import edu.stanford.smi.protegex.owl.model.RDFProperty;
import edu.stanford.smi.protegex.owl.model.RDFResource;
import edu.stanford.smi.protegex.owl.model.RDFSClass;
import edu.stanford.smi.protegex.owl.ui.widget.OWLUI;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;
import java.util.Vector;

import edu.stanford.smi.protege.util.Log;
import java.util.logging.Level;


/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class DataHandler
{
    private OWLWrapper wrapper;
    private NCIEditFilter filter;
    private NCIEditTab tab;
    private OWLModel owlModel = null;

    String debugfile = "debug.out";
    PrintWriter debug_pw = null;

    public DataHandler (NCIEditTab tab)
    {
		this.tab = tab;
		this.owlModel = tab.getOWLModel();
    	this.wrapper = tab.getOWLWrapper();
    	this.filter = tab.getFilter();


    }

	public PrintWriter openPrintWriter(String outputfile)
	{
		try {
			PrintWriter pw = new PrintWriter (new BufferedWriter(new FileWriter(outputfile)));
            return pw;
		}catch(Exception e) {
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
			return null;
		}
    }


    public Vector getSetDifference(Vector old_vec, Vector new_vec)
	{
		// Find what are in new_vec. but not in old_vec
		HashSet hset = new HashSet();
		Vector w = new Vector();
		for (int i=0; i<old_vec.size(); i++)
		{
			String s = (String) old_vec.elementAt(i);
			if (!hset.contains(s))
                hset.add(s);
		}

		for (int i=0; i<new_vec.size(); i++)
		{
			String s = (String) new_vec.elementAt(i);
			if (!hset.contains(s))
			{
                w.add(s);
			}
		}
		hset.clear();
		return w;
	}


	private Vector getHashMapKeys(HashMap hashmap)
	{
		Vector v = new Vector();
		Set keyset = hashmap.keySet();
		Iterator iter = hashmap.keySet().iterator();
		while (iter.hasNext())
		{
			String key = (String) iter.next();
			v.add(key);
		}
		return v;
	}



	private Vector getHashMapObjects(HashMap hashmap, Vector state_vec)
	{
		Vector v = new Vector();
		for (int i=0; i<state_vec.size(); i++)
		{
			String key = (String) state_vec.elementAt(i);
			TreeItem item = (TreeItem) hashmap.get(key);
			v.add(item);
		}
		return v;
	}

/*
	static final int TYPE_CONCEPT = 0;
	static final int TYPE_PROPERTY = 1;
	static final int TYPE_RESTRICTION = 2;
	static final int TYPE_PARENT = 3;
*/
    private Vector getTreeItemsByType(Vector state_vec, int type)
    {
		Vector v = new Vector();
		for (int i=0; i<state_vec.size(); i++)
		{
			TreeItem item = (TreeItem) state_vec.elementAt(i);
			if (item.getType() == type)
			{
				v.add(item);
			}
		}
		return v;
	}


    private Vector getTreeItemsByType(Vector state_vec, int type, Vector exclusion)
    {
		Vector v = new Vector();
		for (int i=0; i<state_vec.size(); i++)
		{
			TreeItem item = (TreeItem) state_vec.elementAt(i);
			if (item.getType() == type && !exclusion.contains(item.getName()))
			{
				v.add(item);
			}
		}
		return v;
	}


    String getItemKey(TreeItem treeitem)
    {
		if (treeitem.getIsDefining())
		{
			return treeitem.toString() + "(defining)";
		}
		return treeitem.toString();
	}

	private HashMap getStateMap(Vector state_vec)
	{
		HashMap hashmap = new HashMap();
		for (int i=0; i<state_vec.size(); i++)
		{
			TreeItem item = (TreeItem) state_vec.elementAt(i);
     		String key = getItemKey(item);

			if (!hashmap.containsKey(key))
			{
				hashmap.put(key, (Object) item);
			}
		}
		return hashmap;
	}


    private boolean diagnosis(Vector state_vec)
    {
		for (int i=0; i<state_vec.size(); i++)
		{
			TreeItem item = (TreeItem) state_vec.elementAt(i);
			if (item.getType() == TreeItem.TYPE_PARENT)
			{
			    if (item.getCls() == null)
			    {
					System.out.println("DataHandler - Parent node contain null cls. " + item.toString());
					return false;
				}
			}
		}
		return true;
	}

	//121906
	private Vector isLangUpdated(Vector initialState, Vector finalState)
	{
		Vector updated_vec = new Vector();

		String pt_slot_name = wrapper.getSlotName(NCIEditTab.PREFERRED_NAME);
		for (int i=0; i<finalState.size(); i++)
		{
			TreeItem item = (TreeItem) finalState.elementAt(i);
			if (item.getType() == TreeItem.TYPE_PROPERTY)
			{

				String language = item.getLanguage();
				if (item.getLanguage() == null || item.getLanguage().compareTo("") == 0)
					language = "en";

				String propertyname = item.getName();
				if (propertyname.compareTo(pt_slot_name) != 0)
				{
					String propertyvalue = item.getValue();
					for (int k=0; k<initialState.size(); k++){
						TreeItem item1 = (TreeItem) initialState.elementAt(k);
						if (item1.getName().equals(propertyname) &&
							item1.getValue().equals(propertyvalue))
						{
							String language1 = item1.getLanguage();
							if (language1 == null || language1.compareTo("") == 0)
								language1 = "en";

							if (!language1.equals(language)){
								updated_vec.add(item);
							}
						}
					}
				}
			}
		}
		return updated_vec;
	}


    private void outputVector(String label, Vector v)
    {
		System.out.println("\n" + label);
		for (int i=0; i<v.size(); i++)
		{
			String s = (String) v.elementAt(i);
			System.out.println("\tKEY: " + s);
		}
	}


    // Find TreeItems in state_vec_2, but not in state_vec_1
    private Vector getTreeItemDiffs(Vector state_vec_1, Vector state_vec_2)
    {
        HashMap hashmap_1 = getStateMap(state_vec_1);
        HashMap hashmap_2 = getStateMap(state_vec_2);

        Vector key_vec_1 = getHashMapKeys(hashmap_1);
        //outputVector("key_vec_1", key_vec_1);


        Vector key_vec_2 = getHashMapKeys(hashmap_2);
        //outputVector("key_vec_2", key_vec_2);


        Vector key_diff = getSetDifference(key_vec_1, key_vec_2);
        //outputVector("key_diff", key_diff);


        Vector object_diff = getHashMapObjects(hashmap_2, key_diff);
        //outputState("object_diff", object_diff);


        return object_diff;
	}


    private Vector getTreeItemDiffs(PrintWriter pw, Vector state_vec_1, Vector state_vec_2)
    {
		if (pw == null)
		{
			return getTreeItemDiffs(state_vec_1, state_vec_2);
		}

		outputState(pw, "state_vec_1", state_vec_1);
		outputState(pw, "state_vec_2", state_vec_2);


        HashMap hashmap_1 = getStateMap(state_vec_1);
        HashMap hashmap_2 = getStateMap(state_vec_2);

        Vector key_vec_1 = getHashMapKeys(hashmap_1);

		for (int i=0; i<key_vec_1.size(); i++)
		{
			String s = (String) key_vec_1.elementAt(i);
			pw.println("state_vec_1 key: " + s);
		}
		pw.println("\n");


        Vector key_vec_2 = getHashMapKeys(hashmap_2);

		for (int i=0; i<key_vec_1.size(); i++)
		{
			String s = (String) key_vec_1.elementAt(i);
			pw.println("state_vec_2 key: " + s);
		}
		pw.println("\n");

        Vector key_diff = getSetDifference(key_vec_1, key_vec_2);


		for (int i=0; i<key_diff.size(); i++)
		{
			String s = (String) key_diff.elementAt(i);
			pw.println("getSetDifference: " + s);
		}
		pw.println("\n");


        Vector object_diff = getHashMapObjects(hashmap_2, key_diff);
        return object_diff;
	}


    public void outputState(PrintWriter pw, String label, Vector v)
    {
		pw.println("\n----------------------" + label);
		if (v.size() == 0)
		{
			pw.println("\nNone");
		}
		for (int i=0; i<v.size(); i++)
		{
			TreeItem item = (TreeItem) v.elementAt(i);
			pw.println("Type: " + item.getType());
			pw.println("Name: " + item.getName());
			pw.println("Value: " + item.getValue());
			pw.println("getItemKey: " + getItemKey(item));
			if (item.getModifier().compareTo("") != 0)
			{
				pw.println("Modifier: " + item.getModifier());
		    }

		    if (item.getCls() != null)
		    {
		    	pw.println("aClass: " + item.getCls().getBrowserText());
		    	if (item.getIsDefining())
		    	{
					pw.println("defining: TRUE");
				}
				else
				{
					pw.println("defining: FALSE");
				}
			}

			pw.println("");

		}
	}


    public void outputState(String label, Vector v)
    {
		System.out.println("\n----------------------" + label);
		if (v.size() == 0)
		{
			System.out.println("\nNone");
		}
		for (int i=0; i<v.size(); i++)
		{
			TreeItem item = (TreeItem) v.elementAt(i);
			System.out.println("Type: " + item.getType());
			System.out.println("Name: " + item.getName());
			System.out.println("Value: " + item.getValue());
			System.out.println("getItemKey: " + getItemKey(item));
			if (item.getModifier().compareTo("") != 0)
			{
				System.out.println("Modifier: " + item.getModifier());
		    }

		    if (item.getCls() != null)
		    {
		    	System.out.println("aClass: " + item.getCls().getBrowserText());
		    	if (item.getIsDefining())
		    	{
					System.out.println("defining: TRUE");
				}
				else
				{
					System.out.println("defining: FALSE");
				}
			}

			System.out.println("");
		}
	}

    public boolean createCls(String name, TreePanel panel)
    {
		try {
			Vector final_state = panel.getFinalState();
			//081406
			return createCls(name, final_state, false);
	    } catch (Exception ex) {
			throw new RuntimeException(ex);
		}
	}

    private boolean createCls(String name, Vector state_vec)
    {
		try {
			return createCls(name, state_vec, true);
	    } catch (Exception ex) {
			throw new RuntimeException(ex);
		}
	}


    public boolean processData(Cls cls, TreePanel panel)
    {
		try {
			return processData(cls, panel, false);
	    } catch (Exception ex) {
			throw new RuntimeException(ex);
		}
	}

    public boolean processData(Cls cls, TreePanel panel, boolean upper)
    {
		if (cls == null)
		{
			return false;
		}
        boolean retval = wrapper.isEditable((OWLNamedClass) cls);
		if (!retval)
		{
			filter.setErrorMessage(cls.getBrowserText() + " is not editable.");
			return false;
		}

        filter.setErrorMessage("");

        Vector init_state = panel.getInitialState();
		Vector final_state = panel.getFinalState();

        boolean success;
		try {
			return processData(cls, init_state, final_state, upper, true);
	    } catch (Exception ex) {
			throw new RuntimeException(ex);
		}
   }


    public boolean processData(Cls cls, EditPanel panel)
    {
		try {
		    return processData(cls, panel, false);
	    } catch (Exception ex) {
			throw new RuntimeException(ex);
		}
	}





    public Vector getTreeItemDiffsByType(Vector survival_state_vec, Vector retiring_state_vec, int type)
    {
		Vector v1 = getTreeItemsByType(survival_state_vec, type);
		Vector v2 = getTreeItemsByType(retiring_state_vec, type);

		Vector v3 = getTreeItemDiffs(v1, v2);
		return v3;
	}


    public Vector getTreeItemDiffsByType(Vector survival_state_vec, Vector retiring_state_vec,
                                         int type, Vector exclusion)
    {
		Vector v1 = getTreeItemsByType(survival_state_vec, type);
		Vector v2 = getTreeItemsByType(retiring_state_vec, type, exclusion);

		Vector v3 = getTreeItemDiffs(v1, v2);
		return v3;
	}

    public boolean deleteTreeItems(Cls cls, Vector items)
    {
		if (cls == null) return false;
		if (items == null) return false;
		for (int i=0; i<items.size(); i++)
		{
			TreeItem item = (TreeItem) items.elementAt(i);
			//boolean retval = deleteTreeItem(cls, item.cloneTreeItem());
			boolean retval = deleteTreeItem(cls, item);
			if (!retval) return false;
		}
		return true;
	}




    public boolean deleteTreeItem(Cls cls, TreeItem item)
    {

System.out.println("**** DataHandler  deleteTreeItem " + item.getCls().getBrowserText());


		if (cls == null) return false;
		if (item == null) return false;
        if (item.getType() == TreeItem.TYPE_PROPERTY)
        {
			// delete property
            String propertyname = item.getName();
            String propertyvalue = item.getValue();
	        boolean retval = wrapper.removeAnnotationProperty((OWLNamedClass) cls,
	                                                  propertyname, propertyvalue);
			if (!retval)
			{
				System.out.println("WARNING: Error encountered while removing property from "
				+ cls.getBrowserText());
				return false;
			}

		}
        else if (item.getType() == TreeItem.TYPE_RESTRICTION)
        {

System.out.println("**** item.getType() == TreeItem.TYPE_RESTRICTION " + item.getCls().getBrowserText());


            boolean retval = false;
			if (item.getIsDefining())
			{


System.out.println("**** wrapper.removeEquivalentClass " + item.getCls().getBrowserText());


                 retval = wrapper.removeEquivalentClass((OWLNamedClass) cls, item.getCls());


System.out.println("**** wrapper.removeEquivalentClass Done ");

			}
			else
			{

System.out.println("**** NOT DEFINING item.getIsDefining() " + item.getCls().getBrowserText());


				 //retval = wrapper.removeRestriction((OWLNamedClass) cls, item.getCls());
				 String browserText = item.getCls().getBrowserText();

System.out.println("**** wrapper.removeSuperclasses " + item.getCls().getBrowserText());

				 retval = wrapper.removeSuperclasses((OWLNamedClass) cls, browserText);

System.out.println("**** wrapper.removeSuperclasses Done ");

    		}

		}

        else if (item.getType() == TreeItem.TYPE_ASSOCIATION)
        {

        	System.out.println("Calling wrapper.removeObjectProperty..." + cls.getBrowserText());
        	System.out.println(item.getName());
        	System.out.println(item.getValue());

			boolean retval = wrapper.removeObjectProperty((OWLNamedClass) cls,
	                                //item.getProperty(),
	                                item.getName(), // property name
	                                //item.getCls());
	                                item.getValue());

		}

        else if (item.getType() == TreeItem.TYPE_PARENT)
        {
			// remove parent concept
			boolean retval = false;
			if (item.getIsDefining())
			{
                 retval = wrapper.removeEquivalentClass((OWLNamedClass) cls, item.getCls());
			}
			else
			{
				 retval = wrapper.removeDirectSuperclass((OWLNamedClass) cls, item.getCls());

			}

			if (!retval)
			{
				System.out.println("WARNING: Error encountered while removing parent concept from "
				+ cls.getBrowserText());
				return false;
			}

	    }

		return true;
	}




/*
	static final int TYPE_CONCEPT = 0;
	static final int TYPE_PROPERTY = 1;
	static final int TYPE_RESTRICTION = 2;
	static final int TYPE_PARENT = 3;
*/


    public boolean addTreeItems(Cls cls, Vector items)
    {
		if (cls == null) return false;
		if (items == null) return false;
		for (int i=0; i<items.size(); i++)
		{
			TreeItem item = (TreeItem) items.elementAt(i);

			boolean retval = addTreeItem(cls, item.cloneTreeItem());
			if (!retval) return false;
		}
		return true;
	}


    public boolean addTreeItem(Cls cls, TreeItem item)
    {
		if (cls == null)
		{
			return false;
		}
		if (item == null)
		{
			return false;
		}
        if (item.getType() == TreeItem.TYPE_PROPERTY)
        {
	        wrapper.addAnnotationProperty((OWLNamedClass) cls, item.getName(), item.getValue());
		}
        else if (item.getType() == TreeItem.TYPE_RESTRICTION)
        {
			if (item.getIsDefining())
			{
				OWLNamedClass owl_cls = (OWLNamedClass) wrapper.getCls(item.getValue());
							boolean retval = wrapper.addEquivalentClass(
												(OWLNamedClass) cls, item.getCls());

			}
			else
			{
				RDFSClass rdfsCls = (RDFSClass) item.getCls();

				boolean retval = wrapper.addRestriction(
									(OWLNamedClass) cls,
									rdfsCls);

			}

		}

        else if (item.getType() == TreeItem.TYPE_ASSOCIATION)
        {
			// add association
			boolean retval = wrapper.addObjectProperty((OWLNamedClass) cls,
			    item.getName(), item.getValue());

/*
			boolean retval = wrapper.addObjectProperty((OWLNamedClass) cls,
			    item.getName(), item.getCls());


			boolean retval = wrapper.addObjectProperty((OWLNamedClass) cls,
			    item.getProperty(), item.getCls());

*/
			if (!retval)
			{
				System.out.println("WARNING: Error encountered while adding association to "
				+ cls.getBrowserText());
				return false;
			}
	    }


        else if (item.getType() == TreeItem.TYPE_PARENT)
        {
			boolean retval = false;

			OWLNamedClass supCls = (OWLNamedClass) item.getCls();
			//Cls supCls = wrapper.getCls(item.getValue());
			//retval = wrapper.addDirectSuperclass(cls, item.getValue());
			if (item.getIsDefining())
			{
				retval = wrapper.addEquivalentClass((OWLNamedClass) cls, (RDFSClass) supCls);
			}
			else
			{
				retval = wrapper.addDirectSuperclass(cls, supCls);
			}

			if (!retval)
			{
				System.out.println("WARNING: Error encountered while adding parent concept to "
				+ cls.getBrowserText());
				return false;
			}
	    }
	    return true;
	}


    private void showMessage(String msg)
    {
        ModalDialog.showMessageDialog(tab, msg);
	}







    public boolean processData(Cls cls, EditPanel panel, boolean upper)
    {
		if (cls == null)
		{
			System.out.println("WARNING: processData cls == null");
			return false;
		}

		if (!wrapper.isEditable((OWLNamedClass) cls))
		{
			System.out.println("WARNING: " + cls.getBrowserText() + " is not editable.");
			filter.setErrorMessage(cls.getBrowserText() + " is not editable.");
			return false;
		}

        Vector init_state = panel.getInitialState();

		//ConceptState cs = new ConceptState(wrapper, (OWLNamedClass) cls);
		// from database
		//Vector init_state = cs.get_state_vec(); // from kb

		Vector final_state = panel.getFinalState();

        try {
			boolean retval = processData(cls, init_state, final_state, upper, false);
			return retval;
	    } catch (Exception ex) {
			throw new RuntimeException(ex);
		}
    }

    public boolean updatePropertyLanguage(Cls cls, Vector updated_vec)
    {
		OWLNamedClass hostClass = (OWLNamedClass) cls;
		String pt_slot_name = wrapper.getSlotName(NCIEditTab.PREFERRED_NAME);
		String full_syn_name = wrapper.getSlotName(NCIEditTab.FULL_SYN);
		for (int i=0; i<updated_vec.size(); i++)
		{
			TreeItem item = (TreeItem) updated_vec.elementAt(i);
			if (item.getType() == TreeItem.TYPE_PROPERTY)
			{

				String language = item.getLanguage();
				if (item.getLanguage() == null || item.getLanguage().compareTo("") == 0)
				{
					language = "en"; // default to English
				}
				String propertyname = item.getName();

				if (propertyname.compareTo(pt_slot_name) != 0)
				//if (propertyname.compareTo(full_syn_name) == 0)
				{
					String propertyvalue = item.getValue();
					RDFResource subject = (RDFResource) hostClass;
					RDFProperty property = (RDFProperty) owlModel.getSlot(wrapper.getSlotName(propertyname));
					if (property == null) return false;

					if (language.equals("en"))
						language="";

					boolean retval = wrapper.resetLanguage(hostClass, propertyname, propertyvalue, language);
					if (!retval)
					{
						return false;
					}
			    }
			}
		}

		return true;
	}





//////////////////////////////////////////////////////////////////////////////////

	private void dumpSuperclasses(OWLNamedClass cls)
	{
		Collection parent_vec = cls.getDirectSuperclasses();//cls.getDirectSuperclasses();
		if (parent_vec == null || parent_vec.isEmpty())
		{
			System.out.println("DEBUG " + cls.getBrowserText() + " has NO superclass.");
			return;
		}

		if (parent_vec != null) {
			Iterator j = parent_vec.iterator();
			while(j.hasNext())
			{
				Object obj = j.next();
				RDFSClass r = (RDFSClass) obj;
				System.out.println("DEBUG: Superclass " + r.getBrowserText());
			}
		}
	}


////////////////////////////////////////////////////////////////////////////////////////////





   private boolean createCls(String name, Vector state_vec, boolean generatecode)
   {
		// By code mode: rdfs:ID is the code -- no need to check the format
		// By name mode: rdfs:ID is class name -- check format for xml nc name compliance

// 041207
		filter.reset();
		filter.setFinalState(state_vec);
		filter.setInitialState(state_vec);

		if (!filter.run())
			return false;


		if (wrapper.codeSlotExists) // by name mode
		{
			boolean rv = filter.runCheckXMLNCNameCompliance(name);
			if (!rv) return false;
		}
		boolean retval = filter.canAdd(state_vec, state_vec, state_vec);
		if (!retval)
		{
			return false;
		}


// Transaction Block
		Cls cls = wrapper.createCls(name, generatecode);
		// 030107
		if (cls == null)
		{
			return false;
		}
		/*
		try {
			cls = wrapper.createCls(name, generatecode);
			// 030107
			if (cls == null)
			{
				System.out.println("WARNING: OWLModel.createCls returns null.");
				Log.getLogger().info("OWLModel.createCls returns null.");

				// 042707
				// OWLWrapper rollback already
				//owlModel.rollbackTransaction();
				return false;
			}
		} catch (Exception ex) {
			owlModel.rollbackTransaction();
			//OWLUI.handleError(owlModel, ex);
			throw new RuntimeException(ex);
			//return false;
		}
		*/

        try {
            // 042707
			owlModel.beginTransaction("Create class " + name, name);
			Vector equiv_items_2 = new Vector();
			Vector finalState = new Vector();

			for (int i=0; i<state_vec.size(); i++)
			{
				TreeItem item = (TreeItem) state_vec.elementAt(i);
				if (item.getIsDefining())
				{
					equiv_items_2.add(item);
				}
				else
				{
					finalState.add(item);
				}
			}

			RDFSClass newDefinition = null;
			if (equiv_items_2.size() == 1)
			{
				TreeItem item = (TreeItem) equiv_items_2.elementAt(0);
				newDefinition = item.getCls();
			}

			else if (equiv_items_2.size() > 1) // intersection class
			{
				OWLIntersectionClass temp_newDefinition = owlModel.createOWLIntersectionClass();
				for (int i=0; i<equiv_items_2.size(); i++)
				{
					TreeItem item = (TreeItem) equiv_items_2.elementAt(i);
					RDFSClass operand = (RDFSClass) item.getCls();
					Log.getLogger().warning("Creating clone of " + operand.getBrowserText());
					temp_newDefinition.addOperand(operand.createClone());
				}
				newDefinition = (RDFSClass) temp_newDefinition;
			}

            //tab.removeFromListenedToClses((OWLNamedClass) cls);
			if (newDefinition != null)
			{
				retval = wrapper.addEquivalentClass((OWLNamedClass) cls, newDefinition);
			}

			///////////////////////////////////////////////////////////////////
			// add equivalent class, if there is one

            try {
				retval = addTreeItems(cls, finalState);
				if (!retval)
				{
					// Remove event lister (KLO, 06/30/06)
					//tab.addToListenedToClses((OWLNamedClass) cls);

					// 042707
					owlModel.rollbackTransaction();
					return false;
				}
			} catch (Exception ex) {
				owlModel.rollbackTransaction();
				//OWLUI.handleError(owlModel, ex);
				throw new RuntimeException(ex);
			}

            owlModel.commitTransaction();
			return true;

	    } catch (Exception e) {

			owlModel.rollbackTransaction();
			//OWLUI.handleError(owlModel, e);
			throw new RuntimeException(e);
			//return false;
		}
	}



   // New implementation of the processData method, which minimizes the cloning at saving.
   public boolean processData(Cls cls, Vector initialState0, Vector finalState0, boolean upper, boolean istree) {
		// this should never happen
		if (cls == null)
			return false;

		if (!wrapper.isEditable((OWLNamedClass) cls))
		{
			Log.getLogger().warning(cls.getBrowserText() + " is not editable.");
			filter.setErrorMessage(cls.getBrowserText() + " is not editable.");
			return false;
		}

		filter.reset();
		filter.setFinalState(finalState0);
		filter.setInitialState(initialState0);

		if (!filter.run())
			return false;


		Vector inserted_items = getTreeItemDiffs(initialState0, finalState0);
		Vector deleted_items = getTreeItemDiffs(finalState0, initialState0);

		Log.getLogger().info("Inserted items:  " + inserted_items);
		Log.getLogger().info("Deleted items:   "  + deleted_items);

		// If no change was made, then don't do anything
		Vector updated_vec = isLangUpdated(initialState0, finalState0);

		boolean success = true;
		long t0;

		if (inserted_items.size() == 0 && deleted_items.size() == 0) {

			// update annotation property language
			// Vector updated_vec = isLangUpdated(initialState0, finalState0);

			if (updated_vec.size()!=0){
				t0 = System.currentTimeMillis();
				try {
					//tab.removeFromListenedToClses((OWLNamedClass) cls);
					owlModel.beginTransaction("Update property language for " + cls.getBrowserText(), cls.getName());
					updatePropertyLanguage(cls, updated_vec);

					Log.getLogger().info("Synchronization between local copy and ontology (ms): " + (System.currentTimeMillis() - t0));
					// 111506
					//tab.addToListenedToClses((OWLNamedClass) cls);
					owlModel.commitTransaction();
					return success;

				} catch (Exception e) {
					success = false;
					owlModel.rollbackTransaction();
					Log.getLogger().warning("Error at saving " + cls);
					//OWLUI.handleError(owlModel, e);
					throw new RuntimeException(e);
				}
			}
		}

		//tt: what does this code do?
		tab.getEventListener().resetHasEvent();
		wrapper.flushEvents();
		if (tab.getEventListener().getHasEvent())
			return false;

		t0 = System.currentTimeMillis();

		try {
			owlModel.beginTransaction("Save diffs for " + cls.getBrowserText(), cls.getName());

			// add to the ontology the newly inserted local items
			if (!filter.canAdd(inserted_items, initialState0, finalState0)) {
				Log.getLogger().warning("Cannot add " + inserted_items + " to " + cls);
			} else {
               // 032207, KLO
                if (istree)
                {
					success = success & addInsertedItems((OWLNamedClass)cls, inserted_items);
					success = success & removeDeletedItems((OWLNamedClass)cls, deleted_items);
			    }
			    else
			    {
					success = success & removeDeletedItems((OWLNamedClass)cls, deleted_items);
					success = success & addInsertedItems((OWLNamedClass)cls, inserted_items);
			    }
			}

			//121906
			//Vector updated_vec = isLangUpdated(initialState0, finalState0);
			boolean exists=false;
			for (int i=0; i<inserted_items.size(); i++){
				exists=false;
				TreeItem item = (TreeItem) inserted_items.elementAt(i);
				for (int k=0; k<updated_vec.size(); k++){
					TreeItem item1 = (TreeItem) updated_vec.elementAt(k);
					if (item1.getName().equals(item.getName()) &&
						item1.getValue().equals(item.getValue()))
					{
						exists=true;
					}
				}
				if (!exists)
					updated_vec.add(item);
			}

			if (updated_vec.size()!=0){
				updatePropertyLanguage(cls, updated_vec);
			}


			Log.getLogger().info("Synchronization between local copy and ontology (ms): " + (System.currentTimeMillis() - t0));
            // 111506
			//tab.addToListenedToClses((OWLNamedClass) cls);
			owlModel.commitTransaction();
			return success;
			//return success;
		} catch (Exception e) {
			owlModel.rollbackTransaction();
			Log.getLogger().warning("Error at saving " + cls);
			//OWLUI.handleError(owlModel, e);
			throw new RuntimeException(e);
			//success = false;
			//return false;
		}
	}

	private boolean addInsertedItems(OWLNamedClass cls, Vector inserted_items) {
		boolean retVal = true;
		for (Iterator iter = inserted_items.iterator(); iter.hasNext();) {
			TreeItem item = (TreeItem) iter.next();

			boolean success = addInsertedItem(cls, item);

			if (!success)
				Log.getLogger().warning("Errors adding " + item);

			retVal = retVal & success;
		}
    	return retVal;
    }

	private boolean addInsertedItem(OWLNamedClass cls, TreeItem item) {

		if (cls == null || item == null)
			return false;

		if (item.getType() == TreeItem.TYPE_PROPERTY) {
			wrapper.addAnnotationProperty(cls, item.getName(), item.getValue());
			return true;
		}

		if (item.getType() == TreeItem.TYPE_RESTRICTION) {
			RDFSClass rdfsCls = (RDFSClass) item.getCls();

			//rdfsCls = rdfsCls.createClone();
			rdfsCls = rdfsCls.createClone();

			if (item.getIsDefining()) {
                return wrapper.addEquivalentDefinition(cls, rdfsCls);


			} else {
				return wrapper.addRestriction(cls, rdfsCls);
			}
		}

		if (item.getType() == TreeItem.TYPE_ASSOCIATION) {
			return wrapper.addObjectProperty(cls, item.getName(), item.getValue());
		}

		if (item.getType() == TreeItem.TYPE_PARENT) {
			OWLNamedClass supCls = (OWLNamedClass) item.getCls();

			if (item.getIsDefining()) {
				return wrapper.addEquivalentDefinition((OWLNamedClass) cls, (RDFSClass) supCls);

			} else {

				return wrapper.addDirectSuperclass(cls, supCls);
			}
		}
		return false;
	}


    private boolean removeDeletedItems(OWLNamedClass cls, Vector deleted_items) {

		boolean retVal = true;
		for (Iterator iter = deleted_items.iterator(); iter.hasNext();) {
			TreeItem item = (TreeItem) iter.next();

			boolean success = removeDeletedItem(cls, item);

			if (!success)
				Log.getLogger().warning("Errors removing " + item);

			retVal = retVal & success;
		}
    	return retVal;
    }

	private boolean removeDeletedItem(OWLNamedClass cls, TreeItem item) {
		if (cls == null || item == null)
		{
			return false;
		}

        if (item.getType() == TreeItem.TYPE_PROPERTY)
        {
			// delete property
            String propertyname = item.getName();
            String propertyvalue = item.getValue();
	        return wrapper.removeAnnotationProperty(cls, propertyname, propertyvalue);
	    }

/* KLO
// (removeEquivalentDefinition method failed???)
        else if (item.getType() == TreeItem.TYPE_RESTRICTION)
        {
           if (item.getIsDefining())
			{
               return wrapper.removeEquivalentDefinition(cls, item.getCls());
               //return wrapper.removeEquivalentClass(cls, item.getCls());
			}
			else
			{
				 return wrapper.removeRestriction((OWLNamedClass) cls, item.getCls());
				// tt: it is dangerous to remove the browser text
				// String browserText = item.getCls().getBrowserText();
				// return wrapper.removeSuperclasses(cls, browserText);
    		}
		}
*/

		else if (item.getType() == TreeItem.TYPE_RESTRICTION)
		{
			boolean retval = false;
			if (item.getIsDefining())
			{
                 return wrapper.removeEquivalentDefinition(cls, item.getCls());
				 //retval = wrapper.removeEquivalentClass((OWLNamedClass) cls, item.getCls());
			}
			else
			{
				 retval = wrapper.removeRestriction((OWLNamedClass) cls, item.getCls());
			}
		}


        else if (item.getType() == TreeItem.TYPE_ASSOCIATION)
        {
        	//tt: check this!!!

			OWLNamedClass targetCls = (OWLNamedClass) item.getCls();
			if (targetCls == null)
			{
				return false;
			}

            String targetClsName = wrapper.getInternalName(targetCls);
			return wrapper.removeObjectProperty(cls, item.getName(), targetClsName);

		}

        else if (item.getType() == TreeItem.TYPE_PARENT)
        {
			if (item.getIsDefining())
			{
				// KLO, 040407
                 //return  wrapper.removeEquivalentClass(cls, item.getCls());
                 return  wrapper.removeEquivalentDefinition(cls, item.getCls());
			}
			else
			{
				return wrapper.removeDirectSuperclass(cls, item.getCls());
			}
	    }

		return true;
	}
}

