/*****************************************************************************
*
* NG Inc 2003. All Rights Reserved.
*
*****************************************************************************/

package gov.nih.nci.protegex.edit;

import java.sql.*;
import java.io.*;
import java.util.Vector;
import java.util.Calendar;
import java.net.InetAddress;
import java.text.SimpleDateFormat;

import edu.stanford.smi.protege.model.*;

import edu.stanford.smi.protege.util.Log;
import java.util.logging.Level;


/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class EditHistory {

	private /*final*/String failure = "Cannot write to history database.";
	private Connection conn;
	private String ip, port, user, pwd, instance;
	private String owner, table, sequence;
	private String protegeUser;
	private OWLWrapper wrapper = null;

	private boolean connected = false;

	public EditHistory(String protegeUser,
		String ip, String port, String user, String pwd, String instance,
		String owner, String table, String sequence
		)
    	{
		this.protegeUser = protegeUser;
		this.ip = ip;
		this.port = port;
		this.user = user;
		this.pwd = pwd;
		this.instance = instance; // db
		this.owner = owner;
		this.table = table;
        this.sequence = sequence;

		System.out.println("ip=" + ip);
		System.out.println("port=" + port);
		System.out.println("user=" + user);
		System.out.println("pwd=" + pwd);
		System.out.println("owner=" + owner);
		System.out.println("table=" + table);
		System.out.println("sequence=" + sequence);

        connected = connect(ip, port, instance, user, pwd);
        if (!connected)
        {
			System.out.println("WARNING: Unable to establish database connection.");
		}
		else
		{
			System.out.println("Database connection established.");
			disconnect();
		}

	}

	public void setOWLWrapper(OWLWrapper wrapper)
	{
		this.wrapper = wrapper;
	}

    // uid: root
    // password: password

 // for Oracle
 /*
	private boolean connect()
	{
	    String url = "jdbc:oracle:thin:@"+ip+":"+port+":"+instance;
		try
		{
			DriverManager.registerDriver (new oracle.jdbc.driver.OracleDriver());
			conn = DriverManager.getConnection (url,user,pwd);
		} catch(Exception exception) {
			System.out.println("Cannot connect to " + url + " " + exception);
			failure = "Unable to log to history database -- cannot connect to database.";
			return false;
		}
		return true;
	}
 */


	// for MySQL
	private boolean connect()
	{
		try{
            String url = "jdbc:mysql://" + ip + "/" + instance;
            Class.forName ("com.mysql.jdbc.Driver").newInstance ();
 			conn = DriverManager.getConnection (url, user, pwd);
			return true;
		} catch (Exception e) {
            Log.getLogger().log(Level.WARNING, "Exception caught", e);
            //System.out.println("Cannot connect to edit history database.");
			return false;
		}
	}

	private boolean connect(String ip, String jdbcport, String dsn, String uid, String pwd)
	{
		try{
			//String url = "jdbc:oracle:thin:@"+ip+":"+jdbcport+":"+dsn;
			//DriverManager.registerDriver (new oracle.jdbc.driver.OracleDriver());
            //String url = "jdbc:mysql://" + ip + ":" + jdbcport + "/" + dsn;

            String url = "jdbc:mysql://" + ip + "/" + dsn;
            Class.forName ("com.mysql.jdbc.Driver").newInstance ();
 			conn = DriverManager.getConnection (url,uid,pwd);
 			return true;
		} catch (Exception e) {
            Log.getLogger().log(Level.WARNING, "Exception caught", e);
            //System.out.println("Cannot connect to edit history database.");
			return false;
		}
	}

 	public boolean disconnect()
 	{
		if (!connected) return false;
 		try
 		{
 			conn.close();
 			return true;
 		}
 		catch(Exception exception) {
 			System.out.println("Cannot disconnect from database.");
 		}
 		return false;
 	}

	public String addApostrophe(String name)
	{
		String retStr = "";
		for (int i=0; i<name.length(); i++)
		{
			char c = name.charAt(i);
			Character ch = new Character(c);

			if (c == '\'')
			{
				retStr = retStr + "''";
			}
			else
			{
				retStr = retStr + ch.toString();
			}
		}
		return retStr;
	}


	private String getHost()
	{
		String hostname="";
		try{
			//InetAddress me = InetAddress.getLocalHost();
			String localhost = InetAddress.getLocalHost().getHostAddress();
			InetAddress me = InetAddress.getByName(localhost);
			hostname = me.getHostName();
		}catch(Exception e){
			System.out.println("Cannot find host "+e);}
		return hostname;
	}


	public boolean logEditEvent(String action,
	    String name1, String code1, String name2, String code2,
	    Vector codes) 	// codesc ontains codes of all superconcepts
		              	//(as defined in OLD_PARENT properties)
	{
        if (!connected) return false;
		String host = getHost();
		if (host.equals(""))
		{
			failure = "Unable to log to history database -- cannot find host '" + ip + "'.";
			return false;
		}

		if (name1 == null) return false;
		Cls cls_1 = wrapper.getCls(name1);
		if (cls_1 == null) return false;

		if (code1 == null || code1.compareTo("0") == 0) {
			code1 = wrapper.getInternalName(cls_1);
			if (code1 == null || code1.compareTo("0") == 0) return false;

			if (code1.compareTo(name1) == 0)
			{
				name1 = wrapper.getSlotValue(cls_1, "rdfs:label");
			}
		}

 		if ((action.compareToIgnoreCase("Split") == 0) ||
		    (action.compareToIgnoreCase("Merge") == 0))
        {
			if (name2 == null) return false;
			Cls cls_2 = wrapper.getCls(name2);
			if (cls_2 == null) return false;

			if (code2 == null || code2.compareTo("0") == 0) {
				code2 = wrapper.getInternalName(cls_2);
				if (code2 == null || code2.compareTo("0") == 0) return false;
			}

			if (code2.compareTo(name2) == 0)
			{
				name2 = wrapper.getSlotValue(cls_2, "rdfs:label");
			}
	    }

/*

		if ((action.compareToIgnoreCase("Split") == 0) ||
		    (action.compareToIgnoreCase("Merge") == 0))
		{
			if (name2 == null || name2.equals(""))
			{
				return false;
			}
			if (code2 == null || code2.equals(""))
			{
				return false;
			}
		}
*/
		boolean canconnect = connect();
		if (!canconnect)
		{
			System.out.println("Unable to connect to " + this.ip + ".");
			return false;
		}

        if (action.compareToIgnoreCase("New") == 0)
		{
			action = "Create";
		}

		String nullVal = null;
		String action_new = "Create";
		String action_retire = "Retire";

		String username = protegeUser;
		//log history records using lower case of user name.
		username = username.toLowerCase();

		String conname = addApostrophe(name1);
		String code = code1;

		String s = "";
		Statement stmt = null;

		String t = "INSERT INTO "+table+"(CONCEPTCODE, ACTION, HOST, "+
	  			   "CONCEPTNAME, EDITNAME, REFERENCECODE) VALUES(";


		try
		{
			if ((action.compareToIgnoreCase("Create") == 0) ||
				(action.compareToIgnoreCase("Modify") == 0))
			{
			    stmt = conn.createStatement();
				//040504
				//s = t + getHistorySeq()+",'"+code+"','"+action+"','"+host+"','"+conname+"','"+username+"',"+nullVal+")";
				s = t + "'"+code+"','"+action+"','"+host+"','"+conname+"','"+username+"',"+nullVal+")";

				stmt.executeUpdate(s);
			    	stmt.close();
			}
			// Concepts will not be deleted
			else if (action.compareToIgnoreCase("Retire") == 0)
			{
				stmt = conn.createStatement();
			    if ( codes == null || codes.size() == 0)
				{
					//040504
					//s = t + getHistorySeq()+",'"+code+"','"+action+"','"+host+"','"+conname+"','"+username+"',"+nullVal+")";
					s = t + "'"+code+"','"+action+"','"+host+"','"+conname+"','"+username+"',"+nullVal+")";
					stmt.addBatch(s);
				}
				else
				{
					for (int i=0; i<codes.size(); i++)
					{
						String parent_code = (String) codes.elementAt(i);
						//040504
						//s = t + getHistorySeq()+",'"+code+"','"+action+"','"+host+"','"+conname+"','"+username+"','"+parent_code+"')";
						s = t + "'"+code+"','"+action+"','"+host+"','"+conname+"','"+username+"','"+parent_code+"')";
						stmt.addBatch(s);
					}
				}
				int [] updateCounts = stmt.executeBatch();
				stmt.close();
			}
			else if (action.compareToIgnoreCase("Split") == 0)
			{
				String ref_code = code2;
				String ref_name = addApostrophe(name2);

				// Log new event
				stmt = conn.createStatement();
				//040504
				//s = t + getHistorySeq()+",'"+ref_code+"','"+action_new+"','"+host+"','"+ref_name+
				s = t + "'"+ref_code+"','"+action_new+"','"+host+"','"+ref_name+
					"','"+username+"',"+nullVal+")";
				stmt.addBatch(s);

				// Log split event
				//040504
				//s = t + getHistorySeq()+",'"+code+"','"+action+"','"+host+"','"+conname+"','"+username+
				s = t + "'"+code+"','"+action+"','"+host+"','"+conname+"','"+username+
					"','"+code+"')";
				stmt.addBatch(s);

				/*s = t + getHistorySeq() + ",'"+ref_code+"','"+action+"','"+host+"','"+ref_name+"','"+username+
					"','"+code+"')";*/
				//040504
				//s = t + getHistorySeq() + ",'"+code+"','"+action+"','"+host+"','"+ref_name+"','"+username+
				s = t + "'"+code+"','"+action+"','"+host+"','"+ref_name+"','"+username+
					"','"+ref_code+"')";
				stmt.addBatch(s);

                		int [] updateCounts = stmt.executeBatch();
			    	stmt.close();
			}
			else if (action.compareToIgnoreCase("Merge") == 0)
			{
				String ref_code = code2;
				String ref_name = addApostrophe(name2);

				// Log merge event
				stmt = conn.createStatement();
				//040504
				//s = t + getHistorySeq() +",'"+code+"','"+action+"','"+host+"','"+conname+"','"+username+
				s = t + "'"+code+"','"+action+"','"+host+"','"+conname+"','"+username+
					"','"+code2+"')";
				stmt.addBatch(s);
				//040504
				//s = s = t + getHistorySeq() +",'"+code2+"','"+action+"','"+host+"','"+ref_name+"','"+
				s = t + "'"+code2+"','"+action+"','"+host+"','"+ref_name+"','"+
					username+"','"+code2+"')";
				stmt.addBatch(s);

				// Log retire event
				//040504
				//s = s = t + getHistorySeq() +",'"+code2+"','"+action_retire+"','"+host+"','"+ref_name+
				s = t + "'"+code1+"','"+action_retire+"','"+host+"','"+conname+
					"','"+username+"',"+nullVal+")";
				stmt.addBatch(s);

                		int [] updateCounts = stmt.executeBatch();
			    	stmt.close();
			}
		}
		catch(Exception exception) {
			System.out.println("logEditEvent " + exception);
			failure = "Unable to log " + action + " event -- please consult with your database administrator.";
			disconnect();
			return false;
		}
		disconnect();
		return true;
	}

	private int getHistorySeq()
	{
		int seq=0;
		try
		{
			String s = "SELECT "+owner+"."+sequence+".NEXTVAL FROM DUAL";
			Statement stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery(s);
			if (rs.next())
				seq = rs.getInt(1);
			rs.close();
			stmt.close();
		}
		catch(Exception exception) {
			System.out.println("workFlow getSeq "+exception);
		}
		return seq;
	}




	private String getToday()
	{
		Calendar cal = Calendar.getInstance();
		//SimpleDateFormat  sdf = new SimpleDateFormat("MMM d, yyyy");
		SimpleDateFormat  sdf = new SimpleDateFormat("MM-dd-yy");
		String today = sdf.format(cal.getTime());
		return today;
	}

/*    	public static void main(String[] args)
    	{

        	String ip = "ncievs-test.nci.nih.gov";
		String port = "1521";
		String usr = "evs_management";//"tde_editor";
		String pwd = "nciev$";//"hi$tory";
		String db = "test";
		String owner = "evs_history";
		String table = "evs_history";
		String sequence = "evs_history_id_seq";

		String username = "testuser";
		EditHistory hist = new EditHistory(username, ip, port, usr, pwd, db,
		                                 owner, table, sequence);

		boolean canconnect = hist.connect();
		if (canconnect)
		{
			System.out.println("Connected to " + ip + "[" + port + "]");
			hist.disconnect();
		}
		else
		{
			System.out.println("Cannot connect to " + ip + "[" + port + "]");
		}

	}*/



}
