package gov.nih.nci.protegex.edit;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import edu.stanford.smi.protege.widget.*;
import edu.stanford.smi.protege.resource.*;
import edu.stanford.smi.protege.ui.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.ui.InstanceDisplay;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protege.widget.ClsWidget;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.ui.cls.*;
import edu.stanford.smi.protegex.owl.ui.clsdesc.DisjointClassesWidget;
import edu.stanford.smi.protegex.owl.ui.conditions.ConditionsWidget;
import edu.stanford.smi.protegex.owl.ui.icons.OWLIcons;
import edu.stanford.smi.protegex.owl.ui.resourcedisplay.ResourceDisplay;
import edu.stanford.smi.protegex.owl.ui.subsumption.InferredSubsumptionTreePanel;
import edu.stanford.smi.protegex.owl.ui.widget.WidgetUtilities;

import edu.stanford.smi.protege.event.*;
import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Model;
import edu.stanford.smi.protegex.owl.model.OWLNamedClass;
import edu.stanford.smi.protegex.owl.model.RDFSClass;

////////////////////////////////
import edu.stanford.smi.protegex.owl.model.event.PropertyValueAdapter;
import edu.stanford.smi.protegex.owl.model.event.PropertyValueListener;

import edu.stanford.smi.protegex.owl.ui.conditions.*;

import edu.stanford.smi.protege.event.FrameAdapter;
import edu.stanford.smi.protege.event.FrameEvent;
import edu.stanford.smi.protege.event.FrameListener;

import edu.stanford.smi.protege.model.KnowledgeBase;

import edu.stanford.smi.protege.model.Slot;

import edu.stanford.smi.protegex.owl.model.classparser.OWLClassParser;
import edu.stanford.smi.protegex.owl.model.event.ClassAdapter;
import edu.stanford.smi.protegex.owl.model.event.ClassListener;
import edu.stanford.smi.protegex.owl.model.impl.AbstractRDFSClass;
import edu.stanford.smi.protegex.owl.model.impl.DefaultOWLIntersectionClass;
import edu.stanford.smi.protegex.owl.ui.ProtegeUI;
import edu.stanford.smi.protegex.owl.ui.widget.OWLUI;

/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class EventListener
{
	boolean hasEvent = false;
	private OWLNamedClass hostClass;

    private Slot superclassesSlot;
    private OWLWrapper wrapper;

    private ClassListener classListener = new ClassAdapter() {

/*
        public void subclassAdded(RDFSClass cls, RDFSClass subclass) {
			OWLNamedClass c1 = (OWLNamedClass ) cls;
			OWLNamedClass c2 = (OWLNamedClass ) subclass;
			if (inListenedToClses(c1) || inListenedToClses(c2))
			{
				String msg = "A subclass of "
				   + c1.getBrowserText()
				   + " has been created.";
				showMessage(msg);
			}
        }

        public void subclassRemoved(RDFSClass cls, RDFSClass subclass) {
			OWLNamedClass c1 = (OWLNamedClass ) cls;
			OWLNamedClass c2 = (OWLNamedClass ) subclass;
			if (inListenedToClses(c1) || inListenedToClses(c2))
			{
				String msg = "A subclass of "
				   + c1.getBrowserText()
				   + " has been removed.";
     			showMessage(msg);
			}
		}

        public void superclassAdded(RDFSClass cls, RDFSClass superclass) {
			OWLNamedClass c1 = (OWLNamedClass ) cls;
			OWLNamedClass c2 = (OWLNamedClass ) superclass;
			if (inListenedToClses(c1) || inListenedToClses(c2))
			{
				String msg = "A superclass of "
				   + c1.getBrowserText()
				   + " has been created.";
     			showMessage(msg);
			}
		}

        public void superclassRemoved(RDFSClass cls, RDFSClass superclass) {
			OWLNamedClass c1 = (OWLNamedClass ) cls;
			OWLNamedClass c2 = (OWLNamedClass ) superclass;
			if (inListenedToClses(c1) || inListenedToClses(c2))
			{
				String msg = "A superclass of "
				   + c1.getBrowserText()
				   + " has been removed.";
     			showMessage(msg);

			}
		}
*/
    };


   /**
     * A FrameListener that detects any changes in own slot values of annotation
     * properties and then updates the table model accordingly.
     */

    private PropertyValueListener valueListener = new PropertyValueAdapter() {
        public void propertyValueChanged(RDFResource resource, RDFProperty property, Collection oldValues) {
            if (resource instanceof OWLNamedClass)
            {
				OWLNamedClass owl_cls = (OWLNamedClass ) resource;
				if (inListenedToClses(owl_cls))
				{
					String propertyName = property.getBrowserText();
					String msg = owl_cls.getBrowserText() + " " + propertyName + " property has been modified.";
					showMessage(msg);
					if (tab != null)
					{
						tab.getEditPanel().getCancelButton().setEnabled(true);
				    }
			    }
			}
        }
    };


    private OWLModel owlModel;

    private NCICodeGenerator generator;

    private NCIEditTab tab;

    private Collection listenedToClses = null;

/////////////////////////////////////////////////////////////////////////////////////////////////

// Constructor
    public EventListener (NCIEditTab tab)//, NCICodeGenerator generator)
    {
		this.tab = tab;
		this.owlModel = tab.getOWLModel();
		this.wrapper = tab.getOWLWrapper();

		owlModel.addPropertyValueListener(valueListener);

		listenedToClses = new ArrayList();

    	addOWLModelListners();
    	//addClsListners();

    }

    // for supporting simulator
    public EventListener (OWLModel owlModel, OWLWrapper wrapper)//, NCICodeGenerator generator)
    {
		this.tab = null;
		this.owlModel = owlModel;
		this.wrapper = wrapper;

		owlModel.addPropertyValueListener(valueListener);

		listenedToClses = new ArrayList();

    	addOWLModelListners();
    	//addClsListners();

    }


    public void resetHasEvent()
    {
		hasEvent = false;
	}

    public boolean getHasEvent()
    {
		return hasEvent;
	}

    public boolean inListenedToClses(OWLNamedClass cls)
    {
 		if (cls == null) return false;
 		if (listenedToClses == null) return false;

 		final Object[] obs = listenedToClses.toArray();
 		if (obs.length == 0) return false;
 		for (Iterator it = listenedToClses.iterator(); it.hasNext();) {
 			OWLNamedClass owl_cls = (OWLNamedClass) it.next();
 			if (owl_cls.getBrowserText().compareTo(cls.getBrowserText()) == 0)
 			{
 				return true;
 			}
 		}
 		return false;
	}


    public boolean addToListenedToClses(OWLNamedClass cls)
    {
		if (cls == null) return false;
		if (!(cls instanceof OWLNamedClass)) return false;
		if (inListenedToClses(cls)) return false;
		listenedToClses.add(cls);
		//cls.addClassListener(classListener);
		cls.addPropertyValueListener(valueListener);

		String internalName = wrapper.getInternalName((Cls) cls);
		//System.out.println("Start event listener " + internalName);

		if (listenedToClses.size() > 2)
		{
			System.out.println("WARNING: EventListener is listening to more than two classes.");
		}

		return true;
	}

    public boolean removeFromListenedToClses(OWLNamedClass cls)
    {
		if (cls == null) return false;
		if (!(cls instanceof OWLNamedClass)) return false;
		if (!inListenedToClses(cls)) return false;
		listenedToClses.remove(cls);
		//cls.removeClassListener(classListener);
		cls.removePropertyValueListener(valueListener);

		String internalName = wrapper.getInternalName((Cls) cls);
		//System.out.println("Remove event listener " + internalName);

		if (listenedToClses.size() == 0)
		{
			System.out.println("EventListener is idle.");
		}

		return true;
	}


    public void clearListenedToClses()
    {
		while (!listenedToClses.isEmpty())
		{
			final Object[] obs = listenedToClses.toArray();
			OWLNamedClass owl_cls = (OWLNamedClass) obs[0];
			removeFromListenedToClses(owl_cls);
		}
		hasEvent = false;
	}


    public void resetListenedToClses(Collection c)
    {
		clearListenedToClses();
		if (c == null) return;
		for (Iterator it = c.iterator(); it.hasNext();) {
			OWLNamedClass owl_cls = (OWLNamedClass) it.next();
			addToListenedToClses(owl_cls);
		}
	}

/*
    private void removeListeners() {
        for (Iterator it = listenedToClses.iterator(); it.hasNext();) {
            RDFSClass cls = (RDFSClass) it.next();
            cls.removeClassListener(classListener);
            //((Cls) cls).removeFrameListener(frameListener);
        }
        listenedToClses.clear();
    }
*/

    private void addOWLModelListners()
    {
		owlModel.addKnowledgeBaseListener(new KnowledgeBaseAdapter() {
/*
			public void clsCreated(KnowledgeBaseEvent event) {
				Cls cls = event.getCls();
				if (!(cls instanceof OWLNamedClass))
				{
					return;
				}
				System.out.println("EVENT: clsCreated " + cls.getBrowserText());
			}
*/

// cannot identify deleted class

			public void clsDeleted(KnowledgeBaseEvent event) {
				Cls cls = event.getCls();
				if (!(cls instanceof OWLNamedClass))
				{
					return;
				}
				//String msg = "EVENT: clsDeleted " + cls.getBrowserText();
				//showMessage(msg);
			}

		});
    }



    private void showMessage(String msg)
    {
        //ModalDialog.showMessageDialog(tab, msg);
        hasEvent = true;
		ProtegeUI.getModalDialogFactory().showMessageDialog(owlModel, msg);
	}


}

