package gov.nih.nci.protegex.edit;

import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.ui.FrameRenderer;
import edu.stanford.smi.protege.util.PopupMenuMouseListener;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.model.triplestore.Triple;
import edu.stanford.smi.protegex.owl.model.triplestore.impl.DefaultTriple;
import edu.stanford.smi.protegex.owl.ui.ResourceRenderer;
import edu.stanford.smi.protegex.owl.ui.actions.CopyPropertyValueAction;
import edu.stanford.smi.protegex.owl.ui.actions.DeleteAllPropertyValuesAction;
import edu.stanford.smi.protegex.owl.ui.actions.DeleteMatchingPropertyValuesAction;
import edu.stanford.smi.protegex.owl.ui.actions.triple.TripleAction;
import edu.stanford.smi.protegex.owl.ui.actions.triple.TripleActionManager;
import edu.stanford.smi.protegex.owl.ui.actions.triple.TripleActionSwingAction;
import edu.stanford.smi.protegex.owl.ui.components.ComponentUtil;
import edu.stanford.smi.protegex.owl.ui.results.TripleDisplay;
import edu.stanford.smi.protegex.owl.ui.widget.OWLUI;

import javax.swing.*;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;
import java.awt.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import java.util.HashSet;
import java.util.Collections;

import edu.stanford.smi.protegex.owl.ui.components.annotations.*;//AnnotationsValueRenderer

/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class NCIAnnotationsTable extends JTable {

    private static HashSet multiLineProperties = new HashSet();

    static {
        multiLineProperties.add(RDFSNames.Slot.COMMENT);
        multiLineProperties.add("rdfs:comment");
    }

    private NCIAnnotationsTableModel tableModel;
    private JTextField textField;
    private JComboBox typeComboBox;
    private Project project;

    TableColumn valueColumn;

    public NCIAnnotationsTable(Project project,
                  NCIAnnotationsTableModel model,
                  final String partialActionName) {

        super(model);
        this.project = project;
        this.tableModel = model;

        model.setTable(this);

        textField = new JTextField();
        OWLUI.addCopyPastePopup(textField);
        setDefaultEditor(Object.class, new DefaultCellEditor(textField));
        valueColumn = getColumnModel().getColumn(tableModel.getColumnCount() - 2);
        TableColumn languageColumn = getColumnModel().getColumn(tableModel.getColumnCount() - 1);

        NCIAnnotationsValueRenderer renderer = new NCIAnnotationsValueRenderer(model);

        valueColumn.setCellRenderer(renderer);
        valueColumn.setPreferredWidth(700);

        OWLModel owlModel = (OWLModel) project.getKnowledgeBase();

		//062206
        if (model.getHasPropertyColumn()){
        	TableColumn propertyColumn = getColumnModel().getColumn(NCIAnnotationsTableModel.COL_PROPERTY);
        	propertyColumn.setCellRenderer(renderer);
        	propertyColumn.setPreferredWidth(200);
		}

        languageColumn.setCellRenderer(renderer);
        languageColumn.setCellEditor(new NCIAnnotationsLangEditor(owlModel, model.getEditPanel(), this));

        if (tableModel.hasTypeColumn()) {
            TableColumn typeColumn = getColumnModel().getColumn(tableModel.getColumnCount() - 2);
            typeColumn.setCellRenderer(new ResourceRenderer());
            Vector datatypes = new Vector();
            typeComboBox = new JComboBox(datatypes);
            typeComboBox.setRenderer(new ResourceRenderer());
            typeColumn.setCellEditor(new DefaultCellEditor(typeComboBox));
        }

        setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
        getTableHeader().setReorderingAllowed(false);
        setRowMargin(0);
        setIntercellSpacing(new Dimension(0, 0));
        setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        addMouseListener(new PopupMenuMouseListener(this) {

            protected JPopupMenu getPopupMenu() {
                //return null;//createPopupMenu(partialActionName);
                //return createPopupMenu(partialActionName);
                return null;
            }

            protected void setSelection(JComponent c, int x, int y) {
                int row = y / getRowHeight();
                if (row >= 0 && row < getRowCount()) {
                    getSelectionModel().setSelectionInterval(row, row);
                }
            }
        });


        setRowHeight(new JTextField().getPreferredSize().height);
        setGridColor(Color.LIGHT_GRAY);
        setShowGrid(true);
        setIntercellSpacing(new Dimension(1, 1));

    }


    public void setMultilineEditor()
    {
		int col_num = tableModel.getColumnCount() - 2;
        valueColumn.setCellEditor(new NCIAnnotationsValueEditor(tableModel.getOWLModel(), this, col_num));
    }


    protected JPopupMenu createPopupMenu(String partialActionName) {
        //TriplesTableModel tableModel = (TriplesTableModel) getModel();
        RDFResource resource = tableModel.getSubject();
        final JPopupMenu menu = new JPopupMenu();
        final int row = getSelectedRow();
        RDFProperty property = tableModel.getPredicate(row);
        Object value = tableModel.getValue(row);
        if (property.isAnnotationProperty()) {
            if (resource instanceof RDFSNamedClass) {
                RDFSNamedClass rootClass = (RDFSNamedClass) resource;
                Collection targetResources = new ArrayList();
                for (Iterator it = rootClass.getSubclasses(true).iterator(); it.hasNext();) {
                    RDFSClass subClass = (RDFSClass) it.next();
                    if (subClass instanceof RDFSNamedClass) {
                        targetResources.add(subClass);
                    }
                }
                menu.add(new CopyPropertyValueAction("subclasses", property, value, targetResources, partialActionName));
                menu.add(new DeleteMatchingPropertyValuesAction("subclasses", property, value, targetResources, partialActionName));
                menu.add(new DeleteAllPropertyValuesAction("subclasses", property, targetResources, partialActionName));
            }
            else if (resource instanceof RDFProperty) {
                Collection targetResources = new ArrayList(((RDFProperty) resource).getSubproperties(true));
                menu.add(new CopyPropertyValueAction("subproperties", property, value, targetResources, partialActionName));
                menu.add(new DeleteMatchingPropertyValuesAction("subproperties", property, value, targetResources, partialActionName));
                menu.add(new DeleteAllPropertyValuesAction("subproperties", property, targetResources, partialActionName));
            }
        }

        final Triple triple = new DefaultTriple(tableModel.getSubject(), property, value);
        TripleActionManager.addTripleActionsToMenu(triple, new TripleActionManager.Adder() {
            public void addTripleAction(TripleAction action) {
                menu.add(new TripleActionSwingAction(action, triple));
            }
        });
        if (menu.getComponentCount() > 0) {
            return menu;
        }
        else {
            return null;
        }
    }


    public boolean displayTriple(Triple triple) {
        if (triple.getSubject().equals(tableModel.getSubject())) {
            int row = tableModel.getPropertyValueRow(triple.getPredicate(), triple.getObject());
            if (row >= 0) {
                getSelectionModel().setSelectionInterval(row, row);
                scrollRectToVisible(getCellRect(row, 0, false));
            }
        }
        return false;
    }


    public void editCell(final int row) {

        getSelectionModel().setSelectionInterval(row, row);
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                //scrollRectToVisible(getCellRect(row, TriplesTableModel.COL_VALUE, true));
                scrollRectToVisible(getCellRect(row, NCIAnnotationsTableModel.COL_VALUE, true));
                textField.requestFocus();
            }
        });
        //editCellAt(row, TriplesTableModel.COL_VALUE);
        editCellAt(row, NCIAnnotationsTableModel.COL_VALUE);

    }


    public TableCellEditor getCellEditor(int row, int column) {
        //TriplesTableModel tableModel = (TriplesTableModel) getModel();
        RDFProperty property = tableModel.getPredicate(row);
        RDFResource range = property.getRange();
        if (property.getOWLModel().getXSDboolean().equals(range)) {
            JComboBox comboBox = new JComboBox(new Boolean[]{
                    Boolean.FALSE,
                    Boolean.TRUE
            });
            return new DefaultCellEditor(comboBox);
        }
        /* 010207
        else if (range instanceof OWLDataRange) {
            Collection allowedValues = ((OWLDataRange) range).getOneOfValues();
            JComboBox comboBox = new JComboBox(allowedValues.toArray());
            return new DefaultCellEditor(comboBox);
        }
        */
        else {
            return super.getCellEditor(row, column);
        }
    }


    public NCIAnnotationsTableModel getTableModel() {
        return tableModel;
    }


    public static Collection getMultiLineProperties() {
        return Collections.unmodifiableCollection(multiLineProperties);
    }


    public static boolean isMultiLineProperty(RDFProperty property) {
        return multiLineProperties.contains(property.getName());
    }

    public static boolean isMultiLineProperty(String propertyName) {
        return multiLineProperties.contains(propertyName);
    }

    public boolean isCellEditable(int rowIndex, int columnIndex)
    {
		//if (tableModel.isComplex()) return true;

		int n = tableModel.getColumnCount() - 1;
		if (columnIndex == n && super.isCellEditable(rowIndex, columnIndex)) return true;
		return false;
	}

}
