package gov.nih.nci.protegex.edit;

import edu.stanford.smi.protegex.owl.ui.components.annotations.*;

import edu.stanford.smi.protegex.owl.model.RDFResource;
import edu.stanford.smi.protegex.owl.ui.ProtegeUI;

import javax.swing.*;
import javax.swing.table.TableCellRenderer;
import javax.swing.text.View;
import java.awt.*;

import javax.swing.border.Border;
import javax.swing.table.TableCellEditor;
import javax.swing.text.JTextComponent;

/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class NCIAnnotationsValueRenderer implements TableCellRenderer {

    private JTextArea textArea;

    private JLabel label;

    private JLabel langLabel;

    public static final int EXTRA_SPACING = 4;

    private AnnotationsTableCellHolder resourceHolder;

    private AnnotationsTableCellHolder plainTextPropertyValHolder;

    private AnnotationsTableCellHolder langHolder;

    private NCIAnnotationsTableModel model;

	private AnnotationsTableCellHolder multiLineHolder;
	private AnnotationsTableCellHolder singleLineHolder;
	private Border focusBorder = UIManager.getBorder("Table.focusCellHighlightBorder");
	private JTextComponent textComponent;
	private JTextField textField;

    public NCIAnnotationsValueRenderer(NCIAnnotationsTableModel model) {
        super();
        this.model = model;
        textArea = new JTextArea();
        textArea.setWrapStyleWord(true);
        textArea.setLineWrap(true);
        textArea.setFocusable(true);
        textArea.setOpaque(false);
//        plainTextPropertyValHolder = new AnnotationsTableCellHolder(textArea, BorderLayout.CENTER);
        label = new JLabel();
        label.setOpaque(false);
	    if(System.getProperty("os.name").toLowerCase().indexOf("windows") != -1) {
		    // Adjust border on label due to some silly windows 'feature'
		    label.setBorder(BorderFactory.createEmptyBorder(1, 0, 0, 0));
	    }
	    resourceHolder = new AnnotationsTableCellHolder(label, BorderLayout.NORTH);
        langLabel = new JLabel();
        langHolder = new AnnotationsTableCellHolder(langLabel, BorderLayout.NORTH);

		multiLineHolder = new AnnotationsTableCellHolder(textArea, BorderLayout.CENTER);
		textField = new JTextField();
		//062806
		textField.setBorder(null);
		textField.setFocusable(true);
		textField.setOpaque(false);

		singleLineHolder = new AnnotationsTableCellHolder(textField, BorderLayout.CENTER);
		//singleLineHolder = new AnnotationsTableCellHolder(textArea, BorderLayout.CENTER);

    }


	private void focusTextField() {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				textComponent.requestFocus();
			}
		});
	}

	public Object getCellEditorValue() {
		return textComponent.getText().trim();
	}

    /*public Component getTableCellRendererComponent(JTable table,
                                                   Object o,
                                                   boolean selected,
                                                   boolean hasFocus,
                                                   int row,
                                                   int col) {
        // Ensure that the row is the correct height.  We want to adjust the
	    // row height if it hasn't been adjusted by the cell editor, so check
	    // to ensure that the value of the property isn't being edited.
        if ((table.getEditingRow() == row && table.getEditingColumn() == NCIAnnotationsTableModel.COL_VALUE) == false) {
            int rowHeight = getRowHeight(table, row);
            if (table.getRowHeight(row) != rowHeight) {
                table.setRowHeight(row, rowHeight);
            }
        }
        if (model.getHasPropertyColumn() && col == NCIAnnotationsTableModel.COL_PROPERTY) {
            return getResourceComponent((RDFResource) o, selected, hasFocus);
        }
        else if (col == NCIAnnotationsTableModel.COL_VALUE) {
            if (o instanceof RDFResource) {
                return getResourceComponent((RDFResource) o, selected, hasFocus);
            }
            else {
                textArea.setText(o != null ? o.toString() : "");
                plainTextPropertyValHolder.setColors(selected, hasFocus);
                return plainTextPropertyValHolder;
            }
        }
        else {
            langLabel.setText(o != null ? o.toString() : "");
            langHolder.setColors(selected, hasFocus);
            return langHolder;
        }
    }*/


    public Component getTableCellRendererComponent(JTable table,
                                                   Object o,
                                                   boolean selected,
                                                   boolean hasFocus,
                                                   int row,
                                                   int col) {
        // Ensure that the row is the correct height.  We want to adjust the
	    // row height if it hasn't been adjusted by the cell editor, so check
	    // to ensure that the value of the property isn't being edited.
        if ((table.getEditingRow() == row && table.getEditingColumn() == NCIAnnotationsTableModel.COL_VALUE) == false) {
            int rowHeight = getRowHeight(table, row);
            if (table.getRowHeight(row) != rowHeight) {
                table.setRowHeight(row, rowHeight);
            }
        }
        if (model.getHasPropertyColumn()){
			if (col == NCIAnnotationsTableModel.COL_PROPERTY) {
				return getResourceComponent((RDFResource) o, selected, hasFocus);
			}
			else if (col == NCIAnnotationsTableModel.COL_VALUE) {
				if (o instanceof RDFResource) {
					return getResourceComponent((RDFResource) o, selected, hasFocus);
				}
				else {
					//iguo
					//textArea.setText(o != null ? o.toString() : "");
					//plainTextPropertyValHolder.setColors(selected, hasFocus);
					//return plainTextPropertyValHolder;
					return getValueCellHolder(table, o, selected, hasFocus, row, col);
				}
			}
			else {
				langLabel.setText(o != null ? o.toString() : "");
				langHolder.setColors(selected, hasFocus);
				return langHolder;
			}
		}else{
			if (col==0){
				if (o instanceof RDFResource)
					return getResourceComponent((RDFResource) o, selected, hasFocus);
				else
					return getValueCellHolder(table, o, selected, hasFocus, row, col);
			}else{
				langLabel.setText(o != null ? o.toString() : "");
				langHolder.setColors(selected, hasFocus);
				return langHolder;
			}
		}
    }

    private AnnotationsTableCellHolder getValueCellHolder(JTable table,
                                                   Object o,
                                                   boolean selected,
                                                   boolean hasFocus,
                                                   int row,
                                                   int col)
	{
		/*
		if (model.getColumnCount()==2){
			textArea.setText(o != null ? o.toString() : "");
			int rowHeight = getRowHeight(table, row);
			if (table.getRowHeight(row) != rowHeight) {
				table.setRowHeight(row, rowHeight);
			}
			textComponent = textArea;
			focusTextField();
			multiLineHolder.setColors(selected, hasFocus);
			return multiLineHolder;
		}else{
			textField.setText(o != null ? o.toString() : "");
			textComponent = textField;
			focusTextField();
			singleLineHolder.setColors(selected, hasFocus);
			return singleLineHolder;
		}
		*/


			textArea.setText(o != null ? o.toString() : "");
			int rowHeight = getRowHeight(table, row);
			if (table.getRowHeight(row) != rowHeight) {
				table.setRowHeight(row, rowHeight);
			}
			textComponent = textArea;
			focusTextField();
			multiLineHolder.setColors(selected, hasFocus);
			return multiLineHolder;
	}



    private JComponent getResourceComponent(RDFResource resource, boolean selected, boolean focused) {
        label.setText(resource.getBrowserText());
        label.setIcon(ProtegeUI.getIcon(resource));
        resourceHolder.setColors(selected, focused);
        return resourceHolder;
    }


/*    private int getRowHeight(JTable table,
                             int row) {
        Object val = table.getValueAt(row, AnnotationsTableModel.COL_VALUE);
        if (val instanceof String) {
            String text = val.toString();
            textArea.setText(text);
            View v = textArea.getUI().getRootView(textArea);
            v.setSize(table.getColumnModel().getColumn(AnnotationsTableModel.COL_VALUE).getWidth(), Integer.MAX_VALUE);
            int height = (int) v.getPreferredSpan(View.Y_AXIS) + 4;
            if (height < table.getRowHeight()) {
                height = table.getRowHeight();
            }
            return height;
        }
        return table.getRowHeight();
    }*/

    private int getRowHeight(JTable table, int row) {
		Object val;
        if (model.getHasPropertyColumn()){
			val = table.getValueAt(row, AnnotationsTableModel.COL_VALUE);
			if (val instanceof String) {
				String text = val.toString();
				textArea.setText(text);

				View v = textArea.getUI().getRootView(textArea);
				v.setSize(table.getColumnModel().getColumn(AnnotationsTableModel.COL_VALUE).getWidth(), Integer.MAX_VALUE);

				int height = (int) v.getPreferredSpan(View.Y_AXIS) + 4;
				if (height < table.getRowHeight()) {
					height = table.getRowHeight();
				}
				return height;
			}
			return table.getRowHeight();

		}else{
			val = table.getValueAt(row, 0);
			if (val instanceof String) {
				String text = val.toString();
				textArea.setText(text);

				View v = textArea.getUI().getRootView(textArea);
				v.setSize(table.getColumnModel().getColumn(0).getWidth(), Integer.MAX_VALUE);
				int height = (int) v.getPreferredSpan(View.Y_AXIS) + 4;
				if (height < table.getRowHeight()) {
					height = table.getRowHeight();
				}
				return height;
			}
			return table.getRowHeight();
		}
    }
}

