package gov.nih.nci.protegex.edit;

import java.io.*;
import java.net.MalformedURLException;
import java.util.Vector;

import org.apache.xmlrpc.XmlRpc;
import org.apache.xmlrpc.XmlRpcClient;
import org.apache.xmlrpc.XmlRpcException;

import edu.stanford.smi.protege.util.ApplicationProperties;

import edu.stanford.smi.protege.util.Log;
import java.util.logging.Level;


/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class NCICodeGenerator
{
	String separator = File.separator;
	String protege_path;
	BufferedReader reader;
	String ip, port;
	boolean server_available;
	XmlRpcClient client;

    public NCICodeGenerator()
	{
		server_available = initialize();
	}

	private boolean initialize()
	{
		int pos;
		ip = port = "";
		String temp;
		String pathname = "";
		String separator = File.separator;
		server_available = false;

		try
		{
		    //XmlRpc.setDriver("org.apache.xerces.parsers.SAXParser");
		    //protege_path = ApplicationProperties.APPLICATION_DIRECTORY;

/*
			pathname = System.getProperty("user.dir") + separator+"plugins"+
				//separator + "edu.stanford.smi.protegex.owl" + separator+
                                separator + "gov.nih.nci.protegex.ui.nciowlclassestab" + separator+
				"codegen-client.cfg";
*/

			pathname = System.getProperty("user.dir") + separator+"codegen-client.cfg";

			System.out.println("Searching codegen-client.cfg " + pathname);

			reader = new BufferedReader(new FileReader(pathname));
			if (reader == null)
			{
				System.out.println("Unable to find codegen-client.cfg in " + pathname);
				return false;
			}

			while ((temp = reader.readLine())!= null)
			{
				if (temp.indexOf("IP") != -1)
				{
					pos = temp.lastIndexOf(':');
					ip = temp.substring(pos+1).trim();
				}
				else if (temp.indexOf("PORT") != -1)
				{
					pos = temp.lastIndexOf(':');
					port= temp.substring(pos+1).trim();
				}
			}
			reader.close();
			client = new XmlRpcClient("http://"+ip+":"+port+"/");
			return true;
		}catch (Exception e){
			System.out.println("Could not find " + pathname);
			Log.getLogger().log(Level.WARNING, "Exception caught", e);;
			return false;
		}
	}

	/** get the next available code.
	 *
	 * @since 1.0
	 * @return The next available code
	 */

	public String getCode()
	{
	    String code = "0";
	    if (!server_available) return code;
		try
		{
			XmlRpc.setDriver("org.apache.xerces.parsers.SAXParser");
			Vector params = new Vector();
			code = (String)client.execute("codegen.getCode", params);
		}catch (ClassNotFoundException e){
			System.out.println("Could not locate SAX Driver");
		}catch (MalformedURLException e){
			System.out.println("Incorrect URL for XML-RPC server format: "+e.getMessage());
		}catch (XmlRpcException e){
			System.out.println("XML-RPC Exception: "+e.getMessage());
		}catch (IOException e){
			System.out.println("IOException: "+e.getMessage());
		}
		return code;
	}

	public String getCode(String name)
	{
	    String code = "0";
	    if (!server_available) return code;
		try
		{
			XmlRpc.setDriver("org.apache.xerces.parsers.SAXParser");
			Vector params = new Vector();
			params.addElement(name);
			code = (String)client.execute("codegen.getCode", params);
		}catch (ClassNotFoundException e){
			System.out.println("Could not locate SAX Driver");
		}catch (MalformedURLException e){
			System.out.println("Incorrect URL for XML-RPC server format: "+e.getMessage());
		}catch (XmlRpcException e){
			System.out.println("XML-RPC Exception: "+e.getMessage());
		}catch (IOException e){
			System.out.println("IOException: "+e.getMessage());
		}
		return code;
	}


	public Boolean hasCode(String name)
	{
		if (!server_available) return null;
		try {
			XmlRpc.setDriver("org.apache.xerces.parsers.SAXParser");
			Vector params = new Vector();
			params.addElement(name);
			Object ob = client.execute("codegen.hasCode", params);
			if (ob == null) return null;
			Boolean BooleanObj = (Boolean) ob;
			return BooleanObj;

		}catch (ClassNotFoundException e){
			System.out.println("Could not locate SAX Driver");
		}catch (MalformedURLException e){
			System.out.println("Incorrect URL for XML-RPC server format: "+e.getMessage());
		}catch (XmlRpcException e){
			System.out.println("XML-RPC Exception: "+e.getMessage());
		}catch (IOException e){
			System.out.println("IOException: "+e.getMessage());
		}
		return null;
	}


	public Boolean isServerAvailable()
	{
		if (!server_available) return null;
		try {
			XmlRpc.setDriver("org.apache.xerces.parsers.SAXParser");
			Vector params = new Vector();
			Object ob = client.execute("codegen.isServerAvailable", params);
			if (ob == null) return null;
			Boolean BooleanObj = (Boolean) ob;
			return BooleanObj;

		}catch (ClassNotFoundException e){
			System.out.println("Could not locate SAX Driver");
		}catch (MalformedURLException e){
			System.out.println("Incorrect URL for XML-RPC server format: "+e.getMessage());
		}catch (XmlRpcException e){
			System.out.println("XML-RPC Exception: "+e.getMessage());
		}catch (IOException e){
			System.out.println("IOException: "+e.getMessage());
		}
		return null;
	}


	/** Get the description of this code generator.
	 *
	 * @return Description of the code generator
	 */
	public String getDescription()
	{
		return "Centralized NCI code generator";
	}

	public static void main(String[] args)
	{
		NCICodeGenerator generator = new NCICodeGenerator();
		for (int i=0; i<10; i++)
		{
			String code = generator.getCode();
			System.out.println(code);
		}
	}
}
