package gov.nih.nci.protegex.edit;

import edu.stanford.smi.protegex.owl.model.RDFProperty;
import edu.stanford.smi.protegex.owl.model.RDFResource;
import edu.stanford.smi.protegex.owl.ui.ProtegeUI;
import edu.stanford.smi.protegex.owl.ui.icons.OWLIcons;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionEvent;

import java.util.Collection;
import java.util.ArrayList;

import edu.stanford.smi.protegex.owl.ui.actions.ResourceAction;

import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.ui.ProtegeUI;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protegex.owl.ui.dialogs.ModalDialogFactory;
import edu.stanford.smi.protege.util.LabeledComponent;
import edu.stanford.smi.protege.ui.DisplayUtilities;

import edu.stanford.smi.protegex.owl.model.impl.DefaultRDFSDatatype;

import java.util.*;

//053006
import edu.stanford.smi.protegex.owl.ui.ProtegeUI;
import edu.stanford.smi.protegex.owl.ui.editors.PropertyValueEditorManager;
import edu.stanford.smi.protegex.owl.ui.dialogs.ModalDialogFactory;
import edu.stanford.smi.protegex.owl.ui.metadata.AnnotationsWidgetPlugin;
import edu.stanford.smi.protegex.owl.ui.editors.PropertyValueEditor;

import edu.stanford.smi.protegex.owl.ui.metadata.DateAnnotationsWidgetPlugin;
import edu.stanford.smi.protegex.owl.ui.metadata.DateTimeAnnotationsWidgetPlugin;
import edu.stanford.smi.protegex.owl.ui.metadata.TimeAnnotationsWidgetPlugin;
import edu.stanford.smi.protege.util.ComponentFactory;
/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class NCICreateValueAction extends ResourceAction {

//053006
private static Set plugins = new HashSet();

	//060106
	JTextArea desc_ta;
	JComboBox syn_group_combo, source_combo, def_group_combo;
	JTextField name_field, code_field, attr_field, reviewdate_field, reviewer_field;

    OWLModel kb;
    NCIEditTab tab;
    Collection properties;
    Collection excludedProperties = null;
    String title = "Select properties to add";
    NCIAnnotationsTableModel tableModel = null;

    String property_name = null;
    String property_value = null;
    boolean cancelBtnPressed = false;

    int type;

    public NCICreateValueAction(NCIEditTab tab, int type) {
        super("Create new annotation value", OWLIcons.getCreateIcon(OWLIcons.ANNOTATION));
        this.tab = tab;
        this.type = type;
        this.kb = tab.getOWLModel();
        this.properties = kb.getVisibleUserDefinedRDFProperties();
    }


    public NCICreateValueAction(NCIEditTab tab, NCIAnnotationsTableModel tableModel,
           Collection properties, int type)
    {
        super("Create new annotation value", OWLIcons.getCreateIcon(OWLIcons.ANNOTATION));
        this.tab = tab;
        this.type = type;
        this.kb = tab.getOWLModel();
        this.properties = properties;
        if (properties == null)
        {
			this.properties = kb.getVisibleUserDefinedRDFProperties();
		}
        this.tableModel = tableModel;
    }

    public NCICreateValueAction(NCIEditTab tab, NCIAnnotationsTableModel tableModel,
           Collection properties, Collection excludedProperties, int type) {

        super("Create new annotation value", OWLIcons.getCreateIcon(OWLIcons.ANNOTATION));


        this.tab = tab;
        this.type = type;
        this.kb = tab.getOWLModel();
        this.properties = properties;
        if (properties == null && excludedProperties == null)
        {
			this.properties = kb.getVisibleUserDefinedRDFProperties();
		}
		else if (excludedProperties != null)
		{
			this.properties = new ArrayList();
			Collection all_properties = kb.getVisibleUserDefinedRDFProperties();
			Object[] a = all_properties.toArray();
            for (int i=0; i<a.length; i++)
            {
                RDFProperty property = (RDFProperty) a[i];
                //if (!excludedProperties.contains(property.getBrowserText()))
                if (!excludedProperties.contains(property.getName()))
                {
					this.properties.add(property);
				}
		    }
		}

        this.tableModel = tableModel;
    }



    private String[] getAllowableValues(RDFProperty property)
    {
		boolean rangeDefined = property.isRangeDefined();
		if (!rangeDefined) return null;
		boolean includingSuperproperties = false;
		Vector v = new Vector();
		Collection c = property.getRanges(includingSuperproperties);
		if (c == null) return null;
		for(Iterator iterator = c.iterator(); iterator.hasNext();){
			Object value = (Object) iterator.next();

			if (value instanceof DefaultRDFSDatatype)
			{
				return null;
			}
			else
			{
				Collection values = ((OWLDataRange) value).getOneOf().getValues();
				if (values == null) return null;
				if (values.size() > 0)
				{
					for(Iterator iterator2 = values.iterator(); iterator2.hasNext();){
						Object value2 = (Object) iterator2.next();
						v.add(value2.toString());
					}
				}
		    }
		}
		Object[] objs = v.toArray();
		String[] allowedvalues = new String[objs.length];

		for (int i=0; i<objs.length; i++)
		{
			allowedvalues[i] = (String) objs[i];
		}
		return allowedvalues;
	}


    public void actionPerformed(ActionEvent e) {
        String name = "";
        String prop_value = "";
        RDFProperty property = null;
        if (this.properties != null && this.properties.size() == 1)
        {
			Object[] objs =	this.properties.toArray();
			property = (RDFProperty) objs[0];
			name = property.getName();
		}
		else
		{
			Collection allowedSlots = properties;
			if (allowedSlots == null)
			{
				allowedSlots = new ArrayList(kb.getSlots());
			}
			Slot slot = DisplayUtilities.pickSlot(tab, allowedSlots, "Select Property");
			if (slot == null) return;

			name = slot.getName();

			if (tab.isReadOnlyProperty(name))
			{
				Component warning_label = new JLabel(name + " is a read-only property.");
				LabeledComponent lc2 = new LabeledComponent("", warning_label);
				int k = ProtegeUI.getModalDialogFactory().showDialog(tab, lc2, "WARNING", ModalDialogFactory.MODE_CLOSE);
				return;
			}

			property = (RDFProperty) slot;
			property_name = name;

			String[] allowedvalues = getAllowableValues(property);
			if (allowedvalues != null)
			{
				JList valueList;
				valueList = ComponentFactory.createList(null);
				valueList.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

                Arrays.sort(allowedvalues);
				java.util.List list = Arrays.asList(allowedvalues);
				valueList.setListData(list.toArray());
				valueList.setSelectedIndex(0);
				JScrollPane comp = new JScrollPane(valueList);

				String label = property.getBrowserText();
				LabeledComponent lc = new LabeledComponent(label, comp);

				lc.setPreferredSize(new Dimension(400, 120));
				int r = ProtegeUI.getModalDialogFactory().showDialog(tab, lc, "Add Annotation", ModalDialogFactory.MODE_OK_CANCEL);
				if (r == ModalDialogFactory.OPTION_OK) {

					prop_value = (String) valueList.getSelectedValue();

					System.out.println(prop_value);

					// check if property name and value exists:
					String key = property_name + ": " + prop_value;
					if (tab.getEditPanel().itemExists(key, TreeItem.TYPE_PROPERTY))
					{
						Component warning_label = new JLabel("Property already exists.");
						LabeledComponent lc2 = new LabeledComponent("", warning_label);
						int k = ProtegeUI.getModalDialogFactory().showDialog(tab, lc2, "WARNING", ModalDialogFactory.MODE_CLOSE);
						property_name = "";
						property_value = "";
						cancelBtnPressed = true;
						return;
					}
					else
					{
						if (tableModel != null && prop_value != null && prop_value.compareTo("") != 0)
						{
							int index = tableModel.addRow(property, prop_value);
							JTable table = tableModel.getTable();
							if (table != null){
								int newrow = tableModel.getRowCount()-1;
								table.setRowSelectionInterval(newrow, newrow);

							}else
							{
								System.out.println("table is null");
							}

							tab.getEditPanel().enableSaveButton(true);

							if (tableModel.getRowCount()==1)
								tab.getEditPanel().enableDelEditHB(type, true);
						}
					}
				}
				else
				{
					cancelBtnPressed = true;
				}
			}
			else
			{
				if (name.compareTo("") == 0) return;
				property_name = name;

				if (name.equals(tab.getSlotName(NCIEditTab.FULL_SYN)) ||
					name.equals(tab.getSlotName(NCIEditTab.GO_ANNOTATION)) ||
					name.equals(tab.getSlotName(NCIEditTab.LONG_DEFINITION)) ||
					name.equals(tab.getSlotName(NCIEditTab.ALT_LONG_DEFINITION)) ||
					name.equals(tab.getSlotName(NCIEditTab.ALT_DEFINITION))) {


					CustomizedAnnotationDialog dialog
						= new CustomizedAnnotationDialog(tab.getEditPanel(), name, "", -1, tableModel);
					prop_value = dialog.getValue();
					property_value = prop_value;

					String key = name + ": " + property_value;
					if (tab.getEditPanel().itemExists(key, TreeItem.TYPE_PROPERTY))
					{
						Component warning_label = new JLabel("Property already exists.");
						LabeledComponent lc2 = new LabeledComponent("", warning_label);
						int k = ProtegeUI.getModalDialogFactory().showDialog(tab, lc2, "WARNING", ModalDialogFactory.MODE_CLOSE);
						name = "";
						property_value = "";
						cancelBtnPressed = true;
						return;
					}


				}
				else {
					JTextArea textArea = new JTextArea(prop_value.toString());
					textArea.setLineWrap(true);
					textArea.setWrapStyleWord(true);
					textArea.setEnabled(true);

					JScrollPane comp = new JScrollPane(textArea);

					String label = tab.getOWLWrapper().getSlot(name).getBrowserText();

					//			LabeledComponent lc = new LabeledComponent(name, comp);
					LabeledComponent lc = new LabeledComponent(label, comp);

					lc.setPreferredSize(new Dimension(400, 120));
					int r = ProtegeUI.getModalDialogFactory().showDialog(tab, lc, "Add Annotation", ModalDialogFactory.MODE_OK_CANCEL);
					if (r == ModalDialogFactory.OPTION_OK) {

						if (textArea.getText().compareTo("") == 0)
						{
							Component warning_label = new JLabel("Property value is mandatory.");
							LabeledComponent lc2 = new LabeledComponent("", warning_label);
							int k = ProtegeUI.getModalDialogFactory().showDialog(tab, lc2, "WARNING", ModalDialogFactory.MODE_CLOSE);
							return;
						}

						prop_value = textArea.getText();
						property_value = prop_value;

						// check if property name and value exists:
						String key = property_name + ": " + property_value;
						if (tab.getEditPanel().itemExists(key, TreeItem.TYPE_PROPERTY))
						{
							Component warning_label = new JLabel("Property already exists.");
							LabeledComponent lc2 = new LabeledComponent("", warning_label);
							int k = ProtegeUI.getModalDialogFactory().showDialog(tab, lc2, "WARNING", ModalDialogFactory.MODE_CLOSE);
							property_name = "";
							property_value = "";
							cancelBtnPressed = true;
							return;
						}
					}
					else
					{
						cancelBtnPressed = true;
					}
				}
			}

			if (tableModel != null && prop_value != null && prop_value.compareTo("") != 0)
			{
				int index = tableModel.addRow(property, prop_value);
				JTable table = tableModel.getTable();
				if (table != null){
					int newrow = tableModel.getRowCount()-1;
					table.setRowSelectionInterval(newrow, newrow);

				}else
				{
					System.out.println("table is null");
				}

				tab.getEditPanel().enableSaveButton(true);

				if (tableModel.getRowCount()==1)
					tab.getEditPanel().enableDelEditHB(type, true);
			}
		}
	}


	public String getPropertyName()
	{
		return property_name;
	}

	public String getPropertyValue()
	{
		return property_value;
	}

    public boolean isSuitable(Component component, RDFResource resource) {
        return resource instanceof RDFProperty && resource.isEditable() && component instanceof JTree;
    }
}

