package gov.nih.nci.protegex.edit;

import java.sql.*;
import java.io.*;
import java.util.*;
import java.net.InetAddress;
import java.text.SimpleDateFormat;
import javax.swing.JOptionPane;

//import org.jdom.JDOMException;

import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import javax.xml.parsers.*;
import org.w3c.dom.*;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import java.net.URL;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;

import org.apache.xerces.parsers.*;
import org.xml.sax.helpers.DefaultHandler;

import edu.stanford.smi.protege.util.Log;
import java.util.logging.Level;


/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */


public class NCIEditFilter {

	private static final String DEFAULT_SAX_DRIVER_CLASS ="org.apache.xerces.parsers.SAXParser";

   	static final String DEF_CURATOR = "Def_Curator";
    private URL xmlURL;

    Hashtable name_val_ht;
	private final String VERSION = "1.0.1";
	private String failure;
	private Connection conn;
	private String ip, port, user, pwd, instance;
	private String owner, table, sequence;

	private String tdeuser, ticketFile;
	private boolean canEditRetired;

	private Vector definition_types;
	private Vector short_definition_types;
	private Vector nci_definition_types;

	//020206
	private final String DEFAULT_TERMTYPE = "SY";
	private final String DEFAULT_SOURCE = "NCI";

    Vector initial_state;
    Vector final_state;

    private String errorMessage = "";

    String  kind;
	private Hashtable roothash;
	private Hashtable historyHash, wfHash, axeHash, codegenHash;
	private HashSet managerHash;
	private Hashtable remoteserverHash;
	private String extension_version;
	private String filter_version;
	private String curator;
	private Hashtable authorization;
	NCIEditTab tab;


	public NCIEditFilter(String ip, String port, NCIEditTab tab) {
        this.ip = ip;
        this.port = port;
        this.tab = tab;

		definition_types = new Vector();
		short_definition_types = new Vector();
		nci_definition_types = new Vector();

        initial_state = new Vector();
        final_state = new Vector();


		definition_types.add(tab.getSlotName(NCIEditTab.DEFINITION));
		definition_types.add(tab.getSlotName(NCIEditTab.ALT_DEFINITION));
		definition_types.add(tab.getSlotName(NCIEditTab.LONG_DEFINITION));
		definition_types.add(tab.getSlotName(NCIEditTab.ALT_LONG_DEFINITION));

		short_definition_types.add(tab.getSlotName(NCIEditTab.DEFINITION));
		short_definition_types.add(tab.getSlotName(NCIEditTab.ALT_DEFINITION));

		nci_definition_types.add(tab.getSlotName(NCIEditTab.DEFINITION));
		nci_definition_types.add(tab.getSlotName(NCIEditTab.LONG_DEFINITION));


		initialize();

	}

    public void reset()
    {
        //initial_state.clear();
        //final_state.clear();
        errorMessage = "";
	}

	void initialize()
	{
		roothash = new Hashtable();
		historyHash = new Hashtable();
		wfHash = new Hashtable();
		axeHash = new Hashtable();
		codegenHash = new Hashtable();
		remoteserverHash = new Hashtable();
		authorization = new Hashtable();
		managerHash = new HashSet();

	    DocumentBuilderFactory docBuilderFactory;
		DocumentBuilder docBuilder = null;
        docBuilderFactory = DocumentBuilderFactory.newInstance();
        docBuilderFactory.setIgnoringElementContentWhitespace(true);

        try {
            docBuilder = docBuilderFactory.newDocumentBuilder();

        }
        catch (ParserConfigurationException e) {
            Log.getLogger().log(Level.WARNING, "Exception caught", e);;
        }

		// Load configuration data file:
		try{
			//101105
			String path = System.getProperty("user.dir");

			System.out.println("User directory: " + path);

			if (ip!=null && port!=null)
			{
				System.out.println("IP: " + ip);
				System.out.println("port: " + port);
				String url = "http://"+ip+":"+port+"/ProtegeStartup.xml";
				try {
					xmlURL = new URL("http://"+ip+":"+port+"/ProtegeStartup.xml");
				}
				catch(MalformedURLException e) {
					//Log.getLogger().log(Level.WARNING, "Exception caught", e);;
					System.out.println("MalformedURLException ...");
					return;
				}

                HttpURLConnection conn = null;
				try {
				   conn = (HttpURLConnection) xmlURL.openConnection();
			    }
				catch(MalformedURLException e) {
					System.out.println("WARNING: Unable to load configuration file "
					+ url);
					return;
					//Log.getLogger().log(Level.WARNING, "Exception caught", e);;
				}

				conn.setRequestProperty("Content-Type", "text/xml");
				conn.setRequestMethod("POST");
				conn.setDoInput(true);
				conn.setDoOutput(true);
//				conn.setRequestProperty("Content-Length", "" + buffer.length());
				try {
					conn.connect();
			    }
				catch (MalformedURLException e) {
					System.out.println("WARNING: Unable to load configuration file "
					+ url);
					return;
					//Log.getLogger().log(Level.WARNING, "Exception caught", e);;
				}
				// Get the response
				Reader reader = new InputStreamReader(conn.getInputStream());
				Document doc = docBuilder.parse(new InputSource(reader));
				if (doc == null)
				{
					System.out.println("WARNING: Unable to load configuration file "
					+ url);
					reader.close();
					return;
				}

				conn.disconnect();

				dumpXML(doc);

			}


		}catch(Exception e){
			//System.out.println("WARNING: Unable to access the configuration file from remote server. ");
		}

	}


	public static String getCharacterDataFromElement(Element e) {
	    Node child = e.getFirstChild();
	    if (child instanceof CharacterData) {
		    CharacterData cd = (CharacterData) child;
		    return cd.getData();
		}
	    return "?";
	}


	public static String getCharacterDataFromChildNode(Element element, String tag) {
		NodeList nodelist = element.getElementsByTagName(tag);
		Element e = (Element) nodelist.item(0);
		String text = getCharacterDataFromElement(e);
        return text;
	}


    protected void dumpXML(Document doc) {
		NodeList nodes = doc.getElementsByTagName("ROOT");
		for (int i = 0; i < nodes.getLength(); i++) {
		    Element element = (Element) nodes.item(i);
		    String kind = getCharacterDataFromChildNode(element, "KIND");
		    String name = getCharacterDataFromChildNode(element, "NAME");
		    roothash.put(kind, name);
		}

 /*
    	<HISTORY>
     		<USER>tde_editor</USER>
     		<PASSWORD>hi$tory</PASSWORD>
     		<IP>cbiodb30.nci.nih.gov</IP>
     		<PORT>1521</PORT>
     		<DB>EVSTEST</DB>
     		<TABLE>evs_history</TABLE>
     		<SEQUENCE>evs_history_id_seq</SEQUENCE>
     		<OWNER>evs_management</OWNER>
    	</HISTORY>
*/

 		nodes = doc.getElementsByTagName("HISTORY");
		for (int i = 0; i < nodes.getLength(); i++) {
		    Element element = (Element) nodes.item(i);
            String id = "USER";
            String value = getCharacterDataFromChildNode(element, id);
            historyHash.put(id, value);

            id = "PASSWORD";
            value = getCharacterDataFromChildNode(element, id);
            historyHash.put(id, value);

            id = "IP";
            value = getCharacterDataFromChildNode(element, id);
            historyHash.put(id, value);

            id = "PORT";
            value = getCharacterDataFromChildNode(element, id);
            historyHash.put(id, value);

            id = "DB";
            value = getCharacterDataFromChildNode(element, id);
            historyHash.put(id, value);

            id = "TABLE";
            value = getCharacterDataFromChildNode(element, id);
            historyHash.put(id, value);

            id = "SEQUENCE";
            value = getCharacterDataFromChildNode(element, id);
            historyHash.put(id, value);

            id = "OWNER";
            value = getCharacterDataFromChildNode(element, id);
            historyHash.put(id, value);
		 }
/*
    <CODEGEN>
    	<IP>localhost</IP>
		<PORT>8585</PORT>
	</CODEGEN>
*/

		nodes = doc.getElementsByTagName("CODEGEN");
		for (int i = 0; i < nodes.getLength(); i++) {
		    Element element = (Element) nodes.item(i);

            String id = "IP";
            String value = getCharacterDataFromChildNode(element, id);
 		    codegenHash.put(id, value);

            id = "PORT";
            value = getCharacterDataFromChildNode(element, id);
 		    codegenHash.put(id, value);
        }


/*
        <AUTHORIZATION>
		<ENTRY>
			<CURATOR>FDA</CURATOR>
			<SCHEMA>wenlings</SCHEMA>
		</ENTRY>
		<ENTRY>
			<CURATOR>NCI Drug Dictionary</CURATOR>
			<SCHEMA>safran</SCHEMA>
		</ENTRY>
    	</AUTHORIZATION>
*/

		nodes = doc.getElementsByTagName("ENTRY");
		for (int i = 0; i < nodes.getLength(); i++) {
		    Element element = (Element) nodes.item(i);

            String id = "CURATOR";
            String key = getCharacterDataFromChildNode(element, id);

            id = "SCHEMA";
            String value = getCharacterDataFromChildNode(element, id);

            addAuthorization(key, value);
		 }
/*
    <MANAGERS>
    	    <MANAGER>Kim Ong</MANAGER>
   	</MANAGERS>
*/

		nodes = doc.getElementsByTagName("MANAGERS");
		for (int i = 0; i < nodes.getLength(); i++) {
		    Element element = (Element) nodes.item(i);

            String id = "MANAGER";
            String value = getCharacterDataFromChildNode(element, id);
 		    managerHash.add(value);
       }

	}

    public boolean isManager(String username)
    {
		return managerHash.contains(username);
	}


    public void setInitialState(Vector v)
    {
		this.initial_state = v;
	}

    public void setFinalState(Vector v)
    {
 		this.final_state = v;
	}

    public void setErrorMessage(String s)
    {
		failure = s;
	}

    public String getErrorMessage()
    {
		if (failure == null) return "";
		return failure;
	}


    public boolean containName(Vector items, int type, String name)
    {
		//vincent add
		if (items == null) return false;

		for (int i=0; i<items.size(); i++)
		{
			TreeItem item = (TreeItem) items.elementAt(i);
			if (item.getType() == type)
			{
				if (item.getName().compareTo(name) == 0)
				{
					return true;
				}
			}
		}
		return false;
	}


    public boolean containValue(Vector items, int type, String value)
    {
		//vincent add
		if (items == null) return false;

		for (int i=0; i<items.size(); i++)
		{
			TreeItem item = (TreeItem) items.elementAt(i);
			if (item.getType() == type)
			{
				if (item.getValue().compareTo(value) == 0)
				{
					return true;
				}
			}
		}
		return false;
	}


    public boolean isRetired(Vector v)
    {
		return containValue(v, TreeItem.TYPE_PARENT, tab.getClsName(NCIEditTab.RETIRED_CONCEPTS));
	}

    private boolean checkRetired()
    {
		if (isRetired(initial_state))
		{
			setErrorMessage("Cannot edit a retired concept.");
			return false;
		}
		return true;
	}

    private String getClsName(Vector items)
    {
		for (int i=0; i<items.size(); i++)
		{
			TreeItem item = (TreeItem) items.elementAt(i);
			if (item.getType() == TreeItem.TYPE_CONCEPT)
			{
				return item.getName();
			}
		}
		return null;
	}

	private boolean checkConceptName()
	{
		String name_1 = getClsName(initial_state);
		String name_2 = getClsName(final_state);
		if (name_1.compareTo(name_2) != 0)
		{
			setErrorMessage("Cannot edit concept name.");
			return false;
		}
		return true;
	}

    private boolean checkPreferredName()
    {
		String pt = "";
		if (containName(initial_state, TreeItem.TYPE_PROPERTY, tab.getSlotName(NCIEditTab.PREFERRED_NAME)) &&
		    !containName(final_state, TreeItem.TYPE_PROPERTY, tab.getSlotName(NCIEditTab.PREFERRED_NAME)))
		{
			setErrorMessage("Missing preferred name.");
			return false;
		}

		int num_pts = 0;
		for (int i=0; i<final_state.size(); i++)
		{
			TreeItem item = (TreeItem) final_state.elementAt(i);
			if (item.getType() == TreeItem.TYPE_PROPERTY)
			{
				if (item.getName().compareTo(tab.getSlotName(NCIEditTab.PREFERRED_NAME)) == 0)
				{
					num_pts++;
					pt = item.getValue();
					pt = pt.trim();
				}
			}
		}

		if (num_pts == 0)
		{
			setErrorMessage("Missing preferred name.");
			return false;
		}
		else if (num_pts > 1)
		{
			setErrorMessage("Cannot have multiple preferred names.");
			return false;
		}
		else if (pt.compareTo("") == 0)
		{
			setErrorMessage("Preferred name has not been assigned.");
			return false;

		}

		// 010407
        String label = "";
		for (int i=0; i<final_state.size(); i++)
		{
			TreeItem item = (TreeItem) final_state.elementAt(i);
			if (item.getType() == TreeItem.TYPE_PROPERTY)
			{
				if (item.getName().compareTo("rdfs:label") == 0)
				{
					label = item.getValue();
					break;
				}
			}
		}

		if (label.compareTo("") == 0)
		{
			setErrorMessage("rdfs:label has not been assigned.");
			return false;
		}
		else if (label.compareTo(pt) != 0)
		{
			setErrorMessage("Preferred name and rdfs:label must be synchronized manually.");
			return false;
		}


		return true;
	}


    private String getKey(TreeItem item)
    {
		if (item.getType() != TreeItem.TYPE_RESTRICTION)
		{
			return item.toString();
		}
		String key = item.getCls().getBrowserText();
		if (item.getIsDefining())
		{
			key = key + "(defining)";
		}
		return key;
	}



    public boolean checkDuplicateValues(Vector items, int type)
    {
		HashSet hset = new HashSet();
		for (int i=0; i<items.size(); i++)
		{
			TreeItem item = (TreeItem) items.elementAt(i);
			if (item.getType() == type)
			{
                String key = getKey(item);

				//if (!hset.contains(item.toString()))
				if (!hset.contains(key))
				{
					//hset.add(item.toString());
					hset.add(key);
				}
				else
				{
					hset.clear();
					if (type == TreeItem.TYPE_PROPERTY)
					{
						setErrorMessage("Duplicate property detected (name: " + item.getName() + ", "
						   + "value: " + item.getValue() + ")");
						return false;
					}
					/*
					else if (type == TreeItem.TYPE_RESTRICTION)
					{
						setErrorMessage("Duplicate restriction detected ("
						   + item.toString()
						   + ")");
						return false;
					}
					*/
					else if (type == TreeItem.TYPE_PARENT)
					{
						setErrorMessage("Duplicate superclass detected ("
						   + item.toString()
						   + ")");
						return false;
					}
					else if (type == TreeItem.TYPE_ASSOCIATION)
					{
						setErrorMessage("Duplicate association detected ("
						   + item.toString()
						   + ")");
						return false;
					}
				}
			}
		}
		return true;
	}


    public boolean checkRestrictionValues(Vector items)
    {
		HashSet hset = new HashSet();
		for (int i=0; i<items.size(); i++)
		{
			TreeItem item = (TreeItem) items.elementAt(i);
            if (item.getType() == TreeItem.TYPE_RESTRICTION)
			{

				if (item.getValue().compareTo(tab.getClsName(NCIEditTab.RETIRED_CONCEPTS)) == 0 ||
					item.getValue().compareTo(tab.getClsName(NCIEditTab.PRERETIRED_CONCEPTS)) == 0 ||
					item.getValue().compareTo(tab.getClsName(NCIEditTab.PREMERGED_CONCEPTS)) == 0)
				{
					setErrorMessage("Invalid filler value found in the restriction\n" +
					   item.toString());
				return false;
				}
			}
		}
		return true;
	}


	public boolean run() {
		if (!runCheckProperties()) return false;

// 041207 switch order
        if (!checkPreferredName()) return false;
        if (!checkFullSyn()) return false;

        if (!checkRestrictionValues(final_state)) return false;
        if (!checkRetired()) return false;
        if (!checkConceptName()) return false;
        if (!checkDuplicateValues(final_state, TreeItem.TYPE_RESTRICTION)) return false;
/*
        if (!checkDefinition()) return false;
*/

        return true;
	}


	private String getDefSource(String prop_name, String source)
	{
		String def_source="";
		if (prop_name.equals(tab.getSlotName(NCIEditTab.LONG_DEFINITION)))
			def_source="long_def_nci";
		else if (prop_name.equals(tab.getSlotName(NCIEditTab.DEFINITION)))
			def_source="def_nci";
		else if (prop_name.equals(tab.getSlotName(NCIEditTab.ALT_DEFINITION)))
			def_source="alt_def_"+source;
		else if (prop_name.equals(tab.getSlotName(NCIEditTab.ALT_LONG_DEFINITION)))
			def_source="alt_long_def_"+source;

		if (source==null && (prop_name.equals(tab.getSlotName(NCIEditTab.ALT_DEFINITION)))
			|| prop_name.equals(tab.getSlotName(NCIEditTab.ALT_LONG_DEFINITION))){
			return null;
		}else{
			return def_source;
		}
	}


	/**
	1. Each concept can have at most one Def_Curator property.
	2. If a concept has a Def_Curator property, then only authorized editors can make
		change to the NCI's DEFINITION property.
	3. The names (schema name) of the users who are authorized to make changes to the
		DEFINITION property will be stored in the server.
	4. DEFINITION and ALT_DEFINITION properties length should not exceed 1024.
	5. Date of review time-stamped by the filter (applicable to both definition properties.)
	6. A concept can have multiple DEFINITION and LONG_DEFINITION*/


// Need to parse a XML-string to a vector of NameAndValue pairs

	private boolean checkDefinition()
	{
		HashMap action2curatorMap = getCuratorActionMap();
		if (action2curatorMap == null)
		{
			failure = "WARNING: Multiple Def_Curator properties encountered.";
			return false;
        }
		// get curator
		if (!action2curatorMap.keySet().isEmpty())
		{
			Iterator it = action2curatorMap.keySet().iterator();
			String action = (String) it.next();
			String curator = (String) action2curatorMap.get((Object) action);
			Boolean isProtected = isCuratorProtected(curator);
			if (isProtected.equals(Boolean.FALSE))
			{
				failure = "Unidentifiable curator value -- " + curator + " encountered.";
				return false;
			}
			boolean isAuthorized = hasAuthorization(curator, tdeuser);

			if (isAuthorized)
			{
				System.out.println(tdeuser + " is authorized." );
			}
			else
			{
				System.out.println(tdeuser + " is not authorized." );
			}

			boolean granted = curatorEditActionGranted(action, isProtected.booleanValue(), isAuthorized);
			if (!granted)
			{
				failure = "User "+tdeuser+
						" does not have sufficient privilege to edit the Def_Curator property of value "
						+ curator + ".";
				return false;
			}
	    }
		int total_defcurator=0;
		String def_owner=null;

		Vector before_def = new Vector();
		Vector after_def = new Vector();


		try{
			String s, prop_value;
			String attr=null;
			String source=null;
			String reviewer=null;
			String reviewdate="";

			Vector v;
			Vector props1 = getProperties(initial_state);
			for (int i=0; i<props1.size(); i++)
			{
				// For NCI Definitions Only
				Property prop = (Property) props1.elementAt(i);
				if (definition_types.contains(prop.getName()))
				{
					prop_value = prop.getValue();
					Vector qualifiers = prop.getQualifiers();
					for (int k=0; k<qualifiers.size(); k++){
						Qualifier qualifier = (Qualifier) qualifiers.elementAt(k);
						String qualifierName = qualifier.getName();
						String qualifierValue = qualifier.getValue();
						if (qualifierName.equals("attr"))
							attr = qualifierValue;
						if (qualifierName.equals("definition_review_date"))
							reviewdate = qualifierValue;
						if (qualifierName.equals("definition_reviewer_name"))
							reviewer = qualifierValue;
						else if (qualifier.equals("def_source"))
							source = qualifierValue;
					}

					String def_sum = getDefSource(prop.getName(), source);
					if (def_sum!=null){
						def_sum+="|"+attr+"|"+reviewdate+"|"+reviewer+"|"+prop_value;
						before_def.add(def_sum);
					}
					source=null;
				}
			}

			boolean isDefChg=false;
			HashMap after_def_prop = new HashMap();
			Vector props2 = getProperties(final_state);
			for (int i=0; i<props2.size(); i++)
			{
				// Definitions types:
				// DEFINITION
				// ALT_DEFINITION
				// LONG_DEFINITION
				// ALT_LONG_DEFINITION
                Property prop = (Property) props2.elementAt(i);
				if (definition_types.contains(prop.getName()))
				{
					attr=null;
					int num_reviewer, num_reviewdate, num_attr, num_source;
					num_reviewer=num_reviewdate=num_attr=num_source=0;

					prop_value = prop.getValue();
					Vector qualifiers = prop.getQualifiers();
					for (int k=0; k<qualifiers.size(); k++){
						Qualifier qualifier = (Qualifier) qualifiers.elementAt(k);
						String qualifierName = qualifier.getName();
						String qualifierValue = qualifier.getValue();
						if (qualifierName.equals("attr")){
							attr = qualifierValue;
							num_attr++;
						}else if (qualifierName.equals("definition_review_date")){
							reviewdate = qualifierValue;
							num_reviewdate++;
						}else if (qualifierName.equals("def-source")){
							source = qualifierValue;
							num_source++;
						}else if (qualifierName.equals("definition_reviewer_name")){
							reviewer = qualifierValue;
							num_reviewer++;
						}
					}
					//121505
					// If it's neither DEFINITION nor LONG_DEFINITION (NCI)
					if (!nci_definition_types.contains(prop.getName())){
						if (num_source!=1 || num_reviewdate!=1 || num_reviewer!=1 || num_attr>1){
							failure = prop.getName() + " property format is incorrect - \n"+
								"can have exactly one Definition_Review_Date, one Definition_Reviewer_Name, \n"+
								"one Definition_Source, and one optional attr.";
							return false;
						}
					}else{
						if (num_source > 0 || num_reviewdate!=1 || num_reviewer!=1 || num_attr>1){
							failure = prop.getName() + " property format is incorrect - \n"+
								"can have exactly one Definition_Review_Date, one Definition_Reviewer_Name, \n"+
								"and one optional attr. \n" +
								"(Note: Definition_Source is not needed; it is NCI by default.)";

							return false;
						}
					}

					// short definition type:
					// DEFINITION
					// ALT_DEFINITION

                    if (short_definition_types.contains(prop.getName()))
                    {
						// 010909
						if (prop.getName().compareToIgnoreCase("DEFINITION") == 0)
						{
						    source = "NCI";
						}

						s = "<def-source>"+source+"</def-source>";
						s += "<def-definition>"+prop_value+"</def-definition>";
						//101105
						if (attr !=null)
							s += "<attr>"+attr+"</attr>";
						if (s.length()>1024){
							failure = "The length of "+prop.getName()+
								" property should not exceed 1024.";
							return false;
						}
				    }

					String def_sum = getDefSource(prop.getName(), source);

					if (def_sum!=null){

						def_sum+="|"+attr+"|"+reviewdate+"|"+reviewer+"|"+prop_value;
						after_def.add(def_sum);


					}
					after_def_prop.put(def_sum, prop);

				}else if (prop.getName().equalsIgnoreCase("Def_Curator")){
					def_owner = prop.getValue();
					total_defcurator++;
					if (total_defcurator>1){
						failure = "Concept '"+getConceptName(final_state)+
							"' can have at most one Def_Curator property.";
						return false;
					}
				}
			}
			QuickSortVecStrings qsvs = new QuickSortVecStrings();
			qsvs.sort(before_def);
			qsvs.sort(after_def);

			//020106
			int max = Math.max(before_def.size(), after_def.size());
			String before_val=null, after_val=null;
			String before_value, after_value;
			String before_date, after_date;

			before_value=after_value=before_date=after_date=null;


/*
			for (int i=0; i<max; i++){

				if (before_def.size() > i){
					before_value = (String) before_def.elementAt(i);
					int pos = before_value.indexOf("|");
					String partial = before_value.substring(pos+1);
					pos = partial.indexOf("|");
					partial = partial.substring(pos+1);
					pos = partial.indexOf("|");
					before_date = partial.substring(0, pos);
					pos = before_value.lastIndexOf("|");
					before_val = before_value.substring(pos+1);
				}

				if (after_def.size() > i){
					after_value = (String) after_def.elementAt(i);
					int pos = after_value.indexOf("|");
					String partial = after_value.substring(pos+1);
					pos = partial.indexOf("|");
					partial = partial.substring(pos+1);
					pos = partial.indexOf("|");
					after_date = partial.substring(0, pos);
					pos = after_value.lastIndexOf("|");
					after_val = after_value.substring(pos+1);
				}

				if (before_val==null){
					if (!after_value.startsWith("alt_"))
					   isDefChg=true;

					Property def_prop = (Property) (after_def_prop.get(after_value));

					Qualifier[] qualifiers = getQualifiers(def_prop);
					ArrayList al = new ArrayList();

					for (int k=0; k<qualifiers.length; k++){
						if (qualifiers[k].getName().equals("Definition_Review_Date"))
							qualifiers[k].setValue(getToday());

						al.add(qualifiers[k]);
					}
					after_con.removeProperty(def_prop);
					def_prop.setQualifiers(al);
					after_con.addProperty(def_prop);
				}else if (after_val==null){
					if (!before_val.startsWith("alt_"))
						isDefChg=true;
				}else{

					if (!before_val.equals(after_val)){
						if (!after_value.startsWith("alt_"))
						{
						   isDefChg=true;
						}
						if (!before_date.equals(after_date)){
							//change the after_date
							Property def_prop = (Property) (after_def_prop.get(after_value));

							Qualifier[] qualifiers = getQualifiers(def_prop);
							ArrayList al = new ArrayList();

							for (int k=0; k<qualifiers.length; k++){
								if (qualifiers[k].getName().equals("Definition_Review_Date"))
									qualifiers[k].setValue(getToday());

								al.add(qualifiers[k]);
							}
							after_con.removeProperty(def_prop);
							def_prop.setQualifiers(al);
							after_con.addProperty(def_prop);
						}
					}
				}

				before_val=null;
				after_val=null;
			}
*/


			boolean isValid=false;

			// 010606
			//if (def_owner==null || !authorization.containsKey(def_owner))
			if (def_owner==null)
				isValid=true;
			else if (!authorization.containsKey(def_owner))
			{
				failure = "Curator value "+def_owner+" is not identifiable.";
				return false;
			}
			//else if (tdeuser.equals(""+authorization.get(def_owner)))
			else if (hasAuthorization(def_owner, tdeuser))
			{
				isValid=true;
			}

			if (!isValid && isDefChg)
			{
				failure = "User "+tdeuser+
					" does not have sufficient privilege to change definition properties.";
				return false;
			}

		}catch(Exception ex){
			failure = ex.getMessage();
			Log.getLogger().log(Level.WARNING, "Exception caught", ex);;

			failure = "Exception thrown in checkDefinition()";
			System.out.println("Exception thrown in checkDefinition()");

			return false;
		}
		return true;
	}


	private String getToday()
	{
		java.util.Date now = new java.util.Date();
		//java.text.DateFormat df = new java.text.SimpleDateFormat("MM/dd/yyyy");
		java.text.DateFormat df = new java.text.SimpleDateFormat("yyMMdd");
		return df.format(now);
	}


	private Qualifier[] getQualifiers(Property property)
	{
		Vector qualifier_vec = new Vector();
		String prop_value = property.getValue();
		//if (isComplexProperty(prop_value))
		{
			//def-source
			//def-definition

			prop_value = ComplexPropertyParser.pipeDelim2XML(prop_value);
			Hashtable name2val_hashmap = parseXML(prop_value);
			Iterator iter = name2val_hashmap.keySet().iterator();
			while (iter.hasNext())
			{
				String key = (String) iter.next();
				String value = (String) name2val_hashmap.get(key);
				Qualifier q = new Qualifier(key, value);
				qualifier_vec.add(q);
			}
		}

		Qualifier[] qualifiers;
		qualifiers = new Qualifier[qualifier_vec.size()];

		for (int i=0; i<qualifier_vec.size(); i++)
			qualifiers[i] = (Qualifier) qualifier_vec.elementAt(i);

		return qualifiers;
	}


	/** if after_con kind is not "Retired_Kind", return true.
	  * if before_con has old_kind property and editor is lead editor,
	  * return true; otherwise return false
	  */
	private boolean canRetire()
	{
		if (isRetired(final_state)) return true;

		if (ticketFile.equals("") || !canEditRetired){
			failure = "Does not have sufficient privilege to retire a concept or"+
				" edit a retired concept.";
			return false;
		}

		if (isRetired(initial_state))
		{
			// Lead editor wants to modify a retired concept
			return true;
		}
		else
		{
			Vector props = getProperties(initial_state);
			for (int i=0; i<props.size(); i++){
				Property prop = (Property) props.elementAt(i);
				if (prop.getName().equals("Editor_Note")){
					String value = prop.getValue();
					if (value.startsWith("premerge_annotation") ||
							value.startsWith("preretire_annotation")){
						return true;
					}
				}
			}
		}

		failure = "The concept has not been flagged as pre-retired.";
		return false;
	}

	private String getKindName(String kind){
		return kind.substring(0, kind.length()-5);
	}

	private boolean runCheckPreferred()
	{
		Vector props = getProperties(final_state);

		int numPreferredTerms = 0;
		String prefName = "";
		for (int i = 0;i < props.size();i++)
		{
			Property prop = (Property) props.elementAt(i);
			if (prop!=null && prop.getName().equals(tab.getSlotName(NCIEditTab.PREFERRED_NAME)))
			{
				numPreferredTerms++;
				prefName = prop.getValue();
			}
		}

		if (numPreferredTerms == 0)
		{
			failure = "Concept does not have a preferred name.";
			return false;
		}
		else if (numPreferredTerms == 1)
		{
			if (!checkValues("Invalid Preferred_Name -- ", prefName))
				return false;
		}
		else
		{
			failure = "Cannot have multiple preferred names.";
			return false;
		}
		return true;
    }



	private boolean runCheckProperties() {
		HashSet prop_set = new HashSet();
        Vector v = getProperties(final_state);
		for (int i=0; i<final_state.size(); i++)
		{
			TreeItem item = (TreeItem) final_state.elementAt(i);
			if (item.getType() == TreeItem.TYPE_PROPERTY)
			{
				String s = item.toString();
				if (!prop_set.contains(s))
				{
					prop_set.add(s);
					Property prop = new Property(item.getName(), item.getValue());

					// check non-printable characters
					if (!checkProperty(prop))
					{
						return false;
					}
				}
				else{
					failure = "Cannot have duplicate properties " + s +".";
					System.out.println(failure);
					return false;
				}
			}
		}
		return true;
	}

	private boolean isEquivalentToPT(String termtype)
	{
		if (termtype.equals("PT")||termtype.equals("HD")||termtype.equals("AQ")) return true;
		return false;
	}


	private boolean checkProperty(Property prop)
	{
		/*
		if (prop==null) return true;

		String prop_name = prop.getName();
		String value = prop.getValue();
		String warning = "Invalid " + prop_name + " -- ";
		//failure = prop_name + " has an invalid format.";

		//For all properties
		return checkNonPrintable(value, warning);
		*/

		if (prop==null) return true;

		String prop_name = prop.getName();
		String value = prop.getValue();
		String warning = "Invalid " + prop_name + ": " + value + ". ";

		return checkPropertyValue(warning, value, false);
		//return true;
	}


    private Property[] convertToArray(Vector v)
    {
		Property[] props = new Property[v.size()];
		for (int i=0;i<v.size(); i++)
		{
			props[i] = (Property) v.elementAt(i);
		}
		return props;
	}

	/**
	1.More than one NCI/PT in FULL_SYN (Note: A concept does not need to
	  have a NCI PT in FULL_SYN)
	2.NCI/PT entry in the FULL_SYN property does not match with the Preferred_Name field
	3.Allow full_syns with no qualifiers to pass through
	4.One full_syn must be either HD or PT with either no source or NCI source
	*/
	//private boolean checkFullSyn(Property[] props){

	private boolean checkFullSyn(){
		Vector properties = getProperties(final_state);
		Property[] props = new Property[properties.size()];
		for (int i=0; i<properties.size(); i++)
		{
			props[i] = (Property) properties.elementAt(i);
		}

		HashSet fullsyn_set = new HashSet();
		String fullsyn_val, source, termtype;
		String pt;

		source=termtype=pt=null;

		int pt_knt = 0;

		for (int i=0; i<props.length; i++)
		{
			if (props[i].getName().compareTo(tab.getSlotName(NCIEditTab.FULL_SYN)) == 0)
			{
                Qualifier[] qualifiers = getQualifiers(props[i]);

				for (int j=0; j<qualifiers.length; j++)
				{
					String name = qualifiers[j].getName();
					String value = qualifiers[j].getValue();
				}

				int num_source=0;
				int num_type=0;
				int num_code=0;

				for (int k=0; k<qualifiers.length; k++){
					if (qualifiers[k].getName().equals("term-source")) {
						source = qualifiers[k].getValue();
						num_source++;
					}else if (qualifiers[k].getName().equals("term-group")) {
						termtype = qualifiers[k].getValue();
						num_type++;
					}else if (qualifiers[k].getName().equals("source-code")) {
						num_code++;
					}else if (qualifiers[k].getName().equals("term-name"))
						fullsyn_set.add(qualifiers[k].getValue());
				}



				//020206
				if (num_type==0){
					termtype = DEFAULT_TERMTYPE;
					num_type++;
				}
				if (num_source==0){
					source = DEFAULT_SOURCE;
					num_source++;
				}

				if (num_source>1 || num_type>1 || num_code>1){
					failure = "FULL_SYN property format is incorrect: "+
						"can have exactly one term-source and one term-group and "+
						"one optional source-code.";
					return false;
				}

                if (isEquivalentToPT(termtype) && source.equals("NCI")) {
					pt_knt ++;

					if (pt_knt > 1)
					{
						failure = "Cannot have more than one NCI PT (or its equivalent) FULL_SYN properties. ";
						return false;
					}
				}

			}else if (props[i].getName().equals(tab.getSlotName(NCIEditTab.PREFERRED_NAME)))
			{
				pt = props[i].getValue();
			}
		}

		if (fullsyn_set.size()!=0 && !fullsyn_set.contains(pt)){
			failure = "NCI/"+termtype+" FULL_SYN property name does not match "+
				"with Preferred_Name.";
			return false;
		}

		if (pt_knt == 0)
		{
			failure = "Must have at least one NCI PT (or its equivalent) FULL_SYN properties. ";
			return false;
		}

		return true;
	}


// to be tested
	private boolean checkNonPrintable(String value, String warning)
	{
		// rdf:comment can be null
		if (value == null) return true;

		int len = value.length();

		if (len == 0) return true;

		char c1 = value.charAt(0);
		char c2 = value.charAt(len-1);
		int trimlen = value.trim().length();

		if (warning == null)
			warning = "";

		if (trimlen != len)
		{
			failure = warning
				+ "No leading or trailing space is allowed.";
			return false;
		}
		else
		{
			int count = 0;
			for (int i=1; i<len; i++)
			{
				char ch = value.charAt(i);
				int charVal = (int)ch;

				//020606
				//utf-8 pass-through (except control chars, dec < 32)
				//if (charVal >= 32){
				if ((charVal >= 32) && (charVal < 127)) {
					if ((c1==':') || (c1=='!') || (c1=='?') || (c1=='.')
						//090705 Disallow @ character in data
						|| (c1=='@'))
					{
						if (ch == ' ') {
							if (count < 2) {
								count++;
								continue;
							}else {
								failure = warning
									+ "Too many spaces between tokens.";
								return false;
							}
						}
					}else if ((ch == ' ') && (c1 == ' ')) {
						failure = warning
							+ "Too many spaces between tokens.";
						return false;
					}
					c1 = ch;
					count=0;
				}else {
					failure = "Cannot have nonprintable character: "+ ch;
					return false;
				}
			}
		}
		return true;
	}

	/* *
	* for the DEFINITION property, only one space between tokens
	* except when preceeded by period (.), exclamation mark (!),
	* or question mark (?), in which case two spaces are allowed.
	* */
	public Vector parseString(String xml)
	{
		Vector v = new Vector();
		StringTokenizer st = new StringTokenizer(xml, "|");

		while (st.hasMoreTokens())
				v.add(st.nextToken());

		return v;
	}


	public boolean checkValues(String warning, String value)
	{
		int len = value.length();
		char prev = value.charAt(0);
		int trimlen = value.trim().length();
		boolean warningOff = false;
		int count = 0;

		if (warning == null)
			warning = "";

		if (trimlen != len) {
			failure = warning
				+ "No leading or trailing space is allowed.";
			return false;
		}

		for (int i=1; i<len; i++) {
			char curr = value.charAt(i);
			int charVal = (int) curr;

			//020606
			//if ((charVal < 32) || (charVal >= 127)) {
			if (charVal < 32){
				failure = "Cannot have nonprintable character: "+ curr;
				return false;
			}

			if ((prev == ' ') && (curr == ' ')) {
				count++;
				if (count > 1) {
					failure = warning
						+ "Too many spaces between tokens.";
					return false;
				}else if (!warningOff && count == 1) {
					failure = warning
						+ "Too many spaces between tokens.";
					return false;
				}
			}

			if ((curr == '!') || (curr == '?') || (curr == '.')) {
				warningOff = true;
				count = 0;
			}
			prev = curr;
		}
		return true;
	}


	public boolean checkPropertyValue(String warning, String value, boolean checknonprintable)
	{
		int len = value.length();
		if (len == 0) return true;

		char prev = value.charAt(0);
		int trimlen = value.trim().length();
		if (trimlen == 0) return true;

		boolean warningOff = false;
		int count = 0;

		if (warning == null)
			warning = "";

		if (trimlen != len) {
			failure = warning
				+ "No leading or trailing space is allowed.";
			return false;
		}

		for (int i=1; i<len; i++) {
			char curr = value.charAt(i);
			int charVal = (int) curr;

			//020606
			//if ((charVal < 32) || (charVal >= 127)) {
			if (checknonprintable)
			{
				if (charVal < 32){
					failure = "Cannot have nonprintable character: "+ curr;
					return false;
				}
		    }

			if ((prev == ' ') && (curr == ' ')) {
				count++;
				if (count > 1) {
					failure = warning
						+ "Too many spaces between tokens.";
					return false;
				}else if (!warningOff && count == 1) {
					failure = warning
						+ "Too many spaces between tokens.";
					return false;
				}
			}

			if ((curr == '!') || (curr == '?') || (curr == '.')) {
				warningOff = true;
				count = 0;
			}
			prev = curr;
		}
		return true;
	}


	/* *
	 * for the DEFINITION property, only one space between tokens
	 * except when preceeded by period (.), exclamation mark (!),
	 * or question mark (?), in which case two spaces are allowed.
	 * */

    private String getFirstPropertyValue(Vector treeitems, String prop_name)
    {
		for (int i=0; i<treeitems.size(); i++)
		{
			TreeItem item = (TreeItem) treeitems.elementAt(i);
			if (item.getName().compareTo(prop_name) == 0)
			{
				return item.getValue();
			}
		}
		return "";
	}


    private String getConceptCode(Vector treeitems)
    {
		return getFirstPropertyValue(treeitems, "code");
	}

    private String getConceptName(Vector treeitems)
    {
		return getFirstPropertyValue(treeitems, "owl:label");
	}

	public String addApostrophe(String name)
	{
		String retStr = "";
		for (int i=0; i<name.length(); i++) {
			char c = name.charAt(i);
			Character ch = new Character(c);

			if (c == '\'')
				retStr = retStr + "''";
			else
				retStr = retStr + ch.toString();
		}
		return retStr;
	}

    // Concept name is not editable
    // Both before and after images of concept name must meet checkXMLNCNameCompliance.

	public boolean runCheckXMLNCNameCompliance(String name)
	{
		String warning = "Invalid concept name " + name;
		return checkXMLNCNameCompliance(warning, name);
	}


	public static boolean checkXMLNCNameCompliance(String value)
	{
        if (value == null) return false;
        if (value.length() == 0) return false;

        boolean retval;
		// Leading character must be a letter or '_'
		char c = value.charAt(0);

		if(! (Character.isLetter(c) || c=='_'))
		{
			return false;
		}

		// All characters must be a letter, a digit, or in {'.', '-', '_'}
		// NCICB disallows period.
		for (int i=1; i<value.length(); i++) {
			c = value.charAt(i);
			if(! (Character.isLetter(c) || Character.isDigit(c) || c=='-' || c=='_'))
			{
				return false;
			}
		}
		return true;
	}


	private boolean checkXMLNCNameCompliance(String warning, String value) {

        boolean retval;
		// Leading character must be a letter or '_'
		char c = value.charAt(0);

		if(! (Character.isLetter(c) || c=='_'))
		{
			failure = warning + "\n Concept name must start with a letter or '_'";
			return false;
		}

		// All characters must be a letter, a digit, or in {'.', '-', '_'}
		// NCICB disallows period.
		for (int i=1; i<value.length(); i++) {
			c = value.charAt(i);
			if(! (Character.isLetter(c) || Character.isDigit(c) || c=='-' || c=='_'))
			{
				failure = warning + "\n Cannot have character: '" + c + "'";
				return false;
			}
		}
		return true;
	}


    private Vector getProperties(Vector treeitems)
    {
		Vector v = new Vector();
		for (int i=0; i<treeitems.size(); i++)
		{
			TreeItem item = (TreeItem) treeitems.elementAt(i);
			if (item.getType() == TreeItem.TYPE_PROPERTY)
			{
				Property p = new Property(item.getName(), item.getValue());
				v.add(p);
			}
		}
		return v;
	}

    // 010606
	private HashMap getCuratorActionMap()
	{
		// return null if multiple Def_Curator properties were found in either image
		HashMap hashmap = new HashMap();
		String Def_Curator = "Def_Curator";
		try{
			//ConceptDef after_con = e.getConceptDef();
			//if (cdq == null) cdq = e.getConceptDefQuery();
			//ConceptDef before_con = cdq.getConceptByCode(after_con.getCode());

			String s, prop_value;
			String before_curator = "";
			String after_curator = "";
			String action;

			Vector props1 = getProperties(initial_state);

// Def_Curator is a property name

			for (int i=0; i<props1.size(); i++)
			{
				Property prop = (Property) props1.elementAt(i);
				if (prop.getName().compareTo(Def_Curator) == 0)
				{
					if (before_curator.compareTo("") == 0)
					{
						before_curator = prop.getValue();
					}
					else
					{
						// multiple Def_Curator properties encountered
						return null;
					}
				}
	        }

			Vector props2 = getProperties(final_state);

			for (int i=0; i<props2.size(); i++)
			{
				Property prop = (Property) props2.elementAt(i);
				if (prop.getName().compareTo(Def_Curator) == 0)
				{
					if (after_curator.compareTo("") == 0)
					{
						after_curator = prop.getValue();
					}
					else
					{
						// multiple Def_Curator properties encountered
						return null;
					}
				}
	        }

	        if (before_curator.compareTo("") == 0 && after_curator.compareTo("") == 0) return hashmap;
	        else if (before_curator.compareTo("") == 0 && after_curator.compareTo("") != 0)
	        {
			    action = "Create";
			    hashmap.put(action, after_curator);
			}
	        else if (before_curator.compareTo("") != 0 && after_curator.compareTo("") == 0)
	        {
			    action = "Delete";
			    hashmap.put(action, before_curator);
			}
	        else if (before_curator.compareTo("") != 0 && after_curator.compareTo("") != 0)
	        {
			    action = "Modify";
			    if (before_curator.compareToIgnoreCase(after_curator) != 0)
			    {
			       hashmap.put(action, before_curator);
			    }
			}
	    } catch (Exception ex) {
            Log.getLogger().log(Level.WARNING, "Exception caught", ex);;
		}
		return hashmap;
	}

    // 010606
	// There is no non-protected Def_Curator values. ie.e, dummy_def_curators, they must
	// all exist in the configuration file.

	private Boolean isCuratorProtected(String curator)
	{
		if (!authorization.containsKey(curator))
		{
			return Boolean.FALSE;
		}
		return Boolean.TRUE;
	}

    // 010606
	public boolean hasAuthorization(String curator, String user)
	{
		System.out.println("Checking authorization ...");
		System.out.println("curator: " + curator);
		System.out.println("user: " + user);

		if (authorization.containsKey(curator))
		{
			Vector v = (Vector) authorization.get(curator);
			for (int i=0; i<v.size(); i++)
			{
				String authorizedUser = (String) v.elementAt(i);
				if (authorizedUser.compareToIgnoreCase(user) == 0) return true;
			}
			return false;
		}
		return true;
	}

    // 010606
	private boolean curatorEditActionGranted(String action, boolean isProtected, boolean isAuthorized)
	{

		System.out.println("Calling curatorEditActionGranted ...");
		System.out.println("input action: " + action);
		if (isAuthorized)
		{
			System.out.println("input isAuthorized: true");
		}
		else
		{
			System.out.println("input isAuthorized: false");
		}

		if (isProtected)
		{
			System.out.println("input isProtected: true");
		}
		else
		{
			System.out.println("input isProtected: false");
		}


		boolean granted = false;
		if (action.compareToIgnoreCase("Create") == 0)
		{
			if (isProtected && isAuthorized) granted = true;
		}
		else if (action.compareToIgnoreCase("Modify") == 0)
		{
			if (isProtected && isAuthorized) granted = true;
		}
		else if (action.compareToIgnoreCase("Delete") == 0)
		{
			if (isProtected && isAuthorized) granted = true;
		}
		return granted;
	}

	 private void addAuthorization(String key, String value)
	 {
		 if (authorization.containsKey(key))
		 {
			 Vector v = (Vector) authorization.get(key);
			 v.add(value);
			 authorization.put(key, v);

			 System.out.println("Key: " + key + " Value: " + value);
		 }
		 else
		 {
			 Vector v = new Vector();
			 v.add(value);
			 authorization.put(key, v);

			 System.out.println("Key: " + key + " Value: " + value);
		 }
	 }


    private Vector getPropertyValues(Vector state_vec, String propertyname)
    {
		Vector v = new Vector();
		for (int i=0; i<state_vec.size(); i++)
		{
			TreeItem item = (TreeItem) state_vec.elementAt(i);
			if (item.getType() == TreeItem.TYPE_PROPERTY)
			{
				if (item.getName().compareTo(propertyname) == 0)
				{
					v.add(item.getValue());
				}
			}
		}
		return v;
	}



    private boolean canAdd(TreeItem item, Vector initialState, Vector finalState)
    {
        if (item.getType() == TreeItem.TYPE_PROPERTY)
        {
			if (item.getName().compareTo(tab.getSlotName(NCIEditTab.DEFINITION)) == 0)
			{
				Vector v = getPropertyValues(initialState, DEF_CURATOR);
				if (v.size() > 0)
				{
					for (int i=0; i<v.size(); i++)
					{
						String curator = (String) v.elementAt(i);
						System.out.println("Curator: " + curator);
						String user = tab.getUserName();
						boolean retval = hasAuthorization(curator, user);
						if (retval)
						{
							System.out.println("User " + user + " can add DEFINITION.");
						}
						else
						{
							failure = "User " + user + " does not have authority to add a DEFINITION.";
							System.out.println(failure);
						    return false;
						}
					}
				}
			}
		}
		return true;
	}


    public boolean canAdd(Vector items, Vector initialState, Vector finalState)
    {
		for (int i=0; i<items.size(); i++)
		{
			TreeItem item = (TreeItem) items.elementAt(i);
			boolean retval = canAdd(item, initialState, finalState);
			if (!retval)
			{
				return false;
			}
		}
		return true;
	}

    public boolean canDelete(Vector items, Vector initialState, Vector finalState)
    {
		for (int i=0; i<items.size(); i++)
		{
			TreeItem item = (TreeItem) items.elementAt(i);
			boolean retval = canDelete(item, initialState, finalState);
			if (!retval)
			{
				return false;
			}
		}
		return true;
	}

    private boolean canDelete(TreeItem item, Vector initialState, Vector finalState)
    {
        if (item.getType() == TreeItem.TYPE_PROPERTY)
        {
			if (item.getName().compareTo(tab.getSlotName(NCIEditTab.DEFINITION)) == 0)
			{
				Vector v = getPropertyValues(initialState, DEF_CURATOR);
				if (v.size() > 0)
				{
					for (int i=0; i<v.size(); i++)
					{
						String curator = (String) v.elementAt(i);
						System.out.println("Curator: " + curator);
						String user = tab.getUserName();
						boolean retval = hasAuthorization(curator, user);
						if (retval)
						{
							System.out.println("User " + user + " can edit a DEFINITION.");
						}
						else
						{
							System.out.println("User " + user + " does not have authority to edit a DEFINITION.");
						    return false;
						}
					}
				}
			}
		}
		return true;

	}



    public boolean canModify(TreeItem item, Vector initialState)
    {
        if (item.getType() == TreeItem.TYPE_PROPERTY)
        {
			if (item.getName().compareTo(tab.getSlotName(NCIEditTab.DEFINITION)) == 0)
			{
				Vector v = getPropertyValues(initialState, DEF_CURATOR);
				if (v.size() > 0)
				{
					for (int i=0; i<v.size(); i++)
					{
						String curator = (String) v.elementAt(i);
						System.out.println("Curator: " + curator);
						String user = tab.getUserName();
						boolean retval = hasAuthorization(curator, user);
						if (retval)
						{
							System.out.println("User " + user + " can edit a DEFINITION.");
						}
						else
						{
							System.out.println("User " + user + " does not have authority to edit a DEFINITION.");
						    return false;
						}
					}
				}
			}
		}
		return true;
	}


	public Hashtable getHistoryHash()
	{
		return historyHash;
	}

 	public Hashtable parseXML(String xml)
	{
		Hashtable name2val_hashmap;
		xml="<?xml version=\"1.0\"?><fake>"+xml+"</fake>";
		try
		{
			name2val_hashmap = new Hashtable();
			DOMParser parser = new DOMParser();
			BufferedReader reader = new BufferedReader(new StringReader(xml));
			InputSource inputsource = new InputSource(reader);
			parser.parse(inputsource);
			Document document = parser.getDocument();
			traverse (document, name2val_hashmap);
			reader.close();
		} catch (Exception e) {
			Log.getLogger().log(Level.WARNING, "Exception caught", e);;
			return null;
		}
		return name2val_hashmap;
	}

    private void traverse (Node node, Hashtable name2val_hashmap)
    {
        int type = node.getNodeType();
        if (type == Node.ELEMENT_NODE)
        {
            String nodename = node.getNodeName();
			NodeList children = node.getChildNodes();
			if (children!=null && children.item(0)!=null){
				String nodevalue = children.item(0).getNodeValue();
				if (nodevalue!=null)
					name2val_hashmap.put(nodename, nodevalue);
			}
		}
		NodeList children = node.getChildNodes();
        if (children != null) {
            for (int i=0; i< children.getLength(); i++)
                traverse (children.item(i), name2val_hashmap);
        }
    }


    private boolean isComplexProperty(String prop_value)
    {
		if (prop_value.contains("<") && prop_value.contains("</"))
		{
			return true;
		}
		return false;
    }


}
