/*
* The contents of this file are subject to the Mozilla Public License
* Version 1.1 (the "License");  you may not use this file except in compliance
* with the License.  You may obtain a copy of the License at
* http://www.mozilla.org/MPL/
*u
* Software distributed under the License is distributed on an "AS IS" basis,
* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
* the specific language governing rights and limitations under the License.
*
* The Original Code is Protege-2000.
*
* The Initial Developer of the Original Code is Stanford University. Portions
* created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
*
* Protege-2000 was developed by Stanford Medical Informatics
* (http://www.smi.stanford.edu) at the Stanford University School of Medicine
* with support from the National Library of Medicine, the National Science
* Foundation, and the Defense Advanced Research Projects Agency.  Current
* information about Protege can be obtained at http://protege.stanford.edu
*
* This code was developed by Northrop Grumman Information Technology (NGIT)
* Contributor(s): Kim Ong, Iris Guo
*/

package gov.nih.nci.protegex.edit;


import java.util.*;
import java.lang.System;

import javax.swing.*;

import edu.stanford.smi.protege.widget.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.resource.*;
import edu.stanford.smi.protege.ui.*;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protegex.owl.model.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;

import javax.swing.tree.*;

import java.awt.FileDialog;

import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import edu.stanford.smi.protegex.owl.ui.resourcedisplay.ResourceDisplay;
import edu.stanford.smi.protegex.owl.ui.resourcedisplay.ResourcePanel;
import edu.stanford.smi.protegex.owl.ui.ProtegeUI;

import edu.stanford.smi.protege.event.*;
import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Model;
import edu.stanford.smi.protegex.owl.model.OWLNamedClass;
import edu.stanford.smi.protegex.owl.model.RDFSClass;

////////
import edu.stanford.smi.protegex.owl.ui.resourcedisplay.*;
import edu.stanford.smi.protegex.owl.ui.components.annotations.*;
import edu.stanford.smi.protegex.owl.ui.conditions.*;
import edu.stanford.smi.protegex.owl.ui.icons.*;
import edu.stanford.smi.protegex.owl.ui.owltable.*;
import edu.stanford.smi.protegex.owl.ui.restrictions.*;
import edu.stanford.smi.protegex.owl.ui.widget.OWLUI;


import edu.stanford.smi.protege.util.LabeledComponent;
import edu.stanford.smi.protegex.owl.ui.components.triples.*;

import edu.stanford.smi.protege.plugin.PluginUtilities;

import javax.swing.border.*;

import edu.stanford.smi.protege.server.metaproject.impl.OperationImpl;
import edu.stanford.smi.protege.server.framestore.RemoteClientFrameStore;

import edu.stanford.smi.protege.util.Log;
import java.util.logging.Level;


public class NCIEditTab extends AbstractTabWidget
implements ActionListener {

    static int BY_NAME = 1;
    static int BY_CODE = 2;

    public static int owl_format;

	static String MERGE_SOURCE = "Merge_Source";
	static String MERGE_TARGET = "Merge_Target";
	static String MERGE_INTO = "Merge_Into";
	static String SPLIT_FROM = "Split_From";
	static String OLD_PARENT = "OLD_PARENT";
	static String OLD_CHILD = "OLD_CHILD";
	static String OLD_SOURCE_ROLE = "OLD_SOURCE_ROLE";
	static String OLD_ROLE = "OLD_ROLE";
	static String EDITOR_NOTE = "Editor_Note";
	static String DESIGNNOTE = "DesignNote";
	static String OLD_STATE = "OLD_STATE";
	static String CONCEPT_STATUS = "Concept_Status";
	static String FULL_SYN = "FULL_SYN";
	static String DEFINITION = "DEFINITION";
	static String PREFERRED_NAME = "Preferred_Name";
	static String DEF_CURATOR = "Def_Curator";
	static String CODE = "code";
	static String ALT_DEFINITION = "ALT_DEFINITION";
	static String LONG_DEFINITION = "LONG_DEFINITION";
	static String GO_ANNOTATION = "GO_Annotation";
	static String GO_ANNOTATION2 = "GO_ANNOTATION";
	static String ALT_LONG_DEFINITION = "ALT_LONG_DEFINITION";

    String focusedCls = "";

   	Vector noneditable_vec = null;
   	Vector requiredProperties = null;
   	Vector readOnlyProperties = null;


	static String RETIRED_CONCEPTS = "Retired_Concepts";
	static String PRERETIRED_CONCEPTS = "Preretired_Concepts";
	static String PREMERGED_CONCEPTS = "Premerged_Concepts";


   	private int readOnlyPropertyCount = 0;
   	private int requiredPropertyCount = 0;

    private NCICodeGenerator generator;
    private OWLWrapper wrapper;
    //private ClsesPanel _clsesPanel;
    private NCIClsesPanel _clsesPanel;

    private ClsInverseRelationshipPanel _inverseRelationshipPanel;
    private InstanceDisplay _instanceDisplay;

    protected ResourcePanel resourcePanel;

    private JPanel leftComponent;
    private JSplitPane mainSplitPane;
    private Instance selectedInstance;
    private JButton fileChooserButton;


    private JButton generateButton_Reportwriter;

	private	JProgressBar progress;
	private	JLabel progress_msg;

    private JLabel status_Reportwriter;

    private JButton clearButton_Reportwriter;
    private JButton saveButton_Reportwriter;

    private JTextField fileName;


    private	JList list;

    private JLabel superConceptLabel;
    private File exportFile;
    private Vector classList;
    private JTextArea reportTextArea;

    private JTextArea loaderTextArea;
    private JTextArea editorTextArea;

	private JPanel textAreaPanel;
	private HashSet nodelist;

	private String newline = "\n";

	Collection selection;

	private JPanel tabbedPanel;

	private JTabbedPane tabbedPane;
	private JTabbedPane tabbedPane_Preretire;

	private JTabbedPane tabbedPane_1;
	private JTabbedPane tabbedPane_Modify;


	private boolean unmergeButtonPressed;
	private boolean unretireButtonPressed;

	private boolean preretireButtonPressed;

	//022806
	JComponent mainSplitter;

    private JButton createButton_Edit;
	private JButton clearButton_Edit;
    private JButton saveButton_Edit;

    private JButton saveButton_Split;
    private JButton saveButton_Premerge;
    private JButton saveButton_Merge;
    private JButton saveButton_Preretire;
    private JButton saveButton_Retire;

    private JButton clearButton_Split;
	private JButton clearButton_Premerge;
	private JButton clearButton_Merge;
	private JButton clearButton_Preretire;
	private JButton clearButton_Retire;

	private JButton subClsButton_Preretire;
	private JButton refClsButton_Preretire;
	private JButton unretireButton_Preretire;
    private JButton preretireButton_Preretire;

	private JButton mergeButton_Merge;
	private JButton unmergeButton_Merge;

    private JButton premergeButton_Premerge;
    private JButton unmergeButton_Premerge;

    private JButton retireButton_Retire;

	private JButton splitButton_Split;

	private JButton unretireButton_Retire;

	private JButton loadButton_Loader;
	private JButton saveButton_Loader;
	private JButton clearButton_Loader;

	private JButton loadButton_Editor;
	private JButton saveButton_Editor;
	private JButton clearButton_Editor;

	private JButton treeButton_Partonomy;
	private JButton saveButton_Partonomy;
	private JButton clearButton_Partonomy;

	private JButton clearButton_Delete;
	private JButton deleteButton_Delete;


	private Component panel1;
	private Component panel2;
	private Component panel3;
	private Component panel4;
	private Component panel5;
	private Component panel6;
	private Component panel7;
	private Component panel8;
	private Component panel9;
	private Component panel10;
	private Component panel11;


	private EditPanel panel12;

	private JScrollPane panel1_upperPane; // edit
	private JScrollPane panel2_upperPane; // split
	private JScrollPane panel2_lowerPane; // split
	private JScrollPane panel3_lowerPane; // premerge
	private JScrollPane panel3_upperPane; // premerge
	private JScrollPane panel4_lowerPane; // merge
	private JScrollPane panel4_upperPane; // merge
	private JScrollPane panel5_upperPane; // preretire
	private JScrollPane panel5_lowerPane; // preretire

	private JScrollPane subclass_Pane; // preretire
	private JScrollPane reference_Pane; // preretire

	private JList subclassList; // preretire
	private JList referenceList; // preretire

    private JScrollPane panel6_upperPane; // retire

    private JScrollPane partonomyTreeScrollPane;


	//042806
	CloneTreePanel upperTreePanel_Clone = null;
	CloneTreePanel lowerTreePanel_Clone = null;

	//private JButton newButton_Clone;
	private JButton cloneButton_Clone;
	private JButton saveButton_Clone;
	private JButton clearButton_Clone;


	private JScrollPane panel11_upperPane; // clone
	private JScrollPane panel11_lowerPane; // clone

	private Component panel0;
	private JScrollPane panel0_upperPane; // delete


    TreePanel upperTreePanel_Split = null;
    TreePanel lowerTreePanel_Split = null;

    TreePanel lowerTreePanel_Premerge = null;
    TreePanel upperTreePanel_Premerge = null;

    TreePanel lowerTreePanel_Merge = null;
    TreePanel upperTreePanel_Merge = null;

    TreePanel upperTreePanel_Retire = null;
    TreePanel leftTreePanel_Preretire = null;

    TreePanel upperTreePanel_Delete = null;

    private Vector subVec, restrictionVec;

	private Collection subClassCollection = null;
	private Collection referringClasses = null;

	private Cls droppedCls = null;
	//private ResourcePanel subrefClsPanel = null;
	private EditPanel subrefClsPanel = null;

	SubMouseListener submouseListener = null;
    ReferenceMouseListener referencemouseListener = null;

    DataHandler datahandler = null;

    static OWLModel owlModel = null;

    private NCIEditFilter editfilter;
    //private EditHistory edithistory = null;

    int num_concepts_premerge = 0;
    int num_concepts_merge = 0;

    boolean can_undo_premerge = false;

    PartonomyPanel partonomyPanel;

	//041706
	int lastSel;
	//042006
	JSplitPane splitPane;

	//DefaultListModel subclassListModel;
	JList subClsList;

	DefaultListModel referenceListModel;
	DefaultListModel subclassListModel;

	EventListener eventListener = null;

	//042006
	JSplitPane clonePane;
	//050206
	TreeItem copied_item;
	//050906
	boolean cloned_cls;

	boolean chgtab_flag = false;

	static final String SUBCLASSOF = "rdfs:subClassOf";

    private InstanceNameComponent name_panel = null;

    JButton reviewButton_Modify = null;
	JButton saveButton_Modify = null;
	JButton cancelButton_Modify = null;

	JLabel codeLabel = null;

    boolean canDelete = false;

    private String username = "";
    private String remoteServerIP = "";
    private String remoteServerPort = "";
    //private String user = "";

	private	HashMap name2rdfID = null;
	private	HashMap rdfID2Label = null;
	private	HashMap rdfID2Slot = null;

	private	HashMap name2Slot = null;
	private	HashMap name2SlotName = null;

	private	HashMap name2Cls = null;
	private	HashMap name2ClsName = null;
	private	HashMap name2rdfIDCls = null;

	private	HashMap externalNameMap = null;

	private HashSet rdfIDHash = null;

    ///////////////////////////////////////////////////
    // EditPanel
    ///////////////////////////////////////////////////

	EditPanel upperEditPanel_Split = null;
	EditPanel lowerEditPanel_Split = null;

	EditPanel upperEditPanel_Premerge = null;
	EditPanel lowerEditPanel_Premerge = null;

	EditPanel upperEditPanel_Merge = null;
	EditPanel lowerEditPanel_Merge = null;

	EditPanel upperEditPanel_Retire = null;
	EditPanel leftEditPanel_Preretire = null;

	int prev_selection = -1;
	int curr_selection = -1;

	boolean historyOn = false;

    public final static String EDIT_READ_ACTION = "EditRead";
    public final static String EDIT_WRITE_ACTION = "EditWrite";


    public final static String EDIT_BASIC_ACTION = "EditBasicRead";
    public final static String EDIT_PROPERTIES_ACTION = "EditPropertiesRead";
    public final static String EDIT_RELATIONS_ACTION = "EditRelationsRead";
    public final static String EDIT_RETIRED_CLASS_ACTION = "EditRetiredClass";


    public final static String GENERATE_REPORT_ACTION = "GenerateReport";
    public final static String MERGE_ACTION = "Merge";
    public final static String COPY_ACTION = "NCICopy";
    public final static String PRE_MERGE_ACTION = "PreMerge";
    public final static String PRE_RETIRE_ACTION = "PreRetire";
    public final static String RETIRE_ACTION = "Retire";
    public final static String SPLIT_ACTION = "Split";
    public final static String LOAD_BATCH_ACTION = "LoadBatch";
    public final static String EDIT_BATCH_ACTION = "EditBatch";
    public final static String PARTONOMY_TREE_ACTION = "Partonomize";
    public final static String EDIT_ACTION = "Edit";
    public final static String DELETE_ACTION = "Delete";
    public final static String EDIT_RETIRED_CLASS = "EditRetiredClass";


    public final static OperationImpl EDIT_READ = new OperationImpl("EditRead");
    public final static OperationImpl EDIT_WRITE = new OperationImpl("EditWrite");


    public final static OperationImpl EDIT_BASIC = new OperationImpl("EditBasicRead");
    public final static OperationImpl EDIT_PROPERTIES = new OperationImpl("EditPropertiesRead");
    public final static OperationImpl EDIT_RELATIONS = new OperationImpl("EditRelationsRead");

    public final static OperationImpl GENERATE_REPORT = new OperationImpl("WriteReport");
    public final static OperationImpl MERGE = new OperationImpl("Merge");
    public final static OperationImpl COPY = new OperationImpl("NCICopy");
    public final static OperationImpl PRE_MERGE = new OperationImpl("PreMerge");
    public final static OperationImpl PRE_RETIRE = new OperationImpl("PreRetire");
    public final static OperationImpl RETIRE = new OperationImpl("Retire");
    public final static OperationImpl SPLIT = new OperationImpl("Split");
    public final static OperationImpl DELETE = new OperationImpl("Delete");
    public final static OperationImpl LOAD_BATCH = new OperationImpl("LoadBatch");
    public final static OperationImpl EDIT_BATCH = new OperationImpl("EditBatch");
    public final static OperationImpl PARTONOMY_TREE = new OperationImpl("Partonomize");
    public final static OperationImpl EDIT_RETIRED = new OperationImpl("EditRetiredClass");


    HashMap permissionHashMap = null;
    Vector action_vec = null;
    boolean isRemoteProject = false;

    int iCtr_progress = 0;

    public HashMap CustomizedAnnotationData_Map;
    //010307
    public Vector customizedAnnotationKeys;

    ReportWriter ReportWriter_dlg = null;

    JScrollPane SubRefClsPanelContainier = null;

    private void initializePermissionHashMap()
    {
		action_vec = new Vector();
		permissionHashMap = new HashMap();

		isRemoteProject = true;

		owlModel = getOWLModel();

		if (owlModel == null)
		{
			System.out.println("WARNING: OWLModel is null.");
			System.exit(0);

		}
		wrapper = new OWLWrapper(getOWLModel());
		if (owlModel.getProject().isMultiUserClient()) {
			isRemoteProject = true;

			System.out.println("Checking user privilege from metaproject...");
			loadPropertyFile("nci.properties", false);
			username = owlModel.getProject().getLocalUser();
			wrapper.setUserName(username);

			permissionHashMap.put(EDIT_READ_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, EDIT_READ)));
			permissionHashMap.put(EDIT_WRITE_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, EDIT_WRITE)));

			permissionHashMap.put(EDIT_BASIC_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, EDIT_BASIC)));
			permissionHashMap.put(EDIT_PROPERTIES_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, EDIT_PROPERTIES)));
			permissionHashMap.put(EDIT_RELATIONS_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, EDIT_RELATIONS)));
			permissionHashMap.put(GENERATE_REPORT_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, GENERATE_REPORT)));
			permissionHashMap.put(MERGE_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, MERGE)));
			permissionHashMap.put(COPY_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, COPY)));
			permissionHashMap.put(PRE_MERGE_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, PRE_MERGE)));
			permissionHashMap.put(PRE_RETIRE_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, PRE_RETIRE)));
			permissionHashMap.put(RETIRE_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, RETIRE)));
			permissionHashMap.put(SPLIT_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, SPLIT)));
			permissionHashMap.put(LOAD_BATCH_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, LOAD_BATCH)));
			permissionHashMap.put(EDIT_BATCH_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, EDIT_BATCH)));
			permissionHashMap.put(PARTONOMY_TREE_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, PARTONOMY_TREE)));

			permissionHashMap.put(EDIT_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, EDIT_BASIC)));
			permissionHashMap.put(DELETE_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, DELETE)));
			permissionHashMap.put(EDIT_RETIRED_CLASS_ACTION, new Boolean(RemoteClientFrameStore.isOperationAllowed(owlModel, EDIT_RETIRED)));
	    }
	    else
	    {
			loadPropertyFile("nci.properties", false); // set user to local user
			getDefaultUser();
			wrapper.setUserName(username);

			permissionHashMap.put(EDIT_READ_ACTION, Boolean.TRUE);
			permissionHashMap.put(EDIT_WRITE_ACTION, Boolean.TRUE);


			permissionHashMap.put(EDIT_BASIC_ACTION, Boolean.TRUE);
			permissionHashMap.put(EDIT_PROPERTIES_ACTION, Boolean.TRUE);
			permissionHashMap.put(EDIT_RELATIONS_ACTION, Boolean.TRUE);

			permissionHashMap.put(GENERATE_REPORT_ACTION, Boolean.TRUE);
			permissionHashMap.put(MERGE_ACTION, Boolean.TRUE);
			permissionHashMap.put(COPY_ACTION, Boolean.TRUE);
			permissionHashMap.put(PRE_MERGE_ACTION, Boolean.TRUE);
			permissionHashMap.put(PRE_RETIRE_ACTION, Boolean.TRUE);
			permissionHashMap.put(RETIRE_ACTION, Boolean.TRUE);

			permissionHashMap.put(SPLIT_ACTION, Boolean.TRUE);

			permissionHashMap.put(LOAD_BATCH_ACTION, Boolean.TRUE);
			permissionHashMap.put(EDIT_BATCH_ACTION, Boolean.TRUE);
			permissionHashMap.put(PARTONOMY_TREE_ACTION, Boolean.TRUE);

			permissionHashMap.put(EDIT_ACTION, Boolean.TRUE);

			permissionHashMap.put(DELETE_ACTION, Boolean.FALSE);
			permissionHashMap.put(EDIT_RETIRED_CLASS_ACTION, Boolean.FALSE);
		}


		action_vec.add(EDIT_ACTION);
		action_vec.add(SPLIT_ACTION);
		action_vec.add(PRE_MERGE_ACTION);
		action_vec.add(MERGE_ACTION);
		action_vec.add(PRE_RETIRE_ACTION);
		action_vec.add(RETIRE_ACTION);
		action_vec.add(GENERATE_REPORT_ACTION);
		action_vec.add(LOAD_BATCH_ACTION);
		action_vec.add(EDIT_BATCH_ACTION);
		action_vec.add(PARTONOMY_TREE_ACTION);
		action_vec.add(COPY_ACTION);
		action_vec.add(DELETE_ACTION);

	}


    public boolean isActionAllowed(String action)
    {
		if (permissionHashMap == null) return false;
		if (!permissionHashMap.containsKey((Object) action))
		{
			return false;
		}
		Boolean allowed = (Boolean) permissionHashMap.get(action);
		return allowed.booleanValue();
	}


    public OWLWrapper getWrapper()
    {
		return wrapper;
	}

    public NCIEditFilter getFilter()
    {
		return editfilter;
	}

	public EventListener getEventListener()
	{
		return eventListener;
	}

	public PartonomyPanel getPartonomyPanel()
	{
		return partonomyPanel;
	}

    public static boolean isSuitable(Project p, Collection errors) {
        if (!(p.getKnowledgeBase() instanceof OWLModel)) {
            errors.add("This tab can only be used with OWL projects.");
            return false;
        }
        else {
            return true;
        }
    }

	public String getUserName()
	{
		/*
		if (isRemoteProject)
		{
			return getOWLModel().getProject().getLocalUser();
		}
		return getDefaultUser();
		*/
		return username;
	}
/*
	private void setUserName(String name)
	{
		getOWLModel().setUserName(name);
	}
*/
    public int incrementConceptsPremerge()
    {
		num_concepts_premerge++;
		return num_concepts_premerge;
	}

    public int incrementConceptsMerge()
    {
		num_concepts_merge++;
		return num_concepts_merge;
	}

    protected JComponent createClsDisplay() {
        _instanceDisplay = new InstanceDisplay(getProject());
        return _instanceDisplay;
    }

    public Slot getPropertySlot(String name)
    {
		if (!name2SlotName.containsKey(name)) return null;
		String key = (String) name2SlotName.get(name);
		if (!name2Slot.containsKey(key))
		{
			return null;
		}
		return (Slot) name2Slot.get(key);

	}


    public String getPropertyLabel(String name)
    {
		if (!name2rdfID.containsKey(name)) return null;
		String rdfID = (String) name2rdfID.get(name);
		if (!rdfID2Label.containsKey(rdfID)) return (String) rdfID;
		return (String) rdfID2Label.get((String) rdfID);
	}

	public boolean isReadOnlyProperty(String name)
	{
		if (readOnlyProperties.contains(name)) return true;
		return false;
	}


	private boolean loadConfigurationFile(String filename)
	{
		readOnlyProperties = new Vector();
		requiredProperties = new Vector();

		readOnlyPropertyCount = 0;
		requiredPropertyCount = 0;

		readOnlyProperties.add(MERGE_SOURCE);
		requiredProperties.add(MERGE_SOURCE);

		readOnlyProperties.add(MERGE_TARGET);
		requiredProperties.add(MERGE_TARGET);

		readOnlyProperties.add(MERGE_INTO);
		requiredProperties.add(MERGE_INTO);

		readOnlyProperties.add(SPLIT_FROM);
		requiredProperties.add(SPLIT_FROM);

		readOnlyProperties.add(OLD_PARENT);
		requiredProperties.add(OLD_PARENT);

		readOnlyProperties.add(OLD_CHILD);
		requiredProperties.add(OLD_CHILD);

		readOnlyProperties.add(OLD_SOURCE_ROLE);
		requiredProperties.add(OLD_SOURCE_ROLE);

		readOnlyProperties.add(OLD_ROLE);
		requiredProperties.add(OLD_ROLE);

		readOnlyProperties.add(OLD_STATE);
		requiredProperties.add(OLD_STATE);
		readOnlyProperties.add(CODE);
		requiredProperties.add(CODE);

		readOnlyProperties.add(DEF_CURATOR);
		requiredProperties.add(DEF_CURATOR);

		requiredProperties.add(EDITOR_NOTE);
		requiredProperties.add(DESIGNNOTE);
		requiredProperties.add(CONCEPT_STATUS);
		requiredProperties.add(FULL_SYN);
		requiredProperties.add(DEFINITION);
		requiredProperties.add(PREFERRED_NAME);
		requiredProperties.add(ALT_DEFINITION);
		requiredProperties.add(GO_ANNOTATION);
		requiredProperties.add(LONG_DEFINITION);
		requiredProperties.add(ALT_LONG_DEFINITION);

		name2rdfID = new HashMap();
		rdfID2Label = new HashMap();
		rdfID2Slot = new HashMap();
		name2Slot = new HashMap();
		name2SlotName = new HashMap();
		externalNameMap  = new HashMap();
		name2ClsName = new HashMap();
		name2Cls = new HashMap();
		name2rdfIDCls = new HashMap();

	    String s;
		// default required property values
		for (int i=0; i<requiredProperties.size(); i++)
		{
			s = (String) requiredProperties.elementAt(i);
			name2rdfID.put(s, s);
			Slot slot = wrapper.getSlot(s);
			if (slot != null)
			{
				rdfID2Slot.put(s, slot);
				name2SlotName.put(s, slot.getName());
				externalNameMap.put(s, slot.getName());
				name2Slot.put(s, slot);
			}
		}

		noneditable_vec = new Vector();
		noneditable_vec.add(PREMERGED_CONCEPTS);
		noneditable_vec.add(RETIRED_CONCEPTS);
		noneditable_vec.add(PRERETIRED_CONCEPTS);



		// default place holder classes
		for (int i=0; i<noneditable_vec.size(); i++)
		{
			s = (String) noneditable_vec.elementAt(i);
			name2ClsName.put(s, s);
			Cls cls = wrapper.getCls(s);
			if (cls != null)
			{
				name2Cls.put(s, cls);
				name2ClsName.put(s, wrapper.getInternalName(cls));
                externalNameMap.put(s, wrapper.getInternalName(cls));
			}
		}

		if (filename != null)
		{
			try
			{
				BufferedReader inFile = new BufferedReader(new FileReader(filename));

				System.out.println("\nLoading configuration file: " + filename);

				while ((s = inFile.readLine()) != null)
				{
					if (s.length() > 0 && s.substring(0, 1).compareTo("#") != 0)
					{
						StringTokenizer st = new StringTokenizer(s, "\t");
						String name = st.nextToken();
						String value = st.nextToken();

						if (noneditable_vec.contains(name))
						{
							// class
							Cls cls = wrapper.getCls(value);
							if (cls != null)
							{
								name2Cls.put(name, cls);
    							name2ClsName.put(name, wrapper.getInternalName(cls));
   							    externalNameMap.put(value, name);
							}
							else
							{
								System.out.println("\nConcept " + value + " NOT found.");
							}
						}
						else
						{
							// properties
							name2rdfID.put(name, value);
							Slot slot = wrapper.getSlot(value);
							if (slot != null)
							{
								rdfID2Slot.put(value, slot);

								name2Slot.remove(name);

								name2Slot.put(value, slot);
                                name2SlotName.put(name, slot.getName());
                                externalNameMap.put(value, name);

								Object obj = slot.getOwnSlotValue(wrapper.getSlot("rdfs:label"));
								if (obj != null)
								{
									String label = obj.toString();
									rdfID2Label.put(value, label);
								}
								else
								{
									rdfID2Label.put(value, value);
								}
							}
							else
							{
								System.out.println("\tUnable to find slot: " + value);
							}
					    }
				    }
				}
				inFile.close();
			}
			catch(Exception e)
			{
				System.err.println(e);
				//return false;
			}
		}

		for (int i=0; i<requiredProperties.size(); i++)
		{
			s = (String) requiredProperties.elementAt(i);
			String t = getSlotName(s);
			if (t.compareTo(s) != 0)
			{
				System.out.println(s
				   + " property: " + getSlotName(s));
				Object obj = requiredProperties.set(i, t);
				for (int k=0; k<readOnlyProperties.size(); k++)
				{
					String str = (String) readOnlyProperties.elementAt(k);
					if (str.compareTo(s) == 0)
					{
						obj = readOnlyProperties.set(k, t);
						break;
					}
				}
			}
		}

		for (int i=0; i<noneditable_vec.size(); i++)
		{
			s = (String) noneditable_vec.elementAt(i);
			String t = getClsName(s);
			if (t.compareTo(s) != 0)
			{
				System.out.println(s
				   + ": " + getClsName(s));
				Object obj = noneditable_vec.set(i, t);
			}
        }

		wrapper.init(noneditable_vec, readOnlyProperties, requiredProperties,
		   name2rdfID, name2SlotName, rdfID2Slot, name2Cls, name2ClsName);

		return true;
	}


	public String getDefaultUser()
	{
		try {
			username = System.getProperty("user.name");
			//owlModel.setUserName(username);

		} catch (Exception e) {
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
			return null;
		}
		return username;
	}


	void loadPropertyFile(String filename, boolean readUser)
	{
		Properties properties = new Properties();
		try {
			properties.load(new FileInputStream(filename));
			remoteServerIP = properties.getProperty("remoteServerIP");
			////System.out.println("Remote Server IP: " + remoteServerIP);
			remoteServerPort = properties.getProperty("remoteServerPort");
			System.out.println("Remote Server Port: " + remoteServerPort);

			if (readUser)
			{
				username = properties.getProperty("user");
				//owlModel.setUserName(username);
		    }
		} catch (IOException e) {
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
		}
	}

    private void createEditFilter()
    {
		//loadPropertyFile("nci.properties");

	    editfilter = new NCIEditFilter(remoteServerIP, remoteServerPort, this);
    	canDelete = false;
	}


    protected NCIClsesPanel createClsesPanel() {
		createEditFilter();
        NCIClsesPanel panel = new NCIClsesPanel(getProject(), canDelete);
        resourcePanel = createResourcePanel();

        panel.addSelectionListener(new SelectionListener() {
            public void selectionChanged(SelectionEvent event) {
                transmitSelection();
            }
        });
//111806
        panel.setNCIEditTab(this);
        return panel;
    }

    protected ResourcePanel createResourcePanel() {
        return ProtegeUI.getResourcePanelFactory().createResourcePanel(getOWLModel(), ResourcePanel.DEFAULT_TYPE_CLASS);
    }

    protected JComponent createClsesSplitter() {
        _clsesPanel = createClsesPanel();
        return _clsesPanel;
    }

    protected ClsInverseRelationshipPanel createInverseRelationshipPanel() {
        final ClsInverseRelationshipPanel panel = new ClsInverseRelationshipPanel(getProject());
        initInverseRelationshipPanelListener(panel);
        return panel;
    }

    protected void initInverseRelationshipPanelListener(final ClsInverseRelationshipPanel panel) {
        panel.addSelectionListener(new SelectionListener() {
            public void selectionChanged(SelectionEvent event) {
                Collection selection = panel.getSelection();
                if (selection.size() == 1) {
                    Cls cls = (Cls) CollectionUtilities.getFirstItem(selection);
                    _clsesPanel.setDisplayParent(cls);
                }
            }
        });
    }





    public void enableButton(String buttonLabel, boolean state)
    {
		if (buttonLabel.compareTo("createButton_Edit") == 0)
		    createButton_Edit.setEnabled(state);


		else if (buttonLabel.compareTo("cloneButton_Clone") == 0)
		    cloneButton_Clone.setEnabled(state);
		else if (buttonLabel.compareTo("saveButton_Clone") == 0)
		    saveButton_Clone.setEnabled(state);
		else if (buttonLabel.compareTo("clearButton_Clone") == 0)
		    clearButton_Clone.setEnabled(state);

		else if (buttonLabel.compareTo("clearButton_Edit") == 0)
		    clearButton_Edit.setEnabled(state);
		else if (buttonLabel.compareTo("clearButton_Edit") == 0)
		    clearButton_Edit.setEnabled(state);

		else if (buttonLabel.compareTo("splitButton_Split") == 0)
		    splitButton_Split.setEnabled(state);

		else if (buttonLabel.compareTo("saveButton_Split") == 0)
		    saveButton_Split.setEnabled(state);
		else if (buttonLabel.compareTo("saveButton_Premerge") == 0)
		    saveButton_Premerge.setEnabled(state);
		else if (buttonLabel.compareTo("saveButton_Merge") == 0)
		    saveButton_Merge.setEnabled(state);
		else if (buttonLabel.compareTo("saveButton_Preretire") == 0)
		    saveButton_Preretire.setEnabled(state);
		else if (buttonLabel.compareTo("saveButton_Retire") == 0)
		    saveButton_Retire.setEnabled(state);

		else if (buttonLabel.compareTo("clearButton_Split") == 0)
		    clearButton_Split.setEnabled(state);
		else if (buttonLabel.compareTo("clearButton_Premerge") == 0)
		    clearButton_Premerge.setEnabled(state);
		else if (buttonLabel.compareTo("clearButton_Merge") == 0)
		    clearButton_Merge.setEnabled(state);
		else if (buttonLabel.compareTo("clearButton_Preretire") == 0)
		    clearButton_Preretire.setEnabled(state);
		else if (buttonLabel.compareTo("clearButton_Retire") == 0)
		    clearButton_Retire.setEnabled(state);
		else if (buttonLabel.compareTo("clearButton_Reportwriter") == 0)
		    clearButton_Reportwriter.setEnabled(state);

		else if (buttonLabel.compareTo("subClsButton_Preretire") == 0)
		    subClsButton_Preretire.setEnabled(state);
		else if (buttonLabel.compareTo("refClsButton_Preretire") == 0)
		    refClsButton_Preretire.setEnabled(state);
		else if (buttonLabel.compareTo("unretireButton_Preretire") == 0)
		    unretireButton_Preretire.setEnabled(state);
		else if (buttonLabel.compareTo("preretireButton_Preretire") == 0)
		    preretireButton_Preretire.setEnabled(state);


		else if (buttonLabel.compareTo("premergeButton_Premerge") == 0)
		    premergeButton_Premerge.setEnabled(state);
		else if (buttonLabel.compareTo("unmergeButton_Premerge") == 0)
		    unmergeButton_Premerge.setEnabled(state);

		else if (buttonLabel.compareTo("mergeButton_Merge") == 0)
		    mergeButton_Merge.setEnabled(state);
		else if (buttonLabel.compareTo("unmergeButton_Merge") == 0)
		    unmergeButton_Merge.setEnabled(state);

		else if (buttonLabel.compareTo("retireButton_Retire") == 0)
		    retireButton_Retire.setEnabled(state);
		else if (buttonLabel.compareTo("unretireButton_Retire") == 0)
		    unretireButton_Retire.setEnabled(state);

		else if (buttonLabel.compareTo("treeButton_Partonomy") == 0)
		    treeButton_Partonomy.setEnabled(state);
		else if (buttonLabel.compareTo("saveButton_Partonomy") == 0)
		    saveButton_Partonomy.setEnabled(state);
		else if (buttonLabel.compareTo("clearButton_Partonomy") == 0)
		    clearButton_Partonomy.setEnabled(state);

		else if (buttonLabel.compareTo("loadButton_Loader") == 0)
		    loadButton_Loader.setEnabled(state);
		else if (buttonLabel.compareTo("saveButton_Loader") == 0)
		    saveButton_Loader.setEnabled(state);
		else if (buttonLabel.compareTo("clearButton_Loader") == 0)
		    clearButton_Loader.setEnabled(state);

		else if (buttonLabel.compareTo("loadButton_Editor") == 0)
		    loadButton_Editor.setEnabled(state);
		else if (buttonLabel.compareTo("saveButton_Editor") == 0)
		    saveButton_Editor.setEnabled(state);
		else if (buttonLabel.compareTo("clearButton_Editor") == 0)
		    clearButton_Editor.setEnabled(state);

		else if (buttonLabel.compareTo("deleteButton_Delete") == 0)
		    deleteButton_Delete.setEnabled(state);
		else if (buttonLabel.compareTo("clearButton_Delete") == 0)
		    clearButton_Delete.setEnabled(state);


		return;
	}

	private void resetEditPanel(){

		panel1_upperPane.setViewportView(new TreePanel(this, null,  getOWLModel()));
		clearListenedToClses();
	}



	private Component getSplitPanel(){
		JPanel panel = new JPanel(false);
		panel.setLayout(new BorderLayout());

		panel2_upperPane=new JScrollPane();

		upperTreePanel_Split = createTreePanel();//new TreePanel(this, null,  getOWLModel());

		upperTreePanel_Split.setType(TreePanel.TYPE_SPLIT);

		lowerTreePanel_Split = createTreePanel();//new TreePanel(this, null,  getOWLModel());

		lowerTreePanel_Split.setCanDrop(false);

		panel2_upperPane.setViewportView(upperTreePanel_Split);
        //panel2_upperPane=new JScrollPane(upperTreePanel_Split, JScrollPane.VERTICAL_SCROLLBAR_NEVER, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);

		panel2_lowerPane=new JScrollPane();
        panel2_lowerPane.setViewportView(lowerTreePanel_Split);


		LabeledComponent lc_upper
		   = new LabeledComponent("Existing Concept", panel2_upperPane);
		LabeledComponent lc_lower
		   = new LabeledComponent("New Concept", panel2_lowerPane);

		splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,lc_upper,lc_lower);

		splitPane.setOneTouchExpandable(true);

        splitPane.setDividerLocation(280);
		panel.add(splitPane,BorderLayout.CENTER);

		JPanel buttonPanel = new JPanel();
		splitButton_Split = new JButton("Split");
		splitButton_Split.addActionListener(this);

		saveButton_Split = new JButton("Save");
		saveButton_Split.addActionListener(this);

		clearButton_Split = new JButton("Cancel");
		clearButton_Split.addActionListener(this);

		buttonPanel.add(splitButton_Split);
		buttonPanel.add(saveButton_Split);
		buttonPanel.add(clearButton_Split);
		panel.add(buttonPanel, BorderLayout.SOUTH);

		splitButton_Split.setEnabled(false);
		saveButton_Split.setEnabled(false);
		clearButton_Split.setEnabled(false);


		return panel;
	}

	//011207
	public JScrollPane getScrollPane(int type, int subtype){
		if (type == TreePanel.TYPE_SPLIT)
		{
			return panel2_upperPane;
		}
		else if (type == TreePanel.TYPE_CLONE)
		{
			int num = getNumClses();
			if (num == 1)
				return panel11_upperPane;
			else if (num == 2)
			{
				if (subtype == TreePanel.TYPE_CLONE_LOWER)
					return panel11_lowerPane;
				else
					return panel11_upperPane;
			}
		}
		else if (type == TreePanel.TYPE_PREMERGE)
		{
			if (subtype == TreePanel.TYPE_PREMERGE_LOWER)
				return panel3_lowerPane;
			else
				return panel3_upperPane;
		}
		else if (type == TreePanel.TYPE_MERGE)
		{
			if (subtype == TreePanel.TYPE_MERGE_LOWER)
				return panel4_lowerPane;
			else
				return panel4_upperPane;
		}
		else if (type == TreePanel.TYPE_PRERETIRE)
		{
			return panel5_upperPane;
		}
		else if (type == TreePanel.TYPE_RETIRE)
		{
			return panel6_upperPane;
		}
		//else if (type == TYPE_DELETE)

		return null;
	}


	private void resetSplitPanel(){

		upperTreePanel_Split = createTreePanel();//new TreePanel(this, null,  getOWLModel());
		upperTreePanel_Split.setType(TreePanel.TYPE_SPLIT);
		lowerTreePanel_Split = createTreePanel();//new TreePanel(this, null,  getOWLModel());
        lowerTreePanel_Split.setCanDrop(false);

		panel2_upperPane.setViewportView(upperTreePanel_Split);
		panel2_lowerPane.setViewportView(lowerTreePanel_Split);

		splitButton_Split.setEnabled(false);
		saveButton_Split.setEnabled(false);
		clearButton_Split.setEnabled(false);

		clearListenedToClses();

	}


    private Vector cloneStateVector(Vector v)
    {
		Vector u = new Vector();
		for (int i=0; i<v.size(); i++)
		{
			TreeItem treeitem = (TreeItem) v.elementAt(i);
			TreeItem new_treeitem = treeitem.cloneTreeItem();
			u.add(new_treeitem);
		}
		return u;
	}

	private boolean populateSplitPanel(String name, String pt){

		String code = generator.getCode();
		if (owl_format == NCIEditTab.BY_CODE)
		{
			if (code == null || code.compareTo("0") == 0)
			return false;
		}

        // By name: use name as identifier
        // By code: use code as identifier
		TreeItem item = new TreeItem();
		item.setType(TreeItem.TYPE_CONCEPT);
		item.setName(code);
		item.setNameValue(code);
		if (wrapper.codeSlotExists)
		{
			item.setName(name);
			item.setNameValue(name);
		}

		// get final state from upperTreePanel_Split
		Vector v = upperTreePanel_Split.getFinalState();

		v = convertNCIPT2NCISY(v);

		//v = cloneStateVector(v);

		Vector w = new Vector();

        int num_label = 0;
        HashSet hset = new HashSet();

		for (int i=0; i<v.size(); i++)
		{
			TreeItem treeitem = (TreeItem) v.elementAt(i);
			if (treeitem.getName().compareTo(getSlotName(CODE)) == 0)
			{
				TreeItem clone = treeitem.cloneTreeItem();
				clone.setValue(code);
				clone.setNameValue(treeitem.getName() + ":" + code);
				w.add(clone);
			}
			else if (treeitem.getName().compareTo(getSlotName(PREFERRED_NAME)) == 0)
			{
				String prop_name = getSlotName(FULL_SYN);
				if (prop_name != null)
				{
					String prop_value = treeitem.getValue();
					String newvalue = prop_value + "|SY|NCI";
					newvalue = ComplexPropertyParser.pipeDelim2XML(newvalue);
					String hash_value = prop_name + ":" + newvalue;

					if (!hset.contains(hash_value))
					{
						TreeItem clone = treeitem.cloneTreeItem();
						clone.setName(prop_name);
						clone.setValue(newvalue);
						clone.setNameValue(prop_name + ":" + newvalue);
						w.add(clone);
						hset.add(hash_value);
					}
			    }
			}
			else if (treeitem.getName().compareTo(getSlotName(FULL_SYN)) == 0)
			{
				String t = treeitem.getValue();
                String hash_value = treeitem.getName() + ":" + t;
                if (!hset.contains(hash_value))
                {
					TreeItem clone = treeitem.cloneTreeItem();
					clone.setValue(t);
					clone.setNameValue(treeitem.getName() + ":" + t);
					w.add(clone);
					hset.add(treeitem.getName() + ":" + t);
			    }
			}

			else if (treeitem.getName().compareTo("rdfs:label") != 0)
			{
				w.add(treeitem.cloneTreeItem());
			}
		}
		hset.clear();

		TreeNode root = new TreeNode(item, -1);

        lowerTreePanel_Split = new TreePanel(this, selectedInstance,  root, code, w);
        boolean retval = addToListenedToClses((OWLNamedClass) selectedInstance);

		//011107
		LabeledComponent lc_lower
		   = new LabeledComponent("New Concept", panel2_lowerPane);

		splitPane.setBottomComponent(lc_lower);
		splitPane.updateUI();

		lowerTreePanel_Split.addProperty("rdfs:label", pt, null);
		lowerTreePanel_Split.addProperty(getSlotName(PREFERRED_NAME), pt, null);


		String prop_name = getSlotName(FULL_SYN);
		String prop_value = pt + "|PT|NCI";
		prop_value = ComplexPropertyParser.pipeDelim2XML(prop_value);

		if (prop_name != null)
		{
			lowerTreePanel_Split.addProperty(prop_name, prop_value, null);
	    }

        // need to copy subclasses when the Save button is pressed
        // subclasses are not visible

		lowerTreePanel_Split.setCanDrop(false);
		upperTreePanel_Split.setCanDrop(false);
		panel2_lowerPane.setViewportView(lowerTreePanel_Split);

		splitButton_Split.setEnabled(false);
		saveButton_Split.setEnabled(true);
		clearButton_Split.setEnabled(true);

		return true;

	}


	private Component getPreMergePanel(){


		JPanel panel = new JPanel(false);
		panel.setLayout(new BorderLayout());

		panel3_lowerPane = new JScrollPane();
		panel3_upperPane = new JScrollPane();

		lowerTreePanel_Premerge = new TreePanel(this, null,  getOWLModel());
		lowerTreePanel_Premerge.setType(TreePanel.TYPE_PREMERGE);
		lowerTreePanel_Premerge.setSubType(TreePanel.TYPE_PREMERGE_LOWER);

		upperTreePanel_Premerge = new TreePanel(this, null,  getOWLModel());
		upperTreePanel_Premerge.setType(TreePanel.TYPE_PREMERGE);
		upperTreePanel_Premerge.setSubType(TreePanel.TYPE_PREMERGE_UPPER);

		panel3_lowerPane.setViewportView(lowerTreePanel_Premerge);
		panel3_upperPane.setViewportView(upperTreePanel_Premerge);


		LabeledComponent lc_upper
		   = new LabeledComponent("Surviving Concept", panel3_upperPane);
		LabeledComponent lc_lower
		   = new LabeledComponent("Retiring Concept", panel3_lowerPane);

		//JSplitPane mergePane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,panel3_lowerPane,panel3_upperPane);
		JSplitPane mergePane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,lc_upper,lc_lower);

        mergePane.setDividerLocation(280);
        mergePane.setOneTouchExpandable(true);

		panel.add(mergePane,BorderLayout.CENTER);

		JPanel buttonPanel = new JPanel();

		unmergeButton_Premerge = new JButton("UnMerge");
		unmergeButton_Premerge.addActionListener(this);

		premergeButton_Premerge = new JButton("PreMerge");
		premergeButton_Premerge.addActionListener(this);
		unmergeButton_Premerge.setEnabled(true);

		saveButton_Premerge = new JButton("Save");
		saveButton_Premerge.addActionListener(this);
		saveButton_Premerge.setEnabled(false);

		clearButton_Premerge = new JButton("Cancel");
		clearButton_Premerge.addActionListener(this);

        buttonPanel.add(unmergeButton_Premerge);
		buttonPanel.add(premergeButton_Premerge);
		buttonPanel.add(saveButton_Premerge);
		buttonPanel.add(clearButton_Premerge);
		panel.add(buttonPanel, BorderLayout.SOUTH);

		unmergeButton_Premerge.setEnabled(false);
		premergeButton_Premerge.setEnabled(false);
		saveButton_Premerge.setEnabled(false);
		clearButton_Premerge.setEnabled(false);

		num_concepts_premerge = 0;

		return panel;
	}

    private void resetPremergePanel(){


		lowerTreePanel_Premerge = new TreePanel(this, null,  getOWLModel());
		lowerTreePanel_Premerge.setType(TreePanel.TYPE_PREMERGE);
		lowerTreePanel_Premerge.setSubType(TreePanel.TYPE_PREMERGE_LOWER);
		upperTreePanel_Premerge = new TreePanel(this, null,  getOWLModel());
		upperTreePanel_Premerge.setType(TreePanel.TYPE_PREMERGE);
		upperTreePanel_Premerge.setSubType(TreePanel.TYPE_PREMERGE_UPPER);

		panel3_lowerPane.setViewportView(lowerTreePanel_Premerge);
		panel3_upperPane.setViewportView(upperTreePanel_Premerge);

		unmergeButton_Premerge.setEnabled(false);
		premergeButton_Premerge.setEnabled(false);
		saveButton_Premerge.setEnabled(false);
		clearButton_Premerge.setEnabled(false);

		num_concepts_premerge = 0;
		clearListenedToClses();

	}


	private Component getMergePanel(){

		JPanel panel = new JPanel(false);
		panel.setLayout(new BorderLayout());

		panel4_lowerPane = new JScrollPane();
		panel4_upperPane = new JScrollPane();

		upperTreePanel_Merge = new TreePanel(this, null,  getOWLModel());
		upperTreePanel_Merge.setCanDrop(true);
        upperTreePanel_Merge.setType(TreePanel.TYPE_MERGE);
        upperTreePanel_Merge.setSubType(TreePanel.TYPE_MERGE_UPPER);
		panel4_upperPane.setViewportView(upperTreePanel_Merge);

		lowerTreePanel_Merge = new TreePanel(this, null,  getOWLModel());
		lowerTreePanel_Merge.setType(TreePanel.TYPE_MERGE);
		lowerTreePanel_Merge.setCanDrop(true);
		lowerTreePanel_Merge.setSubType(TreePanel.TYPE_MERGE_LOWER);
		panel4_lowerPane.setViewportView(lowerTreePanel_Merge);

		LabeledComponent lc_upper
		   = new LabeledComponent("Surviving Concept", panel4_upperPane);
		LabeledComponent lc_lower
		   = new LabeledComponent("Retiring Concept", panel4_lowerPane);

		JSplitPane mergePane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,lc_upper,lc_lower);
        mergePane.setDividerLocation(280);
        mergePane.setOneTouchExpandable(true);

		panel.add(mergePane,BorderLayout.CENTER);

		JPanel buttonPanel = new JPanel();

		unmergeButton_Merge = new JButton("UnMerge");
		unmergeButton_Merge.addActionListener(this);
		unmergeButtonPressed = false;

		mergeButton_Merge = new JButton("Merge");
		mergeButton_Merge.addActionListener(this);

		saveButton_Merge = new JButton("Save");
		saveButton_Merge.addActionListener(this);

		clearButton_Merge = new JButton("Cancel");
		clearButton_Merge.addActionListener(this);

        buttonPanel.add(unmergeButton_Merge);
		buttonPanel.add(mergeButton_Merge);
		buttonPanel.add(saveButton_Merge);
		buttonPanel.add(clearButton_Merge);
		panel.add(buttonPanel, BorderLayout.SOUTH);

		unmergeButton_Merge.setEnabled(false);
		mergeButton_Merge.setEnabled(false);
		saveButton_Merge.setEnabled(false);
		clearButton_Merge.setEnabled(false);


		return panel;
	}

	public String getReferencedClsName(Cls cls)
	{
		return wrapper.getReferencedClsName(cls);
	}

	public Cls getReferencedCls(Cls cls)
	{
		return wrapper.getReferencedCls(cls);
	}


	public void populateMergePanel(String name){
		TreeItem item = new TreeItem();
		item.setType(TreeItem.TYPE_CONCEPT);
		item.setName(name);
		item.setNameValue(name);

		TreeNode dmtn = new TreeNode(item, -1);
		Instance selectedInstance = (Instance) owlModel.getCls(name);

		if (wrapper.hasAnnotationProperty((OWLNamedClass) selectedInstance, getSlotName(MERGE_TARGET)))
		{
			lowerTreePanel_Merge = new TreePanel(this, selectedInstance,  dmtn, null);
			lowerTreePanel_Merge.setCanDrop(true);
			panel4_lowerPane.setViewportView(lowerTreePanel_Merge);
	    }
	    else if (wrapper.hasAnnotationProperty((OWLNamedClass) selectedInstance, getSlotName(MERGE_SOURCE)))
	    {
			upperTreePanel_Merge = new TreePanel(this, selectedInstance,  dmtn, null);
			upperTreePanel_Merge.setCanDrop(true);
			panel4_upperPane.setViewportView(upperTreePanel_Merge);
		}

		saveButton_Merge.setEnabled(false);
		unmergeButton_Merge.setEnabled(true);
		mergeButton_Merge.setEnabled(true);
		clearButton_Merge.setEnabled(true);

		unmergeButtonPressed = false;

		lowerTreePanel_Merge.setCanDrop(false);
		upperTreePanel_Merge.setCanDrop(false);

		boolean retval = addToListenedToClses((OWLNamedClass) selectedInstance);
	}


    private void resetMergePanel(){
		upperTreePanel_Merge = new TreePanel(this, null,  getOWLModel());
        upperTreePanel_Merge.setCanDrop(true);
        upperTreePanel_Merge.setType(TreePanel.TYPE_MERGE);
        upperTreePanel_Merge.setSubType(TreePanel.TYPE_MERGE_UPPER);

		lowerTreePanel_Merge = new TreePanel(this, null,  getOWLModel());
		lowerTreePanel_Merge.setType(TreePanel.TYPE_MERGE);
		lowerTreePanel_Merge.setSubType(TreePanel.TYPE_MERGE_LOWER);
		lowerTreePanel_Merge.setCanDrop(true);

		panel4_lowerPane.setViewportView(lowerTreePanel_Merge);
		panel4_upperPane.setViewportView(upperTreePanel_Merge);

        saveButton_Merge.setEnabled(false);
        mergeButton_Merge.setEnabled(false);
		unmergeButton_Merge.setEnabled(false);
		clearButton_Merge.setEnabled(false);

		unmergeButtonPressed = false;
		clearListenedToClses();

	}



	private Component getPreRetirePanel(){


        JPanel panel = new JPanel(false);
        panel.setLayout(new BorderLayout());

        JPanel leftPanel = new JPanel();
        leftPanel.setLayout(new BorderLayout());

        JPanel subRefTabbedPane = null;
        subRefTabbedPane = createSubRefTabbedPane(); // upperleft

        leftPanel.add(subRefTabbedPane, BorderLayout.NORTH);

		JPanel treePanel = new JPanel();
		JLabel clsLabel = new JLabel("Retiring Class");

		panel5_upperPane = new JScrollPane();
		leftTreePanel_Preretire = new TreePanel(this, null,  getOWLModel());
		leftTreePanel_Preretire.setType(TreePanel.TYPE_PRERETIRE);
		panel5_upperPane.setViewportView(leftTreePanel_Preretire);

		treePanel.setLayout(new BorderLayout());
		treePanel.add(clsLabel, BorderLayout.NORTH);
		treePanel.add(panel5_upperPane, BorderLayout.CENTER);

		leftPanel.add(treePanel, BorderLayout.CENTER); // lowerleft treePanel


/////////////////////////////////////////////////////////////////////
        //subrefClsPanel = createResourcePanel(); // right panel
        OWLNamedClass nullCls = null;
        subrefClsPanel = createEditPanel(nullCls);
		JPanel rightPanel = new JPanel(false);
		rightPanel.setLayout(new BorderLayout());

		SubRefClsPanelContainier = new JScrollPane();
		SubRefClsPanelContainier.setViewportView((Component) subrefClsPanel);

		rightPanel.add(SubRefClsPanelContainier, BorderLayout.CENTER);
		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, leftPanel, rightPanel);
		splitPane.setDividerLocation(220);

        splitPane.setOneTouchExpandable(true);
/////////////////////////////////////////////////////////////////////

		splitPane.resetToPreferredSizes();
		panel.add(splitPane, BorderLayout.CENTER);

		JPanel buttonPanel = new JPanel();
		unretireButton_Preretire = new JButton("Unretire");
		unretireButton_Preretire.addActionListener(this);

		preretireButton_Preretire = new JButton("Preretire");
		preretireButton_Preretire.addActionListener(this);
		preretireButton_Preretire.setEnabled(false);

		saveButton_Preretire = new JButton("Save");
		saveButton_Preretire.addActionListener(this);

		clearButton_Preretire = new JButton("Cancel");
		clearButton_Preretire.addActionListener(this);

		buttonPanel.add(unretireButton_Preretire);
		buttonPanel.add(preretireButton_Preretire);
		buttonPanel.add(saveButton_Preretire);
		buttonPanel.add(clearButton_Preretire);

		panel.add(buttonPanel, BorderLayout.SOUTH);

		preretireButton_Preretire.setEnabled(false);
		saveButton_Preretire.setEnabled(false);
		unretireButton_Preretire.setEnabled(false);
		clearButton_Preretire.setEnabled(false);


		return panel;

	}

    private void resetPreretirePanel()
    {
        if (subclassListModel != null)
        {
			subclassListModel.removeAllElements();
		}
        if (referenceListModel != null)
        {
			referenceListModel.removeAllElements();
		}

		subclassListModel = submouseListener.getDefaultListModel();
		subclassListModel.clear();

		referenceListModel = referencemouseListener.getDefaultListModel();
		referenceListModel.clear();

		//updateSubRefClsPanel(null);

		initializeSubRefClsPanel();


		leftTreePanel_Preretire = new TreePanel(this, null,  getOWLModel());
		leftTreePanel_Preretire.setType(TreePanel.TYPE_PRERETIRE);

		panel5_upperPane.setViewportView(leftTreePanel_Preretire);

		preretireButton_Preretire.setEnabled(false);
		saveButton_Preretire.setEnabled(false);
		unretireButton_Preretire.setEnabled(false);
		clearButton_Preretire.setEnabled(false);

		preretireButtonPressed = false;

		droppedCls = null;
		clearListenedToClses();
	}



	private Component getRetirePanel(){


		JPanel panel = new JPanel(false);
		panel.setLayout(new BorderLayout());

		panel6_upperPane=new JScrollPane();
		upperTreePanel_Retire = new TreePanel(this, null,  getOWLModel());
		upperTreePanel_Retire.setType(TreePanel.TYPE_RETIRE);
		panel6_upperPane.setViewportView(upperTreePanel_Retire);

		panel.add(panel6_upperPane,BorderLayout.CENTER);

		JPanel buttonPanel = new JPanel();

		unretireButton_Retire = new JButton("Unretire");
		unretireButton_Retire.addActionListener(this);

		retireButton_Retire = new JButton("Retire");
		retireButton_Retire.addActionListener(this);

		saveButton_Retire = new JButton("Save");
		saveButton_Retire.addActionListener(this);

		clearButton_Retire = new JButton("Cancel");
		clearButton_Retire.addActionListener(this);

		buttonPanel.add(unretireButton_Retire);
		buttonPanel.add(retireButton_Retire);
		buttonPanel.add(saveButton_Retire);
		buttonPanel.add(clearButton_Retire);

        unretireButton_Retire.setEnabled(false);
        retireButton_Retire.setEnabled(false);
        saveButton_Retire.setEnabled(false);
        clearButton_Retire.setEnabled(false);

        unretireButtonPressed = false;

		panel.add(buttonPanel, BorderLayout.SOUTH);

		return panel;
	}


    private void resetRetirePanel(){

		upperTreePanel_Retire = new TreePanel(this, null,  getOWLModel());
		upperTreePanel_Retire.setType(TreePanel.TYPE_RETIRE);
		panel6_upperPane.setViewportView(upperTreePanel_Retire);

        retireButton_Retire.setEnabled(false);
        saveButton_Retire.setEnabled(false);
        clearButton_Retire.setEnabled(false);
        unretireButtonPressed = false;
        clearListenedToClses();
	}


	private JPanel createTabbedPane() {
		JPanel panel = new JPanel(false);
		ImageIcon icon = new ImageIcon("images/middle.gif");
		tabbedPane = new JTabbedPane();

        panel12 = new EditPanel(this, getSelectedInstance(), getOWLModel());
		tabbedPane.addTab("Edit", icon, panel12, "");
		panel12.disableAll();

		panel2 = getSplitPanel();
		tabbedPane.addTab("Split", icon, panel2, "");

		panel3 = getPreMergePanel();
		tabbedPane.addTab("PreMerge", icon, panel3, "");

		panel4 = getMergePanel();
		tabbedPane.addTab("Merge", icon, panel4, "");

        panel5 = getPreRetirePanel();

        		resetPreretirePanel();



		tabbedPane.addTab("PreRetire", icon, panel5, "");

        panel6 = getRetirePanel();
		tabbedPane.addTab("Retire", icon, panel6, "");

		panel7 = makeReportPanel();
		tabbedPane.addTab("Report Writer", icon, panel7, "");

		panel8 = makeBatchLoaderPanel();
		tabbedPane.addTab("Batch Loader", icon, panel8, "");

		panel9 = makeBatchEditorPanel();
		tabbedPane.addTab("Batch Editor", icon, panel9, "");

		panel10 = makePartonomyTreePanel();
		tabbedPane.addTab("Partonomy Tree", icon, panel10, "");

        panel1 = getClonePanel();
		tabbedPane.addTab("Copy", icon, panel1, "");
/*
        panel0 = getDeletePanel();
		tabbedPane.addTab("Delete", icon, panel0, "");
*/

		//Add the tabbed pane to this panel.
		panel.setLayout(new GridLayout(1, 1));
		panel.add(tabbedPane);

		tabbedPane.setSelectedIndex(0);

		tabbedPane.addChangeListener(new ChangeListener() {

			public void stateChanged(ChangeEvent evt) {

				if (selectedInstance == null)
				{
					tabbedPane.setSelectedIndex(prev_selection);
					return;
				}

				Cls cls = (Cls) selectedInstance;
				if (wrapper.getInternalName(cls).compareTo("owl:Thing") == 0 ||
				    wrapper.getInternalName(cls).compareTo("rdfs:Class") == 0 ||
				    wrapper.getInternalName(cls).compareTo("rdf:Property") == 0)
				{
					tabbedPane.setSelectedIndex(prev_selection);
					return;
				}

				JTabbedPane pane = (JTabbedPane) evt.getSource();
				int sel = pane.getSelectedIndex();

				if (sel == -1) return;

				curr_selection = sel;

				String action = (String) action_vec.elementAt(sel);

				// check permission
				if (!isActionAllowed(action))
				{
					tabbedPane.setSelectedIndex(prev_selection);
					System.out.println("Action: " + action + " NOT ALLOWED.");
					return;
				}

                if (curr_selection != prev_selection) // tab changed
                {
                    // if data has been modified
                    if (lastSel==0 && getEditPanel().dataModified()
					    || lastSel==1 && clearButton_Split.isEnabled()
						|| lastSel==2 && clearButton_Premerge.isEnabled()
						|| lastSel==3 && clearButton_Merge.isEnabled()
						|| lastSel==4 && clearButton_Preretire.isEnabled()
						|| lastSel==5 && clearButton_Retire.isEnabled()
						|| lastSel==7 && clearButton_Loader.isEnabled()
						|| lastSel==8 && clearButton_Editor.isEnabled()
						|| lastSel==9 && clearButton_Partonomy.isEnabled()
						|| lastSel==10 && clearButton_Clone.isEnabled())
					{
						int option = JOptionPane.showConfirmDialog//ok:0 cancel:2
							(null, "No data will be saved. Do you want to continue?", "Confirmation", JOptionPane.YES_NO_OPTION);

						if (option == 0) {  // Yes
							lastSel = sel;
						}
						else { // No
 							tabbedPane.setSelectedIndex(prev_selection);
							return;
						}

					}

					clearLastPanel(prev_selection);
					prev_selection = curr_selection;

					if (curr_selection == 0 && panel12 != null)
					{
						transmitSelection();
					}
                }
			}
		});

		return panel;
	}


	private void clearLastPanel(int index){
		droppedCls = null;
		if (index==0)
		{
			panel12 = new EditPanel(this, getSelectedInstance(), getOWLModel());
			tabbedPane.setComponentAt(index, panel12);//new EditPanel(this, getSelectedInstance(), getOWLModel()));
	    }
		else if (index==1)
		{
			// [#6178] Selecting Split Tab after clonning does not clear the "New Concept" panel
			resetSplitPanel();
			tabbedPane.setComponentAt(index, getSplitPanel());
		}
		else if (index==2)
			tabbedPane.setComponentAt(index, getPreMergePanel());
		else if (index==3)
			tabbedPane.setComponentAt(index, getMergePanel());
		else if (index==4)
		{
			tabbedPane.setComponentAt(index, getPreRetirePanel());
					resetPreretirePanel();

		}
		else if (index==5)
			tabbedPane.setComponentAt(index, getRetirePanel());
		else if (index==6)
			tabbedPane.setComponentAt(index, makeReportPanel());
		else if (index==7)
			tabbedPane.setComponentAt(index, makeBatchLoaderPanel());
		else if (index==8)
			tabbedPane.setComponentAt(index, makeBatchEditorPanel());
		//else if (index==9)
		//	tabbedPane.setComponentAt(index, makePartonomyTreePanel(index));
		else if (index==10)
		{
			// [#6178] Selecting Split Tab after clonning does not clear the "New Concept" panel
			resetClonePanel();
			tabbedPane.setComponentAt(index, getClonePanel());
		}

        clearListenedToClses();

	}


    public JTextArea getEditorTextArea()
    {
		return editorTextArea;
	}

    public JTextArea getLoaderTextArea()
    {
		return loaderTextArea;
	}


	protected Component makeBatchLoaderPanel() {
		JPanel filePanel = new JPanel(new BorderLayout());
		fileName = new JTextField();
		fileName.setColumns(45);

		loadButton_Loader = new JButton("Input");
		loadButton_Loader.addActionListener(this);

		saveButton_Loader = new JButton("Save");
		saveButton_Loader.addActionListener(this);

		clearButton_Loader = new JButton("Clear");
		clearButton_Loader.addActionListener(this);

		JPanel textAreaPanel= new JPanel(new BorderLayout());
		loaderTextArea = new JTextArea(25,45);
		loaderTextArea.setEditable(false);

		loaderTextArea.setTabSize(2);
		JPanel buttonPanel= new JPanel();

		buttonPanel.add(loadButton_Loader);
		buttonPanel.add(saveButton_Loader);
		buttonPanel.add(clearButton_Loader);

		textAreaPanel.add(new JScrollPane(loaderTextArea),BorderLayout.CENTER);
		textAreaPanel.add(buttonPanel,BorderLayout.SOUTH);

       	Box box=new Box(BoxLayout.Y_AXIS);
		box.add(textAreaPanel);

		textAreaPanel.setBorder(BorderFactory.createTitledBorder("Log"));

		JPanel panel = new JPanel(new BorderLayout());
		panel.add(box);

        loadButton_Loader.setEnabled(true);
        saveButton_Loader.setEnabled(false);
        clearButton_Loader.setEnabled(false);

		return panel ;
	}

	protected Component makeBatchEditorPanel() {
		JPanel filePanel = new JPanel(new BorderLayout());
		fileName = new JTextField();
		fileName.setColumns(45);

		loadButton_Editor = new JButton("Input");
		loadButton_Editor.addActionListener(this);

		saveButton_Editor = new JButton("Save");
		saveButton_Editor.addActionListener(this);

		clearButton_Editor = new JButton("Clear");
		clearButton_Editor.addActionListener(this);

		JPanel textAreaPanel= new JPanel(new BorderLayout());
		editorTextArea = new JTextArea(25,45);
		reportTextArea.setTabSize(2);
		JPanel buttonPanel= new JPanel();

		buttonPanel.add(loadButton_Editor);
		buttonPanel.add(saveButton_Editor);
		buttonPanel.add(clearButton_Editor);

		textAreaPanel.add(new JScrollPane(editorTextArea),BorderLayout.CENTER);
		textAreaPanel.add(buttonPanel,BorderLayout.SOUTH);

       	Box box=new Box(BoxLayout.Y_AXIS);
		box.add(textAreaPanel);

		textAreaPanel.setBorder(BorderFactory.createTitledBorder("Log"));

		JPanel panel = new JPanel(new BorderLayout());
		panel.add(box);

        loadButton_Editor.setEnabled(true);
        saveButton_Editor.setEnabled(false);
        clearButton_Editor.setEnabled(false);


		return panel ;
	}

	protected Component makePartonomyTreePanel() {

		JPanel panel = new JPanel(false);
		panel.setLayout(new BorderLayout());

		partonomyTreeScrollPane = new JScrollPane();
		//041306
		//TreePanel treePanel = new TreePanel(this, null,  getOWLModel());
		partonomyPanel =
			new PartonomyPanel(this, null,  getOWLModel(), wrapper);
		partonomyTreeScrollPane.setViewportView(partonomyPanel);

		panel.add(partonomyTreeScrollPane,BorderLayout.CENTER);

		JPanel buttonPanel = new JPanel();

		treeButton_Partonomy = new JButton("Tree");
		treeButton_Partonomy.addActionListener(this);

		saveButton_Partonomy = new JButton("Save");
		saveButton_Partonomy.addActionListener(this);

		clearButton_Partonomy = new JButton("Clear");
		clearButton_Partonomy.addActionListener(this);

		buttonPanel.add(treeButton_Partonomy);
		buttonPanel.add(saveButton_Partonomy);
		buttonPanel.add(clearButton_Partonomy);

        treeButton_Partonomy.setEnabled(true);
        saveButton_Partonomy.setEnabled(false);
        clearButton_Partonomy.setEnabled(false);

		panel.add(buttonPanel, BorderLayout.SOUTH);

		return panel;
	}


	protected Component makeReportPanel() {

		JPanel filePanel = new JPanel(new BorderLayout());
		fileName = new JTextField();
		fileName.setColumns(45);

		superConceptLabel = new JLabel("SuperClass is:");
		superConceptLabel.setName("superConceptLabel");
		superConceptLabel.setSize(45,1);

		generateButton_Reportwriter = new JButton("Generate");
		generateButton_Reportwriter.addActionListener(this);

		clearButton_Reportwriter = new JButton("Clear");
		clearButton_Reportwriter.addActionListener(this);

		//saveButton_Reportwriter = new JButton("Save");
		//saveButton_Reportwriter.addActionListener(this);

		textAreaPanel = new JPanel(new BorderLayout());
		reportTextArea = new JTextArea(25,45);

		reportTextArea.setEditable(false);

		reportTextArea.setTabSize(2);
		JPanel buttonPanel= new JPanel();

		buttonPanel.add(generateButton_Reportwriter);
		//buttonPanel.add(saveButton_Reportwriter);

		//buttonPanel.add(clearButton_Reportwriter);

		//status_Reportwriter = new JLabel("");
		//status_ReportWriter.setEnabled(false);
		//buttonPanel.add(status_Reportwriter);

	    progress = new JProgressBar();
	    progress_msg = new JLabel("");
		progress.setPreferredSize( new Dimension( 250, 10 ) );
		progress.setMinimum( 0 );
		progress.setMaximum( 100 );
		progress.setValue( 0 );

		//showProgressBar(false);

        buttonPanel.add(progress);
        buttonPanel.add(progress_msg);

		generateButton_Reportwriter.setEnabled(true);
		//saveButton_Reportwriter.setEnabled(false);
		//clearButton_Reportwriter.setEnabled(false);

		textAreaPanel.add(new JScrollPane(reportTextArea),BorderLayout.CENTER);
		textAreaPanel.add(buttonPanel,BorderLayout.SOUTH);

       	Box box=new Box(BoxLayout.Y_AXIS);
		box.add(textAreaPanel);

		textAreaPanel.setBorder(BorderFactory.createTitledBorder("Report"));

		JPanel panel = new JPanel(new BorderLayout());
		panel.add(box);

		return panel ;

	}

/*
    public JLabel getStatus_ReportWriter()
    {
		return status_Reportwriter;
	}
*/

	private JPanel createExtensionPane(){
		tabbedPanel = createTabbedPane();
		prev_selection = 0;
		curr_selection = 0;

       	Box box=new Box(BoxLayout.Y_AXIS);
		box.add(tabbedPanel);

		JPanel editPanel = new JPanel(new BorderLayout());
		editPanel.add(box);

		return editPanel ;

	}

	//022806
    private void updateMainSplitter(Instance selectedInstance) {
		leftComponent = createExtensionPane();
		((JSplitPane) mainSplitter).setRightComponent(leftComponent);
    }

    private JComponent createMainSplitter() {
        //resourcePanel = createResourcePanel();

		JSplitPane pane = createLeftRightSplitPane("ClsesTab.left_right", 425);

        JComponent rightComponent = (JComponent)resourcePanel;
        pane.setLeftComponent(createClsesSplitter());

		leftComponent = createExtensionPane();
		pane.setRightComponent(leftComponent);

		Component[] comps = leftComponent.getComponents();

        return pane;
    }


    public void initialize() {
		CustomizedAnnotationData_Map = loadUIConfigurationData("UIConfiguration.dat");

        initializePermissionHashMap();

		File pluginsDirectory = PluginUtilities.getPluginsDirectory();
		System.out.println("pluginsDirectory: " + pluginsDirectory.getAbsolutePath());

        setIcon(Icons.getClsesIcon());
        setLabel("NCI Editor");
        setShortDescription("NCI Editor");

        // Instantiate an OWLWrapper

        loadConfigurationFile("nciedittab.cfg");

        // createMainSplitter (which in turn instantiate editfilter)
        mainSplitter = createMainSplitter();
        add(mainSplitter);
        setInitialSelection();

        // setClsTree
        setClsTree(_clsesPanel.getClsesTree());

        classList=new Vector();
		nodelist = new HashSet();

		//owlModel = getOWLModel();


	    System.out.println("User: " + username);

        // Instantiate NCICodeGenerator
        generator = new NCICodeGenerator();

		subclass_Pane = new JScrollPane(); // preretire
		reference_Pane = new JScrollPane(); // preretire

		subVec = new Vector();
		restrictionVec = new Vector();

	    // Instantiate EventListener
	    eventListener = new EventListener(this);

	    //dumpHashtable(editfilter.getHistoryHash());
/*
    		<USER>root</USER>
    		<PASSWORD>password</PASSWORD>
    		<IP>localhost</IP>
    		<PORT>3360</PORT>
    		<DB>protegeext</DB>
    		<TABLE>concept_history</TABLE>
    		<SEQUENCE>concept_history_id_seq</SEQUENCE>
    		<OWNER>evs_management</OWNER>
*/

        // History records are logged by the Prompt team through post-processing
        if (historyOn)
        {
			Hashtable historyHash = editfilter.getHistoryHash();
			String user = (String) historyHash.get("USER");
			String pwd = (String) historyHash.get("PASSWORD");
			String ip = (String) historyHash.get("IP");
			String port = (String) historyHash.get("PORT");
			String instance = (String) historyHash.get("DB");
			String table = (String) historyHash.get("TABLE");
			String sequence = (String) historyHash.get("SEQUENCE");
			String owner = (String) historyHash.get("OWNER");


			System.out.println("user:" + user);
			System.out.println("ip:" + ip);
			System.out.println("port:" + port);
			System.out.println("instance:" + instance);
			System.out.println("table:" + table);

			/*
			edithistory = new EditHistory(username,
				ip, port, user, pwd, instance,
				owner, table, sequence);

			edithistory.setOWLWrapper(wrapper);
			*/
	    }

        // Instantiate a DataHandler
	    datahandler = new DataHandler(this);

	    checkRequiredData();

	    //initializePermissionHashMap();

	    if (wrapper.codeSlotExists)
	    {
			owl_format = BY_NAME;
			System.out.println("OWL Format: BY_NAME");
		}
		else
		{
			owl_format = BY_CODE;
			System.out.println("OWL Format: BY_CODE");
		}

    }


    private boolean checkRequiredData()
    {
		String msg = "";
		for (int i=0; i<requiredProperties.size(); i++)
		{
			String propertyname = (String) requiredProperties.elementAt(i);
            try {
				Slot slot = getOWLModel().getSlot(propertyname);
				System.out.println("Checking property: " + propertyname + " (" +
				   externalNameMap.get(propertyname) + ") ...");
				if (slot == null)
				{
System.out.println("\tWARNING: Property: " + propertyname + " is not found.");
					msg = msg + propertyname + "\n";
				}
		    } catch (Exception ex) {
				System.out.println("WARNING: Property: " + propertyname + " is not found.");
				msg = msg + propertyname + "\n";
			}
		}

		for (int i=0; i<noneditable_vec.size(); i++)
		{
			String cls_name = (String) noneditable_vec.elementAt(i);
			System.out.println("Checking class: " + cls_name + " (" +
		         externalNameMap.get(cls_name) + ") ...");

			if (getOWLModel().getCls(cls_name) == null)
			{
				System.out.println("\tWARNING: Class: " + cls_name + " is not found.");
				msg = msg + cls_name + "\n";
			}
		}

		if (msg.compareTo("") != 0 && msg.compareTo(CODE + "\n") != 0)
		{
			Object[] options = {"Yes", "No"};
			Object[] option_array = {"OK"};
			int ans = JOptionPane.showOptionDialog(this, "Missing data elements: \n" + msg,
				"WARNING",
			JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
			null, option_array, options[0]);

			return false;
		}
		return true;
	}


    public DataHandler getDataHandler()
    {
		return datahandler;
	}

/*
    public EditHistory getEditHistory()
    {
		return edithistory;
	}
*/

    private void dumpHashtable(Hashtable hashtable)
    {
		if (hashtable == null) return;
		Set keys = hashtable.keySet();
		Iterator it = keys.iterator();
		while (it.hasNext())
		{
			String key = (String) it.next();
			String value = (String) hashtable.get(key);
			System.out.println("key: " + key);
			System.out.println("value: " + value);
		}

	}


    public void setFinderComponent(JComponent c) {
        _clsesPanel.setFinderComponent(c);
    }


    private void setInitialSelection() {
        if (_clsesPanel != null) {
            transmitSelection();
        }
    }

    public OWLModel getOWLModel() {
        return (OWLModel) getProject().getKnowledgeBase();
    }


    public static OWLModel getActiveOWLModel()
    {
		return owlModel;
	}

    public String getCurrSelection() {
        Collection selection = _clsesPanel.getSelection();
        if (selection.size() == 1) {
			return wrapper.getInternalName((Cls) selectedInstance);
        }
        return null;
	}

    protected void transmitSelection() {
		//090606
		if (tabbedPane.getSelectedIndex()==0 && getEditPanel().isSaveButtonEnabled() && selectedInstance != null){

			// To do #1:
			// Need to check if the focused concept has been changed, if it has not, then do not pop-up the following message.

			Object[] options = {"Yes", "No"};
			int ans = JOptionPane.showOptionDialog(this, "Focused class has been modified. Continue?", "Confirmation",
			JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
			null, options, options[0]);

			if (ans == 1) return;
		}

        Collection selection = _clsesPanel.getSelection();
        //Instance selectedInstance = null;
        selectedInstance = null;
        if (selection.size() == 1) {
            selectedInstance = (Instance) CollectionUtilities.getFirstItem(selection);
            OWLModel owlModel = getOWLModel();
            if (//selectedInstance.equals(owlModel.getRootCls()) ||
                    selectedInstance.equals(owlModel.getOWLNothing()) ||
                            selectedInstance.equals(owlModel.getRDFSNamedClass(RDFSNames.Cls.LITERAL)) ||
                            selectedInstance.equals(((KnowledgeBase) owlModel).getCls(Model.Cls.DIRECTED_BINARY_RELATION)) ||
                            selectedInstance.equals(owlModel.getRDFListClass()) ||
                            selectedInstance.equals(owlModel.getRDFUntypedResourcesClass())) {
                selectedInstance = null;
            }
        }

        if(selectedInstance == null)
        {
			return;
		}

        if(!(selectedInstance instanceof OWLNamedClass))
        {
 			return;
		}

        if(selectedInstance == null || selectedInstance instanceof RDFResource) {
            resourcePanel.setResource((RDFResource) selectedInstance);
        }
        else if(resourcePanel instanceof ResourceDisplay) {
            ((ResourceDisplay)resourcePanel).setInstance(selectedInstance);

        }

        if (panel12 != null) // edit panel
        {
			panel12.updateAll();
			if (tabbedPane.getSelectedIndex() == 0)
			{
				if (eventListener != null && selectedInstance != null)
				{
                    clearListenedToClses();
                    OWLNamedClass selectedCls = (OWLNamedClass) selectedInstance;
                    if (!wrapper.isEditable(selectedCls))
                    {
						getEditPanel().disableAll();
						return;
					}
					else
					{
						getEditPanel().enableAddButtons();
					}
					addToListenedToClses((OWLNamedClass) selectedInstance);
				}
		    }
		}
    }

    public void initializeSubRefClsPanel()
    {
		//OWLNamedClass nullCls = null;
		subrefClsPanel = createEditPanel(null);
		subrefClsPanel.setFocusClass(null);
		subrefClsPanel.disableAll();
		SubRefClsPanelContainier.setViewportView((Component) subrefClsPanel);
	}


    public void updateSubRefClsPanel(Instance instance)
    {
		//subrefClsPanel.setResource((RDFResource) instance);
		subrefClsPanel.setFocusClass((OWLNamedClass) instance);
		subrefClsPanel.setAdvancedQuery("AdvancedQuery");

		if (instance != null)
		{
			subrefClsPanel.updateAll();
			subrefClsPanel.enableAddButtons();
	    }
	}


	void updateTextAreaPanelCaption(String title)
	{
		textAreaPanel.setBorder(BorderFactory.createTitledBorder(title));
	}


    private Cls getRootCls(TreePanel panel)
    {
		TreeNode dmtn = panel.getRootNode();
		if (dmtn != null)
		{
			Cls cls = getOWLModel().getCls(dmtn.toString());
			return cls;
		}
		return null;
    }


    public boolean premergeReady()
    {
		if (lowerTreePanel_Premerge == null)
		{
			return false;
		}
		if (upperTreePanel_Premerge == null) return false;

        if (lowerTreePanel_Premerge.getName() == null) return false;
 		if (lowerTreePanel_Premerge.getName().compareTo("") == 0) return false;

        if (upperTreePanel_Premerge.getName() == null) return false;
 		if (upperTreePanel_Premerge.getName().compareTo("") == 0) return false;

		return false;
    }



    public void showProgressMsg()
    {
		progress_msg.setText("Calculating tree size. Please wait ...");
		progress_msg.setVisible(true);
	}


    public void enableReportButton(boolean enabled)
    {
		generateButton_Reportwriter.setEnabled(enabled);
	}



	public void actionPerformed(ActionEvent e) {

        Object[] options = {"Yes", "No"};

		selection = _clsesPanel.getSelection();
		selectedInstance = (Instance) CollectionUtilities.getFirstItem(selection);
/*
		if (e.getSource() ==  saveButton_Reportwriter) {

			if (selectedInstance.equals(getKnowledgeBase().getRootCls())) {
				superConceptLabel.setText("Please select a root from the taxonomy tree.");
				String text = "Please select a root.";
				ModalDialog.showMessageDialog(null, text);
				return;
			}

			JFileChooser fc = new JFileChooser();
			int returnVal = fc.showOpenDialog(this);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				exportFile = fc.getSelectedFile();
				writeToFile(reportTextArea, exportFile);
				nodelist.clear();
				return;
			}
		}

		else
*/
		if (e.getSource() == panel12.getReviewButton()) {
			ReviewClassDialog rcd = new ReviewClassDialog(this, panel12, (OWLNamedClass) selectedInstance);
        }


		else if (e.getSource() == generateButton_Reportwriter) {

			if (selectedInstance.equals(getKnowledgeBase().getRootCls())) {
				superConceptLabel.setText("Please select a root from the taxonomy tree.");
				String text = "Please select a root.";
				ModalDialog.showMessageDialog(null, text);
				return;
			}

			//ReportWriter dlg = new ReportWriter(this, owlModel);

			ReportWriter_dlg = new ReportWriter(this, owlModel);

			if (ReportWriter_dlg.getOKBtnPressed())
			{
				File exportFile = ReportWriter_dlg.getOutputFile();
				try{
					int maxLevel = ReportWriter_dlg.getLevel();
					boolean withAttributes = ReportWriter_dlg.getWithAttributes();

					generateButton_Reportwriter.setCursor(new Cursor(Cursor.WAIT_CURSOR));

					String laf = UIManager.getCrossPlatformLookAndFeelClassName();
					try {
						UIManager.setLookAndFeel(laf);
					}
					catch (Exception ex) {
						Log.getLogger().log(Level.WARNING, "Exception caught", ex);
					}

					JFrame fr = new JFrame("Report Writer");

					WindowListener listener = new WindowAdapter() {
						public void windowClosing(WindowEvent e) {}
					};
					fr.addWindowListener(listener);

					Container contentPane = fr.getContentPane();
					final ProgressBar progress = new ProgressBar(this, exportFile, (Cls) selectedInstance, maxLevel, withAttributes);
					contentPane.setLayout(new GridLayout(1, 0));
					contentPane.add(progress);
					fr.pack();
					fr.show();

					SwingUtilities.invokeLater(new Runnable() {
						public void run() {
							progress.getStartButton().requestFocus();
						}
					});

					generateButton_Reportwriter.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					/*
					clearButton_Reportwriter.setEnabled(true);
					progress_msg.setText("");
					generateButton_Reportwriter.setEnabled(false);
					*/

				}
				catch(Exception ex){
					Log.getLogger().log(Level.WARNING, "Exception caught", ex);
					String text = "Unable to save to file " + exportFile.getPath() + ".";
					ModalDialog.showMessageDialog(null, text);
					generateButton_Reportwriter.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));

				}
		    }

		}

		else if (e.getSource() == createButton_Edit) {
			try {
				//reportTextArea.setText("");
			}
			catch(Exception ex){
				Log.getLogger().log(Level.WARNING, "Exception caught", ex);
			}
		}

		else if (e.getSource() == clearButton_Edit) {
			try {
				int ans = JOptionPane.showOptionDialog(this, "No data will be saved. Do you want to continue?", "Confirmation",
				JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
				null, options, options[0]);

				if (ans == 1)
				{
					return;
				}
				resetEditPanel();
            }
			catch(Exception ex){
				Log.getLogger().log(Level.WARNING, "Exception caught", ex);
			}
		}

		else if (e.getSource() == clearButton_Split) {
			try {
				int ans = JOptionPane.showOptionDialog(this, "No data will be saved. Do you want to continue?", "Confirmation",
				JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
				null, options, options[0]);

				if (ans == 1) return;
				clearButton_Split.setEnabled(false);

				resetSplitPanel();
				//resetAll();
            }
			catch(Exception ex){
				Log.getLogger().log(Level.WARNING, "Exception caught", ex);
			}
		}

		else if (e.getSource() == clearButton_Premerge) {
			try {
				int ans = JOptionPane.showOptionDialog(this, "No data will be saved. Do you want to continue?", "Confirmation",
				JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
				null, options, options[0]);



				if (ans == 1) return;
				clearButton_Premerge.setEnabled(false);

				resetPremergePanel();
				//resetAll();
            }
			catch(Exception ex){
				Log.getLogger().log(Level.WARNING, "Exception caught", ex);
			}
		}

		else if (e.getSource() == clearButton_Merge) {
			try {
				int ans = JOptionPane.showOptionDialog(this, "No data will be saved. Do you want to continue?", "Confirmation",
				JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
				null, options, options[0]);

				if (ans == 1) return;
				resetMergePanel();
            }
			catch(Exception ex){
				Log.getLogger().log(Level.WARNING, "Exception caught", ex);
			}
		}

		else if (e.getSource() == clearButton_Preretire) {
			try {
				int ans = JOptionPane.showOptionDialog(this, "No data will be saved. Do you want to continue?", "Confirmation",
				JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
				null, options, options[0]);
				if (ans == 1)
				{
                   if (preretireButtonPressed)
                   {
						// recover subs and inbound roles
				   }
				   return;
				}

				resetPreretirePanel();
				//resetAll();
            }
			catch(Exception ex){
				Log.getLogger().log(Level.WARNING, "Exception caught", ex);
			}
		}

		else if (e.getSource() == clearButton_Retire) {
			try {
				int ans = JOptionPane.showOptionDialog(this, "No data will be saved. Do you want to continue?", "Confirmation",
				JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
				null, options, options[0]);
				if (ans == 1) return;

				resetRetirePanel();
				//resetAll();
            }
			catch(Exception ex){
				Log.getLogger().log(Level.WARNING, "Exception caught", ex);
			}
		}

		else if (e.getSource() == splitButton_Split) {
			try {
				splitButton_Split.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				OWLNamedClass fromCls = (OWLNamedClass) wrapper.getCls(upperTreePanel_Split.getName());
                if (wrapper.hasAnnotationProperty(fromCls, getSlotName(MERGE_SOURCE)) ||
                    wrapper.hasAnnotationProperty(fromCls, getSlotName(MERGE_TARGET)))
                {
					Object[] option_array = {"OK"};
					splitButton_Split.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					int ans = JOptionPane.showOptionDialog(this, "Cannot split " +
					    fromCls.getName() + " -- concept has been flagged for merge.", "Confirmation",
					JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
					null, option_array, options[0]);
					return;
				}

                String label = "";
                String pt = "";
                boolean cancelled = false;
                boolean hasError = true;
                while (!cancelled)
                {
					NewClsDialog dlg = new NewClsDialog(this);
					dlg.init(label, pt);
					cancelled = dlg.cancelButtonPressed;

                    hasError = false;
					if (cancelled) break;
					label = dlg.getLabel();
					pt = dlg.getPT();

					if (label.compareTo("") == 0 || pt.compareTo("") == 0)
					{
						showDataError("Please complete data entry.");
						hasError = true;
						cancelled = false;
					}
					else if (getOWLModel().getCls(label) != null)
					{
						showConceptExistError(label);
						hasError = true;
						cancelled = false;
					}
					else
					{
						// check if the name is valid:
						boolean nameValid = NCIEditFilter.checkXMLNCNameCompliance(label);
						if (!nameValid)
						{
							String text = "Class name format is incorrect -- " + label;
							ModalDialog.showMessageDialog(null, text);
							hasError = true;
							cancelled = false;
						}
						else
						{
							break;
						}
					}
			    }
			    if (!cancelled && !hasError)
			    {
					splitButton_Split.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					boolean canSplit = populateSplitPanel(label, pt);
					if (!canSplit)
					{
						Object[] option_array = {"OK"};
						int ans = JOptionPane.showOptionDialog(this, "Unable to create a new concept.", "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);
						splitButton_Split.setEnabled(false);
					}
				}
            }
			catch(Exception ex){
				//Log.getLogger().log(Level.WARNING, "Exception caught", ex);
				OWLUI.handleError(owlModel, ex);
			}
			splitButton_Split.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}


		else if (e.getSource() == saveButton_Split) {
            // Split ListenedToClses

 			Cls fromCls = wrapper.getCls(upperTreePanel_Split.getName());
            Cls toCls = getOWLModel().getCls(lowerTreePanel_Split.getName());

			//Cls toCls = wrapper.getCls(lowerTreePanel_Split.getName());
			removeFromListenedToClses((OWLNamedClass) fromCls);
			removeFromListenedToClses((OWLNamedClass) toCls);

            Object[] option_array = {"OK"};
			try {
                owlModel.beginTransaction("Save in split panel. Split class " + upperTreePanel_Split.getName(), fromCls.getName());

                saveButton_Split.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				boolean retval_1 = saveConcept(upperTreePanel_Split);
				if (!retval_1)
				{
					saveButton_Split.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					showError(upperTreePanel_Split);

					addToListenedToClses((OWLNamedClass) fromCls);
					addToListenedToClses((OWLNamedClass) toCls);

					owlModel.rollbackTransaction();
					return;
				}

				boolean retval_2 = saveConcept(lowerTreePanel_Split, lowerTreePanel_Split.getName());
				//Cls fromCls = wrapper.getCls(upperTreePanel_Split.getName());
				//Cls toCls = wrapper.getCls(lowerTreePanel_Split.getName());

				if (!retval_2)
				{
					saveButton_Split.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					showError(lowerTreePanel_Split.getName());


					addToListenedToClses((OWLNamedClass) fromCls);
					//addToListenedToClses((OWLNamedClass) toCls);

					owlModel.rollbackTransaction();
					return;
				}

				// add Split_From annotation property
                //Cls fromCls = wrapper.getCls(upperTreePanel_Split.getName());
				String id_fromCls = wrapper.getCode(fromCls);
				if (id_fromCls == null || id_fromCls.compareTo("0") == 0)
				{
					id_fromCls = wrapper.getInternalName(fromCls);
				}

                //Cls toCls = wrapper.getCls(lowerTreePanel_Split.getName());
				boolean retval = wrapper.addAnnotationProperty(lowerTreePanel_Split.getName(), getSlotName(SPLIT_FROM), id_fromCls);
				if (!retval)
				{

					saveButton_Split.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					addToListenedToClses((OWLNamedClass) fromCls);
					//addToListenedToClses((OWLNamedClass) toCls);

					owlModel.rollbackTransaction();
					return;
				}

                try {

					String name = upperTreePanel_Split.getName();
					if (retval_1 && retval_2)
					{
						toCls = wrapper.getCls(lowerTreePanel_Split.getName());
						// 101306
						RDFSClass c1 = (RDFSClass) fromCls;
						if (c1.isSubclassOf(owlModel.getOWLThingClass()))
						{
							toCls.addDirectSuperclass(owlModel.getOWLThingClass());
						}

						Collection subs = fromCls.getDirectSubclasses();
						for(Iterator iterator = subs.iterator(); iterator.hasNext();)
						{
							RDFSClass sub = (RDFSClass)iterator.next();
							if (sub instanceof OWLNamedClass)
							{
								sub.addDirectSuperclass(toCls);
							}
						}

                        saveButton_Split.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
						int ans = JOptionPane.showOptionDialog(this, "Concept split successful.", "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);

						//logEditEvent("Split",
						//				   upperTreePanel_Split.getName(),
						//				   upperTreePanel_Split.getCode(),
						//				   lowerTreePanel_Split.getName(),
						//				   lowerTreePanel_Split.getCode(),
						//				   null);
                        //

                        // 051007
                        resetSplitPanel();
                        owlModel.commitTransaction();

					}
					else
					{
						saveButton_Split.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
						int ans = JOptionPane.showOptionDialog(this, "Unable to split " + name + ".",
						   "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);


						addToListenedToClses((OWLNamedClass) fromCls);
						addToListenedToClses((OWLNamedClass) toCls);

						owlModel.rollbackTransaction();

						return;
					}

				} catch (Exception ex) {
					owlModel.rollbackTransaction();
					OWLUI.handleError(owlModel, ex);

					addToListenedToClses((OWLNamedClass) fromCls);
					addToListenedToClses((OWLNamedClass) toCls);


				}
				// call logEditEvent in EditHistory to log history data to concept_history table

			}
 			catch(Exception ex){
				owlModel.rollbackTransaction();
				OWLUI.handleError(owlModel, ex);
				addToListenedToClses((OWLNamedClass) fromCls);
				addToListenedToClses((OWLNamedClass) toCls);
			}

			saveButton_Split.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			addToListenedToClses((OWLNamedClass) fromCls);
			addToListenedToClses((OWLNamedClass) toCls);
			ensureClassSelected(fromCls);
		}


		else if (e.getSource() == premergeButton_Premerge) {
			try {
				premergeButton_Premerge.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				String retiringClsName = lowerTreePanel_Premerge.getName();
				String survivingClsName = upperTreePanel_Premerge.getName();
				if (retiringClsName == null || retiringClsName.compareTo("") == 0)
				{
					premergeButton_Premerge.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					return;
				}
				if (survivingClsName == null || survivingClsName.compareTo("") == 0)
				{
					premergeButton_Premerge.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					return;
				}
				if (survivingClsName.compareTo(retiringClsName) == 0)
				{
					premergeButton_Premerge.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					Object[] option_array = {"OK"};
						int ans = JOptionPane.showOptionDialog(this,
						    "Cannot merge a concept with itself. ",
							"Confirmation",
					JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
					null, option_array, options[0]);
					return;
				}
				String editornote = "";
				String designnote = "";
				String prefix = "premerge_annotation";


                NoteDialog dlg = new NoteDialog(this, editornote, designnote, prefix);

				editornote = dlg.getEditorNote();
				designnote = dlg.getDesignNote();

				if (lowerTreePanel_Premerge == null)
				{
					System.out.println("WARNING: lowerTreePanel_Premerge == null");
				}
				else if (dlg.OKBtnPressed())
				{
					JTree edit_tree = lowerTreePanel_Premerge.getTree();

					String prop_name = getSlotName(EDITOR_NOTE);
					TreeNode child_node1 = lowerTreePanel_Premerge.addProperty(prop_name, editornote, null);
					prop_name = getSlotName(DESIGNNOTE);
					TreeNode child_node2 = lowerTreePanel_Premerge.addProperty(prop_name, designnote, null);

					JTree edit_tree2 = upperTreePanel_Premerge.getTree();


					prop_name = getSlotName(MERGE_TARGET);
					TreeNode node1 = lowerTreePanel_Premerge.addProperty(prop_name, survivingClsName, null);

					String prop_value = getClsName(PREMERGED_CONCEPTS);
					Cls premergedCls = wrapper.getCls(prop_value);

					//TreeNode node3 = lowerTreePanel_Premerge.addParent(prop_value);
					TreeNode node3 = lowerTreePanel_Premerge.addParent(premergedCls, false);


					//int n = lowerTreePanel_Premerge.deleteDefiningParents();
					//System.out.println("Numbe of equivalence class removed: " + n);


					prop_name = getSlotName(MERGE_SOURCE);
					TreeNode node2 = upperTreePanel_Premerge.addProperty(prop_name, retiringClsName, null);

					edit_tree.repaint();
					edit_tree2.repaint();

					saveButton_Premerge.setEnabled(true);
					premergeButton_Premerge.setEnabled(false);
					unmergeButton_Premerge.setEnabled(true);

					unmergeButtonPressed = false;

				}
            }
			catch(Exception ex){
				//Log.getLogger().log(Level.WARNING, "Exception caught", ex);
				OWLUI.handleError(owlModel, ex);
			}
			premergeButton_Premerge.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}

// to be modified
		else if (e.getSource() == unmergeButton_Premerge) {
			try {
                unmergeButton_Premerge.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				if (lowerTreePanel_Premerge == null)
				{
					System.out.println("WARNING: lowerTreePanel_Premerge == null");
				}
				else
				{
					JTree edit_tree = lowerTreePanel_Premerge.getTree();
					String retiringClsName = lowerTreePanel_Premerge.getName();
                    deleteNoteProperties(lowerTreePanel_Premerge);

					JTree edit_tree2 = upperTreePanel_Premerge.getTree();
					String survivingClsName = upperTreePanel_Premerge.getName();

					String prop_name = getSlotName(MERGE_TARGET);
					boolean b1 = lowerTreePanel_Premerge.deleteProperty(prop_name);

					String prop_value = getClsName(PREMERGED_CONCEPTS);
					boolean b2 = lowerTreePanel_Premerge.deleteParent(prop_value);

					prop_name = getSlotName(MERGE_SOURCE);
					prop_value = upperTreePanel_Premerge.getPropertyValue(prop_name);
					boolean b3 = upperTreePanel_Premerge.deleteProperty(prop_name, prop_value, null);

					edit_tree.repaint();
					edit_tree2.repaint();

					saveButton_Premerge.setEnabled(true);
					premergeButton_Premerge.setEnabled(true);
					unmergeButton_Premerge.setEnabled(false);

					unmergeButtonPressed = true;

				}
            }
			catch(Exception ex){
				//Log.getLogger().log(Level.WARNING, "Exception caught", ex);
				OWLUI.handleError(owlModel, ex);
			}
			unmergeButton_Premerge.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}

		else if (e.getSource() == saveButton_Premerge) {

            // Premerge ListenedToClses
			Cls fromCls = wrapper.getCls(upperTreePanel_Premerge.getName());
			Cls toCls = wrapper.getCls(lowerTreePanel_Premerge.getName());
			removeFromListenedToClses((OWLNamedClass) fromCls);
			removeFromListenedToClses((OWLNamedClass) toCls);

            Object[] option_array = {"OK"};
			try {
                 owlModel.beginTransaction("Premerge " + upperTreePanel_Premerge.getName() + " and "
                    + lowerTreePanel_Premerge.getName(), fromCls.getName());

                saveButton_Premerge.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				boolean retval_1 = saveConcept(lowerTreePanel_Premerge, false);
				boolean retval_2 = saveConcept(upperTreePanel_Premerge, true);


				if (retval_1 && retval_2)
				{
					if (unmergeButtonPressed)
					{
						saveButton_Premerge.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
						int ans = JOptionPane.showOptionDialog(this,
						    "Merge flags have been removed from concepts "
							+ lowerTreePanel_Premerge.getName()
							+ " and "
							+ upperTreePanel_Premerge.getName(),
							"Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);

					}
					else
					{
						saveButton_Premerge.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
						int ans = JOptionPane.showOptionDialog(this, "Concepts "
							+ lowerTreePanel_Premerge.getName()
							+ " and "
							+ upperTreePanel_Premerge.getName()
							+ " have been flagged for merge.", "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);
					}

/*
			        logEditEvent("Modify",
			                           lowerTreePanel_Premerge.getName(),
			                           lowerTreePanel_Premerge.getCode(),
			                           "",
			                           "",
			                           null);

			        logEditEvent("Modify",
			                           upperTreePanel_Premerge.getName(),
			                           upperTreePanel_Premerge.getCode(),
			                           "",
			                           "",
			                           null);
*/

					resetPremergePanel();
					owlModel.commitTransaction();
				}
				else
				{
					saveButton_Premerge.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));

					int ans = JOptionPane.showOptionDialog(this,
					"Unable to flag " + lowerTreePanel_Premerge.getName() + " and "
					+ upperTreePanel_Premerge.getName()  + " for merge -- " + editfilter.getErrorMessage(),
					   "Confirmation",
					JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
					null, option_array, options[0]);

					addToListenedToClses((OWLNamedClass) fromCls);
					addToListenedToClses((OWLNamedClass) toCls);

                    owlModel.rollbackTransaction();
					return;
				}

			}
 			catch(Exception ex){
				owlModel.rollbackTransaction();
				OWLUI.handleError(owlModel, ex);
			}

			saveButton_Premerge.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			addToListenedToClses((OWLNamedClass) fromCls);
			addToListenedToClses((OWLNamedClass) toCls);
			ensureClassSelected(fromCls);

		}


		else if (e.getSource() == preretireButton_Preretire) {
            preretireButton_Preretire.setCursor(new Cursor(Cursor.WAIT_CURSOR));
            if (droppedCls == null)
            {
				preretireButton_Preretire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				String text = "Retiring concept is not specified.";
				ModalDialog.showMessageDialog(null, text);
				System.out.println(text);
				return;
			}

            resetSubRefTabbedPane();

            DefaultListModel listModel1 = submouseListener.getDefaultListModel();
            DefaultListModel listModel2 = referencemouseListener.getDefaultListModel();
			if (listModel1.getSize()>0 || listModel2.getSize()>0)
			{
				preretireButton_Preretire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
				String text = "Please resolve all subclasses and referencing classes.";

				ProtegeUI.getModalDialogFactory().showErrorMessageDialog(owlModel, text);
				System.out.println(text);
				return;
			}

			Cls retiringCls = wrapper.getCls(leftTreePanel_Preretire.getName());
			//Collection oldSubclasses = retiringCls.getDirectSubclasses();

			try {
				String editornote = "";
				String designnote = "";
				String prefix = "preretire_annotation";

                preretireButton_Preretire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
                NoteDialog dlg = new NoteDialog(this, editornote, designnote, prefix);

				editornote = dlg.getEditorNote();
				designnote = dlg.getDesignNote();

                //if (editornote.compareTo("") != 0 && designnote.compareTo("") != 0)
                if (dlg.OKBtnPressed())
                {
					String prop_name = getSlotName(EDITOR_NOTE);
					JTree edit_tree = leftTreePanel_Preretire.getTree();
					TreeNode child_node1 = leftTreePanel_Preretire.addProperty(prop_name, editornote, null);
					prop_name = getSlotName(DESIGNNOTE);
					TreeNode child_node2 = leftTreePanel_Preretire.addProperty(prop_name, designnote, null);

					Collection col = leftTreePanel_Preretire.getDirectSuperClses();//(RDFSClass) retiringCls);
					Iterator iter = col.iterator();
					while (iter.hasNext())
					{
						Cls sup = (Cls) iter.next();
						if (sup instanceof OWLNamedClass)
						{
							//boolean b = leftTreePanel_Preretire.deleteParent(sup);

							boolean b = leftTreePanel_Preretire.deleteParent(sup);

							TreeNode child_node4 = leftTreePanel_Preretire.addProperty(getSlotName(OLD_PARENT),
								wrapper.getInternalName(sup), sup);

						}

						edit_tree.repaint();
					}


					String prop_value = getClsName(PRERETIRED_CONCEPTS);
//082706
					//Cls preretiredCls = wrapper.getCls(PRERETIRED_CONCEPTS);
					Cls preretiredCls = wrapper.getCls(prop_value);


					//TreeNode child_node3 = leftTreePanel_Preretire.addParent(prop_value);

					TreeNode child_node3 = leftTreePanel_Preretire.addParent(preretiredCls, false);


					// OLD_SOURCE_ROLE?????????????????????????????????????????

			        Vector inboundRoles = leftTreePanel_Preretire.getInboundRoles();
					for (int i=0;i<inboundRoles.size(); i++)
					{
						String inboundRole = (String) inboundRoles.elementAt(i);
						TreeNode child_node4 = leftTreePanel_Preretire.addProperty(getSlotName(OLD_SOURCE_ROLE), inboundRole, null);
						leftTreePanel_Preretire.add_old_source_role(inboundRole);
					}

			        Vector oldSubclasses = leftTreePanel_Preretire.getOldSubclasses();
					for (int i=0;i<oldSubclasses.size(); i++)
					{
						Cls oldSubclass = (Cls) oldSubclasses.elementAt(i);
						//TreeNode child_node4 = leftTreePanel_Preretire.addProperty(getSlotName(OLD_CHILD), oldSubclass, null);
						TreeNode child_node4 = leftTreePanel_Preretire.addProperty(getSlotName(OLD_CHILD),
						    wrapper.getInternalName(oldSubclass), oldSubclass);

						leftTreePanel_Preretire.add_old_child(wrapper.getInternalName(oldSubclass));
					}



					preretireButton_Preretire.setEnabled(false);
					saveButton_Preretire.setEnabled(true);
					unretireButton_Preretire.setEnabled(false);
					clearButton_Preretire.setEnabled(true);

					edit_tree.repaint();

					preretireButtonPressed = true;

			    }

            }
			catch(Exception ex){
				//Log.getLogger().log(Level.WARNING, "Exception caught", ex);
				OWLUI.handleError(owlModel, ex);
			}

			preretireButton_Preretire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));

		}

		else if (e.getSource() == unretireButton_Preretire) {
            if (droppedCls == null)
            {
				String text = "Retiring concept is not specified.";
				ModalDialog.showMessageDialog(null, text);
				System.out.println(text);
				return;
			}

            resetSubRefTabbedPane();
            JTree edit_tree = leftTreePanel_Preretire.getTree();
            boolean preretired = leftTreePanel_Preretire.isSubclassOf(getClsName(PRERETIRED_CONCEPTS));

			if (!preretired)
			{
				String text = droppedCls.getBrowserText() + " has not been flagged for retirement.";
				ModalDialog.showMessageDialog(null, text);
				System.out.println(text);
				return;
			}

			try {

                unretireButton_Preretire.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				deleteNoteProperties(leftTreePanel_Preretire);

				String prop_value = getClsName(PRERETIRED_CONCEPTS);

//082705
//				Cls preretiredCls = wrapper.getCls(PRERETIRED_CONCEPTS);

Cls preretiredCls = wrapper.getCls(prop_value);

				boolean b3 = leftTreePanel_Preretire.deleteParent(prop_value);

/*
				Vector sups = leftTreePanel_Preretire.getPropertyValues(getSlotName(OLD_PARENT));
				for (int i=0; i<sups.size(); i++)
				{
					String sup = (String) sups.elementAt(i);
					//TreeNode child_node4 = leftTreePanel_Preretire.addParent(sup);


					Cls supCls = wrapper.getCls(sup);
					//TreeNode child_node4 = leftTreePanel_Preretire.addParent(sup);
					TreeNode child_node4 = leftTreePanel_Preretire.addParent(supCls);

					boolean b4 = leftTreePanel_Preretire.deleteProperty(getSlotName(OLD_PARENT), sup);
				}
*/
				Vector sups = leftTreePanel_Preretire.getPropertyObjectValues(getSlotName(OLD_PARENT));
				for (int i=0; i<sups.size(); i++)
				{
					Cls supCls = (Cls) sups.elementAt(i);
					//TreeNode child_node4 = leftTreePanel_Preretire.addParent(sup);
					//TreeNode child_node4 = leftTreePanel_Preretire.addParent(sup);
					TreeNode child_node4 = leftTreePanel_Preretire.addParent(supCls, false);

					boolean b4 = leftTreePanel_Preretire.deleteProperty(getSlotName(OLD_PARENT), wrapper.getInternalName(supCls), supCls);
				}

/*
				Vector subs = leftTreePanel_Preretire.getPropertyValues(getSlotName(OLD_CHILD));
				for (int i=0; i<subs.size(); i++)
				{
					String sub = (String) subs.elementAt(i);
					boolean b4 = leftTreePanel_Preretire.deleteProperty(getSlotName(OLD_CHILD), sub);
// ???
// move to save button
//String droppedCls_internalName = wrapper.getInternalName(droppedCls);


Cls subCls = wrapper.getCls(sub);
					//wrapper.addDirectSuperclass(sub, droppedCls.getBrowserText());
					wrapper.addDirectSuperclass(subCls, droppedCls);
				}
*/
                //082706
				Vector subs = leftTreePanel_Preretire.getPropertyObjectValues(getSlotName(OLD_CHILD));
				for (int i=0; i<subs.size(); i++)
				{
					Cls subCls = (Cls) subs.elementAt(i);
					boolean b4 = leftTreePanel_Preretire.deleteProperty(getSlotName(OLD_CHILD), wrapper.getInternalName(subCls), subCls);
					wrapper.addDirectSuperclass(subCls, droppedCls);
				}


				Vector roles = leftTreePanel_Preretire.getPropertyValues(getSlotName(OLD_SOURCE_ROLE));
				for (int i=0; i<roles.size(); i++)
				{
					String role = (String) roles.elementAt(i);
					boolean b4 = leftTreePanel_Preretire.deleteProperty(getSlotName(OLD_SOURCE_ROLE), role, null);


// move to save button

				}


				edit_tree.repaint();

				preretireButton_Preretire.setEnabled(false);
				saveButton_Preretire.setEnabled(true);
				unretireButton_Preretire.setEnabled(false);
				clearButton_Preretire.setEnabled(true);


            }
			catch(Exception ex){
				//Log.getLogger().log(Level.WARNING, "Exception caught", ex);
				OWLUI.handleError(owlModel, ex);
			}
			unretireButton_Preretire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}

// MERGE
		else if (e.getSource() == mergeButton_Merge) {
			try {
				mergeButton_Merge.setCursor(new Cursor(Cursor.WAIT_CURSOR));

				Cls retiringCls = wrapper.getCls(lowerTreePanel_Merge.getName());
				Cls survivalCls = wrapper.getCls(upperTreePanel_Merge.getName());

				JTree edit_tree_1 = lowerTreePanel_Merge.getTree();
				JTree edit_tree_2 = upperTreePanel_Merge.getTree();

				String prop_name = getSlotName(MERGE_SOURCE);
				boolean b4 = upperTreePanel_Merge.deleteProperty(prop_name);
				edit_tree_2.repaint();

				prop_name = getSlotName(MERGE_TARGET);
				boolean b3 = lowerTreePanel_Merge.deleteProperty(prop_name);
				edit_tree_1.repaint();

				String prop_value = getClsName(PREMERGED_CONCEPTS);
				boolean b5 = lowerTreePanel_Merge.deleteParent(prop_value);


//int n = lowerTreePanel_Merge.removeEquivalentSuperclasses();
//System.out.println("Number of equivalence class removed: " + n);


// Remove all equivalence classes
// owl:

// merge operation
//
	/*
	i)	Concept one (survival) gains from concept two (retiring)
	(1)	all nonredundant properties from concept two
	*/
	            Vector exclusion = new Vector();
	            exclusion.add(getSlotName(CODE));
	            exclusion.add("ID");
	            exclusion.add(getSlotName(PREFERRED_NAME));
	            exclusion.add(getSlotName(EDITOR_NOTE));
	            exclusion.add(getSlotName(DESIGNNOTE));
	            exclusion.add(getSlotName(OLD_SOURCE_ROLE));
	            exclusion.add(getSlotName(OLD_ROLE));
	            exclusion.add(getSlotName(OLD_PARENT));
	            exclusion.add("rdfs:label");
	            exclusion.add("hasType");

	            // 082706
	            String split_from_id = lowerTreePanel_Merge.getPropertyValue(getSlotName(SPLIT_FROM));
	            if (split_from_id != null)
	            {
					if (split_from_id.compareTo(upperTreePanel_Merge.getName()) == 0)
					{
						exclusion.add(getSlotName(SPLIT_FROM));
					}
					else if (split_from_id.compareTo(upperTreePanel_Merge.getCode()) == 0)
					{
						exclusion.add(getSlotName(SPLIT_FROM));
					}
			    }

	            String merge_into_id = lowerTreePanel_Merge.getPropertyValue(getSlotName(MERGE_INTO));
	            if (merge_into_id != null)
	            {
					if (merge_into_id.compareTo(upperTreePanel_Merge.getName()) == 0)
					{
						exclusion.add(getSlotName(MERGE_INTO));
					}
					else if (merge_into_id.compareTo(upperTreePanel_Merge.getCode()) == 0)
					{
						exclusion.add(getSlotName(MERGE_INTO));
					}
				}

	            Vector survival_state_vec = upperTreePanel_Merge.getFinalState();
	            Vector retiring_state_vec = lowerTreePanel_Merge.getFinalState();

				// 090806
				// change NCI/PT to NCI/SY in retiring concept
				retiring_state_vec = convertNCIPT2NCISY(retiring_state_vec);

	            Vector v = datahandler.getTreeItemDiffsByType(survival_state_vec,
	                                                          retiring_state_vec,
	                                                          TreeItem.TYPE_PROPERTY,
	                                                          exclusion);


	            for (int i=0; i<v.size(); i++)
	            {
					TreeItem item = (TreeItem) v.elementAt(i);
					TreeNode dmtn = upperTreePanel_Merge.addProperty(item.getName(),
					   item.getValue(), null);
				}

				// modify editor_note and designnote:
				String old_editor_note = lowerTreePanel_Merge.getPropertyValue(getSlotName(EDITOR_NOTE));
				if (old_editor_note != null && old_editor_note.indexOf("|") != -1)
				{
					String editor_note = old_editor_note.substring(old_editor_note.indexOf("|")+1);
				    lowerTreePanel_Merge.deleteProperty(getSlotName(EDITOR_NOTE), old_editor_note, null);
				    lowerTreePanel_Merge.addProperty(getSlotName(EDITOR_NOTE), editor_note, null);
				}

				String old_designnote = lowerTreePanel_Merge.getPropertyValue(getSlotName(DESIGNNOTE));
				if (old_designnote != null && old_designnote.indexOf("|") != -1)
				{
					String designnote = old_designnote.substring(old_designnote.indexOf("|")+1);
				    lowerTreePanel_Merge.deleteProperty(getSlotName(DESIGNNOTE), old_designnote, null);
				    lowerTreePanel_Merge.addProperty(getSlotName(DESIGNNOTE), designnote, null);
				}

				// OLD_CHILD properties
/*
				Vector subs = wrapper.getNamedSubclassNames(retiringCls);
				for (int i=0; i<subs.size(); i++)
				{
					String sub = (String) subs.elementAt(i);
					TreeNode dmtn = lowerTreePanel_Merge.addProperty(getSlotName(OLD_CHILD),
					   sub, null);
				}
*/

                //082706
				Collection subs = retiringCls.getDirectSubclasses();
				Object[] obs_2 = subs.toArray();
				for (int j=0; j<obs_2.length; j++)
				{
					if (obs_2[j] instanceof OWLNamedClass)
					{
						OWLNamedClass owlcls_2 = (OWLNamedClass) obs_2[j];
						TreeNode dmtn = lowerTreePanel_Merge.addProperty(getSlotName(OLD_CHILD),
						   wrapper.getInternalName(owlcls_2), owlcls_2);
				    }
				}


/*
	(2)	all nonredundant roles from concept two
*/
	            survival_state_vec = upperTreePanel_Merge.getFinalState();
	            retiring_state_vec = lowerTreePanel_Merge.getFinalState();
	            v = datahandler.getTreeItemDiffsByType(survival_state_vec,
	                                                   retiring_state_vec,
	                                                   TreeItem.TYPE_RESTRICTION,
	                                                   new Vector());
	            for (int i=0; i<v.size(); i++)
	            {
					TreeItem item = (TreeItem) v.elementAt(i);

System.out.println("+++ mergeButton_Merge getTreeItemDiffsByType " + item.getCls().getBrowserText());

					//TreeNode dmtn = upperTreePanel_Merge.addRestriction(item.getName(),
					//   item.getValue(), item.getModifier());

					// need to be cloned??? 040707

					TreeNode dmtn = upperTreePanel_Merge.addRestriction(item.getCls().createClone(), item.getIsDefining());

			    }


/*
	(3)	all nonredundant parents from concept two
*/
	            survival_state_vec = upperTreePanel_Merge.getFinalState();
	            retiring_state_vec = lowerTreePanel_Merge.getFinalState();
	            v = datahandler.getTreeItemDiffsByType(survival_state_vec,
	                                                   retiring_state_vec,
	                                                   TreeItem.TYPE_PARENT,
	                                                   new Vector());
	            for (int i=0; i<v.size(); i++)
	            {
					TreeItem item = (TreeItem) v.elementAt(i);

					//TreeNode dmtn = upperTreePanel_Merge.addParent(item.getValue());

					String sup_name = wrapper.getInternalName((Cls) item.getCls());
					String survivalCls_name = wrapper.getInternalName(survivalCls);
					if (sup_name.compareTo(survivalCls_name) != 0)
					{
						// KLO, 040907
						// TreeNode dmtn = upperTreePanel_Merge.addParent((Cls) item.getCls());
						TreeNode dmtn = upperTreePanel_Merge.addParent((Cls) item.getCls(), item.getIsDefining());
					}
				}


/*
	(4)	all nonredundant children from concept two
        // Note: can store children and inbound roles of the retiring concept
        //    in local variables of the corresponding TreePanel (i.e., lowerTreePanel_Merge)


	ii)	On nonredundant entities:  It is expected that a number of entities (parents, roles, properties) will be present in both concepts.  Do not add a specific entity if concept one already has it.

	(1)	When assessing whether a property, role value, so on, is the same in both concepts, conduct the string comparison in a case?sensitive manner.
	iii)	Concept one also becomes the target of any referencing concepts targeting concept two (see below)
	(1)	Concepts that refer to concept two include children in addition to role sources.
	iv)	The concept name, code, and ID of concept two are ignored.
	v)	In the history table, there will be a "merge" entry for concept one, referencing itself as a descendant in the merge action.


		d)	Merge actions on concept two (retirement actions).
		i)	All the properties are retained
		ii)	Roles are stored in OLD_ROLE property and deleted
		(1)	OLD_ROLE value contains the role name and the role value, delimited by pipes
		(a)	The role value within OLD_ROLE is the concept name of the target concept
		(2)	The roles are deleted after conversion to the OLD_ROLE property
		iii)	Parents are stored in OLD_PARENT property and deleted
		(1)	The concept name of the parent concept is stored as the value of OLD_PARENT
		(2)	The reference to the parent concept(s) is(are) deleted
		(3)	A new parent concept, "Retired Concept" from the Retired_Kind hierarchy is added.
		iv)	Store the kind name in OLD_KIND property
		(1)	The kind name string is stored as a value of OLD_KIND
		(2)	Set the kind to Retired_Kind
		v)	Children concepts are stored in OLD_CHILD property; children are remapped to concept one.
		(1)	Children concept names are stored as values of OLD_CHILD, one property per child
		(2)	The childrent concepts are retrieved and parent references to concept two are deleted
		(3)	Concept one is added as a parent of each child
		vi)	Referring role sources targeting concept two are remapped to concept one.
		(1)	Concepts that refer to concept two via roles are retrieved
		(2)	The roles referring to concept two are identified
		(3)	New roles are created that refer to concept one instead
		(4)	The roles referring to concept two are finally deleted
		vii)	In the history table, the entry for concept two will show a "merge" action with a reference to concept one, and a "retire" action with no reference code.
		(1)	i.e.
		conceptcode	action	editdate	referencecode 	other columns
		concept two	merge	timestamp	concept one
		concept two	retire	timestamp	null
	*/

		//remove Merge_Target property; remove Merged_<kind>_Concepts superconcept
		//remove Editor_Note, DesignNote

                updateNoteProperties(lowerTreePanel_Merge, (RDFSClass)retiringCls);
				attributes2Properties(lowerTreePanel_Merge, (RDFSClass)retiringCls);
				TreeNode child_node = null;

				prop_value = getClsName(RETIRED_CONCEPTS);
				//child_node = lowerTreePanel_Merge.addParent(prop_value);

				Cls retiredCls = wrapper.getCls(prop_value);
				//child_node = lowerTreePanel_Merge.addParent(prop_value);
				child_node = lowerTreePanel_Merge.addParent(retiredCls);

                mergeButton_Merge.setEnabled(false);
                unmergeButton_Merge.setEnabled(false);
                saveButton_Merge.setEnabled(true);
                clearButton_Merge.setEnabled(true);

                edit_tree_1 = null;
                edit_tree_2 = null;


			}
			catch(Exception ex){
				//Log.getLogger().log(Level.WARNING, "Exception caught", ex);
				OWLUI.handleError(owlModel, ex);
			}
			mergeButton_Merge.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}

		else if (e.getSource() == unmergeButton_Merge) {
			// to be implemented
            unmergeButton_Merge.setCursor(new Cursor(Cursor.WAIT_CURSOR));
			JTree edit_tree = lowerTreePanel_Merge.getTree();
			String retiringClsName = lowerTreePanel_Merge.getName();
			deleteNoteProperties(lowerTreePanel_Merge);

			JTree edit_tree2 = upperTreePanel_Merge.getTree();
			String survivingClsName = upperTreePanel_Merge.getName();

			String prop_name = getSlotName(MERGE_TARGET);
			boolean b1 = lowerTreePanel_Merge.deleteProperty(prop_name);

			String prop_value = getClsName(PREMERGED_CONCEPTS);
			boolean b2 = lowerTreePanel_Merge.deleteParent(prop_value);

			prop_name = getSlotName(MERGE_SOURCE);
			prop_value = upperTreePanel_Merge.getPropertyValue(prop_name);
			boolean b3 = upperTreePanel_Merge.deleteProperty(prop_name, prop_value, null);

			prop_value = getClsName(PREMERGED_CONCEPTS);
			boolean b5 = lowerTreePanel_Merge.deleteParent(prop_value);
/*
			Vector sups = lowerTreePanel_Merge.getPropertyValues(getSlotName(OLD_PARENT));
			for (int i=0; i<sups.size(); i++)
			{
				String sup = (String) sups.elementAt(i);


				Cls superCls = wrapper.getCls(sup);

				//TreeNode child_node4 = lowerTreePanel_Merge.addParent(sup);
				TreeNode child_node4 = lowerTreePanel_Merge.addParent(superCls);


				boolean b4 = lowerTreePanel_Merge.deleteProperty(getSlotName(OLD_PARENT), sup);
			}
*/
			Vector sups = lowerTreePanel_Merge.getPropertyObjectValues(getSlotName(OLD_PARENT));
			for (int i=0; i<sups.size(); i++)
			{
				Cls superCls = (Cls) sups.elementAt(i);
				//TreeNode child_node4 = lowerTreePanel_Merge.addParent(sup);
				TreeNode child_node4 = lowerTreePanel_Merge.addParent(superCls);


				boolean b4 = lowerTreePanel_Merge.deleteProperty(getSlotName(OLD_PARENT), wrapper.getInternalName(superCls), superCls);
			}

			edit_tree.repaint();

            unmergeButton_Merge.setEnabled(false);
			mergeButton_Merge.setEnabled(false);

			saveButton_Merge.setEnabled(true);
			clearButton_Merge.setEnabled(true);

			unmergeButtonPressed = true;
            unmergeButton_Merge.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}


		else if (e.getSource() == saveButton_Merge) {

            // Merge ListenedToClses
            Object[] option_array = {"OK"};

 			//Cls fromCls = wrapper.getCls(upperTreePanel_Merge.getName());
 			//Cls toCls = wrapper.getCls(lowerTreePanel_Merge.getName());

  			Cls toCls = wrapper.getCls(upperTreePanel_Merge.getName());
  			Cls fromCls = wrapper.getCls(lowerTreePanel_Merge.getName());

 			removeFromListenedToClses((OWLNamedClass) fromCls);
 			removeFromListenedToClses((OWLNamedClass) toCls);


			try {
				owlModel.beginTransaction("Merge " + lowerTreePanel_Merge.getName() + " into "
				    + upperTreePanel_Merge.getName(), fromCls.getName());
				saveButton_Merge.setCursor(new Cursor(Cursor.WAIT_CURSOR));

				boolean retval_1 = saveConcept(lowerTreePanel_Merge);
				boolean retval_2 = saveConcept(upperTreePanel_Merge);

				if (retval_1 && retval_2)
				{
					if (unmergeButtonPressed)
					{
						int ans = JOptionPane.showOptionDialog(this, "Concepts "
							+ lowerTreePanel_Merge.getName()
							+ " and "
							+ upperTreePanel_Merge.getName()
							+ " are unmerged successfully.", "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);

						resetMergePanel();
					}
					else
					{
						// 07/24/06 add Merge_Into annotation property
						String id_toCls = wrapper.getCode(toCls);
						if (id_toCls == null || id_toCls.compareTo("0") == 0)
						{
							id_toCls = wrapper.getInternalName(toCls);
						}

						boolean retval = wrapper.addAnnotationProperty(fromCls.getName(), getSlotName(MERGE_INTO), id_toCls);
						if (!retval)
						{
							addToListenedToClses((OWLNamedClass) fromCls);
							addToListenedToClses((OWLNamedClass) toCls);
                            owlModel.rollbackTransaction();
							return;
						}

// transfer subconcepts of the retiring concept to suviving concept
// fromCls: survival
// toCls: retiring

						wrapper.removeDirectSuperclasses((RDFSClass) fromCls);
                        wrapper.addDirectSubclasses((RDFSClass) fromCls, (RDFSClass) toCls);
                        wrapper.removeDirectSubclasses((RDFSClass) fromCls);

// redirect inbound roles from the retiring concept to the surviving concept
                        wrapper.redirectReferringClasses(fromCls, toCls);

						Cls cls = getOWLModel().getCls(lowerTreePanel_Merge.getName());
						Deprecatable deprecatedCls = (Deprecatable) cls;
						deprecatedCls.setDeprecated(true);

						wrapper.removeDirectSuperclasses((RDFSClass) fromCls);

						int ans = JOptionPane.showOptionDialog(this, "Concepts "
							+ lowerTreePanel_Merge.getName()
							+ " and "
							+ upperTreePanel_Merge.getName()
							+ " are merged successfully.", "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);

						resetMergePanel();
						owlModel.commitTransaction();


					}
					//resetMergePanel();
				}
				else
				{
					String action = "merge";
					if (unmergeButtonPressed)
					{
						action = "unmerge";
					}
					saveButton_Merge.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					int ans = JOptionPane.showOptionDialog(this,
					"Unable to " + action + " " + lowerTreePanel_Merge.getName()  + " and "
					+ upperTreePanel_Merge.getName() + ".",
					   "Confirmation",
					JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
					null, option_array, options[0]);


					addToListenedToClses((OWLNamedClass) fromCls);
					addToListenedToClses((OWLNamedClass) toCls);

					owlModel.rollbackTransaction();
					return;
				}
				unmergeButtonPressed = false;
			}
 			catch(Exception ex){
				owlModel.rollbackTransaction();
				OWLUI.handleError(owlModel, ex);
			}
			saveButton_Merge.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));

			addToListenedToClses((OWLNamedClass) fromCls);
			addToListenedToClses((OWLNamedClass) toCls);


		}


		else if (e.getSource() == saveButton_Preretire) {

//OLD_SOURCE_ROLE
//OLD_CHILD
//OLD_PARENT

// Process OLD_SOURCE_ROLE
// Process OLD_CHILD

            // Pre-Retire ListenedToClses
			Cls fromCls = wrapper.getCls(leftTreePanel_Preretire.getName());
			removeFromListenedToClses((OWLNamedClass) fromCls);
			try {
                owlModel.beginTransaction("Preretire " + leftTreePanel_Preretire.getName(), fromCls.getName());
                saveButton_Preretire.setCursor(new Cursor(Cursor.WAIT_CURSOR));

				boolean retval = saveConcept(leftTreePanel_Preretire);
				if (retval)
				{
					Object[] option_array = {"OK"};
					String name = leftTreePanel_Preretire.getName();

					if (unretireButtonPressed)
					{
						saveButton_Preretire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
						int ans = JOptionPane.showOptionDialog(this, name + " has been unretired", "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);

					}
					else
					{
						saveButton_Preretire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
						int ans = JOptionPane.showOptionDialog(this, name + " has been flagged for retirement", "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);
				    }
/*
			        logEditEvent("Modify",
			                           leftTreePanel_Preretire.getName(),
			                           leftTreePanel_Preretire.getCode(),
			                           "",
			                           "",
			                           null);

*/
					resetPreretirePanel();
					owlModel.commitTransaction();

				}
				else
				{
					saveButton_Preretire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					showError(leftTreePanel_Preretire.getName());
					System.out.println("Save is incomplete.");
					owlModel.rollbackTransaction();
				}
            }
			catch(Exception ex){
				owlModel.rollbackTransaction();
				OWLUI.handleError(owlModel, ex);
			}

			addToListenedToClses((OWLNamedClass) fromCls);
			saveButton_Preretire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			ensureClassSelected(fromCls);
		}

		else if (e.getSource() == retireButton_Retire) {
			try {
				retireButton_Retire.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				JTree edit_tree_1 = upperTreePanel_Retire.getTree();
                Cls retiringCls = wrapper.getCls(upperTreePanel_Retire.getName());

				updateNoteProperties(upperTreePanel_Retire, (RDFSClass)retiringCls);
				attributes2Properties(upperTreePanel_Retire, (RDFSClass)retiringCls);

				TreeNode child_node = null;
				Vector roles = upperTreePanel_Retire.getPropertyValues(getSlotName(OLD_SOURCE_ROLE));
				for (int i=0; i<roles.size(); i++)
				{
					String role = (String) roles.elementAt(i);
					boolean b4 = upperTreePanel_Retire.deleteProperty(getSlotName(OLD_SOURCE_ROLE), role, null);

				}

				String conceptStatus = upperTreePanel_Retire.getPropertyValue(getSlotName(CONCEPT_STATUS));
				if (conceptStatus != null)
				{
					boolean b4 = upperTreePanel_Retire.deleteProperty(getSlotName(CONCEPT_STATUS), conceptStatus, null);
				}
				TreeNode dmtn = upperTreePanel_Retire.addProperty(getSlotName(CONCEPT_STATUS), "Retired_Concept", null);

				String prop_value = getClsName(PRERETIRED_CONCEPTS);
				boolean b5 = upperTreePanel_Retire.deleteParent(prop_value);

				prop_value = getClsName(RETIRED_CONCEPTS);
				//child_node = upperTreePanel_Retire.addParent(prop_value);

// 082706
				//Cls retiredClses = wrapper.getCls(RETIRED_CONCEPTS);

				Cls retiredClses = wrapper.getCls(prop_value);
				child_node = upperTreePanel_Retire.addParent(retiredClses);



// modify editor_note and designnote:
				String old_editor_note = upperTreePanel_Retire.getPropertyValue(getSlotName(EDITOR_NOTE));
				if (old_editor_note != null && old_editor_note.indexOf("|") != -1)
				{
					String editor_note = old_editor_note.substring(old_editor_note.indexOf("|")+1);
				    upperTreePanel_Retire.deleteProperty(getSlotName(EDITOR_NOTE), old_editor_note, null);
				    upperTreePanel_Retire.addProperty(getSlotName(EDITOR_NOTE), editor_note, null);
				}

				String old_designnote = upperTreePanel_Retire.getPropertyValue(getSlotName(DESIGNNOTE));
				if (old_designnote != null && old_designnote.indexOf("|") != -1)
				{
					String designnote = old_designnote.substring(old_designnote.indexOf("|")+1);
				    upperTreePanel_Retire.deleteProperty(getSlotName(DESIGNNOTE), old_designnote, null);
				    upperTreePanel_Retire.addProperty(getSlotName(DESIGNNOTE), designnote, null);
				}
                unretireButton_Retire.setEnabled(false);
                retireButton_Retire.setEnabled(false);
                saveButton_Retire.setEnabled(true);
                clearButton_Retire.setEnabled(true);

                unretireButtonPressed = false;

			}
			catch(Exception ex){
				//Log.getLogger().log(Level.WARNING, "Exception caught", ex);
				OWLUI.handleError(owlModel, ex);
			}
			retireButton_Retire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}

		else if (e.getSource() == unretireButton_Retire) {
			try {
				unretireButton_Retire.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				JTree edit_tree_1 = upperTreePanel_Retire.getTree();
                Cls retiringCls = wrapper.getCls(upperTreePanel_Retire.getName());
                deleteNoteProperties(upperTreePanel_Retire);

				String prop_value = getClsName(PRERETIRED_CONCEPTS);
				boolean b5 = upperTreePanel_Retire.deleteParent(prop_value);

				prop_value = getClsName(RETIRED_CONCEPTS);
				b5 = upperTreePanel_Retire.deleteParent(prop_value);

                properties2Attributes(upperTreePanel_Retire, (RDFSClass) retiringCls);

                unretireButton_Retire.setEnabled(false);
                retireButton_Retire.setEnabled(false);
                saveButton_Retire.setEnabled(true);
                clearButton_Retire.setEnabled(true);

                unretireButtonPressed = true;

			}
			catch(Exception ex){
				//Log.getLogger().log(Level.WARNING, "Exception caught", ex);
				OWLUI.handleError(owlModel, ex);
			}
			unretireButton_Retire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
		}


		else if (e.getSource() == saveButton_Retire) {
            // Retire ListenedToClses
            Cls fromCls = wrapper.getCls(upperTreePanel_Retire.getName());
            removeFromListenedToClses((OWLNamedClass) fromCls);

			try {
				owlModel.beginTransaction("Retire " + upperTreePanel_Retire.getName(), fromCls.getName());
				saveButton_Retire.setCursor(new Cursor(Cursor.WAIT_CURSOR));
				boolean retval = saveConcept(upperTreePanel_Retire);
				if (retval)
				{
					String name = upperTreePanel_Retire.getName();
					Object[] option_array = {"OK"};

					if (unretireButtonPressed)
					{
						saveButton_Retire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
						int ans = JOptionPane.showOptionDialog(this, name + " has been unretired.", "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);

					}
					else
					{
						saveButton_Retire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
						int ans = JOptionPane.showOptionDialog(this, name + " has been retired.", "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);

						Cls cls = getOWLModel().getCls(upperTreePanel_Retire.getName());
						Deprecatable deprecatedCls = (Deprecatable) cls;
						deprecatedCls.setDeprecated(true);
					}

					resetRetirePanel();
					owlModel.commitTransaction();
				}
				else
				{
					showError(upperTreePanel_Retire.getName());
					System.out.println("Save is incomplete.");
					owlModel.rollbackTransaction();
				}

            }
			catch(Exception ex){
				owlModel.rollbackTransaction();
				OWLUI.handleError(owlModel, ex);
			}
			addToListenedToClses((OWLNamedClass) fromCls);
			saveButton_Retire.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
			ensureClassSelected(fromCls);
		}

		else if (e.getSource() == loadButton_Loader) {

			try {
               BatchEditorDialog dlg = new BatchEditorDialog(this, getOWLModel(), BatchEditorDialog.BATCH_LOADER);

            }
			catch(Exception ex){
				Log.getLogger().log(Level.WARNING, "Exception caught", ex);

			}
		}

		else if (e.getSource() == clearButton_Loader) {
            clearButton_Loader.setEnabled(false);
            loadButton_Loader.setEnabled(true);
            saveButton_Loader.setEnabled(false);
            loaderTextArea.setText("");
		}

		else if (e.getSource() == saveButton_Loader) {
			JFileChooser fc = new JFileChooser();
			int returnVal = fc.showOpenDialog(this);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				exportFile = fc.getSelectedFile();
				writeToFile(loaderTextArea, exportFile);
				return;
			}
		}

		else if (e.getSource() == loadButton_Editor) {

			try {
               BatchEditorDialog dlg = new BatchEditorDialog(this, getOWLModel(), BatchEditorDialog.BATCH_EDITOR);
            }
			catch(Exception ex){
				Log.getLogger().log(Level.WARNING, "Exception caught", ex);
			}
		}

		else if (e.getSource() == clearButton_Editor) {
            clearButton_Editor.setEnabled(false);
            loadButton_Editor.setEnabled(true);
            saveButton_Editor.setEnabled(false);
            editorTextArea.setText("");
		}

		else if (e.getSource() == saveButton_Editor) {
			JFileChooser fc = new JFileChooser();
			int returnVal = fc.showOpenDialog(this);
			if (returnVal == JFileChooser.APPROVE_OPTION) {
				exportFile = fc.getSelectedFile();
				writeToFile(editorTextArea, exportFile);
				return;
			}
		}


		else if (e.getSource() == treeButton_Partonomy) {
            Vector v = getPropertiesList();
            if (v.size() == 0)
            {
				Object[] option_array = {"OK"};
				int ans = JOptionPane.showOptionDialog(this, "No applicable part-of relationship is found.", "Confirmation",
				JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
				null, option_array, options[0]);
                return;
			}

			try {
               SelectRoleDialog dlg = new SelectRoleDialog(this, getOWLModel());
            }
			catch(Exception ex){
				Log.getLogger().log(Level.WARNING, "Exception caught", ex);
			}
		}

		else if (e.getSource() == saveButton_Partonomy) {

			try {

				JFileChooser fc = new JFileChooser();
				int returnVal = fc.showOpenDialog(this);
				if (returnVal == JFileChooser.APPROVE_OPTION) {
					File selectedFile = fc.getSelectedFile();

					if (selectedFile != null)
					{
						String outfile = selectedFile.toString();
						System.out.println("Selected file: " + 	outfile);
						PrintWriter pw = new PrintWriter (new BufferedWriter(new FileWriter(outfile)));
						boolean retval = partonomyPanel.outputTree(pw);

						if (retval)
						{
							// write partonomy tree data to a file
							Object[] option_array = {"OK"};
							int ans = JOptionPane.showOptionDialog(this, "Output file " + outfile + " generated.",
							   "Confirmation",
							   JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
							null, option_array, options[0]);
						}
						pw.close();
					}
					else
					{
						System.out.println("File not found.");
					}
				}
           }
			catch(Exception ex){
				Log.getLogger().log(Level.WARNING, "Exception caught", ex);
			}
		}

		else if (e.getSource() == clearButton_Partonomy) {
			partonomyPanel =
				new PartonomyPanel(this, null, getOWLModel(), wrapper);
			partonomyTreeScrollPane.setViewportView(partonomyPanel);
			clearButton_Partonomy.setEnabled(false);
			saveButton_Partonomy.setEnabled(false);
		}

		else if (e.getSource() == clearButton_Clone) {
			try {
				int ans = JOptionPane.showOptionDialog(this, "No data will be saved. Do you want to continue?", "Confirmation",
				JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
				null, options, options[0]);

				if (ans == 1) return;
				clearButton_Clone.setEnabled(false);

				resetClonePanel();
            }
			catch(Exception ex){
				Log.getLogger().log(Level.WARNING, "Exception caught", ex);
			}
		}


		else if (e.getSource() == cloneButton_Clone) {
			try {

                String label = "";
                String pt = "";
                boolean cancelled = false;
                boolean hasError = true;

                while (!cancelled)
                {
					NewClsDialog dlg = new NewClsDialog(this);
					dlg.init(label, pt);
					cancelled = dlg.cancelButtonPressed;

                    hasError = false;
					if (cancelled) break;
					label = dlg.getLabel();
					pt = dlg.getPT();

					if (label.compareTo("") == 0 || pt.compareTo("") == 0)
					{
						showDataError("Please complete data entry.");
						hasError = true;
						cancelled = false;
					}
					else if (getOWLModel().getCls(label) != null)
					{
						showConceptExistError(label);
						hasError = true;
						cancelled = false;
					}
					else
					{
						// check if the name is valid:
						boolean nameValid = NCIEditFilter.checkXMLNCNameCompliance(label);
						if (!nameValid)
						{
							String text = "Class name format is incorrect -- " + label;
							ModalDialog.showMessageDialog(null, text);
							hasError = true;
							cancelled = false;
						}
						else
						{
							break;
						}
					}
			    }

			    if (!cancelled && !hasError)
			    {
					boolean retval = populateClonePanel(label, pt);
					splitButton_Split.setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
					boolean canSplit = populateSplitPanel(label, pt);
					if (!canSplit)
					{
						Object[] option_array = {"OK"};
						int ans = JOptionPane.showOptionDialog(this, "Unable to create a new concept.", "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);
						cloneButton_Clone.setEnabled(false);
						return;
					}
					cloned_cls=true;
				}

            }
			catch(Exception ex){
				//Log.getLogger().log(Level.WARNING, "Exception caught", ex);
				OWLUI.handleError(owlModel, ex);
			}
		}

		else if (e.getSource() == saveButton_Clone) {
            // Clone ListenedToClses
			Cls fromCls = wrapper.getCls(upperTreePanel_Clone.getName());
			Cls toCls = wrapper.getCls(lowerTreePanel_Clone.getName());
			removeFromListenedToClses((OWLNamedClass) fromCls);
			removeFromListenedToClses((OWLNamedClass) toCls);

            Object[] option_array = {"OK"};
			try {
                owlModel.beginTransaction("Create a copy (clone) of " + upperTreePanel_Clone.getName(), fromCls.getName());
                boolean concept1_exists = false;
			    if (upperTreePanel_Clone.droppedCls != null
			    &&  upperTreePanel_Clone.getName().compareTo("") != 0)
			    {
					concept1_exists = true;
			    }

                boolean concept2_exists = false;
			    if (lowerTreePanel_Clone.getRootNode() != null)
			    {
					concept2_exists = true;
			    }

			    if (concept1_exists && !concept2_exists)
				{
					boolean retval_1 = saveConcept(upperTreePanel_Clone);
					if (!retval_1)
					{
						showError(upperTreePanel_Clone);
						addToListenedToClses((OWLNamedClass) fromCls);
						addToListenedToClses((OWLNamedClass) toCls);
                        owlModel.rollbackTransaction();
						return;
					}
					else
					{
						int ans = JOptionPane.showOptionDialog(this, "Concept " +  upperTreePanel_Clone.getName()
						 + " saved successful.", "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);

						resetClonePanel();
						owlModel.commitTransaction();
					}
			    }
			    else if (!concept1_exists && concept2_exists && !lowerTreePanel_Clone.getIsNew())
				{
					boolean retval_1 = saveConcept(lowerTreePanel_Clone);
					if (!retval_1)
					{
						showError(lowerTreePanel_Clone);
						addToListenedToClses((OWLNamedClass) fromCls);
						addToListenedToClses((OWLNamedClass) toCls);
                        owlModel.rollbackTransaction();
						return;
					}
					else
					{
						int ans = JOptionPane.showOptionDialog(this, "Concept " +  lowerTreePanel_Clone.getName()
						 + " saved successful.", "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);

						resetClonePanel();
						owlModel.commitTransaction();
					}
			    }

			    else if (!concept1_exists && concept2_exists && lowerTreePanel_Clone.getIsNew())
				{
					boolean retval_1 = saveConcept(lowerTreePanel_Clone, lowerTreePanel_Clone.getName());
					if (!retval_1)
					{
						showError(lowerTreePanel_Clone);

						addToListenedToClses((OWLNamedClass) fromCls);
						addToListenedToClses((OWLNamedClass) toCls);

						owlModel.rollbackTransaction();
						return;
					}
					else
					{
						int ans = JOptionPane.showOptionDialog(this, "Concept " +  lowerTreePanel_Clone.getName()
						 + " saved successful.", "Confirmation",
						JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
						null, option_array, options[0]);

						resetClonePanel();
						owlModel.commitTransaction();
					}
			    }

			    else if (concept1_exists && concept2_exists)
                {
					boolean retval_1 = saveConcept(upperTreePanel_Clone);
					if (!retval_1)
					{
						showError(upperTreePanel_Clone);

						addToListenedToClses((OWLNamedClass) fromCls);
						addToListenedToClses((OWLNamedClass) toCls);

						owlModel.rollbackTransaction();
						return;
					}

                    boolean retval_2;
					if (cloned_cls)
						retval_2 = saveConcept(lowerTreePanel_Clone, lowerTreePanel_Clone.getName());
					else
						retval_2 = saveConcept(lowerTreePanel_Clone);

					if (!retval_2)
					{
						showError(lowerTreePanel_Clone.getName());

						addToListenedToClses((OWLNamedClass) fromCls);
						addToListenedToClses((OWLNamedClass) toCls);

						owlModel.rollbackTransaction();
						return;
					}

					int ans = JOptionPane.showOptionDialog(this, "Concepts saved successful.", "Confirmation",
					JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
					null, option_array, options[0]);

					resetClonePanel();
					owlModel.commitTransaction();

                }
			}
 			catch(Exception ex){
				owlModel.rollbackTransaction();
				OWLUI.handleError(owlModel, ex);

				addToListenedToClses((OWLNamedClass) fromCls);
				addToListenedToClses((OWLNamedClass) toCls);

			}
			//050906
			cloned_cls=false;

			addToListenedToClses((OWLNamedClass) fromCls);
			addToListenedToClses((OWLNamedClass) toCls);

		}


	}

    private Vector getPropertiesList() {

		Vector u = new Vector();
		u.add("Anatomic_Structure_is_Physical_Part_of");
		u.add("Biological_Process_Is_Part_of_Process");
		u.add("Conceptual_Part_Of");
		u.add("Gene_Product_is_Physical_Part_of");

		Vector v = new Vector();
		try {
			Collection allProperties = getOWLModel().getVisibleUserDefinedRDFProperties();
			//Collection selectableProperties = new ArrayList();
			for (Iterator it = allProperties.iterator(); it.hasNext();) {
				RDFProperty property = (RDFProperty) it.next();
				if (!property.isAnnotationProperty()) {
				   //selectableProperties.add(property);
				   //if (u.contains(property.getBrowserText()))
				   {
				   		//v.add(property.getBrowserText());
				   		v.add(property.getName());
			       }
				}
			}
			if (v.size() == 0)
			{
				System.out.println("WARNING: No applicable part of relationship available.");
			}


	    } catch (Exception e) {
			//Log.getLogger().log(Level.WARNING, "Exception caught", e);
			OWLUI.handleError(owlModel, e);
		}

        return v;
    }



	private void getSlots(Cls cls, int level,
		String tabString) {
		Collection ownslots = cls.getOwnSlots();
		if (ownslots != null) {
			Iterator j = ownslots.iterator();
			while(j.hasNext()){
				Slot slot = (Slot)j.next();
				Collection slotColl = cls.getOwnSlotValues(slot);
				if(slotColl == null || slotColl.isEmpty())
					continue;
				String slotName = slot.getBrowserText();
				if (slotName.equals(Model.Slot.NAME)) {
					continue;
				}
				else if (slotName.equals(Model.Slot.DIRECT_SUBCLASSES))
					continue;
				else if (slotName.equals(Model.Slot.DIRECT_INSTANCES))
					continue;
				else {
					getOwnslot(slotName, slotColl, tabString);
				}
			}
		}
	}


	private void getEquivalentClasses(RDFSClass cls, String tabString)
	{
		HashSet hset = new HashSet();
		Collection col = cls.getEquivalentClasses();
		for(Iterator iterator = col.iterator(); iterator.hasNext();)
		{
			RDFSClass rdfsCls = (RDFSClass) iterator.next();
			String value = rdfsCls.getBrowserText();
			if (!hset.contains(value))
			{
				classList.add("\t" + tabString + value);
			    hset.add(value);
			}
		}
	}

/*
	private void getAnonymousSuperclasses(RDFSClass cls, String tabString)
	{
		HashSet hset = new HashSet();
        Vector v = wrapper.getDirectSuperclassItems(cls, NCIConditionsTableModel.SET_SUPERCLASS);
        for (int i=0; i<v.size(); i++)
        {
			TreeItem item = (TreeItem) v.elementAt(i);
			String value = SUBCLASSOF + ": " + wrapper.getInternalName(item.getCls());
			if (!hset.contains(value))
			{
				classList.add("\t" + tabString + value + newline);
			    hset.add(value);
			}
		}
        v = wrapper.getDirectSuperclassItems(cls, NCIConditionsTableModel.SET_RESTRICTION);
        for (int i=0; i<v.size(); i++)
        {
			TreeItem item = (TreeItem) v.elementAt(i);
			String value = item.getCls().getBrowserText();
			if (!hset.contains(value))
			{
				classList.add("\t" + tabString + value + newline);
			    hset.add(value);
			}
		}


        v = wrapper.getDefinitionItems(cls, NCIConditionsTableModel.SET_SUPERCLASS);
        for (int i=0; i<v.size(); i++)
        {
			TreeItem item = (TreeItem) v.elementAt(i);
			String value = SUBCLASSOF + ": " + wrapper.getInternalName(item.getCls());
			if (!hset.contains(value))
			{
				classList.add("\t" + tabString + value + newline);
			    hset.add(value);
			}
		}
        v = wrapper.getDefinitionItems(cls, NCIConditionsTableModel.SET_RESTRICTION);
        for (int i=0; i<v.size(); i++)
        {
			TreeItem item = (TreeItem) v.elementAt(i);
			String value = item.getCls().getBrowserText();
 			if (!hset.contains(value))
			{
				classList.add("\t" + tabString + value + newline);
			    hset.add(value);
			}
		}
		//classList.add("\n");
	}
*/

/*
	private void getClsData(BufferedWriter buffWriter, Cls superCls, int level, int maxLevel, boolean withAttributes)
	{
		if (maxLevel != -1 && level > maxLevel) return;

		if (superCls == null)
		{
			return;
		}

		//if (nodelist.contains((Object) superCls.getName()))
		//{
			//return;
		//}


		String tabString="";
		for(int i=0;i<level;i++){
			tabString+="\t";
		}

		try{
			if (withAttributes)
			{
				classList.add(newline);
		    }

            String pt = null;
            try {
				pt = wrapper.getOwnSlotValue(superCls, getSlotName(PREFERRED_NAME));
			} catch (Exception ex) {

			}
			if (pt == null)
			{
				pt = superCls.getName();
			}

			//classList.add(tabString+superCls.getName()+newline);
			classList.add(tabString+pt+newline);

			reportTextArea.append(tabString+pt+newline);
			//reportTextArea.repaint();

			if (withAttributes)
			{
				// output slot data
				getSlots(superCls, level, tabString);
				//getEquivalentClasses((RDFSClass) superCls, tabString);
				getAnonymousSuperclasses((RDFSClass) superCls, tabString);
		    }

			writeToFile(buffWriter);
			classList.clear();

			//nodelist.add((Object) superCls.getName());

			Collection subclasses = wrapper.getDirectSubclasses(superCls);
			level++;
			if(subclasses != null && !subclasses.isEmpty()){
				Iterator i = subclasses.iterator();
				while(i.hasNext()) {
					Cls subcls = (Cls) i.next();
					if (subcls instanceof OWLNamedClass)
					{
						if(subcls.getBrowserText().compareTo(superCls.getBrowserText()) != 0) {
							getClsData(buffWriter, subcls, level, maxLevel, withAttributes);
						}
				    }
				}
			}
		}
		catch(Exception e) {
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
		}

	}
*/
/*

	private void getClsData(Cls superCls, int level) {

		if (superCls == null)
		{
			//System.out.println("superCls == null -- stop");
			return;
		}

		if (nodelist.contains((Object) superCls.getName()))
		{
			return;
		}

		String tabString="";

		for(int i=0;i<level;i++){
			tabString+="\t";
		}

		try{
			classList.add(newline);
			classList.add(tabString+superCls.getName()+newline);

			// output slot data
			getSlots(superCls, level, tabString);
			//getEquivalentClasses((RDFSClass) superCls, tabString);

			getAnonymousSuperclasses((RDFSClass) superCls, tabString);

			writeToTextArea();
			classList.clear();

			nodelist.add((Object) superCls.getName());

			Collection subclasses = wrapper.getDirectSubclasses(superCls);

			level++;
			if(subclasses != null && !subclasses.isEmpty()){
				Iterator i = subclasses.iterator();
				while(i.hasNext()) {
					Cls subcls = (Cls) i.next();
					if (subcls instanceof OWLNamedClass)
					{
						if(subcls.getBrowserText().compareTo(superCls.getBrowserText()) != 0) {
							getClsData(subcls, level);
						}
				    }
				}
			}
		}
		catch(Exception e) {
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
		}

	}
*/

    private void getOwnslot(String slotname, Collection values, String tabString)
    {
		if (values != null) {
			Iterator i = values.iterator();
			while(i.hasNext()) {
				Object obj = i.next();
				ValueType type = wrapper.getObjectValueType(obj);
				String entry = wrapper.convertObjecttoString(obj, type);
				String stringType = type.toString();
				try {
					if (slotname.compareTo(SUBCLASSOF) != 0)
					{
						classList.add("\t" + tabString + slotname + ": " + entry + newline);
				    }
				}
				catch(Exception e){
					Log.getLogger().log(Level.WARNING, "Exception caught", e);
				}
			}
		}
	}


    private void writeToFile(JTextArea textarea, File exportFile){

		try{
			BufferedWriter buffWriter = new BufferedWriter(new FileWriter(exportFile));
			String s = textarea.getText();
			buffWriter.write(s);
			buffWriter.close();
			String text = "Output file " + exportFile + " generated.";
			ModalDialog.showMessageDialog(null, text);
		}
		catch(Exception e){
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
			String text = "Unable to save to file " + exportFile + ".";
			ModalDialog.showMessageDialog(null, text);
		}
    }


    public boolean printable(String line)
    {
		return wrapper.printable(line);
	}

/*
    private void writeToTextArea(){

		try{
			int length=0;
			String conceptDetail="";
			String textAreaText="";
			if(classList != null)
			{
				length=classList.size();
			}

			for(int i=0;i<length;i++) {
				conceptDetail = (String) classList.elementAt(i);
				if (printable(conceptDetail))
				{
					textAreaText+=conceptDetail;
			    }
			}

			//reportTextArea.setText(textAreaText);
			reportTextArea.append(textAreaText);

		}
		catch(Exception e){
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
		}
    }
*/

   private void writeToFile(BufferedWriter buffWriter){

		try{
			int length=0;
			String conceptDetail="";
			String textAreaText="";
			if(classList != null)
			{
				length=classList.size();
			}

			for(int i=0;i<length;i++) {
				conceptDetail = (String) classList.elementAt(i);
				if (printable(conceptDetail))
				{
					//textAreaText+=conceptDetail;
					buffWriter.write(conceptDetail);
			    }
			}
			//buffWriter.write(textAreaText);

		}
		catch(Exception e){
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
		}
    }


   public void writeToFile(BufferedWriter buffWriter, Vector v){

		try{
			int length=0;
			String conceptDetail="";
			String textAreaText="";
			if(v != null)
			{
				length=v.size();
			}

			for(int i=0;i<length;i++) {
				conceptDetail = (String) v.elementAt(i);
				if (printable(conceptDetail))
				{
					//textAreaText+=conceptDetail;
					buffWriter.write(conceptDetail);
			    }
			}
		}
		catch(Exception e){
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
		}
    }


	private JPanel createSubRefTabbedPane() {
		JPanel panel = new JPanel(false);
		ImageIcon icon = new ImageIcon("images/middle.gif");

		//JTabbedPane tabbedPane = new JTabbedPane();
		tabbedPane_Preretire = new JTabbedPane();

        panel8 = createSubclassPanel();
		tabbedPane_Preretire.addTab("Subclasses", icon, panel8, "");

		panel9 = createReferenceClsesPanel();
		tabbedPane_Preretire.addTab("Referencing Classes", icon, panel9, "");

		panel.setLayout(new GridLayout(1, 1));
		panel.add(tabbedPane_Preretire);

		tabbedPane_Preretire.setSelectedIndex(0);

		return panel;
	}




	private Vector toVector(Collection c)
	{
		Vector v = new Vector();
        for (Iterator it = c.iterator(); it.hasNext();) {
            Object obj = (Object) it.next();
            v.add(obj);
        }
        return v;
	}



	private Component createSubclassPanel() {
		subClassCollection = new HashSet();
		Cls cls = null;
		if (droppedCls != null)
		{
			cls = (Cls) droppedCls;
		    subClassCollection = wrapper.getNamedSubclasses(cls);
		}

        JPanel panel = new JPanel();

		submouseListener = new SubMouseListener();
		submouseListener.getJList().addMouseListener(submouseListener);

		DefaultListModel listModel = submouseListener.getDefaultListModel();
		Iterator itr = subClassCollection.iterator();
		while (itr.hasNext())
		{
			//listModel.addElement(((Cls) itr.next()).getBrowserText());
			listModel.addElement(wrapper.getInternalName(((Cls) itr.next())));
		}

		JScrollPane scrollPane = new JScrollPane(submouseListener.getJList());
		panel.setLayout(new BorderLayout());
		panel.add(scrollPane, BorderLayout.CENTER);
		subclass_Pane = scrollPane;
		return panel;
	}

	private Component createReferenceClsesPanel() {
		referringClasses = new HashSet();
		Cls cls = null;
		if (droppedCls != null)
		{
			cls = (Cls) droppedCls;
		    referringClasses = wrapper.getReferringClasses(cls);
		}

        JPanel panel = new JPanel();

		referencemouseListener = new ReferenceMouseListener();
		referencemouseListener.getJList().addMouseListener(referencemouseListener);

		DefaultListModel listModel = referencemouseListener.getDefaultListModel();
		Iterator itr = referringClasses.iterator();
		while (itr.hasNext())
		{
			//listModel.addElement(((Cls) itr.next()).getBrowserText());
			listModel.addElement(wrapper.getInternalName(((Cls) itr.next())));
		}

		JScrollPane scrollPane = new JScrollPane(referencemouseListener.getJList());
		panel.setLayout(new BorderLayout());
		panel.add(scrollPane, BorderLayout.CENTER);
		reference_Pane = scrollPane;
		return panel;
	}

	private Component createReferenceClsesPanel(Collection referringClasses) {
        Cls cls = (Cls) droppedCls;
        JPanel panel = new JPanel();
		JScrollPane scrollPane = null;
		if (referringClasses == null)
		{
			referenceList = new JList(new Vector());
			scrollPane = new JScrollPane(referenceList);
		}
		else
		{
			referenceList = new JList(toVector(referringClasses));
			scrollPane = new JScrollPane(referenceList);
	    }
		panel.setLayout(new BorderLayout());
		panel.add(scrollPane, BorderLayout.CENTER);
		reference_Pane = scrollPane;
		return panel;
	}


    public void setDroppedCls(Cls cls)
    {
		this.droppedCls = cls;
		//System.out.println("Tab set droppedCls to " + droppedCls.getBrowserText());
        resetSubRefTabbedPane();
        panel5_upperPane.repaint();

	}


	private void resetSubRefTabbedPane() {
        if (droppedCls == null)
        {
			System.out.println("resetSubRefTabbedPane == null -- nothing done.");
			subVec.clear();
			restrictionVec.clear();
			DefaultListModel listModel = submouseListener.getDefaultListModel();
			listModel.clear();
			listModel = referencemouseListener.getDefaultListModel();
			listModel.clear();
			clearListenedToClses();
			return;
		}

        Cls cls = (Cls) droppedCls;

		subClassCollection = wrapper.getNamedSubclasses(cls);
		referringClasses = wrapper.getReferringClasses(cls);

		DefaultListModel listModel = submouseListener.getDefaultListModel();
		listModel.clear();

		Vector v = new Vector();
		Iterator itr = subClassCollection.iterator();
		while (itr.hasNext())
		{
			Cls nextcls = (Cls) itr.next();
			//v.addElement(nextcls.getBrowserText());
			v.addElement(wrapper.getInternalName(nextcls));
		}
		if (v.size() > 0)
		{
			try {
                if (wrapper.getSortUtility() == null)
                {
					System.out.println("QuickSort utility not available...");
				}
				else
				{
					if (v.size() > 1)
					{
						wrapper.getSortUtility().QuickSort(v, 0, v.size()-1);
					}
				}
				for (int i=0; i<v.size(); i++)
				{
					listModel.addElement((String) v.elementAt(i));
					subVec.add((String) v.elementAt(i));
				}
		    } catch (Exception ex) {
                Log.getLogger().log(Level.WARNING, "Exception caught", ex);
			}
	    }


		listModel = referencemouseListener.getDefaultListModel();
		listModel.clear();
		itr = referringClasses.iterator();
		v.clear();
		while (itr.hasNext())
		{
			Cls nextcls = (Cls) itr.next();
			//v.addElement(nextcls.getBrowserText());
			v.addElement(wrapper.getInternalName(nextcls));
		}

		if (v.size() > 0)
		{
			try {
                if (wrapper.getSortUtility() == null)
                {
					System.out.println("QuickSort utility not available...");
				}

				else
				{
					if (v.size() > 1)
					{
						wrapper.getSortUtility().QuickSort(v, 0, v.size()-1);
					}
				}
				for (int i=0; i<v.size(); i++)
				{
					listModel.addElement((String) v.elementAt(i));
					restrictionVec.add((String) v.elementAt(i));
				}
		    } catch (Exception ex) {
                Log.getLogger().log(Level.WARNING, "Exception caught", ex);
			}
	    }

		//updateSubRefClsPanel(null);
		initializeSubRefClsPanel();
		preretireButton_Preretire.setEnabled(true);
	}





/*
	public void updateNoteProperties(TreePanel panel, RDFSClass retiringCls)
    {
		JTree edit_tree = panel.getTree();

		String prop_name = getSlotName(EDITOR_NOTE);
		String prop_value = panel.getPropertyValue(prop_name);
		if (prop_value == null) return;

		boolean b1 = panel.deleteProperty(prop_name);
		int pos = prop_value.indexOf("|");
		if (pos != -1)
		{
			prop_value = prop_value.substring(pos+1);
		}
		TreeNode childnode = panel.addProperty(prop_name, prop_value, null);
		edit_tree.repaint();

		prop_name = getSlotName(DESIGNNOTE);
		prop_value = panel.getPropertyValue(prop_name);
		if (prop_value == null) return;
		boolean b2 = panel.deleteProperty(prop_name);
		pos = prop_value.indexOf("|");
		if (pos != -1)
		{
			prop_value = prop_value.substring(pos+1);
		}

		childnode = panel.addProperty(prop_name, prop_value, null);
		edit_tree.repaint();
    }
*/

	public void updateNoteProperties(TreePanel panel, RDFSClass retiringCls)
    {
		JTree edit_tree = panel.getTree();

		String prop_name = getSlotName(EDITOR_NOTE);
		Vector prop_values = panel.getPropertyValues(prop_name);
		if (prop_values != null)
		{
            for (int i=0; i<prop_values.size(); i++)
            {
				String prop_value = (String) prop_values.elementAt(i);
				int pos = prop_value.indexOf("|");
				if (pos != -1)
				{
					boolean b1 = panel.deleteProperty(prop_name);
					prop_value = prop_value.substring(pos+1);
					TreeNode childnode = panel.addProperty(prop_name, prop_value, null);
					edit_tree.repaint();
			    }
		    }
	    }

		prop_name = getSlotName(DESIGNNOTE);
		prop_values = panel.getPropertyValues(prop_name);
		if (prop_values != null)
		{
            for (int i=0; i<prop_values.size(); i++)
            {
				String prop_value = (String) prop_values.elementAt(i);
				int pos = prop_value.indexOf("|");
				if (pos != -1)
				{
					boolean b1 = panel.deleteProperty(prop_name);
					prop_value = prop_value.substring(pos+1);
					TreeNode childnode = panel.addProperty(prop_name, prop_value, null);
					edit_tree.repaint();
			    }
		    }
	    }
    }



	private void attributes2Properties(TreePanel panel, RDFSClass retiringCls)
    {
        //TreeNode child_node = null;
        JTree edit_tree = panel.getTree();

// parent --> OLD_PARENT
		//Collection col = wrapper.getDirectSuperclasses(retiringCls);


		Collection col = panel.getDirectSuperClses();
		Iterator iter = col.iterator();
		while (iter.hasNext())
		{
			Cls sup = (Cls) iter.next();
			if (sup instanceof OWLNamedClass)
			{
				if (wrapper.getInternalName(sup).compareTo(getClsName(PREMERGED_CONCEPTS)) != 0 &&
					wrapper.getInternalName(sup).compareTo(getClsName(PRERETIRED_CONCEPTS)) != 0)
				{
					TreeNode child_node = panel.addProperty(getSlotName(OLD_PARENT), wrapper.getInternalName(sup), sup);
				}
				boolean b = panel.deleteParent(sup);
		    }
			//System.out.println("attributes2Properties Remove parent " + wrapper.getInternalName(sup));
			edit_tree.repaint();
		}

/*
// children --> OLD_CHILD
		col = wrapper.getDirectSubclasses(retiringCls);
		iter = col.iterator();
		while (iter.hasNext())
		{
			String sub = (String) iter.next();
			boolean b = panel.deleteParent(sub);
			child_node = panel.addProperty(getSlotName(OLD_CHILD), sub);

			//System.out.println("Remove parent " + sub);
			edit_tree.repaint();
		}
*/

// roles -->  OLD_ROLE
/*
		Vector roles = panel.getRestrictionStrings();
		for (int i=0; i<roles.size(); i++)
		{
			String s = (String) roles.elementAt(i);
			StringTokenizer st = new StringTokenizer(s, "|");
			String name = st.nextToken();
			String value = st.nextToken();
			String modifier = st.nextToken();
			boolean b0 = panel.deleteRestriction(
					name, value, modifier);
			TreeNode child_node = panel.addProperty(getSlotName(OLD_ROLE), s, null);
		}
*/
		Vector roles = panel.getRestrictions();
		for (int i=0; i<roles.size(); i++)
		{
			RDFSClass r = (RDFSClass) roles.elementAt(i);
			boolean b0 = panel.deleteRestriction(r);
			TreeNode child_node = panel.addProperty(getSlotName(OLD_ROLE), r.getBrowserText(), null);
		}

		// OLD_STATE
		String stateString = panel.getPropertyValue("hasType");

		if (stateString != null && stateString.compareTo("primitive") == 0)
		{
			panel.deleteProperty("hasType", stateString, null);
			panel.addProperty("hasType", "defined", null);
			panel.addProperty(getSlotName(OLD_STATE), stateString, null);
		}

    }


 	public void properties2Attributes(TreePanel panel, RDFSClass retiringCls)
    {
// parent --> OLD_PARENT
/*
		Vector sups = panel.getPropertyValues(getSlotName(OLD_PARENT));
		for (int i=0; i<sups.size(); i++)
		{
			String sup = (String) sups.elementAt(i);
			//TreeNode child_node = panel.addParent(sup);

			Cls superCls = wrapper.getCls(sup);
			TreeNode child_node = panel.addParent(superCls);



			boolean b = panel.deleteProperty(getSlotName(OLD_PARENT), sup);
		}
*/
		Vector sups = panel.getPropertyObjectValues(getSlotName(OLD_PARENT));
		for (int i=0; i<sups.size(); i++)
		{
			Cls superCls = (Cls) sups.elementAt(i);
			if (superCls instanceof OWLNamedClass)
			{
				TreeNode child_node = panel.addParent(superCls);
				boolean b = panel.deleteProperty(getSlotName(OLD_PARENT), wrapper.getInternalName(superCls), superCls);
		    }
		}


		// children --> OLD_CHILD
		String retiringClsName = wrapper.getInternalName(retiringCls);

/*
		Vector subs = panel.getPropertyValues(getSlotName(OLD_CHILD));
		for (int i=0; i<subs.size(); i++)
		{
			String sub = (String) subs.elementAt(i);
			boolean b4 = panel.deleteProperty(getSlotName(OLD_CHILD), sub);

// need to move to save button

            Cls subCls = wrapper.getCls(sub);

            wrapper.addDirectSuperclass(subCls, retiringCls);
		}
*/
        //082606
		Vector subs = panel.getPropertyObjectValues(getSlotName(OLD_CHILD));
		for (int i=0; i<subs.size(); i++)
		{
			Cls subCls = (Cls) subs.elementAt(i);
			if (subCls instanceof OWLNamedClass)
			{
				boolean b4 = panel.deleteProperty(getSlotName(OLD_CHILD), wrapper.getInternalName(subCls), subCls);
				wrapper.addDirectSuperclass(subCls, retiringCls);
		    }
		}


// children --> OLD_SOURCE_ROLE
		Vector roles = panel.getPropertyValues(getSlotName(OLD_SOURCE_ROLE));
		for (int i=0; i<roles.size(); i++)
		{
			String role = (String) roles.elementAt(i);
			boolean b4 = panel.deleteProperty(getSlotName(OLD_SOURCE_ROLE), role, null);

// ???
// need to move to save button

			StringTokenizer st = new StringTokenizer(role, "|");
			String name = st.nextToken();
			String value = st.nextToken();
			String modifier = st.nextToken();

			// recover the inbound role
			OWLNamedClass owl_cls = wrapper.getOWLCls(value);
			wrapper.addRestriction(owl_cls, name, retiringClsName, modifier);

		}



// roles -->  OLD_ROLE
		roles = panel.getPropertyValues(getSlotName(OLD_ROLE));
		for (int i=0; i<roles.size(); i++)
		{
			String s = (String) roles.elementAt(i);
			/*
			StringTokenizer st = new StringTokenizer(s, "|");
			String name = st.nextToken();
			String value = st.nextToken();
			String modifier = st.nextToken();
			TreeNode node = panel.addRestriction(
					name, value, modifier);
			*/

			String parsableText = s;
			RDFSClass aClass = wrapper.parsableText2RDFSClass(s);
			boolean isDefining = false;

			TreeNode node = panel.addRestriction(aClass, isDefining);

			//TreeNode node = panel.addRestriction(item.getCls(), item.getIsDefining());

			boolean b = panel.deleteProperty(getSlotName(OLD_ROLE), s, null);
		}

// OLD_STATE
		String oldState = panel.getPropertyValue(getSlotName(OLD_STATE));
		if (oldState != null)
		{
			boolean b = panel.deleteProperty(getSlotName(OLD_STATE), oldState, null);
            b = panel.deleteProperty("hasType", "defined", null);
            TreeNode node = panel.addProperty("hasType", "primitive", null);

	    }

// roles -->  OLD_SOURCE_ROLE
		roles = panel.getPropertyValues(getSlotName(OLD_SOURCE_ROLE));
		for (int i=0; i<roles.size(); i++)
		{
			String s = (String) roles.elementAt(i);
			boolean b = panel.deleteProperty(getSlotName(OLD_SOURCE_ROLE), s, null);
		}
	}



	public OWLWrapper getOWLWrapper()
	{
		return wrapper;
	}




    class SubMouseListener extends MouseAdapter
    {
//		DefaultListModel subclassListModel;
//		JList subClsList;

		public SubMouseListener()
		{
			subclassListModel = new DefaultListModel();
			subClsList = new JList(subclassListModel);
			DefaultListCellRenderer renderer0 = new DefaultListCellRenderer();
			subClsList.setCellRenderer(renderer0);
			subClsList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		}

		public void mouseClicked(MouseEvent e) {
			if (subclassListModel.isEmpty()) return;

			int selectIndex = subClsList.locationToIndex(e.getPoint());
			if (selectIndex >= 0)
			{
			    String clsName = (String) subclassListModel.getElementAt(selectIndex);
				Cls selectedCls = getOWLModel().getCls(clsName);
				updateSubRefClsPanel((Instance) selectedCls);
			}

			//chgfont(subclassListModel, selectIndex, 0);
			//chgCls(subclassListModel, selectIndex);
		}

		public DefaultListModel getDefaultListModel(){
			return subclassListModel;
		}

		public JList getJList(){
			return subClsList;
		}
	}

    class ReferenceMouseListener extends MouseAdapter
    {
		public ReferenceMouseListener()
		{
     		referenceListModel = new DefaultListModel();
			referenceList = new JList(referenceListModel);
			DefaultListCellRenderer renderer1 = new DefaultListCellRenderer();
			referenceList.setCellRenderer(renderer1);
		}

		public void mouseClicked(MouseEvent e) {
			if (referenceListModel.isEmpty()) return;

			int selectIndex = referenceList.locationToIndex(e.getPoint());
			if (selectIndex >= 0)
			{
			    String clsName = (String) referenceListModel.getElementAt(selectIndex);
				Cls selectedCls = getOWLModel().getCls(clsName);
				updateSubRefClsPanel((Instance) selectedCls);
			}
		}

		public DefaultListModel getDefaultListModel(){
			return referenceListModel;
		}

		public JList getJList(){
			return referenceList;
		}
    }

    private void deleteNoteProperties(TreePanel panel)
    {
		String prop_name = getSlotName(EDITOR_NOTE);
		String editornote = panel.getPropertyValue(prop_name);
		if (panel.deleteProperty(prop_name))
		{
			boolean b = panel.deleteProperty(prop_name);
		}

		prop_name = getSlotName(DESIGNNOTE);
		String designnote = panel.getPropertyValue(prop_name);
		if (panel.deleteProperty(prop_name))
		{
			boolean b = panel.deleteProperty(prop_name);
		}
	}

    public void showError(String conceptname)
    {
		Object[] option_array = {"OK"};
/*
		String errorMessage = "Unable to save " + conceptname + " -- " +
				   editfilter.getErrorMessage();
*/
		String errorMessage = "Unable to save " + conceptname;
		if (editfilter.getErrorMessage().compareTo("") != 0)
		{
			errorMessage = errorMessage + " -- " +
				   editfilter.getErrorMessage();
		}

		int ans = JOptionPane.showOptionDialog(this, errorMessage, "Confirmation",
		JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
		null, option_array, option_array[0]);
		return;
    }

    private void showError(TreePanel panel)
    {
		Object[] option_array = {"OK"};
		String conceptname = panel.getName();

        String errorMessage = "Unable to save " + conceptname;
		if (editfilter.getErrorMessage().compareTo("") != 0)
		{
			errorMessage = errorMessage + " -- " +
				   editfilter.getErrorMessage();
		}
/*
		String errorMessage = "Unable to save " + conceptname + " -- " +
				   editfilter.getErrorMessage();
*/
		int ans = JOptionPane.showOptionDialog(this, errorMessage, "Confirmation",
		JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
		null, option_array, option_array[0]);
		return;
    }



	public void populatePremergePanel(String name){

		String slotname_src = getSlotName(MERGE_SOURCE);
		String slotname_target = getSlotName(MERGE_TARGET);

		TreeItem item = new TreeItem();
		item.setType(TreeItem.TYPE_CONCEPT);
		item.setName(name);
		item.setNameValue(name);

		TreeNode dmtn = new TreeNode(item, -1);

		Instance selectedInstance = (Instance) owlModel.getCls(name);

		OWLNamedClass cls = (OWLNamedClass) selectedInstance;
        if (wrapper.hasAnnotationProperty((OWLNamedClass) cls, slotname_target))
		{
			lowerTreePanel_Premerge = new TreePanel(this, selectedInstance,  dmtn, null);
			lowerTreePanel_Premerge.setCanDrop(false);
			panel3_lowerPane.setViewportView(lowerTreePanel_Premerge);

			saveButton_Premerge.setEnabled(false);
			unmergeButton_Premerge.setEnabled(true);
			premergeButton_Premerge.setEnabled(false);

			can_undo_premerge = true;

	    }
        else if (wrapper.hasAnnotationProperty((OWLNamedClass) cls, slotname_src))
		{
			upperTreePanel_Premerge = new TreePanel(this, selectedInstance,  dmtn, null);
			upperTreePanel_Premerge.setCanDrop(false);
			panel3_upperPane.setViewportView(upperTreePanel_Premerge);

			saveButton_Premerge.setEnabled(false);
			unmergeButton_Premerge.setEnabled(true);
			premergeButton_Premerge.setEnabled(false);

			can_undo_premerge = true;

	    }

	    else
	    {
			saveButton_Premerge.setEnabled(false);
			unmergeButton_Premerge.setEnabled(true);
			premergeButton_Premerge.setEnabled(false);
	    }

		clearButton_Premerge.setEnabled(true);


	}



	//042806
	private Component getClonePanel(){

		JPanel panel = new JPanel(false);
		panel.setLayout(new BorderLayout());

		panel11_upperPane=new JScrollPane();

		upperTreePanel_Clone = new CloneTreePanel(this, null, getOWLModel());
		upperTreePanel_Clone.setType(TreePanel.TYPE_CLONE);
		upperTreePanel_Clone.setSubType(TreePanel.TYPE_CLONE_UPPER);

		lowerTreePanel_Clone = new CloneTreePanel(this, null, getOWLModel());
		lowerTreePanel_Clone.setSubType(TreePanel.TYPE_CLONE_LOWER);

		//lowerTreePanel_Clone.setCanDrop(false);

		panel11_upperPane.setViewportView(upperTreePanel_Clone);

		panel11_lowerPane=new JScrollPane();
		panel11_lowerPane.setViewportView(lowerTreePanel_Clone);

		clonePane = new JSplitPane(JSplitPane.VERTICAL_SPLIT,panel11_upperPane,panel11_lowerPane);
		clonePane.setDividerLocation(280);
		clonePane.setOneTouchExpandable(true);

		panel.add(clonePane,BorderLayout.CENTER);

		JPanel buttonPanel = new JPanel();

		//newButton_Clone = new JButton("New");
		//newButton_Clone.addActionListener(this);

		cloneButton_Clone = new JButton("Clone");
		cloneButton_Clone.addActionListener(this);

		saveButton_Clone = new JButton("Save");
		saveButton_Clone.addActionListener(this);

		clearButton_Clone = new JButton("Cancel");
		clearButton_Clone.addActionListener(this);

        //buttonPanel.add(newButton_Clone);
		buttonPanel.add(cloneButton_Clone);
		buttonPanel.add(saveButton_Clone);
		buttonPanel.add(clearButton_Clone);
		panel.add(buttonPanel, BorderLayout.SOUTH);

		cloneButton_Clone.setEnabled(false);
		saveButton_Clone.setEnabled(false);
		clearButton_Clone.setEnabled(false);

		return panel;
	}


	public TreeItem getCopyItem(){
		return copied_item;
	}


	public void setCopyItem(TreeItem copied_item){
		this.copied_item = copied_item;
	}



	private boolean populateClonePanel(String name, String pt){

		String code = generator.getCode();
		if (owl_format == NCIEditTab.BY_CODE)
		{
			if (code == null || code.compareTo("0") == 0)
			return false;
		}

		TreeItem item = new TreeItem();
		item.setType(TreeItem.TYPE_CONCEPT);

		item.setName(code);
		item.setNameValue(code);
		if (wrapper.codeSlotExists)
		{
			item.setName(name);
			item.setNameValue(name);
		}

		// get final state from upperTreePanel_Split
		Vector v = upperTreePanel_Clone.getFinalState();

		v = convertNCIPT2NCISY(v);

		v = cloneStateVector(v);

		Vector w = new Vector();

        int num_label = 0;
        HashSet hset = new HashSet();

		for (int i=0; i<v.size(); i++)
		{
			TreeItem treeitem = (TreeItem) v.elementAt(i);
			if (treeitem.getName().compareTo(getSlotName(CODE)) == 0)
			{
				TreeItem clone = treeitem.cloneTreeItem();
				clone.setValue(code);
				clone.setNameValue(treeitem.getName() + ":" + code);
				w.add(clone);
			}
			else if (treeitem.getName().compareTo(getSlotName(PREFERRED_NAME)) == 0)
			{
				String prop_name = getSlotName(FULL_SYN);
				if (prop_name != null)
				{
					String prop_value = treeitem.getValue();
					String newvalue = prop_value + "|SY|NCI";
					newvalue = ComplexPropertyParser.pipeDelim2XML(newvalue);
					String hash_value = prop_name + ":" + newvalue;

					if (!hset.contains(hash_value))
					{
						TreeItem clone = treeitem.cloneTreeItem();
						clone.setName(prop_name);
						clone.setValue(newvalue);
						clone.setNameValue(prop_name + ":" + newvalue);
						w.add(clone);
						hset.add(hash_value);
					}
			    }
			}
			else if (treeitem.getName().compareTo(getSlotName(FULL_SYN)) == 0)
			{
				String t = treeitem.getValue();
                String hash_value = treeitem.getName() + ":" + t;
                if (!hset.contains(hash_value))
                {
					TreeItem clone = treeitem.cloneTreeItem();
					clone.setValue(t);
					clone.setNameValue(treeitem.getName() + ":" + t);
					w.add(clone);
					hset.add(treeitem.getName() + ":" + t);
			    }
			}

			else if (treeitem.getName().compareTo("rdfs:label") != 0)
			{
				w.add(treeitem.cloneTreeItem());
			}
		}
		hset.clear();

		TreeNode root = new TreeNode(item, -1);
        lowerTreePanel_Clone = new CloneTreePanel(this, selectedInstance,  root, code, w);
        boolean retval = addToListenedToClses((OWLNamedClass) selectedInstance);


		//042006
		splitPane.setBottomComponent(panel2_lowerPane);
		splitPane.updateUI();

		lowerTreePanel_Clone.addProperty("rdfs:label", pt, null);
		lowerTreePanel_Clone.addProperty(getSlotName(PREFERRED_NAME), pt, null);


		String prop_name = getSlotName(FULL_SYN);
		String prop_value = pt + "|PT|NCI";
		prop_value = ComplexPropertyParser.pipeDelim2XML(prop_value);

		if (prop_name != null)
		{
			lowerTreePanel_Clone.addProperty(prop_name, prop_value, null);
	    }

        // need to copy subclasses when the Save button is pressed
        // subclasses are not visible

		lowerTreePanel_Clone.setCanDrop(false);
		upperTreePanel_Clone.setCanDrop(false);
		panel11_lowerPane.setViewportView(lowerTreePanel_Clone);

        //newButton_Clone.setEnabled(false);
		cloneButton_Clone.setEnabled(false);
		saveButton_Clone.setEnabled(true);
		clearButton_Clone.setEnabled(true);

		return true;

	}


	private void showNewConcept(String label, String code){

		String newname = label;
        Vector v = new Vector();

		TreeItem item = new TreeItem();
		item.setType(TreeItem.TYPE_CONCEPT);
		item.setName(newname);
		item.setNameValue(newname);
		v.add(item);

		TreeNode root = new TreeNode(item, -1);

        int num_pt = 0;
        int num_pt_full_syn = 0;
        int num_label = 0;

		item = new TreeItem();
		item.setType(TreeItem.TYPE_PROPERTY);
		item.setName(getSlotName(CODE));
		item.setValue(code);
		item.setNameValue(getSlotName(CODE) + ": " + code);
		v.add(item);

		item = new TreeItem();
		item.setType(TreeItem.TYPE_PROPERTY);
		item.setName(getSlotName(PREFERRED_NAME));
		item.setValue(newname);
		item.setNameValue(getSlotName(PREFERRED_NAME) + ": " + newname);
		v.add(item);

		String prop_name = getSlotName(FULL_SYN);
		if (prop_name != null)
		{
			item = new TreeItem();
			item.setType(TreeItem.TYPE_PROPERTY);
			item.setName(prop_name);
			String value = newname + "|PT|NCI";
			item.setValue(value);
			item.setNameValue(getSlotName(FULL_SYN) + ": " + value);
			v.add(item);
	    }

		item = new TreeItem();
		item.setType(TreeItem.TYPE_PROPERTY);
		item.setName("rdfs:label");
		item.setValue(newname);
		item.setNameValue("rdfs:label" + ": " + newname);
		v.add(item);

		Cls sup = (Cls) selectedInstance;
		if (sup != null)
		{
			item = new TreeItem();
			item.setType(TreeItem.TYPE_PARENT);
			item.setName(SUBCLASSOF);
			//item.setValue(sup.getBrowserText());
			//item.setNameValue(SUBCLASSOF + ": " + sup.getBrowserText());
			item.setValue(wrapper.getInternalName(sup));
			item.setNameValue(SUBCLASSOF + ": " + wrapper.getInternalName(sup));
			item.setCls((RDFSClass) sup);

			v.add(item);
	    }

		lowerTreePanel_Clone = new CloneTreePanel(this, selectedInstance, root, code, v);
		lowerTreePanel_Clone.setName(label);
		lowerTreePanel_Clone.setCode(code);
		lowerTreePanel_Clone.setIsNew(true);

		//042006
		clonePane.setBottomComponent(panel11_lowerPane);
		clonePane.updateUI();

		lowerTreePanel_Clone.setCanDrop(false);
		upperTreePanel_Clone.setCanDrop(false);
		panel11_lowerPane.setViewportView(lowerTreePanel_Clone);

		cloneButton_Clone.setEnabled(false);
		saveButton_Clone.setEnabled(true);
		clearButton_Clone.setEnabled(true);

	}

	private void resetClonePanel()
	{
		upperTreePanel_Clone = new CloneTreePanel(this, null, getOWLModel());
		upperTreePanel_Clone.setType(CloneTreePanel.TYPE_CLONE);
		lowerTreePanel_Clone = new CloneTreePanel(this, null, getOWLModel());
		lowerTreePanel_Clone.setType(CloneTreePanel.TYPE_CLONE);
		//011207
		upperTreePanel_Clone.setSubType(TreePanel.TYPE_CLONE_UPPER);
		lowerTreePanel_Clone.setSubType(TreePanel.TYPE_CLONE_LOWER);


		panel11_upperPane.setViewportView(upperTreePanel_Clone);
		panel11_lowerPane.setViewportView(lowerTreePanel_Clone);

		cloneButton_Clone.setEnabled(false);
		saveButton_Clone.setEnabled(false);
		clearButton_Clone.setEnabled(false);
		clearListenedToClses();

		resetSplitPanel();
	}


	//050306
	public int getNumClses()
	{
		int num=0;
		if (upperTreePanel_Clone.containsCls())
			num++;

		if (lowerTreePanel_Clone.containsCls())
			num++;

		return num;
	}


	public Instance getSelectedInstance()
	{
		return selectedInstance;
	}

	public EditPanel getEditPanel()
	{
		return panel12;
	}

	public NCIClsesPanel getClassPanel()
	{
		return _clsesPanel;
	}

    private TreePanel createTreePanel()
    {
		TreePanel panel = new TreePanel(this, null,  getOWLModel());
		return panel;
	}

    public EditPanel createEditPanel()
    {
		EditPanel panel = new EditPanel(this, getSelectedInstance(), getOWLModel());
		return panel;
	}

	//120506
	public EditPanel createEditPanel(Cls cls)
	{
		EditPanel panel = new EditPanel(this, cls, getOWLModel(), "AdvancedQuery");
		return panel;
	}

    public void ensureClassSelected(Cls cls) {
        if (cls != null && !_clsesPanel.getSelection().contains(cls)) {
            _clsesPanel.setSelectedCls(cls);
            //_clsesPanel.setSelectedClass(cls, true);
        }
    }

    private void clearListenedToClses()
    {
		if (eventListener == null) return;
		eventListener.clearListenedToClses();
	}


	public boolean addToListenedToClses(OWLNamedClass cls)
    {
		if (eventListener == null) return false;
		return eventListener.addToListenedToClses(cls);
	}


    public boolean removeFromListenedToClses(OWLNamedClass cls)
    {
		if (eventListener == null) return false;
		return eventListener.removeFromListenedToClses(cls);
	}

	public int getCurrentTab()
	{
		return curr_selection;
	}

	public boolean logEditEvent(String action,
	    String name1, String code1, String name2, String code2,
	    Vector codes)
	{
		/*
        if (edithistory == null) return false;
        return logEditEvent(action, name1, code1, name2, code2, codes);
        */
        // no longer need to support history logging
        return true;
	}

	public void showConceptExistError(String name)
	{
		Object[] option_array = {"OK"};
		String errorMessage = "Concept " + name + " already exists.";

		int ans = JOptionPane.showOptionDialog(this, errorMessage, "Confirmation",
		JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
		null, option_array, option_array[0]);
	}

	public void showInvalidConceptNameError(String name)
	{
		Object[] option_array = {"OK"};
		String errorMessage = "Concept " + name + " is invalid.";

		int ans = JOptionPane.showOptionDialog(this, errorMessage, "Confirmation",
		JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
		null, option_array, option_array[0]);
	}


	public void showDataError(String errorMessage)
	{
		Object[] option_array = {"OK"};
		int ans = JOptionPane.showOptionDialog(this, errorMessage, "Confirmation",
		JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
		null, option_array, option_array[0]);
	}

	public String getSlotName(String name)
	{
		if (name2SlotName.containsKey(name))
		{
			return (String) name2SlotName.get(name);
		}
		return name;
	}

	public String getClsName(String name)
	{
		if (name2ClsName.containsKey(name))
		{
			String retstr = (String) name2ClsName.get(name);
			return (String) name2ClsName.get(name);
		}

		return name;
	}

	public String getBrowserText(Cls cls)
	{
		return wrapper.getInternalName(cls);
	}

	public String getExternalName(String nm)
	{
		return (String) externalNameMap.get((String) nm);
	}


    private boolean saveConcept(TreePanel panel)
    {
		return saveConcept(panel, null);
	}


    private boolean saveConcept(TreePanel panel, boolean upper)
    {
		return saveConcept(panel, null, upper);
	}


    private boolean saveConcept(TreePanel panel, String clsName)
    {
		return saveConcept(panel, clsName, false); //lower
	}


    private boolean saveConcept(TreePanel panel, String clsName, boolean upper)
    {
		try {
			if (clsName == null || clsName.compareTo("") == 0)
			{
				Cls cls = panel.getInstance();
				if (cls == null)
				{
					System.out.println("Error saving concept data -- cls is null");
					return false;
				}

				boolean retval = datahandler.processData(cls, panel, upper);
				if (!retval)
				{
					return false;
				}
				return true;
			}
			else
			{
				return datahandler.createCls(clsName, panel); // rdf:ID known
			}

		} catch (Exception e) {
			//Log.getLogger().log(Level.WARNING, "Exception caught", e);
			OWLUI.handleError(owlModel, e);
		}
		return false;
	}


	private boolean isEquivalentToPT(String termtype)
	{
		if (termtype.equals("PT")||termtype.equals("HD")||termtype.equals("AQ")) return true;
		return false;
	}

    private Vector convertNCIPT2NCISY(Vector v)
    {
		Vector u = new Vector();

		String prop_name = getSlotName(FULL_SYN);
		for (int i=0; i<v.size(); i++)
		{
			TreeItem item0 = (TreeItem) v.elementAt(i);
			TreeItem item = item0.cloneTreeItem();


			//if (item.getName().compareTo("FULL_SYN") == 0)
			if (item.getName().compareTo(prop_name) == 0)
			{
				HashMap hmap = ComplexPropertyParser.parseXML(ComplexPropertyParser.pipeDelim2XML(item.getValue()));
				String term_name = (String) hmap.get("term-name");
				String term_group = (String) hmap.get("term-group");
				String term_source = (String) hmap.get("term-source");
   			    String term_code = (String) hmap.get("source-code");

				if (term_source == null) // default
				{
				    term_source = "NCI";
				}
				if (term_group == null) // default
				{
				    term_group = "SY";
				}
                if (term_source.compareTo("NCI") == 0)
                {
					if (isEquivalentToPT(term_group))
					{
						term_group = "SY";
					}
				}
				String t = "";
				t = t + "<term-name>" + term_name + "</term-name>";
				t = t + "<term-group>" + term_group + "</term-group>";
				t = t + "<term-source>" + term_source + "</term-source>";
				if (term_code != null)
				{
					t = t + "<source-code>" + term_code + "</source-code>";
				}
				item.setValue(t);
				item.setNameValue(prop_name + ": " + t);

			}
			u.add(item);
		}

		return u;
	}


    public Vector getCustomizedAnnotationKeys()
    {
		return customizedAnnotationKeys;
	}


	public HashMap loadUIConfigurationData(String filename)
	{
		HashMap hmap = new HashMap();
		if (filename == null) return hmap;
		String s;
		int knt = 0;
		BufferedReader inFile = null;
		try
		{
			inFile = new BufferedReader(new FileReader(filename));
			// 010307
			customizedAnnotationKeys = new Vector();

			while ((s = inFile.readLine()) != null)
			{
				String annotation_name = s;
				String tagname_data = inFile.readLine();
				String component_type_data = inFile.readLine();
				String combobox_data = inFile.readLine();
				String default_value_data = inFile.readLine();
				CustomizedAnnotationData cad = new CustomizedAnnotationData(
				                                   annotation_name,
				                                   tagname_data,
				                                   component_type_data,
				                                   combobox_data,
				                                   default_value_data);
				hmap.put(annotation_name, cad);

				//010307
				String[] tagnames = cad.get_component_names();
				customizedAnnotationKeys.add(tagnames[0]);

			}
			inFile.close();
		}
		catch(Exception e)
		{
			//System.err.println(e);
			System.out.println("File " + filename + " not found.");
		}
		return hmap;
    }


    public CustomizedAnnotationData getCustomizedAnnotationData(String annotation_name)
    {
		if (CustomizedAnnotationData_Map.containsKey(annotation_name))
		{
			return (CustomizedAnnotationData) CustomizedAnnotationData_Map.get(annotation_name);
		}
		return null;
	}



	public Cls getSelectedCls()
	{
        Collection selection = _clsesPanel.getSelection();
        //Instance selectedInstance = null;
        Instance selectedInstance = null;
        if (selection.size() == 1) {
            selectedInstance = (Instance) CollectionUtilities.getFirstItem(selection);
            return (Cls) selectedInstance;
		}
        return null;
	}

	private int getTreeSize(Cls cls)
	{
		int num_subs = 0;
		Collection subclasses = wrapper.getDirectSubclasses(cls);
		Iterator i = subclasses.iterator();
		while(i.hasNext()) {
			Cls subcls = (Cls) i.next();
			if (subcls instanceof OWLNamedClass)
			{
				num_subs = num_subs + getTreeSize(subcls);
			}
		}
		return num_subs+1;
	}

	public JTextArea getReportTextArea()
	{
		return reportTextArea;
	}


    public void showProgressBar(boolean aFlag)
    {
		initializeProgressBar();
		progress.setVisible(aFlag);
		progress.setString("Calculating tree size ...");
	}


    public void initializeProgressBar()
    {
		iCtr_progress = 0;
		progress.setValue( iCtr_progress );
		Rectangle progressRect = progress.getBounds();
		progressRect.x = 0;
		progressRect.y = 0;
		progress.paintImmediately( progressRect );
	}


    public void updateProgressBar()
    {
		// Update the progress indicator and label
		iCtr_progress++;

		if (iCtr_progress%10 != 0) return;

		if (iCtr_progress > 100)
		{
			iCtr_progress = 0;
		}
		progress.setValue( iCtr_progress );
		Rectangle progressRect = progress.getBounds();
		progressRect.x = iCtr_progress-10;
		if (progressRect.x < 0) progressRect.x = 0;
		progressRect.y = 0;

		progress.paintImmediately( progressRect );
	}

	public void initializeEditPanel()
	{
		ensureClassSelected(owlModel.getOWLThingClass());
		transmitSelection();
	}

}

