package gov.nih.nci.protegex.edit;

import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.ui.FrameRenderer;
import edu.stanford.smi.protege.util.PopupMenuMouseListener;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.model.triplestore.Triple;
import edu.stanford.smi.protegex.owl.model.triplestore.impl.DefaultTriple;
import edu.stanford.smi.protegex.owl.ui.ResourceRenderer;
import edu.stanford.smi.protegex.owl.ui.actions.CopyPropertyValueAction;
import edu.stanford.smi.protegex.owl.ui.actions.DeleteAllPropertyValuesAction;
import edu.stanford.smi.protegex.owl.ui.actions.DeleteMatchingPropertyValuesAction;
import edu.stanford.smi.protegex.owl.ui.actions.triple.TripleAction;
import edu.stanford.smi.protegex.owl.ui.actions.triple.TripleActionManager;
import edu.stanford.smi.protegex.owl.ui.actions.triple.TripleActionSwingAction;
import edu.stanford.smi.protegex.owl.ui.components.ComponentUtil;
import edu.stanford.smi.protegex.owl.ui.results.TripleDisplay;
import edu.stanford.smi.protegex.owl.ui.widget.OWLUI;
import edu.stanford.smi.protegex.owl.ui.components.annotations.*;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import java.awt.*;
import java.util.*;

/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class NCIFULLSYNTable extends JTable {

    private NCIFULLSYNTableModel tableModel;
    private JTextField textField;
    private Project project;

	private JComboBox groupComboBox, sourceComboBox, langComboBox;
	private Vector group_vec, source_vec;

    public NCIFULLSYNTable(Project project, NCIFULLSYNTableModel model,
                  final String partialActionName) {

        super(model);
        this.project = project;
        this.tableModel = model;

        model.setTable(this);

		String[] groups = { "AB", "AD", "AQ", "BR", "CA2", "CA3", "CN", "CNU",
							"CS", "DN", "FB", "HD", "PT", "SN", "SY"};

		String[] sources = { "BioCarta", "CADSR", "CDC", "CDISC", "CTCAE", "CTRM", "DCP", "DTP",
							"FDA", "KEGG", "NCI", "NCI-GLOSS", "SEER"};

		String[] def_sources = { "NCI-GLOSS", "CADSR", "CTCAE", "FDA", "NCI"};

		group_vec = new Vector();
		source_vec = new Vector();
		for (int i=0; i<groups.length; i++)
			group_vec.add(groups[i]);
		for (int i=0; i<sources.length; i++)
			source_vec.add(sources[i]);

        JTextField textField = new JTextField();
        OWLUI.addCopyPastePopup(textField);
        setDefaultEditor(Object.class, new DefaultCellEditor(textField));

        TableColumn termnameColumn = getColumnModel().getColumn(NCIFULLSYNTableModel.TERM_NAME);
        TableColumn termgroupColumn = getColumnModel().getColumn(NCIFULLSYNTableModel.TERM_GROUP);
		TableColumn termsourceColumn = getColumnModel().getColumn(NCIFULLSYNTableModel.TERM_SOURCE);
        TableColumn sourcecodeColumn = getColumnModel().getColumn(NCIFULLSYNTableModel.TERM_SOURCE_CODE);
        TableColumn langColumn = getColumnModel().getColumn(NCIFULLSYNTableModel.LANGUAGE);

        NCIFULLSYNValueRenderer renderer = new NCIFULLSYNValueRenderer();

        OWLModel owlModel = (OWLModel) project.getKnowledgeBase();

        termnameColumn.setCellRenderer(renderer);
        termnameColumn.setPreferredWidth(300);

		termgroupColumn.setPreferredWidth(60);
		termgroupColumn.setCellRenderer(renderer);
		groupComboBox = new JComboBox(group_vec);
		groupComboBox.setRenderer(new ResourceRenderer());
		termgroupColumn.setCellEditor(new DefaultCellEditor(groupComboBox));

		termsourceColumn.setCellRenderer(renderer);
		termsourceColumn.setPreferredWidth(60);
		sourceComboBox = new JComboBox(source_vec);
		sourceComboBox.setRenderer(new ResourceRenderer());
		termsourceColumn.setCellEditor(new DefaultCellEditor(sourceComboBox));

        sourcecodeColumn.setCellRenderer(renderer);
        sourcecodeColumn.setPreferredWidth(80);
        //sourcecodeColumn.setCellEditor(new AnnotationsLangEditor(owlModel, this));

		//langComboBox = ComponentUtil.createLanguageComboBox(owlModel, "en");
		//langColumn.setCellEditor(new DefaultCellEditor(langComboBox));
		langColumn.setCellRenderer(renderer);
		langColumn.setCellEditor(new NCIAnnotationsLangEditor(owlModel, model.getEditPanel(), this));

        setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
        getTableHeader().setReorderingAllowed(false);
        setRowMargin(0);
        setIntercellSpacing(new Dimension(0, 0));
        setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        setRowHeight(new JTextField().getPreferredSize().height);
        setGridColor(Color.LIGHT_GRAY);
        setShowGrid(true);
        setIntercellSpacing(new Dimension(1, 1));

    }



    private void initColumnSizes(JTable table) {
        NCIFULLSYNTableModel model = (NCIFULLSYNTableModel)table.getModel();
        TableColumn column = null;
        TableCellRenderer headerRenderer =
            table.getTableHeader().getDefaultRenderer();

        for (int i = 0; i < 5; i++) {
            column = table.getColumnModel().getColumn(i);
        	NCIFULLSYNValueRenderer renderer = new NCIFULLSYNValueRenderer();
        	column.setCellRenderer(renderer);
        }
    }

    public void setUpComboColumn(JTable table, TableColumn groupColumn, Object[] items) {
        //Set up the editor for the sport cells.
		Vector item_vec = new Vector();
		for (int i=0; i<items.length; i++)
			item_vec.add(items[i]);

        JComboBox comboBox = new JComboBox(item_vec);
        groupColumn.setCellEditor(new DefaultCellEditor(comboBox));

        //Set up tool tips for the sport cells.
        DefaultTableCellRenderer renderer =
                new DefaultTableCellRenderer();
        //renderer.setToolTipText("Click for combo box");
        groupColumn.setCellRenderer(renderer);
    }

    public NCIFULLSYNTableModel getTableModel() {
        return tableModel;
    }
}
