package gov.nih.nci.protegex.edit;

import edu.stanford.smi.protegex.owl.ui.components.triples.*;

import com.hp.hpl.jena.datatypes.xsd.impl.XMLLiteralType;
import edu.stanford.smi.protege.ui.FrameComparator;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.model.event.PropertyValueAdapter;
import edu.stanford.smi.protegex.owl.model.event.PropertyValueListener;
import edu.stanford.smi.protegex.owl.model.impl.DefaultRDFSLiteral;
import edu.stanford.smi.protegex.owl.model.impl.XMLSchemaDatatypes;
import edu.stanford.smi.protegex.owl.ui.ProtegeUI;
import edu.stanford.smi.protegex.owl.ui.metadata.AnnotationsWidgetPlugin;
import edu.stanford.smi.protegex.owl.model.impl.DefaultOWLObjectProperty;


import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;

import java.util.*;
import java.net.URI;

public class NCIFULLSYNTableModel extends AbstractTableModel
{
    private ArrayList init_properties = new ArrayList();
    private ArrayList init_values = new ArrayList();

    private RDFResource subject;
    private ArrayList properties = new ArrayList();
    private ArrayList values = new ArrayList();
    private OWLModel owlModel;
    private JTable table;

    public final static int TERM_NAME = 0;
    public final static int TERM_GROUP = 1;
    public final static int TERM_SOURCE = 2;
    public final static int TERM_SOURCE_CODE = 3;
    public final static int LANGUAGE = 4;

    private final String SUBPROPERTYOF = "rdfs:subPropertyOf";

    private Vector u = null;

    private	Vector userDefinedRelevantProperties, excludedProperties;

    private boolean isModified = false;
    private ArrayList languages = new ArrayList();

	private String[] columnNames = {"Term Name",
									"Term Group",
									"Term Source",
				 					"Source Code",
				 					"Lang"};

    private EditPanel editpanel;

    public NCIFULLSYNTableModel(EditPanel editpanel, OWLModel owlModel) {
		this.owlModel = owlModel;
		this.editpanel = editpanel;
		userDefinedRelevantProperties = new Vector();
		excludedProperties = new Vector();
        initialize();
    }


    public NCIFULLSYNTableModel(EditPanel editpanel, RDFResource subject) {
        if (subject != null)
        {
			owlModel = (OWLModel) subject.getKnowledgeBase();
		}
		this.editpanel = editpanel;
		userDefinedRelevantProperties = new Vector();
		excludedProperties = new Vector();
		//objectValuedOnly = false;
        this.subject = subject;

        if (subject != null)
        {
			owlModel = (OWLModel) subject.getKnowledgeBase();
			u = new Vector();
			u.add(owlModel.getOWLDifferentFromProperty());
			u.add(owlModel.getOWLDisjointWithProperty());
			u.add(owlModel.getOWLEquivalentPropertyProperty());
			u.add(owlModel.getOWLSameAsProperty());
			u.add(owlModel.getRDFSIsDefinedByProperty());
			RDFProperty seeAlsoSlot = owlModel.getRDFProperty(RDFSNames.Slot.SEE_ALSO);
			u.add(seeAlsoSlot);
			initialize();
	    }

        refill();
    }

    public NCIFULLSYNTableModel(EditPanel editpanel, RDFResource subject, Vector v) {
        if (subject != null)
        {
			owlModel = (OWLModel) subject.getKnowledgeBase();
		}
		this.editpanel = editpanel;
		userDefinedRelevantProperties = new Vector();
		excludedProperties = new Vector();
		//objectValuedOnly = false;
        this.subject = subject;

        for (int i=0; i<v.size(); i++)
        {
			userDefinedRelevantProperties.add((String) v.elementAt(i));
		}

        if (subject != null)
        {
			owlModel = (OWLModel) subject.getKnowledgeBase();
			u = new Vector();
			u.add(owlModel.getOWLDifferentFromProperty());
			u.add(owlModel.getOWLDisjointWithProperty());
			u.add(owlModel.getOWLEquivalentPropertyProperty());
			u.add(owlModel.getOWLSameAsProperty());
			u.add(owlModel.getRDFSIsDefinedByProperty());
			RDFProperty seeAlsoSlot = owlModel.getRDFProperty(RDFSNames.Slot.SEE_ALSO);
			u.add(seeAlsoSlot);
			initialize();
	    }
        refill();
    }


    public NCIFULLSYNTableModel(EditPanel editpanel, RDFResource subject, Vector v, Vector w) {
        if (subject != null)
        {
			owlModel = (OWLModel) subject.getKnowledgeBase();
		}
		this.editpanel = editpanel;

		userDefinedRelevantProperties = new Vector();
		excludedProperties = new Vector();
        this.subject = subject;

        for (int i=0; i<v.size(); i++)
        {
			userDefinedRelevantProperties.add((String) v.elementAt(i));
		}
        for (int i=0; i<w.size(); i++)
        {
			excludedProperties.add((String) w.elementAt(i));
		}

        if (subject != null)
        {
			owlModel = (OWLModel) subject.getKnowledgeBase();
			u = new Vector();
			u.add(owlModel.getOWLDifferentFromProperty());
			u.add(owlModel.getOWLDisjointWithProperty());
			u.add(owlModel.getOWLEquivalentPropertyProperty());
			u.add(owlModel.getOWLSameAsProperty());
			u.add(owlModel.getRDFSIsDefinedByProperty());
			RDFProperty seeAlsoSlot = owlModel.getRDFProperty(RDFSNames.Slot.SEE_ALSO);
			u.add(seeAlsoSlot);
			initialize();
	    }
		refill();
	}

	public int getColumnCount() {
		return columnNames.length;
	}

    public int getRowCount() {
        return properties.size();
    }

	public String getColumnName(int col) {
		return columnNames[col];
	}

    public Object getValue(int rowIndex) {
		return values.get(rowIndex);
    }

	public Object getValueAt(int row, int col) {
		if (col==NCIFULLSYNTableModel.LANGUAGE)
			return getLanguage(row);

		Object obj = getValue(row);
		if (obj instanceof RDFSLiteral){
			RDFSLiteral value = (RDFSLiteral)obj;
			return getFullSynValue(value.getString(), col);
		}else
			return getFullSynValue((String)obj, col);
	}

	/*
	 * JTable uses this method to determine the default renderer/
	 * editor for each cell.  If we didn't implement this method,
	 * then the last column would contain text ("true"/"false"),
	 * rather than a check box.
	 */
	public Class getColumnClass(int c) {
		if (c==4)
			return String.class;

		return getValueAt(0, c).getClass();
	}

	/*
	 * Don't need to implement this method unless your table's
	 * editable.
	 */
	public boolean isCellEditable(int row, int col) {
		//Note that the data/cell address is constant,
		//no matter where the cell appears onscreen.
		//return true;
		if (col==NCIFULLSYNTableModel.LANGUAGE)
			return true;
		return false;
	}

	/*
	 * Don't need to implement this method unless your table's
	 * data can change.
	 */

    public void setValueAt(Object aValue, int row, int col) {
		setValueAndGetIt(aValue, row, col);
        fireTableCellUpdated(row, col);
        editpanel.enableSaveButton(true);
    }

    private String replaceValue(HashMap hm, String key, String value){
		String newValue="";

		Iterator it = hm.keySet().iterator();
	    while(it.hasNext()){
			String s = (String) it.next();
			if (s.equals(key))
				newValue += "<"+key+">"+value+"</"+key+">";
			else
        		newValue += "<"+key+">"+hm.get(it.next())+"</"+key+">";
		}
		return newValue;
	}

    private Object setLanguage(int row, String newLanguage) {

        RDFProperty property = getPredicate(row);
        String text = (String) getDisplayValue(row);
        Object newValue = createNewValue(property, text, newLanguage);
        if (!subject.getPropertyValues(property).contains(newValue)) {
             Object oldValue = getValue(row);

             //010307
             //removePropertyValue(property, oldValue);
             //addPropertyValue(property, newValue);

             resetValue(property.getName(), oldValue, newValue);
        }

        return newValue;
    }

    private Object createNewValue(RDFProperty property, String text, String language) {
        Object newValue = null;
        if (language == null || language.trim().length() == 0) {
            newValue = text;
        }
        else {
            newValue = property.getOWLModel().createRDFSLiteralOrString(text, language);
        }
        return newValue;
    }




    public Object getDisplayValue(int rowIndex) {
        Object value = values.get(rowIndex);
        if (value instanceof RDFSLiteral) {
            return ((RDFSLiteral) value).getString();
        }
        return value;
    }



    public Object setValueAndGetIt(Object value, int row, int col) {

		if (col == LANGUAGE){
			return setLanguage(row, (String) value);
		}

        String oldValue = (String) getValue(row);

		oldValue = ComplexPropertyParser.pipeDelim2XML(oldValue);
	    HashMap hmap = ComplexPropertyParser.parseXML(oldValue);

	    String key1="term-name";
	    String key2="term-gorup";
	    String key3="term-source";
	    String key4="source-code";
	    String key5="language";


		String newValue;
        if (col == TERM_NAME) {
			newValue = replaceValue(hmap, key1, (String)value);

			if (newValue == null) return oldValue;

            return setValue(newValue, row, col);
        }
        if (col == TERM_GROUP) {
			newValue = replaceValue(hmap, key2, (String)value);

			if (newValue == null) return "";

  			return setValue(newValue, row, col);
        }
        else if (col == TERM_SOURCE) {
			newValue = replaceValue(hmap, key3, (String)value);

			if (newValue == null) return "";

  			return setValue(newValue, row, col);
        }
        else {
			newValue = replaceValue(hmap, key4, (String)value);

			if (newValue == null) return "";

			return setValue(newValue, row, col);
        }
    }

    public String getLanguage(int row) {
        Object value = getValue(row);
        if (value instanceof RDFSLiteral) {
            return ((RDFSLiteral) value).getLanguage();
        }
        return null;
        //if (value==null) return null;
        //return value.toString();
    }


    private Object createNewValue(String text, String language) {
        Object newValue = null;
        if (language == null || language.trim().length() == 0) {
            newValue = text;
        }
        else {
            newValue = owlModel.createRDFSLiteralOrString(text, language);
        }
        return newValue;
    }

    public Object setValue(Object aValue, int row, int col)
    {
        RDFProperty property = getPredicate(row);
        Object oldValue = getValue(row);
        if (oldValue == null || !oldValue.equals(aValue)) {
            RDFResource range = property.getRange();
            try {
                Object newValue=null;
                String str = aValue.toString();
                OWLModel owlModel = property.getOWLModel();
                if (owlModel.getOWLOntologyProperties().contains(property)) {
                    String message = str + " is not a valid URI.";
                    try {
                        new URI(str);
                    }
                    catch (Exception ex) {
                        ProtegeUI.getModalDialogFactory().showErrorMessageDialog(owlModel, message);
                        return oldValue;
                    }
                    if (!str.startsWith("http://") && !str.startsWith("file:")) {
                        ProtegeUI.getModalDialogFactory().showErrorMessageDialog(owlModel, message);
                        return oldValue;
                    }
                }

                String lang = getLanguage(row);
                //?????
                if(aValue instanceof RDFSLiteral && ((RDFSLiteral)aValue).getLanguage() != null) {
                    newValue = aValue;
                }
                else if (lang != null) {
                    newValue = createNewValue(str, lang);
                	return newValue;
                }

                else if (oldValue instanceof RDFSLiteral) {
                    RDFSLiteral oldLiteral = (RDFSLiteral) oldValue;
                    newValue = getOWLModel().createRDFSLiteral(str, oldLiteral.getDatatype());
                }
                else if (oldValue instanceof Boolean) {
                    newValue = Boolean.valueOf(str.equals("true"));
                }
                else if (oldValue instanceof Float) {
                    newValue = Float.valueOf(str);
                }
                else if (oldValue instanceof Integer) {
                    newValue = Integer.valueOf(str);
                }
                else {
                    newValue = str;
                }

                if (!subject.getPropertyValues(property).contains(newValue)) {
                    removePropertyValue(property, oldValue);
                    addPropertyValue(property, newValue);
                }

				values.set(row, aValue);
                return newValue;
            }
            catch (NumberFormatException ex) {
                // Ignore illegal number format
            }
        }
        return oldValue;
    }

    public void setSubject(RDFResource instance) {

        RDFResource oldSubject = this.subject;

        if (oldSubject != null) {
            //oldSubject.removePropertyValueListener(valueListener);
        }
        this.subject = instance;

        init_properties.clear();
        init_values.clear();

        properties.clear();
        values.clear();

		languages.clear();

        if (instance != null) {
            //instance.addPropertyValueListener(valueListener);

            initialize();

			for (int i=0; i<values.size(); i++)
			{
				Object obj = values.get(i);
				String value;
				if (obj instanceof RDFSLiteral)
				{
					RDFSLiteral obj_literal = (RDFSLiteral) obj;
					value = obj_literal.getString();
				}
				else
				{
					value = (String) obj;
				}
			}


            refill();
        }
        fireTableDataChanged();
        isModified = false;
    }


    private void refill() {
        if (subject == null) return;
        OWLModel owlModel = (OWLModel) subject.getKnowledgeBase();

		Collection c = getRelevantProperties();
		RDFProperty[] ss = (RDFProperty[]) c.toArray(new RDFProperty[0]);

		//Arrays.sort(ss, new FrameComparator());
		for (int i = 0; i < ss.length; i++) {
			RDFProperty property = ss[i];
			if (property.isAnnotationProperty() && !(property instanceof DefaultOWLObjectProperty))
			{
				//if (userDefinedRelevantProperties.contains(property.getBrowserText()))
				if (userDefinedRelevantProperties.contains(property.getName()))
				{
					if(property.isVisible()) {
						Collection c2 = subject.getPropertyValues(property);
						for (Iterator it = c2.iterator(); it.hasNext();) {
							Object value = it.next();

							//this.properties.add(property);
							//this.values.add(value);
							String s = ComplexPropertyParser.pipeDelim2XML(value.toString());
							HashMap hmap = ComplexPropertyParser.parseXML(s);
							if (hmap != null && hmap.containsKey("term-name") )
							{
								this.properties.add(property);
								this.values.add(value);
							}
						}
					}
				}

				else if (userDefinedRelevantProperties.size() == 0
					  //&& !excludedProperties.contains(property.getBrowserText()))
					  && !excludedProperties.contains(property.getName()))
				{
//					if(property.isVisible() && property.getBrowserText().compareTo(SUBPROPERTYOF) != 0) {
					if(property.isVisible() && property.getName().compareTo(SUBPROPERTYOF) != 0) {

						Collection values = subject.getPropertyValues(property);
						for (Iterator it = values.iterator(); it.hasNext();) {
							Object value = it.next();

							//this.properties.add(property);
							//this.values.add(value);
							String s = ComplexPropertyParser.pipeDelim2XML(value.toString());
							HashMap hmap = ComplexPropertyParser.parseXML(s);
							if (hmap != null && hmap.containsKey("term-name") )
							{
								this.properties.add(property);
								this.values.add(value);
							}

						}

					}
				}
			}

		}

		for(Iterator it = getDefaultProperties().iterator(); it.hasNext(); ) {
			RDFProperty curProp = (RDFProperty) it.next();
			if(this.properties.contains(curProp) == false) {
				this.properties.add(0, curProp);
				this.values.add(0, null);
			}
		}

		//062906
		if (values.size() >0){
			editpanel.enableDelEditHB(1, true);
		}
    }


	public Collection getDefaultProperties() {
		return Collections.EMPTY_LIST;
	}

	public Collection getProperties() {
		return properties;
	}

	public Collection getValues() {
		return values;
	}

    protected Collection getRelevantProperties() {
        OWLModel owlModel = subject.getOWLModel();
        return owlModel.getRDFProperties();
    }

	public void setTable(JTable table) {
        this.table = table;
    }

	private String getFullSynValue(String value, int column){

		if (value == null || value.equals(""))
		{
			return "";
		}

		value = ComplexPropertyParser.pipeDelim2XML(value);
		HashMap hmap = ComplexPropertyParser.parseXML(value);
		String s = "";
		if (column == 0)
		{
			//s  = (String) hmap.get("Syn_Term_Name");
			s  = (String) hmap.get("term-name");
		}
		else if (column == 1)
		{
			//s  = (String) hmap.get("Syn_Term_Type");
			s  = (String) hmap.get("term-group");
		}
		else if (column == 2)
		{
			//s  = (String) hmap.get("Syn_Source");
			s  = (String) hmap.get("term-source");
		}
		else if (column == 3)
		{
			//s  = (String) hmap.get("Syn_Source_Code");
			s  = (String) hmap.get("source-code");
		}
		if (s == null) return "";
		return s;
	}

    public RDFProperty getPredicate(int rowIndex) {
        return (RDFProperty) properties.get(rowIndex);
    }

    public OWLModel getOWLModel() {
        return subject.getOWLModel();
    }

    public void removePropertyValue(RDFProperty property, Object value) {
        int index = getPropertyValueRow(property, value);
        if (index >= 0 && index < getRowCount())
        {
			properties.remove(index);
			values.remove(index);
			//property_map_1.removeValue((Object) property, (Object) value);
			//updateValues();
		}
	}

    public int getPropertyValueRow(RDFProperty property, Object value) {
        for (int i = 0; i < properties.size(); i++) {
            RDFProperty s = getPredicate(i);
            if (s.equals(property)) {
				Object ob = getValue(i);
				String tableStr;
				if (ob instanceof RDFSLiteral)
					tableStr = ((RDFSLiteral)ob).getString();
				else
					tableStr = (String)ob;

                if (value == null && ob == null) {
                    return i;
                }
                else if (value != null && value.toString().equals(tableStr)) {
                    return i;
                }
            }
        }
        return -1;
    }



    void updateValues() {
        int index = -1;
        if (table != null) {
            index = table.getSelectedRow();
        }

        properties.clear();
        values.clear();

        refill();
        fireTableDataChanged();
        if (table != null && index >= 0 && index < getRowCount()) {
            table.getSelectionModel().setSelectionInterval(index, index);
        }
        else if (table == null)
        {
			System.out.println("table == null");
		}
    }

	private Vector parseFullSyn(String value){
		Vector v = null;
		StringTokenizer st = null;

		if (value != null)
			st = new StringTokenizer(value, "|");

		int num_tokens = st.countTokens();
		if (num_tokens>=3){
			v = new Vector();
			while (st.hasMoreTokens())
				v.add(st.nextToken());
		}
		return v;
	}



    private String getStringValue(Object obj)
    {
		if (obj instanceof RDFSLiteral)
		{
			RDFSLiteral literal = (RDFSLiteral) obj;
            return literal.getString();
		}
		else
		{
			return (String) obj;
		}
	}



    private void initialize() {

        if (subject == null) return;

        OWLModel owlModel = (OWLModel) subject.getKnowledgeBase();

        // For Assocations and other object valued properties

            Collection c = getRelevantProperties();
            RDFProperty[] ss = (RDFProperty[]) c.toArray(new RDFProperty[0]);

            //Arrays.sort(ss, new FrameComparator());
            for (int i = 0; i < ss.length; i++) {
                RDFProperty property = ss[i];
                if (property.isAnnotationProperty() && !(property instanceof DefaultOWLObjectProperty))
                {
					//if (userDefinedRelevantProperties.contains(property.getBrowserText()))
					if (userDefinedRelevantProperties.contains(property.getName()))

					{
						if(property.isVisible()) {


							if (property.getName().compareTo(editpanel.getSlotName(NCIEditTab.FULL_SYN)) == 0)
							{
								Collection values = subject.getPropertyValues(property);
								for (Iterator it = values.iterator(); it.hasNext();) {
									Object value = it.next();

									//this.init_properties.add(property);
									//this.init_values.add(value);

									String str_value = getStringValue(value);

									String s = ComplexPropertyParser.pipeDelim2XML(str_value);

									HashMap hmap = ComplexPropertyParser.parseXML(s);
									if (hmap != null && hmap.containsKey("term-name") )
									{

										this.init_properties.add(property);
										this.init_values.add(value);

									}

								}
						    }
						}
					}

					else if (userDefinedRelevantProperties.size() == 0
						  //&& !excludedProperties.contains(property.getBrowserText()))
						  && !excludedProperties.contains(property.getName()))
					{
						//if(property.isVisible() && property.getBrowserText().compareTo(SUBPROPERTYOF) != 0) {
						if(property.isVisible() && property.getName().compareTo(SUBPROPERTYOF) != 0) {

							Collection values = subject.getPropertyValues(property);
							for (Iterator it = values.iterator(); it.hasNext();) {
								Object value = it.next();

                                String s = ComplexPropertyParser.pipeDelim2XML(value.toString());
								HashMap hmap = ComplexPropertyParser.parseXML(s);
								if (hmap != null && hmap.containsKey("term-name") )
								{

									this.init_properties.add(property);
									this.init_values.add(value);

							    }
							}

						}
					}
			    }

            }

	        for(Iterator it = getDefaultProperties().iterator(); it.hasNext(); ) {
		        RDFProperty curProp = (RDFProperty) it.next();
		        if(this.properties.contains(curProp) == false) {

			        this.init_properties.add(0, curProp);
			        this.init_values.add(0, null);
		        }
	        }

	        //062906
	        editpanel.enableDelEditHB(1, false); // header buttons
    }

    public RDFResource getSubject() {
        return subject;
    }

	public Collection getInitialProperties() {
		return init_properties;
	}

	public Collection getInitialValues() {
		return init_values;
	}


    private boolean hasPropertyValue(RDFProperty property, Object value) {
		if (property == null || value == null) return false;

        int index = getPropertyValueRow(property, value);
        if (index >= 0 && index < getRowCount())
        {
			return true;
		}
		return false;
	}

    public void deleteRow(int index) {
		if (index >= 0 && index < getRowCount())
		{
			properties.remove(index);
			values.remove(index);
			fireTableDataChanged();
			editpanel.enableSaveButton(true);
		}
	}

    public int addRow(RDFProperty property, Object value) {
        if (!hasPropertyValue(property, value)) {
            addPropertyValue(property, value);
            isModified = true;
            editpanel.enableSaveButton(true);
        }

        int index = getPropertyValueRow(property, value);
        return index;
    }

    public void addPropertyValue(RDFProperty property, Object value) {

        int index = getPropertyValueRow(property, value);
        if (index >= 0 && index < getRowCount())
        {
			return;
		}
        properties.add(property);
        values.add(value);
        fireTableDataChanged();
        isModified = true;
        //updateValues();
	}

    public boolean resetValue(String prop_name, Object prop_value_obj, Object new_prop_value_obj)
    {


		String prop_value = getStringValue(prop_value_obj);
		String new_prop_value = getStringValue(new_prop_value_obj);


        for (int i=0; i<properties.size(); i++)
        {
			RDFProperty property = (RDFProperty) properties.get(i);
			//String name = property.getBrowserText();
			String name = property.getName();


			String value = null;//(String) values.get(i);
			Object obj = values.get(i);

			if (obj instanceof RDFSLiteral)
			{
				RDFSLiteral obj_literal = (RDFSLiteral) obj;
			    value = obj_literal.getString();
			}
			else
			{
				value = (String) obj;
			}


			if (name.compareToIgnoreCase(prop_name) == 0 &&
				value.compareToIgnoreCase(prop_value) == 0)
			{
				values.set(i, new_prop_value_obj);
				fireTableDataChanged();
				isModified = true;
				editpanel.enableSaveButton(true);
				return true;
			}
		}
		return false;
	}

    public EditPanel getEditPanel()
    {
		return editpanel;
    }

}

