package gov.nih.nci.protegex.edit;

import edu.stanford.smi.protegex.owl.ui.components.triples.*;

import com.hp.hpl.jena.datatypes.xsd.impl.XMLLiteralType;
import edu.stanford.smi.protege.ui.FrameComparator;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.model.event.PropertyValueAdapter;
import edu.stanford.smi.protegex.owl.model.event.PropertyValueListener;
import edu.stanford.smi.protegex.owl.model.impl.DefaultRDFSLiteral;
import edu.stanford.smi.protegex.owl.model.impl.XMLSchemaDatatypes;
import edu.stanford.smi.protegex.owl.ui.ProtegeUI;
import edu.stanford.smi.protegex.owl.ui.metadata.AnnotationsWidgetPlugin;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import java.net.URI;
import java.util.*;

import javax.swing.*;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;
import java.awt.*;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.Vector;

import edu.stanford.smi.protegex.owl.ui.widget.OWLUI;

import javax.swing.table.DefaultTableModel;

import edu.stanford.smi.protegex.owl.model.impl.DefaultOWLObjectProperty;
import edu.stanford.smi.protegex.owl.ui.components.annotations.*;

import edu.stanford.smi.protegex.owl.model.RDFSNames;

import edu.stanford.smi.protege.model.Cls;


/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class NCIRoleGroupTableModel extends AbstractTableModel {

    private JTextField textField;

    /**
     * The resource being annotated
     */
    private RDFResource subject;
    private HashMap rolegroups = new HashMap();
    private HashMap values = new HashMap();

    private JTable table;
    private boolean isModified = false;

    OWLModel owlModel;
    OWLWrapper wrapper;

    private OWLClass r0;

    /*private PropertyValueListener valueListener = new PropertyValueAdapter() {
        public void propertyValueChanged(RDFResource resource, RDFProperty property, Collection oldValues) {
            if (isRelevantProperty(property)) {
            }
        }
    };*/

    public NCIRoleGroupTableModel(OWLModel owlModel, OWLClass r0) {
		this.owlModel = owlModel;
		this.r0 = r0;
        initialize();
    }


    public NCIRoleGroupTableModel(OWLModel owlModel, RDFResource subject, OWLClass r0) {
		this.owlModel = owlModel;
        this.subject = subject;
		this.r0 = r0;

        if (subject != null) {
            //subject.addPropertyValueListener(valueListener);
        }
        initialize();

        refill();
    }


    private void initialize() {
		populateRoleGroupTable((OWLClass) r0);
    }




	private boolean isSimpleRestriction(Cls r)
	{
		if (r instanceof OWLNamedClass) return false;
		if (r instanceof OWLIntersectionClass) return false;
		if (r instanceof OWLUnionClass) return false;
		return true;
	}




    public int getColumnCount() {
        return 2;
    }

    public int addRow(OWLRestriction restriction){
		return addRow(restriction, 0);
	}

    public int addRow(OWLRestriction restriction, int groupId) {
        if (!hasRoleGroupValue(restriction, new Integer(groupId))) {
            addRoleGroupValue(restriction, new Integer(groupId));

            RoleGroupElement element = new RoleGroupElement(groupId, restriction);
            values.put(values.size(), element);
            isModified = true;

            updateValues();
        }
        return values.size();
    }


    public void addRoleGroupValue(OWLRestriction restriction, Integer groupId) {
		if (restriction == null) return;
		if (hasRoleGroupValue(restriction, groupId)) return;

		Vector roles=new Vector();
		if (rolegroups.containsKey(groupId)){
			roles = (Vector) rolegroups.get(groupId);
		}

		roles.add(restriction);

		rolegroups.put(groupId, roles);
		//dumpRoleGroup(rolegroups);
	}

    private boolean hasRoleGroupValue(OWLRestriction restriction, Integer groupId) {
		if (restriction == null) return false;

		if (rolegroups.containsKey(groupId)){
			Vector roles = (Vector) rolegroups.get(groupId);
			if (roles.contains(restriction))
				return true;
		}

		return false;
	}


    public Object setRoleValue(Object aValue, int row) {

        OWLRestriction r = getRole(row);
        String oldValue = getRoleValue(r);
        if (oldValue == null || !oldValue.equals(aValue)) {
	        updateRoleValue(oldValue, (String)aValue, row);
        }
        return oldValue;
    }

    private String getRoleValue(OWLRestriction r){
		return r.getBrowserText();
	}

    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        setValueAndGetIt(aValue, rowIndex, columnIndex);
    }


    public Object setValueAndGetIt(Object value, int row, int col)
    {
	    if (col == 0)
            return setGroupNumberAndGetIt(value, row);

        return setRoleValueAndGetIt(value, row);
    }

    private Object setGroupNumberAndGetIt(Object value, int row){
		RoleGroupElement element = (RoleGroupElement) values.get(row);
		int oldValue = element.getRoleGroupNumber();
		OWLRestriction r = element.getRestriction();
		element.setRoleGroupNumber(Integer.parseInt((String)value));
		values.put(row, element);

		Vector v1 = (Vector) rolegroups.get(oldValue);
		v1.remove(r);
		rolegroups.put(oldValue, v1);

		Vector v2 = new Vector();
		int newKey = Integer.parseInt((String)value);
		if (rolegroups.containsKey(newKey)){
			v2=(Vector) rolegroups.get(newKey);
		}
		v2.add(r);
		rolegroups.put(newKey, v2);

		return value;
	}



    private Object setRoleValueAndGetIt(Object value, int row){
		setRoleValue(value, row);

		RoleGroupElement element = (RoleGroupElement) values.get(row);
		OWLRestriction r = (OWLRestriction) value;

		if (r==null) return null;

		element.setRestriction(r);
		values.put(row, element);
		return value;
	}


/*
    private Object setRoleValueAndGetIt(Object value, int row){
		setRoleValue(value, row);

		RoleGroupElement element = (RoleGroupElement) values.get(row);


		StringTokenizer st = new StringTokenizer((String)value, "|");
		OWLRestriction r = getRestriction(st.nextToken(), st.nextToken(), st.nextToken());

		if (r==null) return null;

		element.setRestriction(r);
		values.put(row, element);
		return value;
	}
*/

/*
	private OWLRestriction getRestriction(String name, String value, String modifier) {
		OWLNamedClass cls = (OWLNamedClass) subject;
		Collection col = cls.getRestrictions(false);

        for (Iterator it = col.iterator(); it.hasNext();) {
			OWLRestriction r = (OWLRestriction) it.next();

			RDFProperty p = r.getOnProperty();
			String prop_name = p.getName();
			String prop_value = r.getFillerText();
			String prop_modifier = wrapper.getRestrictionType(r);

			if (prop_name.equals(name) && prop_value.equals(value) &&
					prop_modifier.equals(modifier)){
				return r;
			}
		}
		return null;
	}
*/

    public int getRestrictionValueRow(OWLRestriction restriction, int groupId) {
        return -1;
    }


    private Object createDefaultValue(OWLRestriction restriction) {
        return null;
    }

    public String getColumnName(int column)
    {
		if (column == 0)
			return "Group No.";
		else
			return "Restriction";
    }

    public void deleteRow(int rowIndex) {
		//change HashMap values, rolegroups

		RoleGroupElement element = (RoleGroupElement)values.get(rowIndex);
		for (int i=rowIndex; i<values.size()-1; i++){
			values.put(i, values.get(i+1));
		}
		values.remove(values.size()-1);

		int groupnumber = element.getRoleGroupNumber();
		OWLRestriction r = element.getRestriction();

		Vector v = (Vector)rolegroups.get(groupnumber);
		v.remove(r);

		//testing KLO
		if (v.size() > 0)
		{
			rolegroups.put(groupnumber, v);
	    }
	    else
	    {
			Object obj = rolegroups.remove((Object) groupnumber);
		}

		isModified = true;
		updateValues();
    }

    public void updateRoleValue(String oldValue, String value, int index)
    {
		RoleGroupElement element = (RoleGroupElement) values.get(index);
		StringTokenizer st = new StringTokenizer(oldValue, "|");
		String oldname = st.nextToken();
		String oldvalue = st.nextToken();
		String oldmodifier = st.nextToken();

		//wrapper.removeRestriction((OWLNamedClass) subject, oldname, oldvalue, oldmodifier);
		//wrapper.addRestriction((OWLNamedClass) subject, value);

		updateValues();
	}


    public void updateRestriction(OWLRestriction r, int index)
    {
		RoleGroupElement element = (RoleGroupElement) values.get(index);
        int rolegroup_number = element.getRoleGroupNumber();
        OWLRestriction old_restriction = element.getRestriction();
		Vector v = (Vector) rolegroups.get(rolegroup_number);
		v.remove(old_restriction);
		v.add(r);
		rolegroups.put(rolegroup_number, v);

        element.setRestriction(r);
		updateValues();
	}



    void updateValues() {
        int index = -1;
        if (table != null)
            index = table.getSelectedRow();

        fireTableDataChanged();
        if (table != null && index >= 0 && index < getRowCount()) {
            table.getSelectionModel().setSelectionInterval(index, index);
        }
        else if (table != null && getRowCount() > 0) {
            table.getSelectionModel().setSelectionInterval(0, 0);
        }
    }

    public int getRowCount() {
        return values.size();
    }

    public Object getValueAt(int rowIndex, int columnIndex) {
		if (columnIndex==0){
			return getRoleGroupNumber(rowIndex);
		}else{
			OWLRestriction r = getRole(rowIndex);
			return getRoleValue(r);
		}
    }


    public OWLRestriction getRole(int rowIndex) {
		RoleGroupElement element = (RoleGroupElement) values.get(rowIndex);
        return element.getRestriction();
    }

    public int getRoleGroupNumber(int rowIndex) {
		RoleGroupElement element = (RoleGroupElement) values.get(rowIndex);
		return element.getRoleGroupNumber();
    }


    private void refill() {
        if (subject == null) return;

        OWLModel owlModel = (OWLModel) subject.getKnowledgeBase();
    }


	public void setTable(JTable table) {
        this.table = table;
        //091406
		this.table.setFont(new Font("Times New Roman", Font.PLAIN, 12));
    }

	public JTable getTable() {
        return table;
    }



	public void dumpRoleGroup(HashMap rolegroup_hmap) {
		System.out.println("Role groups...");

		Iterator keys = rolegroup_hmap.keySet().iterator();
		while (keys.hasNext()){
			Object key = keys.next();
			Vector v = (Vector) rolegroup_hmap.get(key);
			for (int i=0; i<v.size(); i++){
				OWLRestriction r = (OWLRestriction) v.elementAt(i);
				System.out.println("\t"+key+": "+r.getBrowserText());
			}
		}
	}



    public HashMap getRolegroups(){
		//System.out.println("In role group model: ");
		//dumpRoleGroup(rolegroups);;
		//System.out.println("\n");
		return rolegroups;
	}

    public HashMap getValues(){
		return values;
	}



    private Vector parseRestriction(OWLClass owlcls)
    {
		Vector v = new Vector();

		if (owlcls instanceof OWLUnionClass)
		{
			OWLUnionClass cls = (OWLUnionClass) owlcls;

			Collection c = cls.getOperands();
			Iterator it = c.iterator();
			while (it.hasNext())
			{
				Cls r2 = (Cls) it.next();
				if (r2 instanceof OWLUnionClass)
				{
					OWLUnionClass unionCls = (OWLUnionClass) r2;
					Collection c2 = unionCls.getOperands();
					Iterator it2 = c2.iterator();
					// each operand is a role group
					while (it2.hasNext())
					{	Cls r3 = (Cls) it2.next();
						v.add(r3);
					}
				}else
					v.add(r2);
			}
		}else
			v.add(owlcls);

		return v;
	}


	private void populateRestrictions(Vector v)
	{
		int group_number = 0;
		for (int i=0; i<v.size(); i++)
		{
			Object r = v.elementAt(i);
			if (r instanceof OWLRestriction)
			{
				addRow((OWLRestriction)r);
			}
			else
			{
				if (r instanceof OWLIntersectionClass)
				{
					OWLIntersectionClass IntersectionCls = (OWLIntersectionClass) r;
					group_number++;

					Collection c = IntersectionCls.getOperands();
					Iterator it = c.iterator();
					while (it.hasNext())
					{
						OWLRestriction r2 = (OWLRestriction) it.next();
						addRow(r2, group_number);
					}
			    }
			}
		}
	}



    private void populateRoleGroupTable(OWLClass owlcls)
    {
		if (owlcls == null)
		{
			return;
		}
		int group_number = 0;

		// if it's a form of intersection class
		if (owlcls instanceof OWLIntersectionClass)
		{
			OWLIntersectionClass intersectionCls = (OWLIntersectionClass) owlcls;
			Collection c = intersectionCls.getOperands();
			Iterator it = c.iterator();
			while (it.hasNext())
			{
				Cls r = (Cls) it.next();

			    if (r instanceof OWLIntersectionClass)
			    {
					// not supported.
					System.out.println("NCIRoleGroupTableModel OWLIntersectionClass NOT SUPPORTED." + r.getBrowserText());
				}
				else if (r instanceof OWLUnionClass)
				{
					System.out.println("NCIRoleGroupTableModel OWLUnionClass " + r.getBrowserText());

					// each operand is a role group
					//group_number++;
					OWLUnionClass unionCls = (OWLUnionClass) r;

// for each role group:
					Collection c2 = unionCls.getOperands();
					Iterator it2 = c2.iterator();
					while (it2.hasNext())
					{
						Cls cls = (Cls) it2.next();
						OWLIntersectionClass intersection_cls = (OWLIntersectionClass) cls;
						Collection c3 = intersection_cls.getOperands();

						group_number++;

						Iterator it3= c3.iterator();
						while (it3.hasNext())
						{
							Cls cls3 = (Cls) it3.next();
							OWLRestriction r3 = (OWLRestriction) cls3;
							addRow(r3, group_number);
						}
					}
				}
				else
				{
					addRow((OWLRestriction) r, 0);
				}

			}

		}
		// if it has more than one role groups
		else if (owlcls instanceof OWLUnionClass)
		{
			// each operand is a role group
			OWLUnionClass unionCls = (OWLUnionClass) owlcls;

			Collection c2 = unionCls.getOperands();
			Iterator it2 = c2.iterator();
			while (it2.hasNext())
			{
				Cls cls = (Cls) it2.next();
				OWLIntersectionClass intersection_cls = (OWLIntersectionClass) cls;

				Collection c3 = intersection_cls.getOperands();
				group_number++;

				Iterator it3= c3.iterator();
				while (it3.hasNext())
				{
					Cls cls3 = (Cls) it3.next();
					OWLRestriction r3 = (OWLRestriction) cls3;
					addRow(r3, group_number);
				}
			}
		}
		else // if it's a simple restriction
		{
			OWLRestriction r = (OWLRestriction) owlcls;
			addRow(r, 0);
		}

	}


}
