package gov.nih.nci.protegex.edit;

import java.io.*;
import java.sql.Connection;
import java.util.*;

import java.awt.*;
import java.awt.dnd.*;
import java.awt.datatransfer.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;

import edu.stanford.smi.protege.util.Log;
import edu.stanford.smi.protegex.owl.model.*;

import edu.stanford.smi.protegex.owl.model.classparser.OWLClassParseException;
import edu.stanford.smi.protegex.owl.model.classparser.OWLClassParser;
import edu.stanford.smi.protegex.owl.ui.ProtegeUI;
/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class TreeItem
{
	private int type;
	private String name;
	private String value;
	private String restriction;
	private String modifier;
	private int cardinality; // cardinality constraint filler
	private String nameValue; // cardinality constraint filler
	private String language;

	private boolean isDefining;

	public static final int TYPE_CONCEPT = 0;
	public static final int TYPE_PROPERTY = 1;

	public static final int TYPE_RESTRICTION = 3;
	public static final int TYPE_PARENT = 2;

	public static final int TYPE_ASSOCIATION = 4;

	private RDFSClass aClass;
	private RDFProperty aProperty;

    public TreeItem()
	{
        this.type = -1;
        this.name = "";
        this.value = "";
        this.restriction = "";
        this.modifier = "";
        this.cardinality = -1;
        this.nameValue = "";
        this.isDefining = false;
        this.aClass = null;
        this.aProperty = null;
 	}


    public TreeItem(int type,
                    String name,
                    String value,
                    String restriction,
                    String modifier,
                    int cardinality)
	{
        this.type = type;
        this.name = name;
        this.value = value;
        this.restriction = restriction;
        this.modifier = modifier;
        this.cardinality = cardinality;
        this.nameValue = name + ": " + value;
        this.isDefining = false;
        this.aClass = null;
        this.aProperty = null;
	}

    public RDFSClass getCls() {
        return aClass;
    }

    public void setCls(RDFSClass aClass) {

		if (aClass == null)
		{
			//System.out.println("WARNING: TreeItem setCls -- aClass == null");
			return;
		}

        this.aClass = aClass;
    }

    public RDFProperty getProperty() {
        return aProperty;
    }

    public void setProperty(RDFProperty aProperty) {

		if (aProperty == null)
		{
			return;
		}

        this.aProperty = aProperty;
    }


	public void setLanguage(String language)
	{
		this.language = language;
	}

	public String getLanguage()
	{
		return language;
	}


	public boolean getIsDefining()
	{
		return isDefining;
	}

	public void setIsDefining(boolean defining)
	{
		this.isDefining = defining;
	}

	public void setType(int type)
	{
		this.type = type;
	}

	public void setName(String name)
	{
		this.name = name;
	}



    private String getObjectValue(Object obj)
    {
		if (obj == null) return null;
		Object newValue;
		String str = obj.toString();

		if (obj instanceof RDFSLiteral) {
			newValue = (RDFSLiteral) obj;

			RDFSLiteral literal = (RDFSLiteral) obj;
			if (literal.getLanguage() == null || literal.getLanguage().compareTo("") == 0 )
			{
				str = str.substring(2, str.length());
			}
			else if (str.length() > 6)//if (str.indexOf("~#") != -1)
			{
				str = str.substring(5, str.length());
			}

		}
		else if (obj instanceof Boolean) {
			newValue = Boolean.valueOf(str.equals("true"));
		}
		else if (obj instanceof Float) {
			newValue = Float.valueOf(str);
		}
		else if (obj instanceof Integer) {
			newValue = Integer.valueOf(str);
		}
		else {

			newValue = getPlainString(str);
		}

		return newValue.toString();
	}

	public String getPlainString(String s)	{
		String[] languages = NCIEditTab.owlModel.getUsedLanguages();
		if (s.indexOf("~#") == -1)
			return s;

		for (int i=0; i<languages.length; i++){
			if (s.startsWith("~#"+languages[i]+" "))
				return s.substring(5);
		}

		return s.substring(2);
	}


    private String getStringValue(Object obj)
    {
		if (obj instanceof RDFSLiteral)
		{
			RDFSLiteral literal = (RDFSLiteral) obj;
            return literal.getString();
		}
		else
		{
			return (String) obj;
		}
	}


    private String getObjectLanguage(Object obj)
    {
		if (obj instanceof RDFSLiteral)
		{
			RDFSLiteral literal = (RDFSLiteral) obj;
            return literal.getLanguage();
		}
		else
		{
			return "";
		}
	}


	public void setValue(String value)
	{
		this.value = value;
	}


	public void setValue(Object value)
	{
		this.language = getObjectLanguage(value);
		this.value = getObjectValue(value);
	}


	public void setRestriction(String restriction)
	{
		this.restriction = restriction;
	}

	public void setModifier(String modifier)
	{
		this.modifier = modifier;
	}

	public void setCardinality(int cardinality)
	{
		this.cardinality = cardinality;
	}

	public void setNameValue(String nameValue)
	{
		this.nameValue = nameValue;
	}

	public void setNameValue(String name, Object value)
	{
		setValue(getObjectValue(value));
		this.nameValue = name + ": " + getObjectValue(value);
	}




	public int getType()
	{
		return type;
	}

	public String getName()
	{
		return name;
	}

	public String getValue()
	{
		return value;
	}

	public String getRestriction()
	{
		return restriction;
	}

	public String getModifier()
	{
		return modifier;
	}

	public int getCardinality()
	{
		return cardinality;
	}

	public String toString()
	{
		return nameValue;
	}


    public static RDFSClass parsableText2RDFSClass(String parsableText)
    {
   	    RDFSClass newRestriction = null;
		try {
			OWLClassParser parser = NCIEditTab.owlModel.getOWLClassDisplay().getParser();
			newRestriction = parser.parseClass(NCIEditTab.owlModel, parsableText);
			return newRestriction;
		}
		catch (OWLClassParseException e) {
			ProtegeUI.getModalDialogFactory().showErrorMessageDialog(NCIEditTab.owlModel, "Error at parsing restriction: "
			   + parsableText, "Parse Error");
			return null;
		}
	}

/*
    private RDFSClass cloneAnonymousClass(RDFSClass aClass)
    {
		if (aClass == null) return null;
		OWLIntersectionClass newClass = NCIEditTab.owlModel.createOWLIntersectionClass();

		if (aClass instanceof OWLIntersectionClass)
		{
			OWLIntersectionClass intersectionClass = (OWLIntersectionClass) aClass;
			Collection operands = ((OWLIntersectionClass) aClass).getOperands();
			for (Iterator oit = operands.iterator(); oit.hasNext();) {
				RDFSClass operand = (RDFSClass) oit.next();
				newClass.addOperand(parsableText2RDFSClass(operand.getBrowserText()));
			}
			return newClass;
		}
		else
		{
			return aClass.createClone();
		}

	}
*/

    public TreeItem cloneTreeItem()
	{
		TreeItem newitem = new TreeItem();
        newitem.setType(type);
        newitem.setName(name);
        newitem.setValue(value);
        newitem.setRestriction(restriction);
        newitem.setModifier(modifier);
        newitem.setCardinality(cardinality);
        newitem.setNameValue(toString());

		if (type == TYPE_ASSOCIATION)
		{
			if (aProperty != null)
			{
				newitem.setProperty(aProperty);
			}
		}

        newitem.setIsDefining(isDefining);
        newitem.setLanguage(language);

        if (aClass != null && aClass instanceof OWLAnonymousClass)
        {
        	Log.getLogger().warning("Creating clone of " + aClass.getBrowserText());
        	
         	newitem.setCls(aClass.createClone());
         	//newitem.setCls(cloneAnonymousClass(aClass));
		}
		else if (aClass != null)
		{
			newitem.setCls(aClass);
		}

        return newitem;
	}



}

