package gov.nih.nci.protegex.edit;

import java.awt.* ;
import javax.swing.*;
import javax.swing.event.*;
import java.util.*;
import java.io.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.filechooser.FileFilter;

import javax.swing.tree.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.ui.*;

import edu.stanford.smi.protegex.owl.ui.profiles.ProfilesManager;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protege.util.LabeledComponent;

import edu.stanford.smi.protege.util.*;


import edu.stanford.smi.protegex.owl.model.DefaultTaskManager;
import java.text.SimpleDateFormat;

import edu.stanford.smi.protege.util.Log;
import java.util.logging.Level;


/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class BatchEditorDialog extends JDialog implements ActionListener
{
	JButton fStartButton, fInputButton, fOutputButton;

	JTextField fInputTf, fOutputTf;

    NCIEditTab tab;
    OWLModel owlModel;

	String infile;// = fInputTf.getText();
	String outfile;// = fOutputTf.getText();

	OWLWrapper wrapper = null;

	static final int BATCH_LOADER = 1;
	static final int BATCH_EDITOR = 2;

	int type = -1;

	FileField inputFileField;
	FileField outputFileField;

    public BatchEditorDialog(NCIEditTab tab, KnowledgeBase kb, int type)
    {
		this.tab = tab;
		this.owlModel = (OWLModel) kb;
		this.wrapper = tab.getWrapper();

		this.infile = "";
		this.outfile = "";

		this.type = type;
		setModal(true);

		if (type == BATCH_LOADER)
		{
			this.setTitle("Batch Loader");
		}
		else
		{
			this.setTitle("Batch Editor");
		}
		init();
	}

    public void setBatchProcessType(int type)
    {
		this.type = type;
	}

  	public void init()
  	{
		try
		{
			Container container = this.getContentPane();
			container.setLayout(new BorderLayout());

			this.setLocation(450,300);

			JPanel filePanel = new JPanel();
			filePanel.setLayout(new BorderLayout());

			String label = "Input File";
			String path = "";
			String extension = "dat";
			String description = "";
			inputFileField = new FileField(label, path, extension, description);

			filePanel.add(inputFileField, BorderLayout.NORTH);
			label = "Log File";
			path = "";
			extension = "out";
			description = "";
			outputFileField = new FileField(label, path, extension, description);
			filePanel.add(outputFileField, BorderLayout.CENTER);

			container.add(filePanel, BorderLayout.NORTH);

			fStartButton = new JButton("Start");
			fStartButton.addActionListener(this);

            JPanel btnPanel = new JPanel();
            btnPanel.setLayout(new BorderLayout());
            btnPanel.add(fStartButton, BorderLayout.CENTER);

			container.add(btnPanel, BorderLayout.SOUTH);

			pack();
			this.setVisible(true);

		}
		catch(Exception ex) {
			Log.getLogger().log(Level.WARNING, "Exception caught", ex);

		}
  	}

    public String getInfile()
    {
		//return infile;
		if (inputFileField.getFilePath() == null) return null;
		return inputFileField.getFilePath().getPath();
	}

    public String getOutfile()
    {
		//return outfile;
		if (outputFileField.getFilePath() == null) return null;
		return outputFileField.getFilePath().getPath();
	}

    public String getToday()
    {
		Calendar cal = Calendar.getInstance();
		SimpleDateFormat  sdf = new SimpleDateFormat("MMM d, yyyy");
		String today = sdf.format(cal.getTime());
		return today;
    }

	public void outputErrors(String outputfile, Vector v)
	{
		if (outputfile == null || v == null) return;
        PrintWriter writer = null;
		try {
			writer = new PrintWriter (new BufferedWriter(new FileWriter(outputfile)));
			String msg = getToday() + "\n";
			writer.println(msg);
			if (type == BATCH_LOADER)
			{
				tab.getLoaderTextArea().append(msg);
				tab.getLoaderTextArea().append("\n");
		    }
		    else
		    {
				tab.getEditorTextArea().append(msg);
				tab.getEditorTextArea().append("\n");
			}

			for (int i=0; i<v.size(); i++)
			{
				msg = (String) v.elementAt(i);
				if (type == BATCH_LOADER)
				{
					tab.getLoaderTextArea().append(msg);
					tab.getLoaderTextArea().append("\n");
			    }
			    else
			    {
					tab.getEditorTextArea().append(msg);
					tab.getEditorTextArea().append("\n");
				}
				writer.println(msg);
			}
            writer.close();
		}catch(Exception e) {
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
			//System.out.println("WARNING: Unable to open the output file " + outputfile + ".");
			try {
				writer.close();
			} catch (Exception ex) {
                Log.getLogger().log(Level.WARNING, "Exception caught", ex);

			}
		}
    }

	private Vector getTokenStr(String value, int length)
	{
		Vector tokenValues = new Vector();
		String s = value + "\t\t\t\t\t\t\t\t\t";
		int n = s.indexOf("\t");
		int i = 0;
		while (n != -1 && i < length)
		{
			String t = s.substring(0, n);
			tokenValues.add(t);
			s = s.substring(n+1);
			i++;
			n = s.indexOf("\t");
		}
		for (i=0; i<length; i++)
		{
			s = (String) tokenValues.elementAt(i);
			if (s.compareTo("") == 0)
			{
				tokenValues.setElementAt("NA", i);
			}
		}
		return tokenValues;
   	}


	public void actionPerformed(ActionEvent event)
	{
		Object action = event.getSource();
		if (action == fStartButton)
		{
			infile = getInfile();
			outfile = getOutfile();

			if (infile == null || infile.compareTo("") == 0)
			{
				JOptionPane.showMessageDialog(null,
						"Please specify an input file.",
						"Error", JOptionPane.ERROR_MESSAGE);
						return;
			}

			else if (outfile == null || outfile.compareTo("") == 0)
			{
				JOptionPane.showMessageDialog(null,
						"Please specify an output file.",
						"Error", JOptionPane.ERROR_MESSAGE);
						return;
			}

			else if (infile.equalsIgnoreCase(outfile))
			{
				JOptionPane.showMessageDialog(null,
						"Invalid inputs.",
						"Error", JOptionPane.ERROR_MESSAGE);
			}

			else
			{
                setVisible(false);
                TaskProgressDialog tpd = null;

                BatchTask task = null;
                if (type == BATCH_LOADER)
                {
					task = new BatchLoadTask(tab, owlModel, infile, outfile);
					task.setType(1);
					Vector w = task.getData(infile);
					Vector u = task.validateData(w);
					if (u == null)
					{
						u = new Vector();
						u.add("Invalid input format.");
						outputErrors(outfile, u);
						tab.enableButton("loadButton_Loader", false);
						tab.enableButton("saveButton_Loader", true);
						tab.enableButton("clearButton_Loader", true);

						JOptionPane.showMessageDialog(null,
							"Invalid input format detected. Please correct all errors and try again.",
							"Error", JOptionPane.ERROR_MESSAGE);
						return;
					}
					else if (u.size() > 0)
					{
						outputErrors(outfile, u);
						tab.enableButton("loadButton_Loader", false);
						tab.enableButton("saveButton_Loader", true);
						tab.enableButton("clearButton_Loader", true);

						JOptionPane.showMessageDialog(null,
							"Invalid input format detected.",
							"Error", JOptionPane.ERROR_MESSAGE);

						return;
					}

					task.openPrintWriter(outfile);
					//task.setType(1);
					tpd = new TaskProgressDialog((JFrame)tab.getTopLevelAncestor(), "Batch Load Progress Status", task);

			    }
			    else if (type == BATCH_EDITOR)
			    {
					task = new BatchEditTask(tab, owlModel, infile, outfile);
					task.setType(2);
					Vector w = task.getData(infile);
					Vector u = task.validateData(w);
					if (u.size() > 0)
					{
						outputErrors(outfile, u);
						tab.enableButton("loadButton_Editor", false);
						tab.enableButton("saveButton_Editor", true);
						tab.enableButton("clearButton_Editor", true);

						JOptionPane.showMessageDialog(null,
							"Invalid input format detected. Please correct all errors and try again.",
							"Error", JOptionPane.ERROR_MESSAGE);
						return;
					}
					task.openPrintWriter(outfile);
					//task.setType(2);
					tpd = new TaskProgressDialog((JFrame)tab.getTopLevelAncestor(), "Batch Edit Progress Status", task);
				}



			    if (tpd != null)
			    {
					tpd.run();
			    }

				dispose();

                if (tpd != null)
                {
					ModalDialog.showMessageDialog(null, "Completed actions: " + tpd.getNumCompleted());
			    }
                try {
                    task.closePrintWriter();
				} catch (Exception e) {

				}

                if (type == BATCH_LOADER)
                {
					if (task.isCancelled())
					{
						tab.enableButton("clearButton_Loader", true);
					}
					else
					{
						tab.enableButton("loadButton_Loader", false);
						tab.enableButton("saveButton_Loader", true);
						tab.enableButton("clearButton_Loader", true);
				    }
			    }
			    else if (type == BATCH_EDITOR)
			    {
					if (task.isCancelled())
					{
						tab.enableButton("clearButton_Editor", true);
					}
					else
					{
						tab.enableButton("loadButton_Editor", false);
						tab.enableButton("saveButton_Editor", true);
						tab.enableButton("clearButton_Editor", true);
				    }
			    }
			}

		}

	}
}
