package gov.nih.nci.protegex.edit;
import java.io.*;
import java.text.*;
import java.sql.*;
import java.lang.Math.*;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import java.net.InetAddress;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.model.Task;

import edu.stanford.smi.protege.ui.*;

import edu.stanford.smi.protegex.owl.ui.profiles.ProfilesManager;
import edu.stanford.smi.protege.util.*;

import edu.stanford.smi.protegex.owl.model.DefaultTaskManager;

import edu.stanford.smi.protege.util.Log;
import java.util.logging.Level;

/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class BatchTask {

    /**
     * Requests that the task be cancelled. This
     * method will only be called if the task
     * can be cancelled.
     */

    static final int TYPE_LOAD = 1;
    static final int TYPE_EDIT = 2;

    NCIEditTab tab = null;
    OWLModel owlModel;
    OWLWrapper wrapper;

    boolean done = false;

    int total = 0;

    String message;
    int max = 10000;
    int min = 0;
    boolean cancelled = false;
    boolean canCancel = true;
    String title = null;

    PrintWriter pw = null;
    int batchtype = 1;

    public BatchTask(NCIEditTab tab, KnowledgeBase kb)
    {
		this.tab = tab;
        this.owlModel = (OWLModel) kb;
        wrapper = tab.getWrapper();
		setMax(10000);
		cancelled = false;
		String title = "Batch Processing";
		setTitle(title);
		setMessage("Batch processing in progress, please wait ...");
	}

    public void initialize()
    {

	}

	public void setType(int type)
	{
		batchtype = type;
	}

    private void setMax(int max)
    {
		this.max = max;
	}


    public void cancelTask()
    {
		closePrintWriter();
        cancelled = true;
	}


    /**
     * Gets the title for this task.
     */

    public void setTitle(String title)
    {
		this.title = title;
	}


    public String getTitle()
    {
		return title;
	}

    public void setMessage(String message)
    {
		this.message = message;
	}

    public String getMessage()
    {
		return message;
	}


    public void setPrintWriter(PrintWriter pw)
    {
		this.pw = pw;
	}

	public PrintWriter openPrintWriter(String outputfile)
	{
		if (outputfile == null) return null;
		try {
			PrintWriter writer = new PrintWriter (new BufferedWriter(new FileWriter(outputfile)));
			//System.out.println("Output file " + outputfile + " opened.");
			this.pw = writer;
			print(getToday() + "\n");

            return writer;
		}catch(Exception e) {
			//Log.getLogger().log(Level.WARNING, "Exception caught", e);
			//System.out.println("WARNING: Unable to open the output file " + outputfile + ".");
			return null;
		}
    }

    public void print(String msg)
    {
		if (pw != null)
		{
			pw.println(msg);
	    }
		if (batchtype == TYPE_LOAD)
		{
			tab.getLoaderTextArea().append(msg);
			tab.getLoaderTextArea().append("\n");

	    }
		else {
			tab.getEditorTextArea().append(msg);
			tab.getEditorTextArea().append("\n");
	    }
	}

	public void closePrintWriter()
	{
		if (pw == null) return;
		try {
			pw.close();
			pw = null;
		}catch(Exception e) {
			Log.getLogger().log(Level.WARNING, "Exception caught", e);
		}
	}


    /**
     * Gets the minimum progress value for
     * this task.
     */
    public int getProgressMin()
    {
		return min;
	}


    /**
     * Gets the maximum progress value for
     * this task.
     */
    public int getProgressMax()
    {
		return max;
	}


    /**
     * Checks whether this Task has been cancelled.
     * Unless either method is overloaded, this will return true after cancelTask
     * has been called (e.g., via the cancel button).
     *
     * @return true  if this has been cancelled
     */
    public boolean isCancelled()
    {
		return cancelled;
	}

    public void setCanCancel(boolean b)
    {
		canCancel = b;
	}

    /**
     * Determines if the task can be cancelled
     *
     * @return <code>true</code> if the task can
     *         be cancelled, or <code>false</code> if the
     *         task cannot be cancelled.
     */
    public boolean isPossibleToCancel()
	{
		return canCancel;
	}


    public boolean processTask(int taskId)
    {
		return true;
	}


    public String getToday()
    {
		Calendar cal = Calendar.getInstance();
		SimpleDateFormat  sdf = new SimpleDateFormat("MMM d, yyyy");
		String today = sdf.format(cal.getTime());
		return today;
    }


	public Vector getData(String filename)
	{
		Vector v = new Vector();
		return v;
	}

	private Vector getTokenStr(String value, int length)
	{
		Vector tokenValues = new Vector();
		String s = value + "\t\t\t\t\t\t\t\t\t";
		int n = s.indexOf("\t");
		int i = 0;
		while (n != -1 && i < length)
		{
			String t = s.substring(0, n);
			tokenValues.add(t);
			s = s.substring(n+1);
			i++;
			n = s.indexOf("\t");
		}
		for (i=0; i<length; i++)
		{
			s = (String) tokenValues.elementAt(i);
			if (s.compareTo("") == 0)
			{
				tokenValues.setElementAt("NA", i);
			}
		}
		return tokenValues;
   	}

    public Vector validateData(Vector v)
    {
	    return new Vector();
	}
}
