package gov.nih.nci.protegex.edit;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;

import java.text.SimpleDateFormat;

import edu.stanford.smi.protegex.owl.ui.ProtegeUI;
import edu.stanford.smi.protegex.owl.ui.dialogs.ModalDialogFactory;
import edu.stanford.smi.protege.util.LabeledComponent;



import edu.stanford.smi.protegex.owl.model.OWLObjectProperty;
import edu.stanford.smi.protegex.owl.model.RDFIndividual;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protege.storage.database.DatabaseKnowledgeBaseFactory;
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protege.event.ClsAdapter;
import edu.stanford.smi.protege.event.ClsEvent;
import edu.stanford.smi.protege.util.CollectionUtilities;
import edu.stanford.smi.protegex.owl.ui.profiles.ProfilesManager;

/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class CustomizedAnnotationConstructor
{
	EditPanel edit_panel;
	JComponent[] components;

	int numfields;
	String[] component_names;
	String[] component_types;
	HashMap  combobox_values;
	HashMap  default_values;


	public CustomizedAnnotationConstructor(EditPanel edit_panel, CustomizedAnnotationData cad)
	{
		this.edit_panel = edit_panel;
		this.component_names = cad.get_component_names();
		this.numfields = component_names.length;
		this.component_types = cad.get_component_types();
		this.combobox_values = cad.get_combobox_values_hmap();
		this.default_values = cad.get_default_values_hmap();
	}



	public CustomizedAnnotationConstructor(	EditPanel edit_panel,
	                                        String[] component_names,
											String[] component_types,
											HashMap  combobox_values,
											HashMap  default_values)
	{
		this.edit_panel = edit_panel;
		this.numfields = component_names.length;
		this.component_names = component_names;
		this.component_types = component_types;
		this.combobox_values = combobox_values;
		this.default_values = default_values;
	}


	public JPanel createPanel(String value)
	{
        String[] values = new String[numfields];

        HashMap hmap = new HashMap();
        if (value != null && value.compareTo("") != 0)
        {
 			value = ComplexPropertyParser.pipeDelim2XML(value);
 		    hmap = ComplexPropertyParser.parseXML(value);
 	    }

 		if (!hmap.isEmpty())
 		{
			for (int i=0; i<numfields; i++)
			{
				values[i] = (String) hmap.get(component_names[i]);
			}
		}

		JPanel panel = new JPanel();
		panel.setLayout(new GridLayout(2, 1));

		JPanel desc_panel = new JPanel();
		desc_panel.setLayout(new BorderLayout());

		components = new JComponent[numfields];
		JLabel[] labels = new JLabel[numfields];

		for (int i=0; i<numfields; i++)
		{
			labels[i] = new JLabel(component_names[i]);
		}

		JTextArea c0 = new JTextArea("");
		boolean wrap = true;
		c0.setLineWrap(wrap);
		c0.setWrapStyleWord(wrap);

		if (values[0] != null)
		{
			c0.setText(values[0]);
		}
		components[0] = (JComponent) c0;
		desc_panel.add(labels[0], BorderLayout.NORTH);
		desc_panel.add(c0, BorderLayout.CENTER);


		for (int i=1; i<numfields; i++)
		{
			//labels[i] = new JLabel(component_names[i]);
			String type = component_types[i];
			if (type.compareTo("JTextArea") == 0)
			{
				if (values[i] != null)
				{
					JTextArea c = new JTextArea(values[i]);
					components[i] = (JComponent) c;
				}
				else
				{
					JTextArea c = new JTextArea("");
					components[i] = new JTextArea("");
				}
			}
			else if (type.compareTo("JTextField") == 0)
			{
				if (values[i] != null)
				{
					JTextField c = new JTextField(values[i]);
					components[i] = (JComponent) c;
				}
				else
				{
					JTextField c = new JTextField("");
					components[i] = (JComponent) c;
					String default_value = (String) default_values.get(component_names[i]);
					if (default_value != null)
					{
						c.setText(default_value);
					}
			    }
			}

			else if (type.compareTo("Date") == 0)
			{
				/*
				if (values[i] != null)
				{
					JTextField c = new JTextField(values[i]);
					c.setEditable(false);
					components[i] = (JComponent) c;
				}
				else
				*/
				{
					SimpleDateFormat sdf = new SimpleDateFormat ("yyMMdd");
					String mydate = sdf.format(new Date());
					JTextField c = new JTextField(mydate);
					c.setEditable(false);
					components[i] = (JComponent) c;
				}
			}


			else if (type.compareTo("User") == 0)
			{
				String username = edit_panel.getUserName();
				JTextField c = new JTextField(username);
				c.setEditable(false);
				components[i] = (JComponent) c;
			}


			else if (type.compareTo("JComboBox") == 0)
			{
				String[] groups = (String[]) combobox_values.get(component_names[i]);
				JComboBox c = new JComboBox(groups);
				if (values[i] != null)
				{
					boolean found = false;
					for (int k =0; k<groups.length; k++){
						if (groups[k].equalsIgnoreCase(values[i]))
						{
							found = true;
							break;
						}
					}
					if (!found)
					{
						c.addItem(values[i]);
					}
					c.setSelectedItem(values[i]);
				}
				else
				{
					String selection = (String) default_values.get(component_names[i]);
					c.setSelectedItem(selection);
				}
				components[i] = (JComponent) c;
			}
		}


		JPanel other_panel = new JPanel();
		other_panel.setLayout(new GridLayout(numfields-1, 2));

		for (int j=1; j<numfields; j++)
		{
			other_panel.add(labels[j]);
			other_panel.add(components[j]);
		}

		panel.add(desc_panel);
		panel.add(other_panel);

		return panel;
	}


	public String getValue(int icomponent)
	{
		String value = "";
		JComponent component = components[icomponent];
		if (component instanceof JTextArea)
		{
			JTextArea c = (JTextArea) component;
			value = c.getText();
		}
		else if (component instanceof JTextField)
		{
			JTextField c = (JTextField) component;
			value = c.getText();
		}
		else if (component instanceof JComboBox)
		{
			JComboBox c = (JComboBox) component;
			value = (String) c.getSelectedItem();
		}
		return value;
    }

	public String getValue()
	{
		String s = "";
		for (int i=0; i<components.length; i++)
		{
			String value = "";
			JComponent component = components[i];
			if (component instanceof JTextArea)
			{
				JTextArea c = (JTextArea) component;
				value = c.getText();
			}
			else if (component instanceof JTextField)
			{
				JTextField c = (JTextField) component;
				value = c.getText();
			}
			else if (component instanceof JComboBox)
			{
				JComboBox c = (JComboBox) component;
				value = (String) c.getSelectedItem();
			}

			String key = (String) component_names[i];
			s = s + "<";
			s = s + key;
			s = s + ">";
			s = s + value;
			s = s + "</";
			s = s + key;
			s = s + ">";
		}
		return s;
	}
}


