package gov.nih.nci.protegex.edit;

import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.tree.*;
import java.awt.*;

import java.text.SimpleDateFormat;

import edu.stanford.smi.protegex.owl.ui.ProtegeUI;
import edu.stanford.smi.protegex.owl.ui.dialogs.ModalDialogFactory;
import edu.stanford.smi.protege.util.LabeledComponent;



import edu.stanford.smi.protegex.owl.model.OWLObjectProperty;
import edu.stanford.smi.protegex.owl.model.RDFIndividual;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protege.storage.database.DatabaseKnowledgeBaseFactory;
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protege.event.ClsAdapter;
import edu.stanford.smi.protege.event.ClsEvent;
import edu.stanford.smi.protege.util.CollectionUtilities;
import edu.stanford.smi.protegex.owl.ui.profiles.ProfilesManager;




/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class CustomizedAnnotationDialog
{
	EditPanel edit_panel;
    private String username;
	private JTextArea desc_ta;
	private JComboBox syn_group_combo, source_combo, def_group_combo;
	private JTextField code_field, attr_field, reviewdate_field, reviewer_field;
    private JTextArea name_field;
	private String newValue = null;

	private JTextField id_field;
	private JTextField source_date_field;
	private JTextField evidence_field;
	private JComboBox go_group_combo;

	public boolean cancelBtnPressed = false;

	private Dimension dimension = new Dimension(500, 300);


	private Vector supportedSlotNames = null;
	private Vector tags = null;

	String title = null;

	static String[] definition_qualifiers = new String[] {
		                                     "Definition_Review_Date",
	                                         "def-source",
	                                         "Definition_Reviewer_Name",
	                                         "attr"};

	static String[] goannotation_qualifiers = new String[] {
		                                     "source-date",
	                                         "go-source",
	                                         "go-evi",
	                                         "go-id"};


	public CustomizedAnnotationDialog(EditPanel edit_panel,
	        String annotation_name,
			Object annotation_value,
			int row,
			NCIAnnotationsTableModel model)
	{
		this.edit_panel = edit_panel;
		cancelBtnPressed = false;
		this.newValue = null;

		this.username = edit_panel.getUserName();
		String label = "";
		JPanel panel = null;
		Component comp = null;
		LabeledComponent lc = null;

		boolean isLiteral = false;
		String language = null;

		String anno_value = "";

		this.supportedSlotNames = new Vector();

		OWLWrapper wrapper = edit_panel.getOWLWrapper();
		OWLModel owlModel = edit_panel.getOWLModel();

		if (annotation_value instanceof RDFSLiteral)
		{
			RDFSLiteral value = (RDFSLiteral) annotation_value;
			anno_value = value.getString();
			isLiteral = true;
			language = value.getLanguage();
		}
		else
		{
			anno_value = (String) annotation_value;
		}


        //  Definitions:
		if (annotation_name.compareTo(edit_panel.getSlotName(NCIEditTab.DEFINITION)) == 0 ||
		    annotation_name.compareTo(edit_panel.getSlotName(NCIEditTab.ALT_DEFINITION)) == 0 ||
		    annotation_name.compareTo(edit_panel.getSlotName(NCIEditTab.LONG_DEFINITION)) == 0 ||
		    annotation_name.compareTo(edit_panel.getSlotName(NCIEditTab.ALT_LONG_DEFINITION)) == 0
		    )
		{
			String tagname_data = "def-definition$Definition_Review_Date$def-source$Definition_Reviewer_Name$attr";
			String component_type_data = "JTextArea$Date: 2007/05/15 19:51:20 $JComboBox$User$JTextField";
			String combobox_data = "$$CADSR|CTCAE|FDA|NCI|NCI-GLOSS$$";
			String default_value_data = "$$NCI$$";

			CustomizedAnnotationData cad = getCustomizedAnnotationData("DEFINITION");
			if (cad != null)
			{
				cad.setAnnotationName(annotation_name);
			}
			else
			{
				cad = new CustomizedAnnotationData(annotation_name,
					 tagname_data, component_type_data, combobox_data, default_value_data);
			}

			//122006
			//String xmlvalue = (String) annotation_value;
			String xmlvalue = anno_value;

			if (annotation_value instanceof RDFSLiteral)
			{
				RDFSLiteral value = (RDFSLiteral) annotation_value;
				xmlvalue = value.getString();
			}

			CustomizedAnnotationConstructor dlg_constructor = new CustomizedAnnotationConstructor(edit_panel, cad);

			panel = dlg_constructor.createPanel(xmlvalue);


			comp = new JScrollPane(panel);
			label = edit_panel.getOWLWrapper().getSlot(annotation_name).getBrowserText();

			lc = new LabeledComponent(label, comp);
			lc.setPreferredSize(dimension);

			String ext_annotation_name = edit_panel.getNCIEditTab().getExternalName(annotation_name);
			title = "Edit " + ext_annotation_name + " Annotation Property";

			if (anno_value == null || anno_value.compareTo("") == 0)
			{
				title = "Create " + ext_annotation_name + " Annotation Property";
		    }

			int r = ProtegeUI.getModalDialogFactory().showDialog(edit_panel, lc, title, ModalDialogFactory.MODE_OK_CANCEL);
			if (r == ModalDialogFactory.OPTION_OK)
			{
                String def_field_value = dlg_constructor.getValue(0);
				//if (desc_ta.getText().compareTo("") == 0)
				if (def_field_value.compareTo("") == 0)
				{
					Component warning_label = new JLabel("Property value is mandatory.");
					LabeledComponent lc2 = new LabeledComponent("", warning_label);
					int k = ProtegeUI.getModalDialogFactory().showDialog(edit_panel, lc2, "WARNING", ModalDialogFactory.MODE_CLOSE);
					return;
				}

                newValue = dlg_constructor.getValue();

				if (row >= 0)
				{
					if (isLiteral)
					{
						RDFProperty property = (RDFProperty) owlModel.getSlot(wrapper.getSlotName(annotation_name));
						Object new_value = wrapper.createNewValue(property, newValue, language);
						if (new_value instanceof RDFSLiteral)
						{
							RDFSLiteral value = (RDFSLiteral) new_value;
							String new_language = value.getLanguage();
						}
                        model.resetValue(annotation_name, annotation_value, (Object) new_value);
					}
					else
					{

						RDFProperty property = (RDFProperty) owlModel.getSlot(wrapper.getSlotName(annotation_name));
						model.resetValue(annotation_name, (Object) anno_value, (Object) newValue);
				    }
                    model.fireTableDataChanged();
					edit_panel.enableSaveButton(true);
				}
			}
			else
			{
				newValue = anno_value;
				cancelBtnPressed = true;
			}

		} else if (annotation_name.compareTo(edit_panel.getSlotName(NCIEditTab.GO_ANNOTATION)) == 0) {
			this.newValue = null;

			String tagname_data = "go-term$source-date$go-source$go-evi$go-id";
			String component_type_data = "JTextArea$Date: 2007/05/15 19:51:20 $JComboBox$JTextField$JTextField";
			String combobox_data = "$$CADSR|CTCAE|FDA|NCI|NCI-GLOSS$$";
			String default_value_data = "$$NCI$";

			CustomizedAnnotationData cad = getCustomizedAnnotationData("GO_ANNOTATION");
			if (cad != null)
			{
				cad.setAnnotationName(annotation_name);
			}
			else
			{
				cad = new CustomizedAnnotationData(annotation_name,
					 tagname_data, component_type_data, combobox_data, default_value_data);
			}


			String xmlvalue = (String) annotation_value;

			if (annotation_value instanceof RDFSLiteral)
			{
				RDFSLiteral value = (RDFSLiteral) annotation_value;
				xmlvalue = value.getString();
			}

			CustomizedAnnotationConstructor dlg_constructor = new CustomizedAnnotationConstructor(edit_panel, cad);

			panel = dlg_constructor.createPanel(xmlvalue);


			comp = new JScrollPane(panel);

			label = edit_panel.getOWLWrapper().getSlot(annotation_name).getBrowserText();
			lc = new LabeledComponent(label, comp);

			lc.setPreferredSize(dimension);
			title = "Edit " + annotation_name + " Annotation Property";
			if (anno_value == null || anno_value.compareTo("") == 0)
			{
				title = "Create " + annotation_name + " Annotation Property";
		    }
			int r = ProtegeUI.getModalDialogFactory().showDialog(edit_panel, lc, title, ModalDialogFactory.MODE_OK_CANCEL);
			if (r == ModalDialogFactory.OPTION_OK)
			{

                String desc_ta_value = dlg_constructor.getValue(0);
                if (desc_ta_value.compareTo("") == 0)
				//if (desc_ta.getText().compareTo("") == 0)
				{
					Component warning_label = new JLabel("Property value is mandatory.");
					LabeledComponent lc2 = new LabeledComponent("", warning_label);
					int k = ProtegeUI.getModalDialogFactory().showDialog(edit_panel, lc2, "WARNING", ModalDialogFactory.MODE_CLOSE);
					return;
				}

				newValue = dlg_constructor.getValue();

				if (row >= 0)
				{
					if (isLiteral)
					{
						RDFProperty property = (RDFProperty) owlModel.getSlot(wrapper.getSlotName(annotation_name));
						Object new_value = wrapper.createNewValue(property, newValue, language);
						if (new_value instanceof RDFSLiteral)
						{
							RDFSLiteral value = (RDFSLiteral) new_value;
							String new_language = value.getLanguage();
						}
                        model.resetValue(annotation_name, annotation_value, (Object) new_value);
					}
					else
					{
						RDFProperty property = (RDFProperty) owlModel.getSlot(wrapper.getSlotName(annotation_name));
						model.resetValue(annotation_name, (Object) anno_value, (Object) newValue);
				    }
                    model.fireTableDataChanged();
					edit_panel.enableSaveButton(true);
				}
			}
			else
			{
				newValue = anno_value;
				cancelBtnPressed = true;
			}

		}
	}



	public CustomizedAnnotationDialog(EditPanel edit_panel,
	        String annotation_name,
			Object annotation_value,
			int row,
			NCIFULLSYNTableModel model)
	{
		this.edit_panel = edit_panel;
		this.newValue = null;
		cancelBtnPressed = false;
		OWLWrapper wrapper = edit_panel.getOWLWrapper();
		OWLModel owlModel = edit_panel.getOWLModel();

		this.supportedSlotNames = new Vector();
		boolean isLiteral = false;
		String language = null;

		String anno_value = "";

		if (annotation_value instanceof RDFSLiteral)
		{
			RDFSLiteral value = (RDFSLiteral) annotation_value;
			anno_value = value.getString();
			isLiteral = true;
			language = value.getLanguage();
		}
		else
		{
			anno_value = (String) annotation_value;
		}

		if (annotation_name.compareTo(edit_panel.getSlotName(NCIEditTab.FULL_SYN)) == 0){

			//String annotation_name = "FULL_SYN";

			String tagname_data = "term-name$term-group$term-source$source-code";
			String component_type_data = "JTextArea$JComboBox$JComboBox$JTextField";
			String combobox_data = "$AB|AD|AQ|AQS|BR|CA2|CA3|CN|CNU|CS|DN|FB|HD|PT|SN|SY$BioCarta|CADSR|CDC|CDISC|CTCAE|CTRM|DCP|DTP|FDA|KEGG|MTH|NCI|NCI-GLOSS|SEER$";
			String default_value_data = "$SY$NCI$";

			CustomizedAnnotationData cad = getCustomizedAnnotationData("FULL_SYN");
			if (cad != null)
			{
				cad.setAnnotationName(annotation_name);
			}
			else
			{
				cad = new CustomizedAnnotationData(annotation_name,
					 tagname_data, component_type_data, combobox_data, default_value_data);
			}

			//CustomizedAnnotationData cad = getCustomizedAnnotationData("FULL_SYN");
			//cad.setAnnotationName(annotation_name);

			//122006
			//String xmlvalue = (String) annotation_value;
			String xmlvalue = anno_value;

			if (annotation_value instanceof RDFSLiteral)
			{
				RDFSLiteral value = (RDFSLiteral) annotation_value;
				xmlvalue = value.getString();
			}

			CustomizedAnnotationConstructor dlg_constructor = new CustomizedAnnotationConstructor(edit_panel, cad);

			JPanel panel = dlg_constructor.createPanel(xmlvalue);

			Component comp = new JScrollPane(panel);

			String label = edit_panel.getOWLWrapper().getSlot(annotation_name).getBrowserText();
			LabeledComponent lc = new LabeledComponent(label, comp);

			lc.setPreferredSize(dimension);
			title = "Edit FULL_SYN Annotation Property";

			if (annotation_value instanceof RDFSLiteral)
			{
				RDFSLiteral value = (RDFSLiteral) annotation_value;
			    anno_value = value.getString();
			}
			else
			{
				anno_value = (String) annotation_value;
			}


			if (anno_value == null || anno_value.compareTo("") == 0)
			{
				title = "Create FULL_SYN Annotation Property";
		    }

			int r = ProtegeUI.getModalDialogFactory().showDialog(edit_panel, lc, title, ModalDialogFactory.MODE_OK_CANCEL);
			if (r == ModalDialogFactory.OPTION_OK) {

                String name_field_value = dlg_constructor.getValue(0);

				//if (name_field.getText().compareTo("") == 0)
				if (name_field_value.compareTo("") == 0)
				{
					Component warning_label = new JLabel("Term name is mandatory.");
					LabeledComponent lc2 = new LabeledComponent("", warning_label);
					int k = ProtegeUI.getModalDialogFactory().showDialog(edit_panel, lc2, "WARNING", ModalDialogFactory.MODE_CLOSE);
					return;
				}

                newValue = dlg_constructor.getValue();

				if (row >= 0)
				{
					if (isLiteral)
					{

						RDFProperty property = (RDFProperty) owlModel.getSlot(wrapper.getSlotName(annotation_name));
						Object new_value = wrapper.createNewValue(property, newValue, language);
						if (new_value instanceof RDFSLiteral)
						{
							RDFSLiteral value = (RDFSLiteral) new_value;
							String new_language = value.getLanguage();
						}
                        model.resetValue(annotation_name, annotation_value, (Object) new_value);

					}
					else
					{

						RDFProperty property = (RDFProperty) owlModel.getSlot(wrapper.getSlotName(annotation_name));
						model.resetValue(annotation_name, (Object) anno_value, (Object) newValue);
				    }
                    model.fireTableDataChanged();
					edit_panel.enableSaveButton(true);

				}
			}
			else
			{
				newValue = anno_value;
				cancelBtnPressed = true;
			}

		}
	}


    public String getValue()
    {
		return newValue;
	}


	private CustomizedAnnotationData getCustomizedAnnotationData(String annotation_name)
	{
		return edit_panel.getNCIEditTab().getCustomizedAnnotationData(annotation_name);
	}


}


