package gov.nih.nci.protegex.edit;

import edu.stanford.smi.protegex.owl.ui.conditions.*;

import edu.stanford.smi.protege.event.FrameAdapter;
import edu.stanford.smi.protege.event.FrameEvent;
import edu.stanford.smi.protege.event.FrameListener;
import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.KnowledgeBase;
import edu.stanford.smi.protege.model.Model;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.model.classparser.OWLClassParser;
import edu.stanford.smi.protegex.owl.model.event.ClassAdapter;
import edu.stanford.smi.protegex.owl.model.event.ClassListener;
import edu.stanford.smi.protegex.owl.model.impl.AbstractRDFSClass;
import edu.stanford.smi.protegex.owl.model.impl.DefaultOWLIntersectionClass;
import edu.stanford.smi.protegex.owl.ui.ProtegeUI;
import edu.stanford.smi.protegex.owl.ui.owltable.OWLTableModel;
import edu.stanford.smi.protegex.owl.ui.widget.OWLUI;

import javax.swing.*;
import javax.swing.table.AbstractTableModel;
import java.util.*;

import java.awt.*;
import edu.stanford.smi.protege.util.LabeledComponent;
import edu.stanford.smi.protegex.owl.ui.dialogs.ModalDialogFactory;

import edu.stanford.smi.protegex.owl.ui.profiles.ProfilesManager;

import edu.stanford.smi.protege.util.Log;
import java.util.logging.Level;

import edu.stanford.smi.protege.util.ApplicationProperties;

/**
 * The TableModel used by the AssertedConditionsWidget.
 *
 * @author NGIT
 */
public class NCIConditionsTableModel extends AbstractTableModel
        implements ConditionsTableConstants, OWLTableModel {

	public static final int SET_RESTRICTION = 1;
	public static final int SET_SUPERCLASS = 2;

	private static final String SHOW_INHERITED_RESTRICTIONS = "restriction.show.inherited";

	int set_type = 0;


    private ClassListener classListener = new ClassAdapter() {
        public void subclassAdded(RDFSClass cls, RDFSClass subclass) {
            //refill();
        }


        public void subclassRemoved(RDFSClass cls, RDFSClass subclass) {
            //refill();
        }


        public void superclassAdded(RDFSClass cls, RDFSClass superclass) {
			//refill();
        }


        public void superclassRemoved(RDFSClass cls, RDFSClass superclass) {
			//refill();
         }
    };


    private FrameListener frameListener = new FrameAdapter() {
        public void ownSlotValueChanged(FrameEvent event) {
            if (event.getSlot().equals(superclassesSlot)) {
                //refill();
            }
        }
    };

    /**
     * The edited class
     */
    private OWLNamedClass hostClass;

    /**
     * One Item object for each row
     */
    private java.util.List init_items = new ArrayList();
    private java.util.List items = new ArrayList();


    private Collection listenedToClses = new ArrayList();
    private Collection equiv_class_operands = new ArrayList();

    private OWLModel owlModel;

    /**
     * Needed to select the most recently edited row after closing the expression editor
     */
    public Cls previouslyEditedCls;

    private Slot superclassesSlot;
    //062906
	EditPanel edit_panel;

    /**
     * Constructs a new ConditionsTableModel with default superslot.
     *
     * @param owlModel the OWLModel
     */
    public NCIConditionsTableModel(OWLModel owlModel) {
        this(((KnowledgeBase) owlModel).getSlot(Model.Slot.DIRECT_SUPERCLASSES));
    }


    /**
     * Constructs a new ConditionsTableModel with customized superslot.
     *
     * @param superclassesSlot the Slot (either inferred or asserted superclasses)
     */
    public NCIConditionsTableModel(Slot superclassesSlot) {
        this.superclassesSlot = superclassesSlot;
        this.owlModel = (OWLModel) superclassesSlot.getKnowledgeBase();
    }


    /**
     * Constructs a new ConditionsTableModel for a given Cls.
     *
     * @param hostCls          the initially displayed class
     * @param superclassesSlot the Slot (either inferred or asserted superclasses)
     */
    public NCIConditionsTableModel(OWLNamedClass hostCls, Slot superclassesSlot) {
        this(superclassesSlot);
        this.set_type = 0;
        setCls(hostCls);
    }

    public NCIConditionsTableModel(OWLNamedClass hostCls, Slot superclassesSlot, int type,
    		EditPanel edit_panel) {
        this(superclassesSlot);
        this.edit_panel = edit_panel;
        this.set_type = type;
        setCls(hostCls);
    }

    public int addEmptyDefinitionBlock() {
        int type = getType(0) + 1;
        if (separatorExists(type)) return -1;
        final NCIConditionsTableItem separator = NCIConditionsTableItem.createSeparator(type);
        int row = 0;
        items.add(row, separator);
        fireTableRowsInserted(row, row);
        return row;
    }


    public int addEmptyRow(int selectedRow) {
        /*if (selectedRow == 0 && !isSeparator(1)) {   // Add new Definition block
            int newType = getType(0) + 1;
            addItem(0, NCIConditionsTableItem.createSeparator(newType));
            addItem(1, NCIConditionsTableItem.createNew(newType));
            fireTableRowsInserted(0, 1);
            return 1;
        }
        else {  */
        NCIConditionsTableItem item = NCIConditionsTableItem.createNew(getType(selectedRow));
        int index = selectedRow + 1;
        addItem(index, item);
        fireTableRowsInserted(index, index);
        return index;
        // }
    }


    private void addItem(int index, NCIConditionsTableItem item) {
        items.add(index, item);
        updateLocalIndices();
    }


    /**
     * Adds a given Item, unless it represents a restriction that has been overloaded
     * in the existing entries.  A restriction is overloaded iff a restriction of the
     * same type (OWLAllValuesFrom, or a cardinality restriction), and restricted slot
     * already exists in the items list.  With AllRestrictions the filler must be
     * a subclass of the filler above.
     *
     * @param aClass    the aClassass to add as an item
     * @param originCls the class where aClassass has been defined
     */

// Note: NCI Restrictions are more general than OWLSomeValuesFrom, etc.


/*
     private void addItemUnlessOverloaded(RDFSClass aClass, OWLNamedClass originCls) {
         if (aClass instanceof OWLRestriction) {
             RDFSClass directType = aClass.getProtegeType();
             RDFProperty property = ((OWLRestriction) aClass).getOnProperty();
             if (aClass instanceof OWLHasValue) {
                 String browserText = aClass.getBrowserText();
                 for (Iterator it = items.iterator(); it.hasNext();) {
                     NCIConditionsTableItem existing = (NCIConditionsTableItem) it.next();
                     if (!existing.isSeparator() && browserText.equals(existing.aClass.getBrowserText())) {
                         return;  // Don't add if entry with same browser text exists
                     }
                 }
             }
             else if (aClass instanceof OWLSomeValuesFrom) {
                 final OWLSomeValuesFrom someRestriction = ((OWLSomeValuesFrom) aClass);

                 if (someRestriction.getFiller() instanceof RDFSClass) {

                     RDFSClass someClass = (RDFSClass) someRestriction.getFiller();
                     String browserText = aClass.getBrowserText();

                     for (Iterator it = items.iterator(); it.hasNext();) {

                         NCIConditionsTableItem existing = (NCIConditionsTableItem) it.next();

						 if (existing.aClass != null)
						 {
							 if (!existing.isSeparator() && existing.aClass instanceof OWLSomeValuesFrom) {
								 if (browserText.equals(existing.aClass.getBrowserText())) {
									 return;  // Don't add if entry with same browser text exists
								 }
								 OWLSomeValuesFrom other = (OWLSomeValuesFrom) existing.aClass;

									if (other.getOnProperty() == null)
									{
									System.out.println("WARNING: other.getOnProperty() == null");
									return;
									}

								 if (other.getOnProperty().equals(property)) {


									 if (other.getFiller() instanceof RDFSClass) {
										 RDFSClass otherSomeClass = (RDFSClass) other.getFiller();
										 if (otherSomeClass.isSubclassOf(someClass)) {
											 return;  // Don't add if OWLSomeValuesFrom with a subclass exists
										 }
									 }
								 }
							 }

					     }
                     }
                 }
             }
             else if (aClass instanceof OWLAllValuesFrom && ((OWLAllValuesFrom) aClass).getFiller() instanceof RDFSClass) {

                 OWLAllValuesFrom newRestriction = (OWLAllValuesFrom) aClass;
                 OWLNamedClass newSourceClass = newRestriction.getOwner();
                 for (Iterator it = items.iterator(); it.hasNext();) {
                     NCIConditionsTableItem existing = (NCIConditionsTableItem) it.next();
                     if (!existing.isSeparator() && directType.equals(existing.aClass.getProtegeType()) &&
                             property.equals(((OWLRestriction) existing.aClass).getOnProperty())) {
                         OWLAllValuesFrom existingRestriction = (OWLAllValuesFrom) existing.aClass;
                         OWLNamedClass existingSourceCls = existingRestriction.getOwner();
                         if (((RDFSClass) existingRestriction.getFiller()).isSubclassOf((RDFSClass) newRestriction.getFiller()) &&
                                 !existingSourceCls.equals(newSourceClass)) {
                             return;
                         }
                     }
                 }
             }
             else {
                 boolean qcr = false;
                 if (aClass instanceof OWLCardinalityBase) {
                     OWLCardinalityBase base = (OWLCardinalityBase) aClass;
                     qcr = base.isQualified();
                 }
                 if (!qcr) {
                     for (Iterator it = items.iterator(); it.hasNext();) {
                         NCIConditionsTableItem existing = (NCIConditionsTableItem) it.next();
                         if (!existing.isSeparator() && directType.equals(existing.aClass.getProtegeType()) &&
                                 property.equals(((OWLRestriction) existing.aClass).getOnProperty())) {
                             return;  // Don't add if entry with same type exists
                         }
                     }
                 }
             }
         }
         addInheritedSeparator();
         items.add(NCIConditionsTableItem.createInherited(aClass, originCls));
    }
*/

    private void addInheritedSeparator() {
        for (Iterator it = items.iterator(); it.hasNext();) {
            NCIConditionsTableItem item = (NCIConditionsTableItem) it.next();
            if (item.isSeparator() && item.getType() == TYPE_INHERITED) {
                return;
            }
        }
        items.add(NCIConditionsTableItem.createSeparator(TYPE_INHERITED));  //-2
    }




    /**
     * Adds a given Cls to the class conditions specified by a given row.
     *
     * @param aClass      the aClassass to add
     * @param selectedRow the row to add to
     * @return true if the row has been added
     */


    public boolean addRow(RDFSClass aClass, int selectedRow) {


        //String browserText = aClass.getBrowserText();

		if (aClass == null)
		{
			 return false;
		}

        String browserText = edit_panel.getNCIEditTab().getOWLWrapper().getInternalName(aClass);
        if (isDefinition(selectedRow))
        {
            boolean retval = addRowAllowMove(aClass, selectedRow);
            return retval;
		}

        else if (!((AbstractRDFSClass) hostClass).hasPropertyValueWithBrowserText(((KnowledgeBase) owlModel).getSlot(Model.Slot.DIRECT_SUPERCLASSES), browserText))
        {
            boolean retval = addRowAllowMove(aClass, selectedRow);
            return retval;

        }
        else {
			boolean retval = addRowAllowMove(aClass, selectedRow);
            return retval;
        }
    }

    public boolean addRow(RDFSClass aClass, int selectedRow, boolean isDefining) {
        //String browserText = aClass.getBrowserText();

        //String browserText = edit_panel.getNCIEditTab().getOWLWrapper().getInternalName(aClass);
/*
        if (isDefinition(selectedRow) ||
                !((AbstractRDFSClass) hostClass).hasPropertyValueWithBrowserText(((KnowledgeBase) owlModel).getSlot(Model.Slot.DIRECT_SUPERCLASSES), browserText)) {
            return addRowAllowMove(aClass, selectedRow, isDefining);
        }
        else {
            return false;
        }

*/

        return addRowAllowMove(aClass, selectedRow, isDefining);
    }


    public boolean addRowAllowMove(RDFSClass aClass, int selectedRow) {
		return addRowAllowMove(aClass, selectedRow, false);
	}




// check
    public boolean addRowAllowMove(RDFSClass aClass, int selectedRow, boolean isDefining) {
		if (aClass.equals(hostClass))
		{
			System.out.println("Invalid input -- restriction not added.");
			return false;
		}

		if (isDefining)
		{
			if (aClass instanceof OWLNamedClass)
			{
				addDefiningSuperclass(aClass, SET_SUPERCLASS);
			}
			else
			{
				addDefiningRestriction(aClass);
			}
		}
		else
		{
			addSuperclass(aClass);
			/*
			if (aClass instanceof OWLNamedClass)
			{
				addSuperclass(aClass, SET_SUPERCLASS);
			}
			else
			{
				addSuperclass(aClass, SET_RESTRICTION);
			}
			*/
		}

		fireTableDataChanged();
        return true;
    }

/*
    public void deleteRow(int index) {
		NCIConditionsTableItem item = (NCIConditionsTableItem) items.get(i);
		if (item.isSeparator())
		{
			return;
		}

        try {
            owlModel.beginTransaction("Delete condition " + getClass(i).getBrowserText() +
                    " from " + getEditedCls().getBrowserText());
            deleteRow(index, false);
        }
        catch (Exception ex) {
            OWLUI.handleError(owlModel, ex);
        }
        finally {
            owlModel.endTransaction();
        }


		items.remove(index);
		fireTableDataChanged();
	}
*/

    public boolean isSeperator(int index)
    {
		NCIConditionsTableItem item = (NCIConditionsTableItem) items.get(index);
		if (item.isSeparator())
		{
			return true;
		}
		return false;
	}

    public void deleteRow(int index) {
		if (index < 0 && index >= items.size()) return;
        int num_superclasses = 0;
        if (set_type == SET_RESTRICTION)
        {
			num_superclasses = getRestrictionCount();
		}
		else
		{
			num_superclasses = getStrictNamedSuperClassCount();
		}

		NCIConditionsTableItem tableItem = (NCIConditionsTableItem) items.get(index);
		if (tableItem.isSeparator())
		{
			System.out.println("Cannot delete a separator ");
			return;
		}
		else if (tableItem.isInherited())
		{
			Component warning_label = new JLabel("Cannot delete a inherited restriction.");
			LabeledComponent lc2 = new LabeledComponent("", warning_label);
			int k = ProtegeUI.getModalDialogFactory().showDialog(null, lc2, "WARNING", ModalDialogFactory.MODE_CLOSE);
			return;
		}
		else
		{
			/*
			boolean is_equivalentclass = false;
			if (index >= 0 && index < items.size())
			{
				RDFSClass aClass = (RDFSClass) tableItem.getCls();//superCls;
				is_equivalentclass = isEquivalentClass(aClass);

				if (!is_equivalentclass)
				{
					if (set_type == SET_SUPERCLASS)
					{
						int num_sups = getStrictNamedSuperClassCount();

						if (num_sups <= 1)
						{
							Component warning_label = new JLabel("Cannot delete the last named superclass.");
							LabeledComponent lc2 = new LabeledComponent("", warning_label);
							int k = ProtegeUI.getModalDialogFactory().showDialog(null, lc2, "WARNING", ModalDialogFactory.MODE_CLOSE);
						}

					}
				}
			}
			*/
			items.remove(index);
			fireTableDataChanged();
		}
    }




    public void displaySemanticError(String message) {
        ProtegeUI.getModalDialogFactory().showErrorMessageDialog(owlModel, message);
    }


    public void dispose() {
        if (hostClass != null) {
            hostClass.removeClassListener(classListener);
            ((Cls) hostClass).removeFrameListener(frameListener);
        }
        removeListeners();
    }


    // Debugging support only
    private void dumpItems() {
        System.err.println("Items:");
        for (Iterator it = items.iterator(); it.hasNext();) {
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) it.next();
            System.err.println(" - " + tableItem + " (" + tableItem.getType() + ")");
        }
    }


    private void fillItems() {
        Collection coveredClses = new HashSet();
        final int classificationStatus = getEditedCls().getClassificationStatus();

        if (superclassesSlot.getName().equals(Model.Slot.DIRECT_SUPERCLASSES) ||
                classificationStatus != OWLNames.CLASSIFICATION_STATUS_UNDEFINED) {

            fillDefinitionItems(coveredClses);


            fillDirectSuperclassItems(coveredClses);

            if (set_type == SET_RESTRICTION)
            {
            	fillInheritedItems(coveredClses);
		    }

            sortItems();
        }

        //071906
		if (items.size() > 2)  // two separators (necessary and sufficient, necessary)
		{
			edit_panel.enableDelEditHB(6, true);
			edit_panel.enableDelEditHB(7, true);
		}
		else
		{
			edit_panel.enableDelEditHB(6, false);
			edit_panel.enableDelEditHB(7, false);
		}

    }


    private boolean separatorExists(int type)
    {
		for (int i=0; i<items.size(); i++)
		{
			NCIConditionsTableItem item = (NCIConditionsTableItem) items.get(i);
            if (item.isSeparator() && item.getType() == type)
            {
				return true;
			}
		}
		return false;
	}




	public boolean isEquivalentClass(RDFSClass aClass)
	{
		if (items == null || items.size() == 0) return false;
        int numSeparators = 0;
        int count = 0;
 		Object[] objs = items.toArray();
 		for (int i=0; i<objs.length; i++)
 		{
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) objs[i];
            if (tableItem.isSeparator())
            {
				numSeparators++;
				if (numSeparators == 2)
				{
					return false;
				}
			}
            else
            {
				String name1 = edit_panel.getNCIEditTab().getBrowserText(tableItem.getCls());
				String name2 = edit_panel.getNCIEditTab().getBrowserText(aClass);
				if (name1.compareTo(name2) == 0)
				{
					return true;
				}
			}
		}
		return false;
	}


    private void fillInheritedItems(Collection coveredClses) {
        coveredClses.removeAll(getNamedDefinitionClses(false));
        for (Iterator it = ((Cls) hostClass).getDirectOwnSlotValues(superclassesSlot).iterator(); it.hasNext();) {
            Cls superCls = (Cls) it.next();
            if (superCls instanceof OWLNamedClass && !superCls.equals(hostClass)) {
                OWLNamedClass namedSuperclass = (OWLNamedClass) superCls;
                fillInheritedItems(namedSuperclass, coveredClses);
                listenedToClses.add(superCls);
                namedSuperclass.addClassListener(classListener);
                ((Cls) namedSuperclass).addFrameListener(frameListener);
            }
        }
    }


    private void fillInheritedItems(OWLNamedClass originCls, Collection coveredClses) {
        fillInheritedAnonymousClses(originCls, coveredClses);
        for (Iterator it = ((Cls) originCls).getDirectOwnSlotValues(superclassesSlot).iterator(); it.hasNext();) {
            Cls ss = (Cls) it.next();
            if (ss instanceof OWLNamedClass && !coveredClses.contains(ss)) {
                coveredClses.add(ss);
                fillInheritedItems((OWLNamedClass) ss, coveredClses);
            }
        }
    }


/*
    private void fillInheritedAnonymousClses(OWLNamedClass originCls, Collection coveredClses) {
        for (Iterator it = originCls.getSuperclasses(false).iterator(); it.hasNext();) {
            Cls ss = (Cls) it.next();
            if (ss instanceof OWLAnonymousClass) {
                if (ss instanceof OWLIntersectionClass) {
                    Collection operands = ((OWLIntersectionClass) ss).getOperands();
                    for (Iterator oit = operands.iterator(); oit.hasNext();) {
                        RDFSClass operand = (RDFSClass) oit.next();
                        if (operand instanceof OWLAnonymousClass) {
                            addItemUnlessOverloaded(operand, originCls);
                        }
                    }
                }
                else if (!coveredClses.contains(ss)) {
                    addItemUnlessOverloaded((OWLAnonymousClass) ss, originCls);
                }
            }
        }
    }
*/

    public int getBlockSize(int blockStartRow) {
        int count = 0;
        int type = getType(blockStartRow - 1);
        for (int i = blockStartRow; i < items.size() && getItem(i).getType() == type; i++) {
            count++;
        }
        return count;
    }




    /**
     * Gets a string of the form <browserText> [& <recursion>] from all
     * rows of a given block.
     *
     * @param type the type
     * @return the string
     */
    public String getBlockText(int type) {
        int i = 0;
        while (isSeparator(i) || getType(i) != type) {
            i++;
        }
        //String str = getClass(i++).getBrowserText();
        String str = edit_panel.getNCIEditTab().getBrowserText(getClass(i++));
        while (i < getRowCount() && getType(i) == type) {

           // str += " & " + getClass(i++).getBrowserText();
              str += " & " + edit_panel.getNCIEditTab().getBrowserText(getClass(i++));

        }
        return str;
    }


    public int getClassRow(RDFSClass cls) {
        if (cls != null) {
            int row = 0;
            for (Iterator it = items.iterator(); it.hasNext(); row++) {
                NCIConditionsTableItem item = (NCIConditionsTableItem) it.next();
                if (item != null && cls.equals(item.aClass)) {
                    return row;
                }
            }
        }
        return -1;
    }


    /**
     * Gets the RDFSClass displayed at a given row.
     *
     * @param row the row to get the Class from
     * @return the RDFSClass there
     */
    public RDFSClass getClass(int row) {
        NCIConditionsTableItem item = getItem(row);
        if (item != null) {
            return item.aClass;
        }
        else {
            return null;
        }
    }


    // Implements TableModel
    public int getColumnCount() {
        return COL_COUNT;
    }


    // Implements TableModel
    public Class getColumnClass(int columnIndex) {
        if (columnIndex == COL_EXPRESSION) {
            return String.class;
        }
        return null;
    }


    /**
     * If a given row displays an operand from an equivalent class intersection,
     * then this method returns the OWLIntersectionClass hosting it.
     *
     * @param rowIndex the index of the row to get the OWLIntersectionClass from
     * @return an OWLIntersectionClass or null
     */
    public OWLIntersectionClass getDefinition(int rowIndex) {
        return getItem(rowIndex).getDefinition();
    }


    public OWLNamedClass getEditedCls() {
        return hostClass;
    }


    public Icon getIcon(RDFResource resource) {
        return ProtegeUI.getIcon(resource);
    }


    public Icon getIcon(int rowIndex, int rowHeight) {
        return getItem(rowIndex).getIcon(rowHeight);
    }


    public NCIConditionsTableItem getItem(int rowIndex) {
        return (NCIConditionsTableItem) items.get(rowIndex);
    }


    private Collection getNamedDefinitionClses(boolean allowDirectSuperclasses) {
        Collection result = new HashSet();
        for (Iterator it = hostClass.getEquivalentClasses().iterator(); it.hasNext();) {
            RDFSClass aClass = (RDFSClass) it.next();
            if (aClass instanceof OWLIntersectionClass) {
                for (Iterator oit = ((OWLIntersectionClass) aClass).getOperands().iterator(); oit.hasNext();) {
                    Cls operand = (Cls) oit.next();
                    if (operand instanceof OWLNamedClass) {
                        result.add(operand);
                    }
                }
            }
            else if (aClass instanceof OWLNamedClass && allowDirectSuperclasses) {
                result.add(aClass);
            }
        }
        return result;
    }


    /**
     * Gets the class where the entry from a given row has been inherited from.
     * This only makes sense for rows which have the type TYPE_INHERITED.
     *
     * @param rowIndex the index of the row to query
     * @return the origin class
     */
    public OWLNamedClass getOriginClass(int rowIndex) {
        return getItem(rowIndex).getOriginCls();
    }


    public RDFProperty getPredicate(int row) {
        if (row == 0 || (row >= 0 && isDefinition(row))) {
            return owlModel.getOWLEquivalentClassProperty();
        }
        return owlModel.getRDFSSubClassOfProperty();
    }


    public RDFResource getRDFResource(int row) {
        return getClass(row);
    }


    public RDFResource getSubject() {
        return hostClass;
    }


    public int getSymbolColumnIndex() {
        return COL_EXPRESSION;
    }


    // Implements TableModel
    public int getRowCount() {
        return items.size();
    }


    /**
     * Gets the type of the entry at a certain row.
     *
     * @param rowIndex the index of the row to get the type of
     * @return one of ConditionsTableColumns.TYPE_xxx
     */
    public int getType(int rowIndex) {
        NCIConditionsTableItem item = getItem(rowIndex);
        if (item != null) {
            return item.getType();
        }
        else {
            return TYPE_SUPERCLASS;
        }
    }


    // Implements TableModel
    public Object getValueAt(int rowIndex, int columnIndex) {
        NCIConditionsTableItem item = getItem(rowIndex);
        if (columnIndex == COL_EXPRESSION) {
            return item.toString();
        }
        else {
            return null;
        }
    }


    private void handleDuplicateEntry(RDFSClass newClass) {
        displaySemanticError("The class " + newClass.getBrowserText() + " is already in the list.");
        if (newClass instanceof OWLAnonymousClass) {
            newClass.delete();
        }
    }


    /**
     * Checks whether the "add named class" function is enabled for a given row.
     *
     * @param rowIndex the index of the row where a named class shall be added
     * @return true if a named class could be added
     */

    public boolean isAddEnabledAt(int rowIndex)
    {
		return true;
	}


    // Implements TableModel
    public boolean isCellEditable(int rowIndex, int columnIndex) {
		/*
        if (columnIndex == COL_EXPRESSION &&
                !isSeparator(rowIndex) &&
                !getItem(rowIndex).isInherited() &&
                superclassesSlot.getName().equals(Model.Slot.DIRECT_SUPERCLASSES)) {
            Cls cls = getClass(rowIndex);
            if (cls == null) {
                return true;
            }
            if (isCreateEnabledAt(rowIndex)) { //getEditedCls().isEditable()) {
                if (cls instanceof OWLAnonymousClass) {
                    return cls.isEditable();
                }
                else {
                    return true;
                }
            }
        }
        */
        return false;
    }

    /**
     * Checks whether an anonymous class could be added/created at a given row.
     *
     * @param row the index of the row to add a new anonymous class
     * @return true if yes
     */
    public boolean isCreateEnabledAt(int row) {
        if (isDefinition(row)) {
            OWLIntersectionClass ic = getDefinition(row);
            if (ic != null) {
                return ic.isEditable();
            }
            else {
                RDFSClass cls = getClass(row);
                if (cls == null) {
                    return isCreateEnabledAt(row + 1);
                }
                else {
                    return owlModel.getTripleStoreModel().isActiveTriple(getEditedCls(),
                            owlModel.getOWLEquivalentClassProperty(),
                            cls);
                }
            }
        }
        else {
            return getType(row) == TYPE_SUPERCLASS;
        }
    }


    public boolean isDeleteEnabledFor(RDFSClass cls) {
        int rowIndex = getClassRow(cls);
        if (cls instanceof OWLAnonymousClass) {
            if (!isSeparator(rowIndex)) {
                return isCellEditable(rowIndex, 0);
            }
            else {
                return false;
            }
        }
        else {
            return isRemoveEnabledFor(rowIndex);
        }
    }


    public boolean isDefinition(int rowIndex) {
        return getItem(rowIndex).isDefinition();
    }


    public boolean isEditable() {
        return hostClass.isEditable();
    }


    public boolean isRemoveEnabledFor(int rowIndex) {
        if (!isSeparator(rowIndex)) {
            RDFSClass superClass = getClass(rowIndex);
            if (superClass instanceof RDFSNamedClass) {
                if (getItem(rowIndex).isDefinition()) {
                    OWLIntersectionClass ic = getDefinition(rowIndex);
                    if (ic != null) {
                        return ic.isEditable();
                    }
                }
                OWLNamedClass editedClass = getEditedCls();
                if (owlModel.getTripleStoreModel().isEditableTriple(editedClass,
                        owlModel.getRDFSSubClassOfProperty(),
                        superClass) ||
                        owlModel.getTripleStoreModel().isEditableTriple(editedClass,
                                owlModel.getOWLEquivalentClassProperty(),
                                superClass)) {
                    Collection nss = new HashSet(hostClass.getNamedSuperclasses());
                    if (nss.contains(superClass)) {
                        nss.remove(superClass);
                        return nss.size() > 0 ||
                                !owlModel.getOWLThingClass().equals(superClass);
                    }
                }
            }
        }
        return false;
    }


    public boolean isSeparator(int rowIndex) {
        return getItem(rowIndex).isSeparator();
    }


    private void refill() {
        items.clear();
        removeListeners();
        fillItems();
        fireTableDataChanged();
    }


    public void refresh() {
        items.clear();
        if (hostClass != null) {
            fillItems();
        }
        fireTableDataChanged();
    }


    public void removeEmptyRow() {
        for (int i = getRowCount() - 1; i >= 0; i--) {
            if (getItem(i).isNew()) {
                items.remove(i);
                fireTableRowsDeleted(i, i);
            }
        }
        updateLocalIndices();
    }


    private void removeListeners() {
        for (Iterator it = listenedToClses.iterator(); it.hasNext();) {
            RDFSClass cls = (RDFSClass) it.next();
            cls.removeClassListener(classListener);
            ((Cls) cls).removeFrameListener(frameListener);
        }
        listenedToClses.clear();
    }


    private void replaceItemType(int oldType, int newType) {
        for (Iterator it = items.iterator(); it.hasNext();) {
            NCIConditionsTableItem item = (NCIConditionsTableItem) it.next();
            if (item.getType() == oldType) {
                item.setType(newType);
            }
        }
    }





    public void setValueAt(Object value, int rowIndex, int columnIndex) {
        if (rowIndex >= getRowCount()) {
            return;
        }
        if (columnIndex == COL_EXPRESSION) {
            String text = (String) value;
            OWLModel owlModel = hostClass.getOWLModel();
            try {
                setValueAt(rowIndex, owlModel, text);
            }
            catch (Exception ex) {
                Log.getLogger().log(Level.WARNING, "Exception caught", ex);;
            }
        }
    }


    public void setValueAt(int rowIndex, OWLModel owlModel, String parsableText)
            throws Exception {

        try {
            owlModel.beginTransaction("NCIEditTab: Set condition at " + getEditedCls().getBrowserText() + " to " + parsableText, getEditedCls().getName());

            OWLClassParser parser = owlModel.getOWLClassDisplay().getParser();
            RDFSClass newClass = parser.parseClass(owlModel, parsableText);
            if (getEditedCls().equals(newClass)) {
				// 050707
				owlModel.rollbackTransaction();
                return;
            }

            NCIConditionsTableItem item = getItem(rowIndex);
            item.setCls(newClass);

            owlModel.commitTransaction();
        }
        catch (Exception ex) {
        	owlModel.rollbackTransaction();
            OWLUI.handleError(owlModel, ex);
        }
    }


    /**
     * Sorts the items according to their <CODE>compareTo</CODE> method.
     *
     * @see NCIConditionsTableItem#compareTo
     */
    private void sortItems() {
        Collections.sort(items);
        updateLocalIndices();
    }


    /**
     * Assumes that the items list contains nothing but at least two non-
     * empty definition blocks.
     */
    private void sortSufficientBlocks(java.util.List separators) {
        boolean changed = false;
        do {
            changed = false;
            for (int i = 0; i < separators.size() - 1; i++) {
                NCIConditionsTableItem separatorA = (NCIConditionsTableItem) separators.get(i);
                int indexA = items.indexOf(separatorA);
                NCIConditionsTableItem a = getItem(indexA + 1);
                NCIConditionsTableItem separatorB = (NCIConditionsTableItem) separators.get(i + 1);
                int indexB = items.indexOf(separatorB);
                NCIConditionsTableItem b = getItem(indexB + 1);
                if (a.compareToWithSameType(b.getCls()) < 0) {
                    changed = true;
                    swapBlocks(separatorA, separatorB);
                    separators.remove(separatorB);
                    separators.add(i, separatorB);
                }
            }
        }
        while (changed);
    }


    private void swapBlocks(NCIConditionsTableItem separatorA, NCIConditionsTableItem separatorB) {
        int typeA = separatorA.getType();
        int typeB = separatorB.getType();
        replaceItemType(typeA, -10);
        replaceItemType(typeB, typeA);
        replaceItemType(-10, typeB);
        sortItems();
    }


    private void updateLocalIndices() {
        int row = 0;
        for (Iterator it = items.iterator(); it.hasNext(); row++) {
            NCIConditionsTableItem item = (NCIConditionsTableItem) it.next();
            if (item.isSeparator()) {
                updateLocalIndices(row + 1);
            }
        }
    }

////////////////////////////////////////////////////////////////////////////////////////////////

    private void updateLocalIndices(int startRow) {
        for (int i = startRow; i < items.size() && !getItem(i).isSeparator(); i++) {
            getItem(i).setLocalIndex(i - startRow);
        }
    }

    private void fillInitItems()
    {
		init_items.clear();
		if (hostClass == null) return;
        for (Iterator it = items.iterator(); it.hasNext();)
        {
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) it.next();
            //System.err.println(" - " + tableItem + " (" + tableItem.getType() + ")");

			 if (!tableItem.isSeparator())
			 {
				 if (tableItem.getCls() != null)
				 {
					 init_items.add(tableItem.createClone());
				 }
			}
		}
    }

    public void dumpInitItems() {
		if (hostClass == null) return;
        System.err.println("Initial Items of " + hostClass.getBrowserText());
        for (Iterator it = init_items.iterator(); it.hasNext();) {
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) it.next();
            System.err.println(" - " + tableItem + " (" + tableItem.getType() + ")");
        }
    }


    private boolean hasSuperclass(RDFSClass aClass, int type)
    {
		if (hostClass == null || aClass == null) return false;
		if (items == null || items.size() == 0) return false;
        for (Iterator it = items.iterator(); it.hasNext();) {
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) it.next();
            if (!tableItem.isSeparator())
            {

				String str1 = edit_panel.getNCIEditTab().getBrowserText(tableItem.getCls());
				String str2 = edit_panel.getNCIEditTab().getBrowserText(aClass);
				if (str1.compareTo(str2) == 0

				//if (tableItem.getCls().getBrowserText().compareTo(aClass.getBrowserText()) == 0


					&& tableItem.getType() == type)
				{
					return true;
				}
			}
        }
		return false;
	}


    private boolean removeSuperclass(RDFSClass aClass, int type)
    {
		if (hostClass == null || aClass == null) return false;
		if (items == null || items.size() == 0) return false;

 		Object[] objs = items.toArray();
 		for (int i=0; i<objs.length; i++)
 		{
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) objs[i];

			String str1 = edit_panel.getNCIEditTab().getBrowserText(tableItem.getCls());
			String str2 = edit_panel.getNCIEditTab().getBrowserText(aClass);
			if (str1.compareTo(str2) == 0

            //if (tableItem.getCls().getBrowserText().compareTo(aClass.getBrowserText()) == 0
                && tableItem.getType() == type)
			{
				items.remove(i);
				return true;
			}
		}
		//dumpItems();

		return false;
	}



    private int getFirstNonSeperatorItem()
    {
		if (hostClass == null) return -1;
		if (items == null || items.size() == 0) return -1;

 		Object[] objs = items.toArray();
 		for (int i=0; i<objs.length; i++)
 		{
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) objs[i];
            if (tableItem.getType() == TYPE_SUPERCLASS) return i;
		}
		return -1;
	}


    public void setCls(OWLNamedClass cls) {

		if (items != null && items.size() > 0)
		{
			int index = getFirstNonSeperatorItem();
			while (index != -1)
			{
				items.remove(index);
				index = getFirstNonSeperatorItem();
			}
			fireTableDataChanged();
			items.clear();
	    }

		if (hostClass != null) {
			hostClass.removeClassListener(classListener);
			((Cls) hostClass).removeFrameListener(frameListener);
		}

        hostClass = cls;
	    if (hostClass != null) {
			hostClass.addClassListener(classListener);
			((Cls) hostClass).addFrameListener(frameListener);

			fillItems();
			fireTableDataChanged();

            fillInitItems();
        }
    }

    public Collection getItems()
    {
		ArrayList a = new ArrayList();
        for (Iterator it = items.iterator(); it.hasNext();) {
            NCIConditionsTableItem item = (NCIConditionsTableItem) it.next();
            if (!item.isSeparator()) {
                a.add(item);
            }
        }
        return a;
	}

    public Collection getInitialItems()
    {
		return init_items;
	}




	public int getNamedSuperClassCount()
	{
		if (hostClass == null) return -1;
		if (items == null || items.size() == 0) return 0;

        int count = 0;
 		Object[] objs = items.toArray();
 		for (int i=0; i<objs.length; i++)
 		{
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) objs[i];
            if (tableItem.getType() == TYPE_SUPERCLASS)
            {
				RDFSClass sup = (RDFSClass) tableItem.getCls();
				if (sup instanceof OWLNamedClass)// && !tableItem.isDefining) // testing
				{
					count++;
				}
			}
		}
		return count;
	}


	public int getRestrictionCount()
	{
		if (hostClass == null) return -1;
		if (items == null || items.size() == 0) return 0;

        int count = 0;
 		Object[] objs = items.toArray();
 		for (int i=0; i<objs.length; i++)
 		{
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) objs[i];
            if (tableItem.getType() == TYPE_SUPERCLASS)
            {
				RDFSClass sup = (RDFSClass) tableItem.getCls();
				if (sup instanceof OWLAnonymousClass)
				{
					count++;
				}
			}
		}
		return count;
	}

/*
	public int getDefinitionCount()
	{
		if (hostClass == null) return -1;
		if (items == null || items.size() == 0) return 0;

        int count = 0;
 		Object[] objs = items.toArray();
 		for (int i=0; i<objs.length; i++)
 		{
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) objs[i];
            if (tableItem.isDefining())
            {
				count++;
			}
		}
		return count;
	}
*/


	public int findFirstRowInNecessaryBlock()
	{
		if (hostClass == null) return -1;
		if (items == null || items.size() == 0) return 0;
        int numSeparators = 0;
        int count = 0;
 		Object[] objs = items.toArray();
 		for (int i=0; i<objs.length; i++)
 		{
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) objs[i];
            if (tableItem.isSeparator())
            {
				numSeparators++;
				if (numSeparators == 2)
				{
					int retval = i + 1;
					return i + 1;
				}
			}
		}
		return objs.length;
	}


	public int getDefinitionCount()
	{
		if (hostClass == null) return -1;
		if (items == null || items.size() == 0) return 0;
        int numSeparators = 0;
        int count = 0;
 		Object[] objs = items.toArray();
 		for (int i=0; i<objs.length; i++)
 		{
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) objs[i];
            if (tableItem.isSeparator())
            {
				numSeparators++;
				if (numSeparators == 2) return count;
			}
            else
            {
				count++;
			}
		}
		return count;
	}




    public boolean addEquivalentClass(RDFSClass aClass)
    {
		if (hostClass == null || aClass == null) return false;
		if (items == null || items.size() == 0) return false;
		/*
		if (hasSuperclass(aClass, type))
		{
			return false;
		}
		*/

    	NCIConditionsTableItem item = NCIConditionsTableItem.create(aClass, 0);
    	item.setIsDefining(true);

		items.add(item);

		sortItems();

		java.util.List separators = new ArrayList();

 		Object[] objs = items.toArray();
 		for (int i=0; i<objs.length; i++)
 		{
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) objs[i];
            if (tableItem.isSeparator())
            {
				separators.add(tableItem);
			}
		}

		//sortSufficientBlocks(separators);
		fireTableDataChanged();
		//dumpItems();

		return true;
	}


    public boolean addDefiningRestriction(RDFSClass aClass)
    {
		if (hostClass == null || aClass == null) return false;
		if (items == null || items.size() == 0) return false;

        RDFSClass newCls = null;
        if (aClass != null)
        {
        	//clone not necessary
        	//Log.getLogger().warning("Creating clone of " + aClass.getBrowserText());
            //newCls = aClass.createClone();
        	newCls = aClass;
	    }

    	NCIConditionsTableItem item = NCIConditionsTableItem.create(newCls, 0);

    	item.setIsDefining(true);
    	items.add(1, item);
		fireTableDataChanged();
		return true;
	}


    private boolean addDefiningSuperclass(RDFSClass aClass, int type)
    {
		if (hostClass == null || aClass == null) return false;
		if (items == null || items.size() == 0) return false;

    	NCIConditionsTableItem item = NCIConditionsTableItem.create(aClass, -1);
    	item.setIsDefining(true);

    	//item.setDefinition();
		items.add(1, item);

		fireTableDataChanged();
		//dumpItems();


		return true;
	}

    public String getRestrictionModifier(OWLWrapper wrapper, NCIConditionsTableItem item)
    {
		if (!(item.getCls() instanceof OWLRestriction)) return null;
		OWLRestriction cls = (OWLRestriction) item.getCls();
        return wrapper.getRestrictionType(cls);
	}

    public String getRestrictionName(OWLWrapper wrapper, NCIConditionsTableItem item)
    {
		if (!(item.getCls() instanceof OWLRestriction)) return null;
		OWLRestriction cls = (OWLRestriction) item.getCls();
		RDFProperty property = (RDFProperty) cls.getOnProperty();
		return property.getBrowserText();
	}

    public String getRestrictionValue(OWLWrapper wrapper, NCIConditionsTableItem item)
    {
		if (!(item.getCls() instanceof OWLRestriction)) return null;
		OWLRestriction cls = (OWLRestriction) item.getCls();
		return cls.getFillerText();
	}

    public boolean itemInSet(NCIConditionsTableItem item, java.util.List list)
    {
		if (list.isEmpty()) return false;
		for (int i=0; i<list.size(); i++)
		{
			NCIConditionsTableItem nextItem = (NCIConditionsTableItem) list.get(i);
			if (sameItem(item, nextItem)) return true;
		}
		return false;
	}

    public boolean sameItem(NCIConditionsTableItem item_1, NCIConditionsTableItem item_2)
    {
		// aClass

		String str1 = edit_panel.getNCIEditTab().getBrowserText(item_1.aClass);
		String str2 = edit_panel.getNCIEditTab().getBrowserText(item_2.aClass);
		if (str1.compareTo(str2) != 0) return false;


        //if (item_1.aClass.getBrowserText().compareTo(item_2.aClass.getBrowserText()) != 0) return false;

        // boolean isDefining
        if (item_1.isDefining() && !item_2.isDefining()) return false;
        if (!item_1.isDefining() && item_2.isDefining()) return false;

        // int type
        if (item_1.getType() != item_2.getType()) return false;

        // OWLIntersectionClass definition;
        if (item_1.getDefinition() == null && item_2.getDefinition() != null) return false;
        if (item_1.getDefinition() != null && item_2.getDefinition() == null) return false;
        if (item_1.getDefinition().getBrowserText().compareTo(
			item_2.getDefinition().getBrowserText()) != 0) return false;

        return true;
	}

    public java.util.List getNewItems()
    {
		java.util.List list = new ArrayList();
		for (int i=0; i<items.size(); i++)
		{
			NCIConditionsTableItem item = (NCIConditionsTableItem) items.get(i);
			if (!itemInSet(item, init_items))
			{
				list.add(item);
			}
		}
		return list;
	}

    public java.util.List getDeletedItems()
    {
		java.util.List list = new ArrayList();
		for (int i=0; i<init_items.size(); i++)
		{
			NCIConditionsTableItem item = (NCIConditionsTableItem) items.get(i);
			if (!itemInSet(item, items))
			{
				list.add(item);
			}
		}
		return list;
	}

/*
    private boolean addSuperclass(RDFSClass aClass, int type)
    {
		if (hostClass == null || aClass == null) return false;
		if (items == null || items.size() == 0) return false;

    	NCIConditionsTableItem item = NCIConditionsTableItem.create(aClass, -1);
		int index = findFirstRowInNecessaryBlock();

		items.add(index, item);
		return true;

	}
*/
    private boolean addSuperclass(RDFSClass aClass)
    {
		if (hostClass == null || aClass == null) return false;
		if (items == null || items.size() == 0) return false;

    	NCIConditionsTableItem item = NCIConditionsTableItem.create(aClass, -1);
		int index = findFirstRowInNecessaryBlock();

		items.add(index, item);
		return true;

	}

	public int replaceNamedSuperClass(String oldClsName, String newClsName, RDFSClass aClass)
	{
		if (newClsName == null) return 0;
		if (items == null || items.size() == 0) return 0;

        int count = 0;
 		Object[] objs = items.toArray();
 		for (int i=0; i<objs.length; i++)
 		{
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) objs[i];
            if (tableItem.getType() == TYPE_SUPERCLASS)
            {
				RDFSClass sup = (RDFSClass) tableItem.getCls();

				if (sup == null)
				{
					System.out.println("WARNING: sup == null");
				}


				if (sup instanceof OWLNamedClass)// && !tableItem.isDefining) // testing
				{


					if (sup.getBrowserText().equals(oldClsName))
					{
						//tableItem.setCls((RDFSClass) newCls);


						tableItem.setCls((RDFSClass) aClass);
						count++;
					}

				}
			}
		}

		if (count > 0)
		{
			fireTableDataChanged();
		}
		return count;
	}


    public String getTableItemName(int row) {
        RDFSClass cls = (RDFSClass) getClass(row);
        if (cls == null) return null;
        return cls.getBrowserText();
    }


    public void updateTableItem(int selIndex, RDFSClass aClass)
    {
		NCIConditionsTableItem tableItem = (NCIConditionsTableItem) this.getItem(selIndex);
		tableItem.setCls(aClass);
		fireTableDataChanged();

	}

    public void updateTableItem(int selIndex, RDFSClass aClass, boolean isDefining)
    {
		NCIConditionsTableItem tableItem = (NCIConditionsTableItem) this.getItem(selIndex);
		tableItem.setCls(aClass);
		tableItem.setIsDefining(isDefining);
		fireTableDataChanged();

	}


    public boolean containsDefinition(OWLNamedClass hostClass, RDFSClass aClass)
    {
		if (aClass == null) return false;

		RDFSClass definition = null;
		Collection c = hostClass.getEquivalentClasses();

		if (c.size() == 0) return false;

		Iterator it = c.iterator();
		while (it.hasNext())
		{
			definition = (RDFSClass) it.next();
			break;
		}

		if (definition.getBrowserText().compareTo(aClass.getBrowserText()) == 0)
		{
			return true;
		}
		if (definition instanceof OWLIntersectionClass)
		{
			OWLIntersectionClass intersectionCls = (OWLIntersectionClass) definition;
			if (intersectionCls.hasOperandWithBrowserText(aClass.getBrowserText()))
			    return true;
		}
		return false;
	}





    private void fillDirectSuperclassItems(Collection coveredClses) {
        items.add(NCIConditionsTableItem.createSeparator(TYPE_SUPERCLASS));
		if (set_type == SET_SUPERCLASS)
		{
			for (Iterator it = ((Cls) hostClass).getDirectOwnSlotValues(superclassesSlot).iterator(); it.hasNext();) {
				Cls superCls = (Cls) it.next();
				if (superCls instanceof OWLNamedClass && !coveredClses.contains(superCls)) {

					if (!containsDefinition(hostClass, (RDFSClass) superCls))
					{
						RDFSClass aClass = (RDFSClass) superCls;
						coveredClses.add(aClass);
						NCIConditionsTableItem item = NCIConditionsTableItem.create(aClass, TYPE_SUPERCLASS);
						item.setCls(aClass);
						item.setIsDefining(false);
						items.add(NCIConditionsTableItem.create(aClass, TYPE_SUPERCLASS));

				    }
				}
			}

	    }
	    else if (set_type == SET_RESTRICTION)
	    {
			for (Iterator it = hostClass.getSuperclasses(false).iterator(); it.hasNext();) {
				Cls superCls = (Cls) it.next();
				if (superCls instanceof OWLAnonymousClass && !coveredClses.contains(superCls)) {
					if (!containsDefinition(hostClass, (RDFSClass) superCls))
					{
						RDFSClass aClass = (RDFSClass) superCls;
						coveredClses.add(aClass);
						NCIConditionsTableItem item = NCIConditionsTableItem.create(aClass, TYPE_SUPERCLASS);
						item.setCls(aClass);
						item.setIsDefining(false);
						items.add(NCIConditionsTableItem.create(aClass, TYPE_SUPERCLASS));
				    }
				}
			}
	    }
    }

    // 112206, KLO
    private boolean containsSimpleRestrictionsOnly(RDFSClass equivalentClass)
    {
		if (equivalentClass instanceof OWLIntersectionClass)
		{
			Collection operands = ((OWLIntersectionClass) equivalentClass).getOperands();
			for (Iterator oit = operands.iterator(); oit.hasNext();) {
				RDFSClass operand = (RDFSClass) oit.next();
				if (operand instanceof OWLIntersectionClass) return false;
				else if (operand instanceof OWLUnionClass) return false;
				else if (operand instanceof OWLNamedClass) return false;
			}
			return true;
		}
		else if (equivalentClass instanceof OWLUnionClass)
		{
			return false;
		}
		else
		{
			return false;
		}
	}


    private void fillDefinitionItems(Collection coveredClses) {
        java.util.List separators = new ArrayList();
        Slot slot = ((KnowledgeBase) hostClass.getOWLModel()).getSlot(Model.Slot.DIRECT_SUPERCLASSES);

        int index = TYPE_DEFINITION_BASE;
                    NCIConditionsTableItem separator0 = NCIConditionsTableItem.createSeparator(index);
						items.add(separator0);
						separators.add(separator0);

        boolean first = true;
        for (Iterator it = ((Cls) hostClass).getDirectOwnSlotValues(slot).iterator(); it.hasNext();) {
            Cls superCls = (Cls) it.next();
            if (superCls instanceof RDFSClass && superCls.getDirectOwnSlotValues(slot).contains(hostClass)) {

                if (!first) {
                    NCIConditionsTableItem separator = NCIConditionsTableItem.createSeparator(index);
					int knt = countSeperators(NCIConditionsTableItem.SUFFICIENT);
					items.add(separator);
					separators.add(separator);
                }

                first = false;
                RDFSClass equivalentClass = (RDFSClass) superCls;
                coveredClses.add(equivalentClass);


/*
                if (equivalentClass instanceof OWLIntersectionClass) {

                    OWLIntersectionClass intersectionCls = (OWLIntersectionClass) equivalentClass;


System.out.println("===== NCIConditionTableModel: fillDefinitionItems " + intersectionCls.getBrowserText());


                    Collection operands = ((OWLIntersectionClass) equivalentClass).getOperands();


System.out.println("===== NCIConditionTableModel: fillDefinitionItems operands " + operands.size());


                    for (Iterator oit = operands.iterator(); oit.hasNext();) {
                        RDFSClass operand = (RDFSClass) oit.next();
						if (set_type == SET_RESTRICTION && operand instanceof OWLAnonymousClass)
						{
							coveredClses.add(operand);

							NCIConditionsTableItem item = NCIConditionsTableItem.createSufficient(operand, index, intersectionCls);
							item.setIsDefining(true);
							items.add(item);

					    }

					    else if (set_type == SET_SUPERCLASS && operand instanceof OWLNamedClass)
					    {
							coveredClses.add(operand);
							NCIConditionsTableItem item = NCIConditionsTableItem.createSufficient(operand, index, intersectionCls);
							item.setIsDefining(true);
							items.add(item);
						}
                    }
                }
*/

                if (equivalentClass instanceof OWLIntersectionClass) {
					OWLIntersectionClass intersectionCls = (OWLIntersectionClass) equivalentClass;
                    if (set_type == SET_RESTRICTION && containsSimpleRestrictionsOnly(equivalentClass))
                    {
						coveredClses.add(intersectionCls);
						NCIConditionsTableItem item = NCIConditionsTableItem.createSufficient(intersectionCls, index, intersectionCls);
						item.setIsDefining(true);
						items.add(item);
					}

                    else
                    {
                        Collection operands = ((OWLIntersectionClass) equivalentClass).getOperands();
						for (Iterator oit = operands.iterator(); oit.hasNext();) {
							RDFSClass operand = (RDFSClass) oit.next();
							if (set_type == SET_RESTRICTION && operand instanceof OWLAnonymousClass)
							{
								coveredClses.add(operand);
								NCIConditionsTableItem item = NCIConditionsTableItem.createSufficient(operand, index, intersectionCls);
								item.setIsDefining(true);
								items.add(item);

							}

							else if (set_type == SET_SUPERCLASS && operand instanceof OWLNamedClass)
							{
								coveredClses.add(operand);
								NCIConditionsTableItem item = NCIConditionsTableItem.createSufficient(operand, index, intersectionCls);
								item.setIsDefining(true);
								items.add(item);
							}
						}
 				    }
                }

                else {

					if (set_type == SET_RESTRICTION && equivalentClass instanceof OWLAnonymousClass)
					{
						NCIConditionsTableItem item = NCIConditionsTableItem.create(equivalentClass, index);
						item.setIsDefining(true);
                    	items.add(item);
					}
					else if (set_type == SET_SUPERCLASS && equivalentClass instanceof OWLNamedClass)
					{
						NCIConditionsTableItem item = NCIConditionsTableItem.create(equivalentClass, index);
						item.setIsDefining(true);
						items.add(item);
					}
                }
                index++;
            }
        }
        /*
        if (separators.size() > 1) {
            sortItems();
            sortSufficientBlocks(separators);
        }
        */
    }





	public int getStrictNamedSuperClassCount()
	{
		if (set_type != SET_SUPERCLASS)
		{
			return 0;
		}
		if (hostClass == null) return -1;

		if (items == null || items.size() == 0)
		{
			return 0;
		}

        int count = 0;
 		Object[] objs = items.toArray();
 		for (int i=0; i<objs.length; i++)
 		{
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) objs[i];

			RDFSClass sup = (RDFSClass) tableItem.getCls();
			//if (sup instanceof OWLNamedClass && !tableItem.getIsDefining()) // testing
			if (sup instanceof OWLNamedClass) // testing
			{
				//System.out.println(sup.getBrowserText());
				count++;
			}
		}
		return count;
	}


    public void modifyRow(int index, RDFSClass aClass, boolean isDefining)
    {
		deleteRow(index);
		if (isDefining)
		{
			if (aClass instanceof OWLNamedClass)
			{
				addDefiningSuperclass(aClass, SET_SUPERCLASS);
			}
			else
			{
				addDefiningRestriction(aClass);
			}
		}
		else
		{
			addSuperclass(aClass);
		}
		fireTableDataChanged();
	}


/*
    public void modifyRow2(int index, RDFSClass aClass, boolean isDefining)
    {
		NCIConditionsTableItem tableItem = (NCIConditionsTableItem) items.get(index);
        tableItem.setCls(aClass);
        tableItem.setIsDefining(isDefining);
        fireTableDataChanged();
	}
*/

	private int countSeperators(String separatorString)
	{
		int knt = 0;
 		Object[] objs = items.toArray();
 		for (int i=0; i<objs.length; i++)
 		{
            NCIConditionsTableItem tableItem = (NCIConditionsTableItem) objs[i];
            if (tableItem.toString().compareTo(separatorString) == 0)
            {
				knt++;
			}
		}
		return knt;
	}



// New code from TT, 030607 (inherited restrictions)
    private void addItemUnlessOverloaded(RDFSClass aClass, OWLNamedClass originCls) {
        if (aClass instanceof OWLRestriction) {
            RDFSClass directType = aClass.getProtegeType();
            RDFProperty property = ((OWLRestriction) aClass).getOnProperty();
            if (aClass instanceof OWLHasValue) {
                String browserText = aClass.getBrowserText();
                for (Iterator it = items.iterator(); it.hasNext();) {
                    //ConditionsTableItem existing = (ConditionsTableItem) it.next();
                    NCIConditionsTableItem existing = (NCIConditionsTableItem) it.next();
                    if (!existing.isSeparator() && browserText.equals(existing.aClass.getBrowserText())) {
                        return;  // Don't add if entry with same browser text exists
                    }
                }
            }
            else if (aClass instanceof OWLSomeValuesFrom) {
                final OWLSomeValuesFrom someRestriction = ((OWLSomeValuesFrom) aClass);
                if (someRestriction.getFiller() instanceof RDFSClass) {
                    RDFSClass someClass = (RDFSClass) someRestriction.getFiller();
                    String browserText = aClass.getBrowserText();
                    for (Iterator it = items.iterator(); it.hasNext();) {
                        //ConditionsTableItem existing = (ConditionsTableItem) it.next();
                        NCIConditionsTableItem existing = (NCIConditionsTableItem) it.next();
                        if (!existing.isSeparator() && existing.aClass instanceof OWLSomeValuesFrom) {
                            if (browserText.equals(existing.aClass.getBrowserText())) {
                                return;  // Don't add if entry with same browser text exists
                            }
                            OWLSomeValuesFrom other = (OWLSomeValuesFrom) existing.aClass;
                            if (other.getOnProperty().equals(property)) {
                                if (other.getFiller() instanceof RDFSClass) {
                                    RDFSClass otherSomeClass = (RDFSClass) other.getFiller();
                                    if (otherSomeClass.isSubclassOf(someClass)) {
                                        return;  // Don't add if OWLSomeValuesFrom with a subclass exists
                                    }
                                }
                            }
                        }
                    }
                }
            }
            else if (aClass instanceof OWLAllValuesFrom && ((OWLAllValuesFrom) aClass).getFiller() instanceof RDFSClass) {
                OWLAllValuesFrom newRestriction = (OWLAllValuesFrom) aClass;
                OWLNamedClass newSourceClass = newRestriction.getOwner();
                for (Iterator it = items.iterator(); it.hasNext();) {
                    //ConditionsTableItem existing = (ConditionsTableItem) it.next();
                    NCIConditionsTableItem existing = (NCIConditionsTableItem) it.next();
                    if (!existing.isSeparator() && directType.equals(existing.aClass.getProtegeType()) &&
                            property.equals(((OWLRestriction) existing.aClass).getOnProperty())) {
                        OWLAllValuesFrom existingRestriction = (OWLAllValuesFrom) existing.aClass;
                        OWLNamedClass existingSourceCls = existingRestriction.getOwner();
                        if (((RDFSClass) existingRestriction.getFiller()).isSubclassOf((RDFSClass) newRestriction.getFiller()) &&
                                !existingSourceCls.equals(newSourceClass)) {
                            return;
                        }
                    }
                }
            }
            else {
                boolean qcr = false;
                if (aClass instanceof OWLCardinalityBase) {
                    OWLCardinalityBase base = (OWLCardinalityBase) aClass;
                    qcr = base.isQualified();
                }
                if (!qcr) {
                    for (Iterator it = items.iterator(); it.hasNext();) {
                        //ConditionsTableItem existing = (ConditionsTableItem) it.next();
                        NCIConditionsTableItem existing = (NCIConditionsTableItem) it.next();
                        if (!existing.isSeparator() && directType.equals(existing.aClass.getProtegeType()) &&
                                property.equals(((OWLRestriction) existing.aClass).getOnProperty())) {
                            return;  // Don't add if entry with same type exists
                        }
                    }
                }
            }
        }
        addInheritedSeparator();
        //items.add(ConditionsTableItem.createInherited(aClass, originCls));
        items.add(NCIConditionsTableItem.createInherited(aClass, originCls));
    }







// New code from TT, 030607 (inherited restrictions)
    private void fillInheritedAnonymousClses(OWLNamedClass originCls, Collection coveredClses) {
    	String showInheritedRestrictionsString = ApplicationProperties.getApplicationOrSystemProperty(SHOW_INHERITED_RESTRICTIONS, "false");

    	boolean showInheritedRestrictions = (showInheritedRestrictionsString.equals("true"));

        for (Iterator it = originCls.getSuperclasses(false).iterator(); it.hasNext();) {
            Cls ss = (Cls) it.next();
            if (ss instanceof OWLAnonymousClass) {
                if (ss instanceof OWLIntersectionClass) {
                    Collection operands = ((OWLIntersectionClass) ss).getOperands();
                    for (Iterator oit = operands.iterator(); oit.hasNext();) {
                        RDFSClass operand = (RDFSClass) oit.next();
                        if (operand instanceof OWLAnonymousClass) {
                  		/* TT:
						 * The following line should be commented out if the performance is bad
    	              	 * The following line adds the inherited conditons to the table
						 */
                        if (showInheritedRestrictions) {
                        	addItemUnlessOverloaded(operand, originCls);
                        } else {
                        	items.add(NCIConditionsTableItem.createInherited(operand, originCls));
                        }

                        }
                    }
                }
                else if (!coveredClses.contains(ss)) {
                  	/* TT:
					 * The following line should be commented out if the performance is bad
                  	 * The following line adds the inherited conditons to the table
					 */
                    if (showInheritedRestrictions) {
                    	addItemUnlessOverloaded((OWLAnonymousClass) ss, originCls);
                    } else {
                    	items.add(NCIConditionsTableItem.createInherited((OWLAnonymousClass)ss, originCls));
                    }
                    //addItemUnlessOverloaded((OWLAnonymousClass) ss, originCls);
                   // items.add(ConditionsTableItem.createInherited((OWLAnonymousClass)ss, originCls));
                }
            }
        }
    }


}
