package gov.nih.nci.protegex.edit;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.KnowledgeBase;
import edu.stanford.smi.protege.model.Model;
import edu.stanford.smi.protege.ui.FrameComparator;
import edu.stanford.smi.protege.ui.FrameRenderer;
import edu.stanford.smi.protege.util.CollectionUtilities;
import edu.stanford.smi.protege.util.ComponentFactory;
import edu.stanford.smi.protege.util.LabeledComponent;
import edu.stanford.smi.protege.widget.ClsWidget;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protegex.owl.model.impl.OWLUtil;
import edu.stanford.smi.protegex.owl.ui.ProtegeUI;
import edu.stanford.smi.protegex.owl.ui.resourcedisplay.ResourcePanel;
import edu.stanford.smi.protegex.owl.ui.resourcedisplay.InstanceNameComponent;
import edu.stanford.smi.protegex.owl.ui.widget.OWLUI;
import edu.stanford.smi.protegex.owl.ui.code.OWLSymbolPanel;
import edu.stanford.smi.protegex.owl.ui.dialogs.ModalDialogFactory;
import edu.stanford.smi.protegex.owl.ui.icons.OWLIcons;
import edu.stanford.smi.protegex.owl.ui.profiles.ProfilesManager;
import edu.stanford.smi.protegex.owl.ui.search.ResourceListFinder;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import java.awt.*;
import java.awt.event.*;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Iterator;

import edu.stanford.smi.protegex.owl.ui.restrictions.FillerTextArea;
import edu.stanford.smi.protegex.owl.ui.restrictions.RestrictionKindRenderer;
import edu.stanford.smi.protege.ui.*;

import edu.stanford.smi.protegex.owl.ui.resourceselection.ResourceSelectionAction;
import edu.stanford.smi.protegex.owl.ui.code.SymbolPanel;


import edu.stanford.smi.protegex.owl.model.impl.DefaultOWLSomeValuesFrom;
import edu.stanford.smi.protegex.owl.model.impl.DefaultOWLAllValuesFrom;
import edu.stanford.smi.protegex.owl.model.impl.DefaultOWLHasValue;
import edu.stanford.smi.protegex.owl.model.impl.DefaultOWLMaxCardinality;
import edu.stanford.smi.protegex.owl.model.impl.DefaultOWLMinCardinality;

import edu.stanford.smi.protegex.owl.ui.dialogs.ModalDialogFactory;
import edu.stanford.smi.protege.util.*;


/**
 * A panel that allows to completely define a restriction.
 * This displays a searchable propertyList of slots, radiobuttons to select the restriction kind,
 * and a textfield that is supported by a symbol panel.
 *
 * @author Holger Knublauch  <holger@knublauch.com>
 */
//public class NCICreateRolePanel extends JDialog implements ActionListener {//extends JComponent implements ModalDialogFactory.CloseCallback {
public class NCICreateRolePanel extends JComponent implements ActionListener {//extends JComponent implements ModalDialogFactory.CloseCallback {


    private Action createDatatypePropertyAction;

    private Action createObjectPropertyAction;

    private Action createRDFPropertyAction;

    //private FillerTextArea fillerTextArea;

    private JList modifierList;

    OWLModel owlModel;

    private JList propertyList;

    private OWLSymbolPanel symbolPanel;

    private RDFSClass targetClass;

    private Action viewAction;

    private static String recentPropertyName;

    private JButton okButton, cancelButton;

    boolean cancelBtnPressed;
	private NCIRoleGroupEditorPanel rolegroupPanel;

	private edu.stanford.smi.protege.model.Cls metaCls = null;

	JTextField filler_field;
    private Cls selectedCls = null;
    RDFProperty property;

    JButton selectClsButton;
    JButton selectIndividualButton;
    private OWLRestriction restriction = null;
    String fillerText;


    public NCICreateRolePanel(NCIRoleGroupEditorPanel rolegroupPanel,
							  OWLModel owlModel,
                              edu.stanford.smi.protege.model.Cls metaCls,
                              RDFProperty property,
                              String fillerText,
                              RDFSClass targetClass, OWLRestriction r) {

        this.owlModel = owlModel;
        this.rolegroupPanel = rolegroupPanel;
        this.targetClass = targetClass;

        this.metaCls = metaCls;
        this.property = property;
        this.restriction = r;
        this.fillerText = fillerText;
        initialize();

   }





    public NCICreateRolePanel(NCIRoleGroupEditorPanel rolegroupPanel,
							  OWLModel owlModel,
                              edu.stanford.smi.protege.model.Cls metaCls,
                              RDFProperty property,
                              String fillerText,
                              RDFSClass targetClass) {

        this.owlModel = owlModel;
        this.rolegroupPanel = rolegroupPanel;
        this.targetClass = targetClass;

        this.metaCls = metaCls;
        this.property = property;
        this.restriction = null;
        this.fillerText = fillerText;
        initialize();

	}


    private void initialize()
    {
		/*
        this.owlModel = owlModel;
        this.rolegroupPanel = rolegroupPanel;
        this.targetClass = targetClass;

        this.metaCls = metaCls;
        this.property = property;

        */

        createActions();

        propertyList = ComponentFactory.createList(null);
        propertyList.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        propertyList.setCellRenderer(FrameRenderer.createInstance());
        propertyList.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                changeProperty((RDFProperty) propertyList.getSelectedValue());
                enableActions();
            }
        });
        propertyList.addMouseListener(new MouseAdapter() {
            public void mouseClicked(MouseEvent e) {
                if (e.getClickCount() == 2 && viewAction.isEnabled()) {
                    viewAction.actionPerformed(null);
                }
            }
        });

        updatePropertiesList();

        // initial selection of property
        propertyList.setSelectedValue(property, true);


        JPanel propertyPanel = new JPanel(new BorderLayout());
        propertyPanel.setLayout(new BorderLayout());
        JScrollPane propertyScrollPane = new JScrollPane(propertyList);
        propertyScrollPane.setPreferredSize(new Dimension(240, 150));
        propertyPanel.add(BorderLayout.CENTER, propertyScrollPane);
        propertyPanel.add(BorderLayout.SOUTH, new ResourceListFinder(propertyList, "Find"));

        edu.stanford.smi.protege.model.Cls[] metaClses = ProfilesManager.getSupportedRestrictionMetaClses(owlModel);
        modifierList = new JList(metaClses);
        modifierList.setCellRenderer(new RestrictionKindRenderer());
        modifierList.getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        // initial selection of modifier
        modifierList.setSelectedValue(metaCls, true);


        JScrollPane modifierScrollPane = new JScrollPane(modifierList);
        modifierScrollPane.setPreferredSize(new Dimension(160, 150));
        modifierList.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
            public void valueChanged(ListSelectionEvent e) {
                changeMetaClsSelection((edu.stanford.smi.protege.model.Cls) modifierList.getSelectedValue());
            }
        });

        JPanel kindPanel = new JPanel();
        kindPanel.setLayout(new BorderLayout());
        kindPanel.add(BorderLayout.NORTH, new LabeledComponent("Restriction", modifierScrollPane));
        kindPanel.add(BorderLayout.CENTER, new JPanel());

		final OWLModel owl_model=owlModel;


		fillerText = trimFillerText(fillerText);

        if (fillerText == null)
        {
			filler_field = new JTextField("");
	    }
	    else
	    {
			filler_field = new JTextField("");
		}

		setInitialFillerValue();



		filler_field.setEditable(false);
		filler_field.setPreferredSize(new Dimension (300, 20));
		LabeledComponent lc = new LabeledComponent("Filler", filler_field);

		Action SelectPropertyValueAction =
			new AbstractAction("Select a Named Class (filler)", OWLIcons.getAddIcon("PrimitiveClass")) {

				public void actionPerformed(ActionEvent e)
				{
					java.awt.TextField textfield = new TextField();
					String label = "Select a named class";

					Collection clses = owl_model.getRootClses();
					/*
					Cls cls = DisplayUtilities.pickConcreteCls(textfield, owl_model, clses);

					if (cls == null)
					{
						//System.out.println("cls is null");
					}
					else
					{
						String name = OWLWrapper.getInternalName(cls);
						filler_field.setText(name);
						selectedCls = cls;
					}
					*/

					final NCISelectClsesPanel p = new NCISelectClsesPanel(owl_model, clses);
					int result = ModalDialog.showDialog(textfield, p, label, ModalDialog.MODE_OK_CANCEL);
					if (result == ModalDialogFactory.OPTION_OK)
					{
						Collection c = p.getSelection();
						if (c != null && c.size() > 0)
						{
							Iterator it = c.iterator();
							Object obj = it.next();
							selectedCls = (Cls) obj;


				//010407
				Object[] option_array = {"OK"};
				Object[] options = {"Yes", "No"};

				boolean invaldSelection = rolegroupPanel.getNCIEditTab().getOWLWrapper().isPremerged((OWLNamedClass) selectedCls);
				if (invaldSelection)
				{
					int ans = JOptionPane.showOptionDialog(rolegroupPanel, "Cannot select a pre-merged concept.", "Confirmation",
					JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
					null, option_array, options[0]);
					return;
				}
				invaldSelection = rolegroupPanel.getNCIEditTab().getOWLWrapper().isPreretired((OWLNamedClass) selectedCls);
				if (invaldSelection)
				{
					int ans = JOptionPane.showOptionDialog(rolegroupPanel, "Cannot select a pre-retired concept.", "Confirmation",
					JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
					null, option_array, options[0]);
					return;
				}
				invaldSelection = rolegroupPanel.getNCIEditTab().getOWLWrapper().isRetired((OWLNamedClass) selectedCls);
				if (invaldSelection)
				{
					int ans = JOptionPane.showOptionDialog(rolegroupPanel, "Cannot select a retired concept.", "Confirmation",
					JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
					null, option_array, options[0]);
					return;
				}


							String name = OWLWrapper.getInternalName(selectedCls);
							filler_field.setText(name);
						}

					}
				}
			};

	    selectClsButton = lc.addHeaderButton(SelectPropertyValueAction);


		Action SelectIndividualAction = new InsertIndividualAction() {

				//public void actionPerformed(ActionEvent e)
				//{
				//	java.awt.TextField textfield = new TextField();
				//	String label = "Select a property value";

				//	Collection clses = owl_model.getRootClses();
				//	Cls cls = DisplayUtilities.pickConcreteCls(textfield, owl_model, clses);
				//	if (cls == null)
				//	{
				//		System.out.println("cls is null");
				//	}
				//	else
				//	{
				//		String name = cls.getBrowserText();
				//		filler_field.setText(name);
				//		selectedCls = cls;
				//	}
				//}

			};

	    selectIndividualButton = lc.addHeaderButton(SelectIndividualAction);

        setLayout(new BorderLayout(10, 10));
        LabeledComponent lc2 = new LabeledComponent("Restricted Property", propertyPanel);
        // lc.addHeaderButton(viewAction);
        if (OWLUtil.hasRDFProfile(owlModel)) {
            lc2.addHeaderButton(createRDFPropertyAction);
        }

        //lc2.addHeaderButton(createDatatypePropertyAction);
        //lc2.addHeaderButton(createObjectPropertyAction);

        JPanel panel = new JPanel();
        panel.setLayout(new BorderLayout());
        panel.add(BorderLayout.CENTER, lc2);
        panel.add(BorderLayout.EAST, kindPanel);
        panel.add(BorderLayout.SOUTH, lc);

        add(BorderLayout.CENTER, panel);

		if (metaCls != null && property != null)
		{
			filler_field.setEditable(false);
			selectClsButton.setEnabled(true);
			selectIndividualButton.setEnabled(false);

			if (metaCls.getName().equals(OWLNames.Cls.HAS_VALUE_RESTRICTION))
			{
				if (property instanceof OWLObjectProperty)
				{
					filler_field.setEditable(false);
					selectClsButton.setEnabled(false);
					selectIndividualButton.setEnabled(true);

				}
				else
				{
					filler_field.setEditable(true);
					filler_field.setText(fillerText);
					selectClsButton.setEnabled(false);
					selectIndividualButton.setEnabled(false);
				}
			}
			else if (metaCls.getName().equals(OWLNames.Cls.MAX_CARDINALITY_RESTRICTION)) {
				filler_field.setEditable(true);
				selectClsButton.setEnabled(false);
				selectIndividualButton.setEnabled(false);
			}
			else if (metaCls.getName().equals(OWLNames.Cls.MIN_CARDINALITY_RESTRICTION)) {
				filler_field.setEditable(true);
				selectClsButton.setEnabled(false);
				selectIndividualButton.setEnabled(false);
			}
			else if (metaCls.getName().equals(OWLNames.Cls.CARDINALITY_RESTRICTION)) {
				filler_field.setEditable(true);
				selectClsButton.setEnabled(false);
				selectIndividualButton.setEnabled(false);
			}
	    }

		setLocation(260,200);
		setPreferredSize(new Dimension(500, 320));

    }





    private String trimFillerText(String s)
    {
		if (s == null) return "";
		if (s.length() < 2) return s;
		char quote = '"';
		String quote_str = "" + quote;
		if (s.substring(0, 1).compareTo(quote_str) == 0)
		{
			s = s.substring(1, s.length());
		}
		if (s.substring(s.length()-1, s.length()).compareTo(quote_str) == 0)
		{
			s = s.substring(0, s.length()-1);
		}
		return s;
	}

    private void addNewProperty(RDFProperty newProperty) {
        newProperty.setFunctional(false);
        newProperty.setDomainDefined(false);
        showModalPropertyWidget(newProperty);
        updatePropertiesList();
        propertyList.setSelectedValue(newProperty, true);
    }


    public boolean canClose(int result) {
        if (result == ModalDialogFactory.OPTION_OK) {
            RDFProperty selectedProperty = (RDFProperty) propertyList.getSelectedValue();
            if (selectedProperty != null) {
				//091506
                //String uniCodeText = fillerTextArea.getText();
                String uniCodeText = filler_field.getText();
                if (uniCodeText.length() == 0) {
                    //symbolPanel.displayError("Please enter a filler");
                    return false;
                }
            }
        }
        //else {
            return true;
        //}
    }


    public Cls getFillerCls()
    {
		return selectedCls;
	}

    private void createActions() {

        viewAction = new AbstractAction("View Property...", OWLIcons.getViewIcon()) {

            public void actionPerformed(ActionEvent arg0) {
                RDFProperty property = (RDFProperty) propertyList.getSelectedValue();
                showModalPropertyWidget(property);
            }
        };
        viewAction.setEnabled(false);

        createDatatypePropertyAction = new AbstractAction("Create datatype property...",
                OWLIcons.getCreatePropertyIcon(OWLIcons.OWL_DATATYPE_PROPERTY)) {

            public void actionPerformed(ActionEvent arg0) {
                OWLProperty newProperty = owlModel.createOWLDatatypeProperty(null);
                addNewProperty(newProperty);
            }
        };

        createObjectPropertyAction = new AbstractAction("Create object property...",
                OWLIcons.getCreatePropertyIcon(OWLIcons.OWL_OBJECT_PROPERTY)) {

            public void actionPerformed(ActionEvent arg0) {
                OWLProperty newProperty = owlModel.createOWLObjectProperty(null);
                addNewProperty(newProperty);
            }
        };

        createRDFPropertyAction = new AbstractAction("Create RDF property...",
                OWLIcons.getCreatePropertyIcon(OWLIcons.RDF_PROPERTY)) {

            public void actionPerformed(ActionEvent arg0) {
                RDFProperty newProperty = owlModel.createRDFProperty(null);
                addNewProperty(newProperty);
            }
        };
    }


    private OWLRestriction createRestriction(edu.stanford.smi.protege.model.Cls metaCls,
                                             RDFProperty property, String text) throws Exception
    {
        Collection parents = CollectionUtilities.createCollection(((KnowledgeBase) owlModel).getCls(OWLNames.Cls.ANONYMOUS_ROOT));
        KnowledgeBase kb = owlModel;

    	try {            
            kb.beginTransaction("Creating local restriction on property " + property + " with filler " + text + " of type " + metaCls.getName());
            
            OWLRestriction restriction = (OWLRestriction) kb.createCls(null, parents, metaCls);
            restriction.setOnProperty(property);
            restriction.setFillerText(text);
            
            kb.commitTransaction();
            return restriction;
			
		} catch (Exception e) {
			kb.rollbackTransaction();
			OWLUI.handleError(owlModel, e);		
		}
		
		return null;
    }


    private void enableSymbolPanel() {
        edu.stanford.smi.protege.model.Cls metaCls = getSelectedMetaCls();
        symbolPanel.setEnabled(true);
        symbolPanel.enableActions(getSelectedProperty(), metaCls);
    }


    private void enableActions() {
        viewAction.setEnabled(true);
    }


    public OWLRestriction getResult() {
        try {
            String text = filler_field.getText().trim();

            // 101606
            if (text.compareTo("") == 0)
            {
				Object[] options = {"OK"};
				int ans = JOptionPane.showOptionDialog(null, "No class is selected.", "Confirmation",
				JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
				null, options, options[0]);
				return null;

			}


            if (getSelectedMetaCls().equals(OWLNames.Cls.HAS_VALUE_RESTRICTION) &&
                getSelectedProperty() instanceof OWLDatatypeProperty)
			{
				text = "" + text;
				text = text + "";
			}

            return createRestriction(getSelectedMetaCls(), getSelectedProperty(), text);

        }
        catch (Exception ex) {
			String text = filler_field.getText().trim();
			System.out.println("Unable to create restricton with filler " + text);
           return null;
        }
    }


    private RDFProperty getRestrictionProperty(Cls metaCls) {
        if (metaCls.getName().equals(OWLNames.Cls.ALL_VALUES_FROM_RESTRICTION)) {
            return owlModel.getRDFProperty(OWLNames.Slot.ALL_VALUES_FROM);
        }
        else if (metaCls.getName().equals(OWLNames.Cls.SOME_VALUES_FROM_RESTRICTION)) {
            return owlModel.getRDFProperty(OWLNames.Slot.SOME_VALUES_FROM);
        }
        else if (metaCls.getName().equals(OWLNames.Cls.HAS_VALUE_RESTRICTION)) {
            return owlModel.getRDFProperty(OWLNames.Slot.HAS_VALUE);
        }
        else if (metaCls.getName().equals(OWLNames.Cls.MAX_CARDINALITY_RESTRICTION)) {
            return owlModel.getRDFProperty(OWLNames.Slot.MAX_CARDINALITY);
        }
        else if (metaCls.getName().equals(OWLNames.Cls.MIN_CARDINALITY_RESTRICTION)) {
            return owlModel.getRDFProperty(OWLNames.Slot.MIN_CARDINALITY);
        }
        else {
            return owlModel.getRDFProperty(OWLNames.Slot.CARDINALITY);
        }
    }


    edu.stanford.smi.protege.model.Cls getSelectedMetaCls() {
        return (edu.stanford.smi.protege.model.Cls) modifierList.getSelectedValue();
    }


    RDFProperty getSelectedProperty() {
        return (RDFProperty) propertyList.getSelectedValue();
    }


    public /*static*/ OWLRestriction showCreateDialog(Component parent,
                                                  RDFSClass targetClass,
                                                  edu.stanford.smi.protege.model.Cls metaCls,
                                                  RDFProperty property) {
        return showCreateDialog(parent, targetClass, metaCls, property, null);
    }


    public /*static*/ OWLRestriction showCreateDialog(Component parent,
                                                  RDFSClass targetClass,
                                                  edu.stanford.smi.protege.model.Cls metaCls,
                                                  RDFProperty property,
                                                  String fillerText) {
        OWLModel owlModel = targetClass.getOWLModel();
        if (property == null && recentPropertyName != null) {
            RDFResource resource = owlModel.getRDFResource(recentPropertyName);
            if (resource instanceof RDFProperty) {
                property = (RDFProperty) resource;
            }
        }
        NCICreateRolePanel panel = new NCICreateRolePanel(rolegroupPanel, owlModel, metaCls, property, fillerText, targetClass);

		int option = JOptionPane.showConfirmDialog
			(null,
			 "Create Restriction. Do you want to continue?", "Confirmation", JOptionPane.YES_NO_OPTION);

		if (option==0){
             return panel.getResult();
        }
        else {
            return null;
        }

    }



    public /*static*/ OWLRestriction showCreateDialog(Component parent, RDFSClass targetClass, edu.stanford.smi.protege.model.Cls metaCls) {
        return showCreateDialog(parent, targetClass, metaCls, null);
    }


    private void showModalPropertyWidget(RDFProperty property) {
        ClsWidget widget = owlModel.getProject().createRuntimeClsWidget(property);
        InstanceNameComponent nameComponent = new InstanceNameComponent();
        nameComponent.setInstance(property);

        JPanel panel = new JPanel(new BorderLayout(6, 6));
        panel.add(nameComponent, BorderLayout.NORTH);
        panel.add((JComponent)widget, BorderLayout.CENTER);

        ProtegeUI.getModalDialogFactory().showDialog(this, panel, "Property " + property.getBrowserText(),
                ModalDialogFactory.MODE_CLOSE);
    }


    private void updatePropertiesList() {
        Collection allProperties = owlModel.getVisibleUserDefinedRDFProperties();
        Collection selectableProperties = new ArrayList();
        for (Iterator it = allProperties.iterator(); it.hasNext();) {
            RDFProperty property = (RDFProperty) it.next();
            if (!property.isAnnotationProperty()) {
                selectableProperties.add(property);
            }
        }
        RDFSNamedClass dbrClass = owlModel.getRDFSNamedClass(Model.Cls.DIRECTED_BINARY_RELATION);
        if (targetClass != null && ((Cls) targetClass).hasSuperclass(dbrClass)) {
            RDFProperty fromProperty = owlModel.getRDFProperty(Model.Slot.FROM);
            if (fromProperty.isVisible()) {
                selectableProperties.add(fromProperty);
            }
            RDFProperty toProperty = owlModel.getRDFProperty(Model.Slot.TO);
            if (toProperty.isVisible()) {
                selectableProperties.add(toProperty);
            }
        }
        Object[] propertiesArray = selectableProperties.toArray();
        Arrays.sort(propertiesArray, new FrameComparator());
        java.util.List propertiesList = Arrays.asList(propertiesArray);
        propertyList.setListData(propertiesList.toArray());

        propertyList.setSelectedIndex(0);
    }



    private class InsertIndividualAction extends ResourceSelectionAction {


        InsertIndividualAction() {
            super("Insert individual...", OWLIcons.getImageIcon(OWLIcons.RDF_INDIVIDUAL));
        }


        public void actionPerformed(ActionEvent e) {
            //Collection sels = ProtegeUI.getSelectionDialogFactory().selectResourcesByType(SymbolPanel.this, owlModel,
            Collection sels = ProtegeUI.getSelectionDialogFactory().selectResourcesByType(null, owlModel,
                    Collections.singleton(owlModel.getOWLThingClass()), "Select the resources to insert");
            for (Iterator it = sels.iterator(); it.hasNext();) {
                Frame frame = (Frame) it.next();
                if (frame instanceof RDFResource) {

					RDFResource rdf_resource = (RDFResource) frame;
                    resourceSelected((RDFResource) frame);

					String name = rdf_resource.getBrowserText();
					filler_field.setText(name);

                }
            }
        }


        public void resourceSelected(RDFResource resource) {
            //insertIndividual(resource);
        }


        public Collection getSelectableResources() {
            Collection frames = owlModel.getOWLIndividuals();
            java.util.List copy = new ArrayList(frames);
            Collections.sort(copy, new FrameComparator());
            return copy;
        }


        public RDFResource pickResource() {
            Collection resources = getSelectableResources();
           // return ProtegeUI.getSelectionDialogFactory().selectResourceFromCollection(SymbolPanel.this,
           return ProtegeUI.getSelectionDialogFactory().selectResourceFromCollection(null,
                    owlModel, resources, "Select the resource to insert");
        }
    }





    public void actionPerformed(ActionEvent event)
	{
		Object action = event.getSource();
		if (action == cancelButton) {
			cancelBtnPressed = true;
			//dispose();
		}
		else if (action == okButton) {
			cancelBtnPressed = false;
			//rolegroupPanel.addRolegroup(this);

			System.out.println("CreateRolePanel okButton pressed.");
			//dispose();
		}
	}

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    private void changeProperty(RDFProperty property)
    {
		// property is the current selected property

		if (modifierList == null) return;

		// modifier

		edu.stanford.smi.protege.model.Cls metaCls = (edu.stanford.smi.protege.model.Cls) modifierList.getSelectedValue();
		if (metaCls == null) return;

		if (!(property instanceof OWLObjectProperty)) // data type
		{
			modifierList.setSelectedIndex(2);
			if (!(metaCls.getName().equals(OWLNames.Cls.HAS_VALUE_RESTRICTION)))
			{
				filler_field.setEditable(false);
				selectClsButton.setEnabled(false);
				selectIndividualButton.setEnabled(false);
				return;
			}
 			filler_field.setEditable(true);
			selectClsButton.setEnabled(false);
			selectIndividualButton.setEnabled(false);
		}
		else
		{
			modifierList.setSelectedIndex(1);
 			filler_field.setEditable(false);
			selectClsButton.setEnabled(true);
			selectIndividualButton.setEnabled(false);
		}
	}



    private void changeMetaClsSelection(edu.stanford.smi.protege.model.Cls metaclass)
    {
        RDFProperty property = (RDFProperty) propertyList.getSelectedValue();
        if (property == null) return;
		if (property instanceof OWLObjectProperty)
		{
			if (metaclass.getName().equals(OWLNames.Cls.ALL_VALUES_FROM_RESTRICTION)) {
				filler_field.setEditable(false);
				selectClsButton.setEnabled(true);
				selectIndividualButton.setEnabled(false);
			}
			else if (metaclass.getName().equals(OWLNames.Cls.SOME_VALUES_FROM_RESTRICTION)) {
				filler_field.setEditable(false);
				selectClsButton.setEnabled(true);
				selectIndividualButton.setEnabled(false);
			}
			else if (metaclass.getName().equals(OWLNames.Cls.HAS_VALUE_RESTRICTION)) {
				filler_field.setEditable(false);
				selectClsButton.setEnabled(false);
				selectIndividualButton.setEnabled(true);
			}
			else if (metaclass.getName().equals(OWLNames.Cls.MAX_CARDINALITY_RESTRICTION)) {
				filler_field.setEditable(true);
				selectClsButton.setEnabled(false);
				selectIndividualButton.setEnabled(false);
			}
			else if (metaclass.getName().equals(OWLNames.Cls.MIN_CARDINALITY_RESTRICTION)) {
				filler_field.setEditable(true);
				selectClsButton.setEnabled(false);
				selectIndividualButton.setEnabled(false);
			}
			else {
				filler_field.setEditable(true);
				selectClsButton.setEnabled(false);
				selectIndividualButton.setEnabled(false);
			}
		} else { // not object valued

  		    if (metaclass.getName().equals(OWLNames.Cls.HAS_VALUE_RESTRICTION))
			{
				filler_field.setEditable(true);
				selectClsButton.setEnabled(false);
				selectIndividualButton.setEnabled(false);
			}
			else
			{
				filler_field.setEditable(false);
				filler_field.setText("");
				selectClsButton.setEnabled(false);
				selectIndividualButton.setEnabled(false);
			}
		}
    }




    private void setInitialFillerValue()
    {
		if (restriction == null) return;
		if (restriction instanceof OWLSomeValuesFrom)
		{
			DefaultOWLSomeValuesFrom cls = (DefaultOWLSomeValuesFrom) restriction;
			RDFResource res = (RDFResource) cls.getSomeValuesFrom();
			OWLNamedClass owl_cls = (OWLNamedClass) res;
			String name = OWLWrapper.getInternalName(owl_cls);
			System.out.println(res.getBrowserText());
			filler_field.setText(name);
		}
		else if (restriction instanceof OWLAllValuesFrom)
		{
			DefaultOWLAllValuesFrom cls = (DefaultOWLAllValuesFrom) restriction;
			RDFResource res = (RDFResource) cls.getAllValuesFrom();
			OWLNamedClass owl_cls = (OWLNamedClass) res;
			String name = OWLWrapper.getInternalName(owl_cls);
			System.out.println(res.getBrowserText());
			filler_field.setText(name);
		}
		else if (restriction instanceof OWLHasValue)
		{
			DefaultOWLHasValue cls = (DefaultOWLHasValue) restriction;
			String s = cls.getFillerText();
			filler_field.setText(s);
		}
		else //if (restriction instanceof OWLMaxCardinality)
		{
			//DefaultOWLMaxCardinality cls = (OWLMaxCardinality) restriction;
			//RDFProperty property = cls.getFillerProperty();
			filler_field.setText(fillerText);
		}
	}

}
