package gov.nih.nci.protegex.edit;

import java.util.*;
import java.rmi.*;
import java.net.*;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.text.SimpleDateFormat;

import edu.stanford.smi.protege.server.*;
import edu.stanford.smi.protegex.owl.storage.*;
import edu.stanford.smi.protegex.owl.jena.*;
import edu.stanford.smi.protegex.owl.model.impl.*;
import edu.stanford.smi.protegex.owl.ProtegeOWL;
import edu.stanford.smi.protegex.owl.jena.Jena;
import edu.stanford.smi.protegex.owl.jena.JenaOWLModel;
import edu.stanford.smi.protegex.owl.model.OWLAnonymousClass;
import edu.stanford.smi.protegex.owl.model.OWLDatatypeProperty;
import edu.stanford.smi.protegex.owl.model.OWLIntersectionClass;
import edu.stanford.smi.protegex.owl.model.OWLNamedClass;
import edu.stanford.smi.protegex.owl.model.OWLObjectProperty;
import edu.stanford.smi.protegex.owl.model.RDFIndividual;
import edu.stanford.smi.protegex.owl.model.OWLModel;
import edu.stanford.smi.protegex.owl.model.RDFSClass;
import edu.stanford.smi.protege.storage.database.DatabaseKnowledgeBaseFactory;
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protegex.owl.model.*;
import edu.stanford.smi.protege.event.ClsAdapter;
import edu.stanford.smi.protege.event.ClsEvent;
import edu.stanford.smi.protege.util.CollectionUtilities;
import edu.stanford.smi.protege.util.Log;
import edu.stanford.smi.protegex.owl.ui.profiles.ProfilesManager;


import edu.stanford.smi.protegex.owl.model.util.CloneFactory;
import edu.stanford.smi.protegex.owl.model.util.ResourceCopier;

import edu.stanford.smi.protegex.owl.model.classparser.OWLClassParseException;
import edu.stanford.smi.protegex.owl.model.classparser.OWLClassParser;
import edu.stanford.smi.protegex.owl.ui.ProtegeUI;

import edu.stanford.smi.protegex.owl.ui.widget.OWLUI;

import edu.stanford.smi.protege.util.Log;
import java.util.logging.Level;


/**
 *@Author: NGIT, Kim Ong; Iris Guo
 */

public class OWLWrapper
{
	private QuickSortVecStrings sort;
    private Instance instance;
    //private JFrame frame;
    private Cls cls;

	private JFrame pFrame;

	private String SUBCLASSOF = "rdfs:subClassOf";
	private RDFSClass retireCls;// = (RDFSClass) instance;
	private RDFSClass survivalCls;// = (RDFSClass) survival;
	private RDFSClass tempCls;

	String referenced_code = null;

	private Vector old_child_vec;
	private Vector old_parent_vec;
	private Vector old_role_vec;

	private HashSet to_annotation_set;
	private HashSet to_named_sups_set;
	private HashSet to_anonymous_sups_set;

    private static RemoteSession _session = null;
	private static RemoteServer _server = null;

    private static OWLModel owlModel;
    private static DefaultKnowledgeBase default_kb;

	private static Project current_project = null;

    private static String _host;
    private static String _user;
    private static String _password;
    private static String _projectname;

	private boolean _projectAvailable;

	private String uri;

   	private NCICodeGenerator generator;

   	private boolean connectionStatus = false;

   	Vector noneditable_vec = null;

   	String username = null;

   	Vector requiredProperties = null;
   	Vector readOnlyProperties = null;

   	private int readOnlyPropertyCount = 0;
   	private int requiredPropertyCount = 0;

	HashMap	name2rdfID = null;
	HashMap	name2SlotName = null;
	HashMap	rdfID2Slot = null;

	HashMap name2Cls = null;
	HashMap name2ClsName = null;

	String slotname_src = null;//tab.getSlotName(NCIEditTab.MERGE_SOURCE);
	String slotname_target = null; //tab.getSlotName(NCIEditTab.MERGE_TARGET);

    boolean codeSlotExists = false;
    String codeSlotName = null;

	public OWLWrapper(OWLModel owlModel)
	{
        this._host = null;
        this._user = null;
        this._password = null;
        this._projectname = null;

		this.generator = new NCICodeGenerator();
		if (this.generator == null)
		{
            System.out.println("Unable to instantiate NCICodeGenerator.");
		}

		this.owlModel = owlModel;
		this.default_kb = (DefaultKnowledgeBase) owlModel;
		sort = new QuickSortVecStrings();

	}

    public void setUserName(String name)
    {
		username = name;
	}


    public String getCodeSlotName()
    {
		return codeSlotName;
	}


    public void init(Vector noneditable_vec, Vector readOnlyProperties, Vector requiredProperties,
        HashMap name2rdfID, HashMap name2SlotName, HashMap rdfID2Slot,
        HashMap name2Cls, HashMap name2ClsName)
    {
		this.noneditable_vec = (Vector)noneditable_vec.clone();
		this.readOnlyProperties = (Vector)readOnlyProperties.clone();
		this.requiredProperties = (Vector)requiredProperties.clone();
		this.name2rdfID = name2rdfID;
		this.name2SlotName = name2SlotName;
		this.rdfID2Slot = rdfID2Slot;
		this.name2Cls = name2Cls;
		this.name2ClsName = name2ClsName;

		slotname_src = (String) name2SlotName.get("Merge_Source");//tab.getSlotName(NCIEditTab.MERGE_SOURCE);
		slotname_target = (String) name2SlotName.get("Merge_Target"); //tab.getSlotName(NCIEditTab.MERGE_TARGET);

        codeSlotExists = false;
        codeSlotName = (String) name2SlotName.get("code");
        if (codeSlotName == null) return;
		Slot codeSlot = owlModel.getSlot(codeSlotName);
		if (codeSlot != null)
		{
			codeSlotExists = true;
		}
	}

    public String getMergeSourceSlotName()
    {
		return slotname_src;
	}

    public String getMergeTargetSlotName()
    {
		return slotname_target;
	}

	public Collection getDirectSuperclasses(RDFSClass cls)
	{
		Collection parent_vec = new Vector();
		Collection sups = cls.getDirectSuperclasses();

		for(Iterator iterator = sups.iterator(); iterator.hasNext();){
			RDFSClass sup = (RDFSClass)iterator.next();
			if (sup instanceof OWLNamedClass)
			{
				String internalName = getInternalName(sup);
				parent_vec.add(internalName);
			}
		}
		return parent_vec;
	}




    boolean annotationPropertyExists(String propertyname)
    {
		Slot slot = owlModel.getSlot(propertyname);
		if (slot == null) return false;
        return true;
	}


    public String getOwnSlotValue(Cls cls, String slotname)
    {
		Slot slot = owlModel.getSlot(slotname);
		if (slot == null)
		{
			System.out.println("Slot name " + slotname + " not defined.");
			return null;
		}

		Collection slot_values = null;
		try {
			slot_values = cls.getOwnSlotValues(slot);
			if (slot_values != null)
			{
				// return the first one
				for(Iterator iterator3 = slot_values.iterator(); iterator3.hasNext();)
				{
					Object slot_value = (Object) iterator3.next();
					return (String) slot_value;
				}
			}
		} catch (Exception e) {
			Log.getLogger().log(Level.WARNING, "Exception caught", e);;
		}
		return null;
    }



	public static String getPlainString(String s)	{
		String[] languages = owlModel.getUsedLanguages();
		if (s.indexOf("~#") == -1)
			return s;

		for (int i=0; i<languages.length; i++){
			if (s.startsWith("~#"+languages[i]+" "))
				return s.substring(5);
		}

		return s.substring(2);
	}


    public String getReferencedClsName(Cls cls)
    {
		String referenceClsName = getOwnSlotValue(cls, slotname_target);
		if (referenceClsName != null)
		{
			return referenceClsName;
		}
		referenceClsName = getOwnSlotValue(cls, slotname_src);
		if (referenceClsName != null)
		{
			return referenceClsName;
		}
		return null;
	}

    public Cls getReferencedCls(Cls cls)
    {
		String referenceClsName = getOwnSlotValue(cls, slotname_src);
		if (referenceClsName != null)
		{
			return owlModel.getCls(getPlainString(referenceClsName));
		}
		referenceClsName = getOwnSlotValue(cls, slotname_target);
		if (referenceClsName != null)
		{
			return owlModel.getCls(getPlainString(referenceClsName));
		}

		return null;
	}

	public boolean removeRestriction(OWLNamedClass owl_cls,
	                                String name,
	                                String value,
	                                String modifier)
	{
		if (owl_cls == null) return false;
		boolean includingSuperclassRestrictions = false;
		java.util.Collection col = owl_cls.getRestrictions(includingSuperclassRestrictions);
		int i=0;
        for (Iterator it = col.iterator(); it.hasNext();) {
			OWLRestriction r = (OWLRestriction) it.next();

			RDFProperty p = r.getOnProperty();
			String prop_name = p.getName();
			String prop_value = r.getFillerText();
			String prop_modifier = getRestrictionType(r);

			i++;

String value_with_brackets = "(" + prop_value + ")";

			if (name.compareTo(prop_name) == 0 &&
				value.compareTo(prop_value) == 0 &&
				modifier.compareToIgnoreCase(prop_modifier) == 0)
			{
				owl_cls.removeSuperclass((RDFSClass) r);
				r.delete();

				return true;
			}
			else if (name.compareTo(prop_name) == 0 &&
				value.compareTo(value_with_brackets) == 0 &&
				modifier.compareToIgnoreCase(prop_modifier) == 0)
			{
				owl_cls.removeSuperclass((RDFSClass) r);
				r.delete();

				return true;
			}
	    }
	    return false;
	}


    public String getRestrictionType(OWLRestriction restriction)
    {
		if (restriction instanceof OWLCardinality) {
			return "exactly";
		}
		else if (restriction instanceof OWLMinCardinality) {
			return "min";
		}
		else if (restriction instanceof OWLMaxCardinality) {
			return "max";
		}
		else if (restriction instanceof OWLAllValuesFrom) {
			return "only";
		}
		else if (restriction instanceof OWLSomeValuesFrom) {
			return "some";
		}
		else if (restriction instanceof OWLHasValue) {
			return "has";
		}
		return "undefined";
	}

    public String getRestrictionType(String restriction)
    {
		if (restriction.compareToIgnoreCase("owl:CardinalityRestriction") == 0) {
			return "exactly";
		}
		else if (restriction.compareToIgnoreCase("owl:MinCardinalityRestriction") == 0) {
			return "min";
		}
		else if (restriction.compareToIgnoreCase("owl:MaxCardinalityRestriction") == 0) {
			return "max";
		}
		else if (restriction.compareToIgnoreCase("owl:AllValuesFromRestriction") == 0) {
			return "only";
		}
		else if (restriction.compareToIgnoreCase("owl:SomeValuesFromRestriction") == 0) {
			return "some";
		}
		else if (restriction.compareToIgnoreCase("owl:HasValueRestriction") == 0) {
			return "has";
		}
		return restriction;
	}

    public String restrictionType2Label(String type)
    {
		if (type.compareToIgnoreCase("exatcly") == 0)
		{
			return "owl:CardinalityRestriction";
		}
		else if (type.compareToIgnoreCase("min") == 0)
		{
			return "owl:MinCardinalityRestriction";
		}
		else if (type.compareToIgnoreCase("max") == 0)
		{
			return "owl:MaxCardinalityRestriction";
		}
		else if (type.compareToIgnoreCase("all") == 0 ||
		         type.compareToIgnoreCase("only") == 0)
		{
			return "owl:AllValuesFromRestriction";
		}
		else if (type.compareToIgnoreCase("some") == 0)
		{
			return "owl:SomeValuesFromRestriction";
		}
		else if (type.compareToIgnoreCase("has") == 0)
		{
			return "owl:HasValueRestriction";
		}

		return type;
	}


	public Vector getRestrictionStrings(OWLNamedClass owl_cls)
	{
		if (owl_cls == null) return null;
		Vector v = new Vector();
		boolean includingSuperclassRestrictions = false;
		java.util.Collection col = owl_cls.getRestrictions(includingSuperclassRestrictions);

		if (col != null)
		{
			final Object[] obs = col.toArray();
			if (obs.length > 0)
			{
				for (int i=0; i<obs.length; i++)
				{
					OWLRestriction r = (OWLRestriction) obs[i];
					if (!(r instanceof OWLNamedClass))
					{
						RDFProperty p = r.getOnProperty();
						String prop_name = p.getName();
						String prop_value = r.getFillerText();
						String prop_modifier = getRestrictionType(r);
						String s = prop_name + "|" + prop_value + "|" + prop_modifier;
						v.add(s);
					}
				}
			}
	    }
	    return v;
	}


	public Vector getRestrictionTargets(OWLNamedClass owl_cls)
	{
		if (owl_cls == null) return null;
		Vector v = new Vector();
		boolean includingSuperclassRestrictions = false;
		java.util.Collection col = owl_cls.getRestrictions(includingSuperclassRestrictions);

		if (col != null)
		{
			final Object[] obs = col.toArray();
			if (obs.length > 0)
			{
				for (int i=0; i<obs.length; i++)
				{
					OWLRestriction r = (OWLRestriction) obs[i];
					if (!(r instanceof OWLNamedClass))
					{
						RDFProperty p = r.getOnProperty();
						String prop_name = p.getName();
						String prop_value = r.getFillerText();
						OWLNamedClass cls = (OWLNamedClass) owlModel.getOWLNamedClass(prop_value);
						v.add(cls);
					}
				}
			}
	    }
	    return v;
	}




	public Vector getPropertyStrings(OWLNamedClass owl_cls)
	{
		Vector v = new Vector();
		HashSet hset = getAnnotations((RDFSClass) owl_cls);
        Iterator iter = hset.iterator();
        while (iter.hasNext())
        {
			String s = (String) iter.next();
			v.add(s);
		}
	    return v;
	}



    public Vector getSetDifference(Vector old_vec, Vector new_vec)
	{
		// Find what are in new_vec. but not in old_vec
		HashSet hset = new HashSet();
		Vector w = new Vector();
		for (int i=0; i<old_vec.size(); i++)
		{
			String s = (String) old_vec.elementAt(i);
			if (!hset.contains(s))
                hset.add(s);
		}

		for (int i=0; i<new_vec.size(); i++)
		{
			String s = (String) new_vec.elementAt(i);
			if (!hset.contains(s))
			{
                w.add(s);
			}
		}
		hset.clear();
		return w;
	}


	public Vector getOwnSlotStrings(Cls cls)
	{
		Vector v = new Vector();
		Collection ownslots = cls.getOwnSlots();
		Collection hset = owlModel.getOWLAnnotationProperties();

		if (ownslots != null)
		{
			Iterator j = ownslots.iterator();
			while(j.hasNext())
			{
				Slot slot = (Slot)j.next();
				Collection slotColl = cls.getOwnSlotValues(slot);
				if(slotColl == null || slotColl.isEmpty())
					continue;

				String slotName = slot.getName();

				if (slotName.equals(Model.Slot.NAME))
				{
					continue;
				} else if (slotName.equals(Model.Slot.DIRECT_SUBCLASSES))
					continue;
				else if (slotName.equals(Model.Slot.DIRECT_INSTANCES))
					continue;
				else
				{
					if (slotColl != null)
					{
						Iterator i = slotColl.iterator();
						while(i.hasNext()) {
							Object obj = i.next();
							if (hset.contains(slotName))
							{
								ValueType type = getObjectValueType(obj);
								String entry = convertObjecttoString(obj, type);

								try {
									//if (slotName.compareTo(SUBCLASSOF) != 0)
									if (slotName.compareTo(SUBCLASSOF) != 0 && slotName.compareTo("owl:equivalentClass") != 0)
									{
										String s = slotName + "|" + entry;
										v.add(s);
									}
								}
								catch(Exception e){
									Log.getLogger().log(Level.WARNING, "Exception caught", e);;
								}
							}
						}
					}
				}
			}
		}
		return v;
	}

	public boolean conceptExists(String name)
	{
		try {
			Cls cls = owlModel.getCls(name);
			if (cls == null) return false;
			return true;
	    } catch (Exception e) {
			return false;
		}
    }


	public OWLNamedClass getOWLCls(String name)
	{
		return (OWLNamedClass) owlModel.getCls(name);
    }



	public Cls getConceptDefByName(String name)
	{
		return owlModel.getCls(name);
	}

/*
	public boolean isPropertyDefined(String propertyName)
	{
		// to be implemented
		return true;
	}
*/

// not applicable
	public boolean isQualifierDefined(String qualifiername)
	{
		return false;
	}

	public Vector getSuperconceptNames(String name)
	{
		Vector sups = new Vector();
		RDFSClass cls = (RDFSClass) getConceptDefByName(name);
		Collection c = getDirectSuperclasses(cls);
		for (Iterator it = c.iterator(); it.hasNext();) {
			RDFSClass sup = (RDFSClass) it.next();
			//sups.add(sup.getBrowserText());
			sups.add(getInternalName(sup));
		}
		return sups;
	}


    public String getAnnotationPropertyValue(Cls cls, String slot_name)
    {

		Vector v = new Vector();
		Collection ownslots = cls.getOwnSlots();
		Collection hset = owlModel.getOWLAnnotationProperties();

		if (ownslots != null)
		{
			Iterator j = ownslots.iterator();
			while(j.hasNext())
			{
				Slot slot = (Slot)j.next();
				String slotName = slot.getName();

				if (slotName.equals(slot_name))
				{
					Collection slotColl = cls.getOwnSlotValues(slot);

					if (slotColl != null)
					{
						Iterator i = slotColl.iterator();
						while(i.hasNext()) {
							Object obj = i.next();
							if (hset.contains(slotName))
							{
								ValueType type = getObjectValueType(obj);
								String entry = convertObjecttoString(obj, type);
								return entry;
							}
						}
					}
				}
			}
		}
		return null;
	}



	////////////////////////////////////////////////////////////
	// batch load
	////////////////////////////////////////////////////////////
	private String formatFULL_SYN(String name, String term_group, String source)
    {
		return name + "|" + term_group + "|" + source;
	}


    // 101906, KLO, Tracker #3119
	private String formatDEFINITION(String source, String text)
    {
		//SimpleDateFormat sdf = new SimpleDateFormat ("yyyy-MM-dd");
		SimpleDateFormat sdf = new SimpleDateFormat ("yyMMdd");
		String mydate = sdf.format(new Date());

		String xml = "";
		xml = xml + "<def-source>";
		xml = xml + source;
		xml = xml + "</def-source>";
		xml = xml + "<def-definition>";
		xml = xml + text;
		xml = xml + "</def-definition>";
		xml = xml + "<Definition_Review_Date>";
		xml = xml + mydate;
		xml = xml + "</Definition_Review_Date>";
		xml = xml + "<Definition_Reviewer_Name>";
		xml = xml + username;
		xml = xml + "</Definition_Reviewer_Name>";

		return xml;
	}



	////////////////////////////////////////////////////////////
	// batch editing
	////////////////////////////////////////////////////////////

	public boolean modifyAnnotationProperty(OWLNamedClass cls,
	                                        String propertyname,
	                                        String propertyvalue,
	                                        String newpropertyvalue)
	{
		if (cls == null) return false;
		boolean retval = removeAnnotationProperty(cls, propertyname, propertyvalue);
		if (!retval) return retval;
		return createAnnotationProperty(cls, propertyname, newpropertyvalue);
	}

	public boolean modifyAnnotationProperty(String name, String propertyname,
	                                        String propertyvalue,
	                                        String newpropertyvalue)
	{
		boolean conceptexists = conceptExists(name);
		if (!conceptexists) return false;
		Cls cls = owlModel.getCls(name);
		boolean retval = removeAnnotationProperty((OWLNamedClass) cls, propertyname, propertyvalue);
		if (!retval) return retval;
		return createAnnotationProperty((OWLNamedClass) cls, propertyname, newpropertyvalue);
	}

	public boolean removeAnnotationProperty(String name, String propertyname, String propertyvalue)
	{
		boolean conceptexists = conceptExists(name);
		if (!conceptexists) return false;
		return removeAnnotationProperty((OWLNamedClass) cls, propertyname, propertyvalue);
	}

	public boolean addRestriction(String clsName,
	                                String name,
	                                String value,
	                                String modifier)

    {
	    OWLNamedClass owl_cls = (OWLNamedClass) getConceptDefByName(clsName);
	    if (owl_cls == null) return false;
	    return addRestriction(owl_cls, name, value, modifier);
    }


	public boolean removeRestriction(String clsName,
	                                String name,
	                                String value,
	                                String modifier)
	{
	    OWLNamedClass owl_cls = (OWLNamedClass) getConceptDefByName(clsName);
	    if (owl_cls == null) return false;
	    return removeRestriction(owl_cls, name, value, modifier);
	}

	public boolean modifyRestriction(String clsName,
	                                String name,
	                                String value,
	                                String modifier,
	                                String newvalue,
	                                String newmodifier
	                                )
	{
	    OWLNamedClass owl_cls = (OWLNamedClass) getConceptDefByName(clsName);
	    if (owl_cls == null) return false;

	    boolean retval = removeRestriction(owl_cls, name, value, modifier);
	    if (retval == false) return retval;
	    return addRestriction(owl_cls, name, newvalue, newmodifier);

	}


    public boolean hasAnnotationProperty(OWLNamedClass cls, String slotname)
    {
		if (cls == null)
		{
			return false;
		}

		Slot slot = owlModel.getSlot(slotname);
		if (slot == null)
		{
			System.out.println("Slot name " + slotname + " not defined.");
			return false;
		}

		Collection slot_values = null;
		try {
			slot_values = cls.getOwnSlotValues(slot);
			if (slot_values != null && slot_values.size() > 0)
			{
				return true;
			}
			else
			{
                return false;
			}
		} catch (Exception e) {
			Log.getLogger().log(Level.WARNING, "Exception caught", e);;
		}
		return false;
    }

   boolean isRetired(OWLNamedClass retireCls)
    {
		OWLNamedClass retire_root = owlModel.getOWLNamedClass(getClsName(NCIEditTab.RETIRED_CONCEPTS));
		if (retire_root != null)
		{
			if (retireCls.isSubclassOf(retire_root))
			{
				return true;
			}
			else if (getInternalName(retireCls).compareTo(getClsName(NCIEditTab.RETIRED_CONCEPTS)) == 0)
			{
				return true;
			}
		}
		else
		{
			System.out.println("WARNING: Retired_Concepts is not defined.");
		}
		return false;
	}


    boolean isPreretired(OWLNamedClass retireCls)
    {
		OWLNamedClass retire_root = owlModel.getOWLNamedClass(getClsName(NCIEditTab.PRERETIRED_CONCEPTS));
		if (retire_root != null)
		{
			if (retireCls.isSubclassOf(retire_root))
			{
				return true;
			}
			else if (getInternalName(retireCls).compareTo(getClsName(NCIEditTab.PRERETIRED_CONCEPTS)) == 0)
			{
				return true;
			}
		}
		return false;
	}

    // 010407
    boolean isPremerged(OWLNamedClass cls)
    {
		OWLNamedClass premerged_root = owlModel.getOWLNamedClass(getClsName(NCIEditTab.PREMERGED_CONCEPTS));
		if (premerged_root != null)
		{
			if (cls.isSubclassOf(premerged_root))
			{
				return true;
			}
			else if (getInternalName(cls).compareTo(getClsName(NCIEditTab.PREMERGED_CONCEPTS)) == 0)
			{
				return true;
			}
		}
		return false;
	}


    public Boolean isReadOnly(String slotname)
    {
		Slot slot = owlModel.getSlot(slotname);
		if (slot == null)
		{
			System.out.println("Unidentifiable slot name: " + slotname);
			return Boolean.FALSE;
		}
		RDFProperty prop = (RDFProperty) slot;
		return prop.isReadOnly();
	}

	//041306
	public Vector getRestrictionSources(String con_name, String role_name)
	{
		if (con_name==null || role_name==null) return null;
		Vector v = new Vector();
		Cls targetCls = owlModel.getOWLNamedClass(con_name);
		RDFResource cls = (RDFResource) targetCls;
		Collection referenceCollection = cls.getReferringAnonymousClasses();
        if (referenceCollection != null)
        {
			Object[] obs = referenceCollection.toArray();
			for (int i=0; i<obs.length; i++)
			{
			    OWLAnonymousClass owlcls = (OWLAnonymousClass) obs[i];
				if (owlcls instanceof OWLRestriction)
				{
	//			    String restrication_name = ((OWLRestriction)owlcls).getOnProperty().getName();
	//092506
	String restrication_name = ((OWLRestriction)owlcls).getOnProperty().getBrowserText();

					if (restrication_name.equals(role_name))
					{
						Collection subs = owlcls.getDirectSubclasses();
						Object[] obs_2 = subs.toArray();
						for (int j=0; j<obs_2.length; j++)
						{
							OWLNamedClass owlcls_2 = (OWLNamedClass) obs_2[j];

	//						if (targetCls.getBrowserText().compareTo(owlcls_2.getBrowserText()) != 0)

	if (getInternalName(targetCls).compareTo(getInternalName(owlcls_2)) != 0)
								v.add(owlcls_2);
						}
					}
				}
			}
         }
         return v;
	}


	public Vector getRestrictionSources(String con_name)
	{
		if (con_name==null) return null;
		Vector v = new Vector();
		Cls targetCls = owlModel.getOWLNamedClass(con_name);
		RDFResource cls = (RDFResource) targetCls;
		Collection referenceCollection = cls.getReferringAnonymousClasses();
        if (referenceCollection != null)
        {
			Object[] obs = referenceCollection.toArray();
			for (int i=0; i<obs.length; i++)
			{
			    OWLAnonymousClass owlcls = (OWLAnonymousClass) obs[i];
                if (owlcls instanceof OWLRestriction)
				{
					Collection subs = owlcls.getDirectSubclasses();
					Object[] obs_2 = subs.toArray();
					for (int j=0; j<obs_2.length; j++)
					{
						OWLNamedClass owlcls_2 = (OWLNamedClass) obs_2[j];

if (getInternalName(targetCls).compareTo(getInternalName(owlcls_2)) != 0)

//						if (targetCls.getBrowserText().compareTo(owlcls_2.getBrowserText()) != 0)
							v.add(owlcls_2);
					}
				}
			}
         }
         return v;
	}

	//041306
	public Vector getInverseRestrictionStrings(String con_name, String role_name)
	{
		if (con_name==null || role_name==null) return null;
		Vector v = new Vector();
		Cls targetCls = owlModel.getOWLNamedClass(con_name);
		RDFResource cls = (RDFResource) targetCls;

		Collection referenceCollection = cls.getReferringAnonymousClasses();
        if (referenceCollection != null)
         {
			Object[] obs = referenceCollection.toArray();
			for (int i=0; i<obs.length; i++)
			{
			    OWLAnonymousClass owlcls = (OWLAnonymousClass) obs[i];
                if (owlcls instanceof OWLRestriction)
				{
                    OWLRestriction r = (OWLRestriction) owlcls;
                    String prop_modifier = getRestrictionType(r);
					RDFProperty p = r.getOnProperty();
					String prop_name = p.getName();
					String prop_value = r.getFillerText();

					if (prop_value.compareTo(con_name) == 0 && prop_name.compareTo(role_name) == 0)
					{

//						String s = prop_name + "|" + owlcls.getBrowserText() + "|" + prop_modifier;
						String s = prop_name + "|" + getInternalName(owlcls) + "|" + prop_modifier;

						v.add(s);
					}
				}
			}
         }
         return v;
	}

	//041306
	public Vector getInverseRestrictionStrings(OWLNamedClass cls)
	{
		if (cls==null) return null;
		Vector v = new Vector();

		//String con_name = cls.getBrowserText();
		String con_name = getInternalName(cls);

		Collection referringClasses = getReferringClasses(cls);
		Iterator itr = referringClasses.iterator();
		while (itr.hasNext())
		{
			OWLNamedClass referringClass = (OWLNamedClass) itr.next();
			//String referringClassName = referringClass.getBrowserText();
			String referringClassName = getInternalName(referringClass);

			Collection referenceCollection = referringClass.getDirectSuperclasses();
			if (referenceCollection != null)
			 {
				Object[] obs = referenceCollection.toArray();
				for (int i=0; i<obs.length; i++)
				{
					//OWLAnonymousClass owlcls = (OWLAnonymousClass) obs[i];
					//if (owlcls instanceof OWLRestriction)
					if (obs[i] instanceof OWLRestriction)
					{
						OWLRestriction r = (OWLRestriction) obs[i];
						String prop_modifier = getRestrictionType(r);
						RDFProperty p = r.getOnProperty();
						String prop_name = p.getName();
						String prop_value = r.getFillerText();

						if (prop_value.compareTo(con_name) == 0)
						{
							String s = prop_name + "|" + referringClassName + "|" + prop_modifier;
							v.add(s);
						}
					}
				}
			 }
		}
		return v;
	}

	//041306
	public OWLNamedClass getOWLNamedClass(String name){
		return owlModel.getOWLNamedClass(name);
	}

	//041306
	public Vector getRestrictionTargetStrings(OWLNamedClass owl_cls)
	{
		if (owl_cls == null) return null;
		Vector v = new Vector();
		boolean includingSuperclassRestrictions = false;
		java.util.Collection col = owl_cls.getRestrictions(includingSuperclassRestrictions);

		if (col != null)
		{
			final Object[] obs = col.toArray();
			if (obs.length > 0)
			{
				for (int i=0; i<obs.length; i++)
				{
					OWLRestriction r = (OWLRestriction) obs[i];

					if (!(r instanceof OWLNamedClass))
					{
						RDFProperty p = r.getOnProperty();
						String prop_name = p.getName();
						String prop_value = r.getFillerText();

						//OWLNamedClass targetCls = owlModel.getOWLNamedClass(prop_value);
						v.add(prop_name+"$"+prop_value);
					}
				}
			}
	    }
	    return v;
	}


	public boolean addDirectSubclasses(RDFSClass fromCls, RDFSClass toCls)
	{
		if (fromCls == null)
		{
			System.out.println("fromCls is null.");
			return false;
		}
		if (toCls == null)
		{
			System.out.println("toCls is null.");
			return false;
		}
		try {
			Collection subs = fromCls.getDirectSubclasses();
			for(Iterator iterator = subs.iterator(); iterator.hasNext();)
			{
				RDFSClass sub = (RDFSClass)iterator.next();
				addDirectSubclass(toCls, sub);
			}
			return true;
	    } catch (Exception e) {
			return false;
		}
	}


	public boolean addDirectSuperclass(RDFSClass hostClass, RDFSClass superCls){

		if (hostClass == null)
		{
			System.out.println("hostClass is null.");
			return false;
		}
		if (superCls == null)
		{
			System.out.println("superCls is null.");
			return false;
		}

		try {
			Collection sups = hostClass.getDirectSuperclasses();
			boolean toAdd = true;
			for(Iterator iterator = sups.iterator(); iterator.hasNext();){
				RDFSClass sup = (RDFSClass)iterator.next();
				if (sup instanceof OWLNamedClass)
				{
					if (getInternalName(sup).compareTo(getInternalName((Cls) superCls)) == 0)
					{
						toAdd = false;
						break;
					}
				}
			}
			if (toAdd) hostClass.addDirectSuperclass(superCls);

			if (hostClass.isSubclassOf(owlModel.getOWLThingClass())) {
				hostClass.removeSuperclass(owlModel.getOWLThingClass());
			}

			return true;
	    } catch (Exception e) {
			return false;
		}

	}


	public boolean addDirectSubclass(RDFSClass hostClass, RDFSClass subCls){

		if (hostClass == null)
		{
			System.out.println("hostClass is null.");
			return false;
		}
		if (subCls == null)
		{
			System.out.println("subCls is null.");
			return false;
		}

		Collection subs = hostClass.getDirectSubclasses();
		boolean toAdd = true;
		for(Iterator iterator = subs.iterator(); iterator.hasNext();)
		{
			RDFSClass sub = (RDFSClass)iterator.next();
			if (sub instanceof OWLNamedClass)
			{
				if (getInternalName(sub).compareTo(getInternalName((Cls) subCls)) == 0)
				{
					toAdd = false;
					break;
				}
			}
		}
		if (toAdd)
		{
        	subCls.addDirectSuperclass(hostClass);
	    }

        if (subCls.isSubclassOf(owlModel.getOWLThingClass())) {
			subCls.removeSuperclass(owlModel.getOWLThingClass());
		}
		return true;
    }

	public void removeDirectSubclasses(RDFSClass retireCls)
    {
		Collection subclasses = retireCls.getDirectSubclasses();
        if (subclasses != null)
        {
			final Object[] obs = subclasses.toArray();
			if (obs.length > 0)
			{
			    for (int i=0; i<obs.length; i++)
			    {
			    	Cls c = (Cls) obs[i];
					if (c instanceof OWLNamedClass)
					{
			    	    c.removeDirectSuperclass(retireCls);
					}
				}
			}
		}
	}

    public void redirectReferringClasses(Cls fromCls, Cls toCls)
    {
		Set hashset = new HashSet();

		RDFResource from_cls = (RDFResource) fromCls;
        RDFResource to_cls = (RDFResource) toCls;

		Collection referenceCollection = from_cls.getReferringAnonymousClasses();
        if (referenceCollection != null)
        {
			final Object[] obs = referenceCollection.toArray();
			for (int i=0; i<obs.length; i++)
			{
                if (obs[i] instanceof OWLRestriction)
				{
                    OWLRestriction r = (OWLRestriction) obs[i];
					RDFProperty p = r.getOnProperty();
					if (p != null)
					{
						String prop_name = p.getName();
						String s = r.getFillerText();
						if (s != null)
						{
							if (s.compareTo(getInternalName(fromCls)) != 0)
							{
								try {
									r.setFillerText(getInternalName(toCls));

								} catch (Exception e) {
									Log.getLogger().log(Level.WARNING, "Exception caught", e);;
								}
							}
						}
					}
				}

			}
         }
	 }




    public Vector getNonEditableClasses()
    {
		return noneditable_vec;
	}

	public boolean isNonEditable(OWLNamedClass owl_cls)
	{
		if (owl_cls == null) return false;

		//String name = owl_cls.getBrowserText();
		String name = getInternalName(owl_cls);
		return noneditable_vec.contains(name);
	}






	public boolean removeObjectProperty(OWLNamedClass owl_cls,
	                                String name,
	                                String value)
	{
		if (owl_cls == null) return false;

		if (name.compareTo(RDFSNames.Slot.IS_DEFINED_BY) == 0)
		{
			java.util.Collection col = owl_cls.getIsDefinedBy();
			if (col != null)
			{
				final Object[] obs = col.toArray();
				if (obs.length > 0)
				{
					for (int i=0; i<obs.length; i++)
					{
						RDFResource r = (RDFResource) obs[i];
						if (r instanceof OWLNamedClass)
						{
							//if (r.getBrowserText().compareTo(value) == 0)
							if (getInternalName((Cls) r).compareTo(value) == 0)
							{
								owl_cls.removeIsDefinedBy(r);
								return true;
							}

						}
					}
				}
			}
			return false;
	    }

		else if (name.compareTo(OWLNames.Slot.SAME_AS) == 0)
		{
			java.util.Collection col = owl_cls.getSameAs();
			if (col != null)
			{
				final Object[] obs = col.toArray();
				if (obs.length > 0)
				{
					for (int i=0; i<obs.length; i++)
					{
						RDFResource r = (RDFResource) obs[i];
						if (r instanceof OWLNamedClass)
						{
							//if (r.getBrowserText().compareTo(value) == 0)
							if (getInternalName((Cls) r).compareTo(value) == 0)
							{
								owl_cls.removeSameAs(r);
								return true;
							}

						}
					}
				}
			}
			return false;
	    }

		else if (name.compareTo(RDFSNames.Slot.SEE_ALSO) == 0)
		{
			OWLNamedClass slotvalue = (OWLNamedClass) owlModel.getCls(value);
			if (slotvalue == null)
			{
				return false;
			}
			//OWLObjectProperty slot = (Slot) owlModel.getOWLObjectProperty(name);
			Slot slot = (Slot) owlModel.getOWLObjectProperty(name);
			if (slot != null)
			{
				owlModel.removeOwnSlotValue(owl_cls, slot, slotvalue);
				return true;
		    }
			return false;
	    }

	    //
		else if (name.compareTo(OWLNames.Slot.DIFFERENT_FROM) == 0)
		{
			OWLNamedClass slotvalue = (OWLNamedClass) owlModel.getCls(value);
			if (slotvalue == null)
			{
				return false;
			}
			//OWLObjectProperty slot = (Slot) owlModel.getOWLObjectProperty(name);
			RDFProperty property = owlModel.getOWLDifferentFromProperty();

			Slot slot = (Slot) property;//.getOWLObjectProperty(name);
			if (slot != null)
			{
				owlModel.removeOwnSlotValue(owl_cls, slot, slotvalue);
				return true;
		    }
			return false;
	    }


		else if (name.compareTo(owlModel.getOWLEquivalentPropertyProperty().getName()) == 0
		      || name.compareTo(owlModel.getOWLDisjointWithProperty().getName()) == 0)

		{
			OWLNamedClass slotvalue = (OWLNamedClass) owlModel.getCls(value);
			if (slotvalue == null)
			{
				return false;
			}

			RDFProperty slot = (RDFProperty) owlModel.getRDFProperty(name);
			if (slot != null)
			{
				owlModel.removeOwnSlotValue(owl_cls, slot, slotvalue);
				return true;
		    }
			return false;
	    }


		else
		{
			OWLNamedClass slotvalue = (OWLNamedClass) owlModel.getCls(value);
			if (slotvalue == null)
			{
				return false;
			}

            Slot slot = null;

			Collection allProperties = owlModel.getVisibleUserDefinedRDFProperties();
			Collection selectableProperties = new ArrayList();
			for (Iterator it = allProperties.iterator(); it.hasNext();) {
				RDFProperty property = (RDFProperty) it.next();
				if (property.isAnnotationProperty()
				    && property.hasObjectRange()
				    && property instanceof DefaultOWLObjectProperty)
				{
					if (property.getName().compareTo(name) == 0)
					{
						slot = (Slot) property;
					}
				}
			}
			if (slot == null)
			{
				System.out.println("Protege.Slot: " + name + " not found.");
				return false;
			}
			else
			{
				System.out.println("removeObjectProperty slotvalue: " + slotvalue.getBrowserText());
			}
			System.out.println("Calling owlModel.removeOwnSlotValue slot " + slot.getBrowserText());
			owlModel.removeOwnSlotValue(owl_cls, slot, slotvalue);
			System.out.println("Exit owlModel.removeOwnSlotValue slot " + slot.getBrowserText());

		}
		return true;
	}



    public edu.stanford.smi.protege.model.Cls getMetaClsByName(String name)
    {
        edu.stanford.smi.protege.model.Cls[] metaClses = ProfilesManager.getSupportedRestrictionMetaClses(owlModel);
        for (int i=0; i<metaClses.length; i++)
        {
			edu.stanford.smi.protege.model.Cls cls = metaClses[i];
			if (cls.getBrowserText().compareTo(name) == 0) return cls;
		}
 		return null;
	}


    private RDFProperty getRestrictionProperty(Cls metaCls) {
		if (metaCls == null)
		{
			return owlModel.getRDFProperty(OWLNames.Slot.SOME_VALUES_FROM);
		}

        if (metaCls.getName().equals(OWLNames.Cls.ALL_VALUES_FROM_RESTRICTION)) {
            return owlModel.getRDFProperty(OWLNames.Slot.ALL_VALUES_FROM);
        }
        else if (metaCls.getName().equals(OWLNames.Cls.SOME_VALUES_FROM_RESTRICTION)) {
            return owlModel.getRDFProperty(OWLNames.Slot.SOME_VALUES_FROM);
        }
        else if (metaCls.getName().equals(OWLNames.Cls.HAS_VALUE_RESTRICTION)) {
            return owlModel.getRDFProperty(OWLNames.Slot.HAS_VALUE);
        }
        else if (metaCls.getName().equals(OWLNames.Cls.MAX_CARDINALITY_RESTRICTION)) {
            return owlModel.getRDFProperty(OWLNames.Slot.MAX_CARDINALITY);
        }
        else if (metaCls.getName().equals(OWLNames.Cls.MIN_CARDINALITY_RESTRICTION)) {
            return owlModel.getRDFProperty(OWLNames.Slot.MIN_CARDINALITY);
        }
        else {
            return owlModel.getRDFProperty(OWLNames.Slot.CARDINALITY);
        }
    }


    private Vector getPropertiesList() {
		Vector v = new Vector();
		try {
			if (owlModel == null)
			{
				System.out.println("WARNING: owlModel == null");
				return v;
			}

			Collection allProperties = owlModel.getVisibleUserDefinedRDFProperties();
			Collection selectableProperties = new ArrayList();
			for (Iterator it = allProperties.iterator(); it.hasNext();) {
				RDFProperty property = (RDFProperty) it.next();
				if (!property.isAnnotationProperty()) {
					v.add(property);
				}
			}
	    } catch (Exception e) {
			Log.getLogger().log(Level.WARNING, "Exception caught", e);;
		}

        return v;
    }



    private OWLRestriction createRestriction(edu.stanford.smi.protege.model.Cls metaCls,
                                             RDFProperty property, String text) throws Exception {
        Collection parents = CollectionUtilities.createCollection(((KnowledgeBase) owlModel).getCls(OWLNames.Cls.ANONYMOUS_ROOT));
        KnowledgeBase kb = owlModel;
        OWLRestriction restriction = (OWLRestriction) kb.createCls(null, parents, metaCls);
        restriction.setOnProperty(property);
        restriction.setFillerText(text);
        return restriction;
    }



    public OWLRestriction createOWLRestriction(
		                   String metaClsName,  // modifier name
                           String propertyName, // role name
                           String text) throws Exception  // filler
    {
        edu.stanford.smi.protege.model.Cls[] metaClses = ProfilesManager.getSupportedRestrictionMetaClses(owlModel);

		edu.stanford.smi.protege.model.Cls metaCls = null;

		String metaCls_name = restrictionType2Label(metaClsName);
		for (int i=0; i<metaClses.length; i++)
		{
			if (metaClses[i].getBrowserText().compareTo(metaCls_name) == 0)
			{
				metaCls = metaClses[i];
				break;
			}
		}

		if (metaCls == null)
		{
			return null;
		}

		RDFProperty property = null;
		Vector properties = getPropertiesList();
		for (int i=0; i<properties.size(); i++)
		{
			RDFProperty p = (RDFProperty) properties.elementAt(i);
			//if (p.getBrowserText().compareTo(propertyName) == 0)
			if (p.getName().compareTo(propertyName) == 0)
			{
				property = p;
				break;
			}
		}
		if (property == null)
		{
			System.out.println("property " + propertyName + " not found -- return null");
			return null;
		}

        Collection parents = CollectionUtilities.createCollection(((KnowledgeBase) owlModel).getCls(OWLNames.Cls.ANONYMOUS_ROOT));
        KnowledgeBase kb = owlModel;
        OWLRestriction restriction = (OWLRestriction) kb.createCls(null, parents, metaCls);
        restriction.setOnProperty(property);
        restriction.setFillerText(text);

        return restriction;
    }






	public boolean addRestriction(OWLNamedClass owl_cls, String restrictionString)
	{
		StringTokenizer st = new StringTokenizer(restrictionString, "|");
		String name = st.nextToken();
		String value = st.nextToken();
		String modifier = st.nextToken();
		return addRestriction(owl_cls, name, value, modifier);
	}


	public boolean addRestriction(OWLNamedClass owl_cls,
	                                String name,
	                                String value,
	                                String modifier)
	{
		if (getCls(value) == null)
		{
			try {
    			return createOWLRestriction(owl_cls, modifier, name, value);
			} catch (Exception e) {
				System.out.println("Exception thrown in OWLWrapper addRestriction.");
				return false;
			}
		}

		Collection allProperties = owlModel.getVisibleUserDefinedRDFProperties();
		Collection selectableProperties = new ArrayList();
		RDFProperty property = null;
		for (Iterator it = allProperties.iterator(); it.hasNext();) {
			property = (RDFProperty) it.next();
			if (!property.isAnnotationProperty()) {
				//if (name.compareTo(property.getBrowserText()) == 0)
				if (name.compareTo(property.getName()) == 0)
				{
					break;
				}
			}
		}

		if (property == null)
		{
			System.out.println("WARNING: addRestriction -- cannot find property");
			return false;

		}


        OWLNamedClass target_cls = (OWLNamedClass) getFrameNameMatches(value);
		if (target_cls == null)
		{
			System.out.println("WARNING: class " + value + " not found,");
			return false;
		}

		if (modifier.compareToIgnoreCase("all") == 0 || modifier.compareToIgnoreCase("only") == 0)
		{
			 //OWLNamedClass target_cls = owlModel.getOWLNamedClass(value);

             owl_cls.addSuperclass(owlModel.createOWLAllValuesFrom(property, target_cls));
             return true;
		}
		else if (modifier.compareToIgnoreCase("some") == 0)
		{
			 //OWLNamedClass target_cls = owlModel.getOWLNamedClass(value);

             owl_cls.addSuperclass(owlModel.createOWLSomeValuesFrom(property, target_cls));
             return true;
		}
		else if (modifier.compareToIgnoreCase("has") == 0)
		{
			 //OWLNamedClass target_cls = owlModel.getOWLNamedClass(value);

             owl_cls.addSuperclass(owlModel.createOWLHasValue(property, target_cls));
             return true;
		}
		else if (modifier.compareToIgnoreCase("exactly") == 0)
		{
			int intValue = Integer.parseInt(value);
            owl_cls.addSuperclass(owlModel.createOWLCardinality(property, intValue));
            return true;
		}
		else if (modifier.compareToIgnoreCase("min") == 0)
		{
			int intValue = Integer.parseInt(value);
            owl_cls.addSuperclass(owlModel.createOWLMinCardinality(property, intValue));
            return true;
		}
		else if (modifier.compareToIgnoreCase("max") == 0)
		{
			int intValue = Integer.parseInt(value);
            owl_cls.addSuperclass(owlModel.createOWLMaxCardinality(property, intValue));
            return true;
		}
		return false;
	}




    public boolean createOWLRestriction(
		                   OWLNamedClass owl_cls,
		                   String metaClsName,  // abbrev modifier
                           String propertyName, // role name
                           String text) throws Exception  // filler
    {
        edu.stanford.smi.protege.model.Cls[] metaClses = ProfilesManager.getSupportedRestrictionMetaClses(owlModel);

		edu.stanford.smi.protege.model.Cls metaCls = null;

		String s = restrictionType2Label(metaClsName);

		for (int i=0; i<metaClses.length; i++)
		{
			if (metaClses[i].getBrowserText().compareTo(s) == 0)
			{
				metaCls = metaClses[i];
				break;
			}
		}

		if (metaCls == null)
		{
			return false;
		}

		RDFProperty property = null;
		Vector properties = getPropertiesList();
		for (int i=0; i<properties.size(); i++)
		{
			RDFProperty p = (RDFProperty) properties.elementAt(i);
			//if (p.getBrowserText().compareTo(propertyName) == 0)
			if (p.getName().compareTo(propertyName) == 0)
			{
				property = p;
				break;
			}
		}
		if (property == null)
		{
			return false;
		}

        Collection parents = CollectionUtilities.createCollection(((KnowledgeBase) owlModel).getCls(OWLNames.Cls.ANONYMOUS_ROOT));
        KnowledgeBase kb = owlModel;
        OWLRestriction restriction = (OWLRestriction) kb.createCls(null, parents, metaCls);
        restriction.setOnProperty(property);
        restriction.setFillerText(text);

        owl_cls.addSuperclass(restriction);

        return true;
    }


    public String assignCode(Cls cls)
    {
		if (cls == null) return null;
		Slot codeSlot = owlModel.getSlot(getSlotName(NCIEditTab.CODE));
		String code = null;
		if (codeSlot != null)
		{
			if (cls.getOwnSlotValueCount(codeSlot) == 0) {
				code = generator.getCode();
				owlModel.addOwnSlotValue(cls, codeSlot, code);
				return code;
			}
		}
		return null;
	}




    public String assignName(Cls cls, String internalName)
    {
		if (cls == null)
		{
			System.out.println("cls == null");
			return null;
		}
		SystemFrames _systemFrames = owlModel.getSystemFrames();
		Slot name_slot = _systemFrames.getNameSlot();

		if (name_slot == null)
		{
			System.out.println("name_slot == null -- nothing done");
			return null;
		}

		if (internalName == null || internalName.compareTo("") == 0) {
			if (generator != null)
			{
				internalName = generator.getCode();
			}
			else
			{
				System.out.println("WARNING: Code generator is not available.");
                return null;
			}
		}

		cls.setDirectOwnSlotValue(name_slot, internalName);
		return internalName;
	}



    public static String getSlotValue(Cls cls, String slotname)
    {
		if (cls == null) return null;
		Slot slot = owlModel.getSlot(slotname);
		if (slot != null)
		{
			Collection slot_values = null;
			try {
				slot_values = cls.getOwnSlotValues(slot);
				if (slot_values != null && slot_values.size() > 0)
				{
					Object[] obs = slot_values.toArray();
					return (String) obs[0];
				}
			}
			catch (Exception e) {
				Log.getLogger().log(Level.WARNING, "Exception caught", e);;
			}
		}
		return null;
	}



    public static String getInternalName(Cls cls)
    {
		if (cls == null) return null;
		SystemFrames _systemFrames = owlModel.getSystemFrames();
		Slot name_slot = _systemFrames.getNameSlot();

		if (name_slot == null)
		{
			System.out.println("name_slot == null -- nothing done");
			return null;
		}
        return getSlotValue(cls, name_slot.getBrowserText());
	}

	public Cls createCls(String name, String pt, Cls supCls, String def)
	{
		/*
		try {
			return createCls(name, pt, supCls, def, true);
	    } catch (Exception e) {
			throw new RuntimeException(e);
		}
		*/
		return createCls(name, pt, supCls, def, true);
	}

    public Cls createCls(String name, String pt, String sup)
    {
		Cls supCls = getCls(sup);
		if (supCls == null) return null;
		return createCls(name, pt, supCls, null);

		/*
		try {
			Cls supCls = getCls(sup);
			if (supCls == null) return null;
			return createCls(name, pt, supCls, null);
	    } catch (Exception e) {
			throw new RuntimeException(e);
		}
		*/
	}

    public Cls createCls(String name, boolean generatecode)
    {
		return createCls(name, null, null, null, generatecode);
		/*
		try {
	        return createCls(name, null, null, null, generatecode);
	    } catch (Exception e) {
			throw new RuntimeException(e);
		}
		*/
	}

	public Cls createCls(String name, String pt, Cls supCls, String def, boolean generatecode)
	{
        Cls cls = null;
		String internalName = null;
		String code = "";

		if (generatecode)
		{
			code = generator.getCode();
		}
		else
		{
			code = name;
		}

		if (NCIEditTab.owl_format == NCIEditTab.BY_CODE)
		{
			if (code == null || code.compareTo("0") == 0)
			{
				System.out.println("Unable to create class -- code generator not available.");
				return null;
			}
		}

        try {
			owlModel.beginTransaction("Create class " + name, name);
            // if codeSlotExists, then Cls name is name; otherwise, Cls name is code
            cls = owlModel.createCls(null, new Vector());
            if (cls == null)
            {
				owlModel.rollbackTransaction();
				return null;
			}

            if (NCIEditTab.owl_format == NCIEditTab.BY_NAME)
            {
				internalName = assignName(cls, name);
				if (code != null && code.compareTo("0") != 0)
				{
					if (generatecode)
					{
						boolean retval = addAnnotationProperty(cls, codeSlotName, code);
					}
				}
		    }
		    else
		    {
				if (code != null && code.compareTo("0") != 0)
				{
					internalName = assignName(cls, code);
				}
				else
				{
					// Unable to generate code, return null;
					internalName = assignName(cls, name);
				}
			}
            if (generatecode)
            {
			    //addAnnotationProperty(cls, "rdfs:label", name);
			    addAnnotationProperty(cls, "rdfs:label", pt);
			}

			if (supCls != null)
			{
				boolean retval = addDirectSuperclass(cls, supCls);
			}
			String term_group = "PT";
			String source = "NCI";

			if (pt != null && pt.compareTo("") != 0)
			{
				addAnnotationProperty(cls, (String) name2SlotName.get(NCIEditTab.PREFERRED_NAME), pt);
				addAnnotationProperty(cls, (String) name2SlotName.get(NCIEditTab.FULL_SYN), formatFULL_SYN(pt, term_group, source));
		    }
			if (def != null && def.compareTo("") != 0)
			{
				addAnnotationProperty(cls, (String) name2SlotName.get(NCIEditTab.DEFINITION), formatDEFINITION(source, def));
			}
			owlModel.commitTransaction();
	    } catch (Exception e) {
			owlModel.rollbackTransaction();
			//OWLUI.handleError(owlModel, e);
			//throw new RuntimeException(e);
			return null;
		}
		return cls;
	}


    public boolean printable(String line)
    {
		if (line.contains(":DIRECT-SUPERCLASSES")) return false;
		if (line.contains(":DIRECT-TYPE")) return false;
		if (line.contains("@_")) return false;
		if (line.contains("rdf:type")) return false;

		return true;
	}


    public OWLNamedClass getClsByLabel(String label)
    {
		Slot slot = owlModel.getSlot("rdfs:label");
		Collection c = owlModel.getMatchingResources(
			   (RDFProperty) slot, label, 1);
		if (c.isEmpty()) return null;

		Iterator it = c.iterator();
		while (it.hasNext()) {
			OWLNamedClass cls = (OWLNamedClass) it.next();
			return cls;
		}
		return null;
	}

	public Cls getCls(String name)
	{
		Cls cls = null;
		try {
			cls = owlModel.getCls(name);
		    if (cls != null) return cls;
            return getClsByLabel(name);

		} catch (Exception e) {
			return null;
		}
	}

	public Slot getSlot(String slotname)
	{
		Slot slot = null;
		try {
		    slot = owlModel.getSlot(slotname);
		    return slot;
		} catch (Exception e) {
			System.out.println("Slot " + slotname + " does not exist.");
			return null;
		}
	}



	public void flushEvents()
	{
		default_kb.flushEvents();
	}





    public String getCode(Cls cls)
    {
		if (cls == null) return null;
		Slot codeSlot = owlModel.getSlot(getSlotName(NCIEditTab.CODE));
		if (codeSlot != null)
		{
			Collection slot_values = null;
			try {
				slot_values = cls.getOwnSlotValues(codeSlot);
				if (slot_values != null && slot_values.size() > 0)
				{
					Object[] obs = slot_values.toArray();
					return (String) obs[0];
				}
			}
			catch (Exception e) {
				Log.getLogger().log(Level.WARNING, "Exception caught", e);;
			}
		}
		//KLO 070806
		//return cls.getBrowserText();//null;
		return "0";
	}


    public String getObjectValue(Object obj)
    {
		if (obj == null) return null;
		Object newValue;
		String str = obj.toString();
		if (obj instanceof RDFSLiteral) {
			RDFSLiteral oldLiteral = (RDFSLiteral) obj;
			newValue = owlModel.createRDFSLiteral(str, oldLiteral.getDatatype());
		}
		else if (obj instanceof Boolean) {
			newValue = Boolean.valueOf(str.equals("true"));
		}
		else if (obj instanceof Float) {
			newValue = Float.valueOf(str);
		}
		else if (obj instanceof Integer) {
			newValue = Integer.valueOf(str);
		}
		else {
			newValue = str;
		}
		return newValue.toString();
	}


	public boolean isReadOnlyProperty(String name)
	{
	    if (readOnlyProperties.contains(name)) return true;
	    return false;
	}

	public Vector getReadOnlyProperties()
	{
		return readOnlyProperties;
	}

	public boolean canModify(String name)
	{
		return (!isReadOnlyProperty(name));
	}

	/////////////////////////////////
    public Vector getNamedSubclassNames(Cls cls)
    {
		Vector v = new Vector();
		if (cls == null) return v;
		Collection subClassCollection=cls.getDirectSubclasses();
	 	if (subClassCollection != null)
		{
			final Object[] obs = subClassCollection.toArray();
			for (int i=0; i<obs.length; i++)
			{
				Cls c = (Cls) obs[i];
			    if (c instanceof OWLNamedClass)
				{
					//v.add(c.getBrowserText());
					v.add(getInternalName(c));
				}
			}
		}
		return v;
	 }

    public Collection getDirectSubclasses(Cls superCls)
    {
 	    return superCls.getDirectSubclasses();
	}

    public QuickSortVecStrings getSortUtility()
    {
		return this.sort;
	}

   public String convertObjecttoString(Object obj, ValueType type) {
		if(type.equals(ValueType.STRING))
			return obj.toString();
		else if(type.equals(ValueType.FLOAT))
			return obj.toString();
		else if(type.equals(ValueType.INTEGER))
			return obj.toString();
		else if(type.equals(ValueType.ANY))
			return obj.toString();
		else if(type.equals(ValueType.BOOLEAN))
			return obj.toString();
		else if(type.equals(ValueType.CLS)) {
			Cls cls = (Cls) obj;
			return cls.getName();
		}
		else if(type.equals(ValueType.INSTANCE)) {
			Instance inst = (Instance) obj;
			return inst.getName();
			//			return inst.getName()+"("+inst.getDirectType().getName()+")";
		}
		else if(type.equals(ValueType.SYMBOL))
			return obj.toString();
		else return "Unknown";
    }

    public ValueType getObjectValueType(Object obj) {
		if (obj instanceof Boolean)
			return ValueType.BOOLEAN;
        else if (obj instanceof Cls)
			return ValueType.CLS;
        else if (obj instanceof Float)
			return ValueType.FLOAT;
        else if (obj instanceof Instance)
			return ValueType.INSTANCE;
        else if (obj instanceof Integer)
			return ValueType.INTEGER;
        else if (obj instanceof String)
			return ValueType.STRING;
        else
			return ValueType.ANY;
    }

    public Collection getNamedSubclasses(Cls cls)
    {
		Set hashset = new HashSet();
		if (cls == null) return hashset;
		Collection subClassCollection=cls.getDirectSubclasses();
	 	if (subClassCollection != null)
		{
			final Object[] obs = subClassCollection.toArray();
			for (int i=0; i<obs.length; i++)
			{
				Cls c = (Cls) obs[i];
			    if (c instanceof OWLNamedClass)
				{
					hashset.add(c);
				}
			}
		}
		return hashset;
	 }





    public Collection getNamedSuperclasses(Cls cls)
    {
		Set hashset = new HashSet();
		if (cls == null) return hashset;
		Collection subClassCollection=cls.getDirectSuperclasses();
	 	if (subClassCollection != null)
		{
			final Object[] obs = subClassCollection.toArray();
			for (int i=0; i<obs.length; i++)
			{
				Cls c = (Cls) obs[i];
			    if (c instanceof OWLNamedClass)
				{
					hashset.add(c);
				}
			}
		}
		return hashset;
	 }

    public Collection getReferringClasses(Cls targetCls)
    {
		//Vector v = new Vector();
		Set hashset = new HashSet();
		if (targetCls == null) return hashset;
		//Vector referenceClasses = new Vector();
		RDFResource cls = (RDFResource) targetCls;

		//Collection referenceCollection = getReferringAnonymousClasses(cls);
		Collection referenceCollection = cls.getReferringAnonymousClasses();
        if (referenceCollection != null)
        {
			final Object[] obs = referenceCollection.toArray();
			for (int i=0; i<obs.length; i++)
			{
			    OWLAnonymousClass owlcls = (OWLAnonymousClass) obs[i];
			    Collection sups = owlcls.getDirectSubclasses();
			    final Object[] obs_2 = sups.toArray();
				for (int j=0; j<obs_2.length; j++)
			    {
				    OWLNamedClass owlcls_2 = (OWLNamedClass) obs_2[j];

if (getInternalName(targetCls).compareTo(getInternalName(owlcls_2)) != 0)

                    //if (targetCls.getBrowserText().compareTo(owlcls_2.getBrowserText()) != 0)
					{
						hashset.add(owlcls_2);
					}
		        }
			}
         }
         return hashset;
	 }

	private boolean hasEquivalentClasses(OWLNamedClass cls)
	{
        Collection col = cls.getEquivalentClasses();
		final Object[] obs = col.toArray();
        if (obs.length == 0) return false;
		return true;
	}


	public boolean addAnnotationProperty(OWLNamedClass cls, String propertyname, String propertyvalue)
	{
		if (cls == null)
		{
			return false;
		}

		Slot slot = owlModel.getSlot(propertyname);
		if (slot == null)
		{
		    System.out.println("Unable to add property -- " + propertyname + " not defined.");
		    return false;
		}
		owlModel.addOwnSlotValue(cls, slot, propertyvalue);
		return true;
	}


// for association (object valued annotation property
	public boolean addAnnotationProperty(OWLNamedClass cls, String propertyname, OWLNamedClass propertyvalue)
	{
		if (cls == null) return false;

		Collection allProperties = owlModel.getVisibleUserDefinedRDFProperties();

		allProperties.add(owlModel.getOWLDifferentFromProperty());
		allProperties.add(owlModel.getOWLDisjointWithProperty());
		allProperties.add(owlModel.getOWLEquivalentPropertyProperty());
		allProperties.add(owlModel.getOWLSameAsProperty());
		allProperties.add(owlModel.getRDFSIsDefinedByProperty());
		allProperties.add(owlModel.getRDFProperty(RDFSNames.Slot.SEE_ALSO));

		RDFProperty property = null;

		for (Iterator it = allProperties.iterator(); it.hasNext();) {
			property = (RDFProperty) it.next();
			if (property.isAnnotationProperty() && property.hasObjectRange()
				&& property instanceof DefaultOWLObjectProperty) {

//				if (propertyname.compareTo(property.getBrowserText()) == 0)
if (propertyname.compareTo(property.getName()) == 0)
				{
					break;
				}
			}
		}

		if (property == null)
		{
			System.out.println("WARNING: addRestriction -- cannot find property");
			return false;

		}

		owlModel.addOwnSlotValue(cls, property, propertyvalue);
		return true;
	}


	private void addAnnotationProperties(OWLNamedClass cls, String propertyname, Vector v)
	{
		if (cls == null) return;
		if (v == null) return;
		if (v.size() == 0) return;

		Slot slot = owlModel.getSlot(propertyname);
		if (slot == null)
		{
		    System.out.println("Unable to add properties -- " + propertyname + " not defined.");
		    return;
		}
	    for (int i=0; i<v.size(); i++)
		{
		    String propertyvalue = (String) v.elementAt(i);
		    owlModel.addOwnSlotValue(cls, slot, propertyvalue);
		}
	}

    public boolean isEditable(String cls_name)
    {
		if (noneditable_vec.contains(cls_name)) return false;
		return true;
	}

    public boolean isEditable(OWLNamedClass cls)
    {
		// 080506
		if (cls.equals(owlModel.getOWLThingClass())) return false;

		Cls owl_cls = owlModel.getCls(getClsName(NCIEditTab.RETIRED_CONCEPTS));
		if (owl_cls == null) return true;

		owl_cls = owlModel.getCls(getClsName(NCIEditTab.PRERETIRED_CONCEPTS));
		if (owl_cls == null) return true;

		owl_cls = owlModel.getCls(getClsName(NCIEditTab.PREMERGED_CONCEPTS));
		if (owl_cls == null) return true;

		OWLNamedClass retiredCls = (OWLNamedClass) owlModel.getCls(getClsName(NCIEditTab.RETIRED_CONCEPTS));
		OWLNamedClass preretiredCls = (OWLNamedClass) owlModel.getCls(getClsName(NCIEditTab.PRERETIRED_CONCEPTS));
		OWLNamedClass premergedCls = (OWLNamedClass) owlModel.getCls(getClsName(NCIEditTab.PREMERGED_CONCEPTS));

		if (cls.equals(owlModel.getOWLThingClass()) ||
			cls.equals(retiredCls) ||
			cls.equals(preretiredCls) ||
			cls.equals(premergedCls))
		{
			return false;
		}
        return true;
	}

	public HashSet getAnnotations(RDFSClass cls)
	{
		HashSet hset = new HashSet();

		if (cls == null)
		{
			System.out.println("getAnnotations input parameter RDFSClass is null.");
			return hset;
		}
		else
		{
			System.out.println("getAnnotations " + cls.getName());
		}

		Collection annotationSlots = owlModel.getOWLAnnotationProperties();
		if (annotationSlots == null)
		{
			System.out.println("WARNING: owlModel.getOWLAnnotationProperties in getAnnotations returns null.");
			return hset;
		}
		for(Iterator iterator = annotationSlots.iterator(); iterator.hasNext();)
		{
			Slot slot = (Slot) iterator.next();
			String slotName = slot.getName();
			System.out.println("AnnotationSlots: " + slotName);

			Collection slot_values = null;
			try {
			    slot_values = cls.getOwnSlotValues(slot);
				if (slot_values != null)
				{
					for(Iterator iterator3 = slot_values.iterator(); iterator3.hasNext();)
					{
						Object slot_value = (Object) iterator3.next();
						String value = slotName + "|" + slot_value.toString();
				 		System.out.println("AnnotationSlots value: " + value);
						if (!hset.contains(value))
						{
							hset.add(value);
						}
					}
				}
				else
				{
					System.out.println("WARNING: cls.getOwnSlotValues in getAnnotations returns null.");
				}
			}
			catch (Exception e)
			{
				System.out.println("cls.getOwnSlotValues threw exception.");
				Log.getLogger().log(Level.WARNING, "Exception caught", e);;
			}
		}
		return hset;
    }

	public String getSlotName(String name)
	{
		if (name2SlotName == null) return name;

		if (name2SlotName.containsKey(name))
		{
			return (String) name2SlotName.get(name);
		}
		return name;
	}

	public String getClsName(String name)
	{
		if (name2ClsName == null) return name;

		if (name2ClsName.containsKey(name))
		{
			return (String) name2ClsName.get(name);
		}
		return name;
	}



    public Cls getFrameNameMatches(String name)
    {
		Collection c = default_kb.getFrameNameMatches(name, 1);

		for (Iterator it = c.iterator(); it.hasNext();) {
			RDFSClass cls = (RDFSClass) it.next();
			return (Cls) cls;
		}

		return null;
	}



/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    private void dumpDIRECT_SUPERCLASSES(String label, OWLNamedClass hostClass)
    {
		Slot slot = ((KnowledgeBase) hostClass.getOWLModel()).getSlot(Model.Slot.DIRECT_SUPERCLASSES);
        for (Iterator it = ((Cls) hostClass).getDirectOwnSlotValues(slot).iterator(); it.hasNext();)
        {
            Cls superCls = (Cls) it.next();


            if (superCls instanceof RDFSClass && superCls.getDirectOwnSlotValues(slot).contains(hostClass))
            {
				System.out.println("DIRECT_SUPERCLASSES=======equivalent class: " + superCls.getBrowserText());
			}
		}
	}


    private void dumpEquivalentClasses(String label, OWLNamedClass hostClass)
    {
		Collection c = hostClass.getEquivalentClasses();
		Iterator it = c.iterator();

		int knt = 0;
		while (it.hasNext())
		{
			Cls superCls = (Cls) it.next();
		}
	}




    public RDFSClass parsableText2RDFSClass(String parsableText)
    {
   	    RDFSClass newRestriction = null;
    	//RDFSClass oldRestriction = (RDFSClass) getClass(rowIndex);
		try {
			OWLClassParser parser = owlModel.getOWLClassDisplay().getParser();
			newRestriction = parser.parseClass(owlModel, parsableText);
			return newRestriction;
		}
		catch (OWLClassParseException e) {
			ProtegeUI.getModalDialogFactory().showErrorMessageDialog(owlModel, "Error at parsing restriction: " + parsableText, "Parse Error");
			return null;
		}
	}


    private boolean hasRestriction(OWLNamedClass hostClass, RDFSClass rdfsclass)
    {
	    Collection col = hostClass.getRestrictions();
	    if (col.size() == 0) return false;
        for (Iterator it = col.iterator(); it.hasNext();) {
			OWLRestriction r = (OWLRestriction) it.next();
			if (r.getBrowserText().compareTo(rdfsclass.getBrowserText()) == 0)
			{
				return true;
			}
		}
		return false;
	}


    public RDFSClass getEquivalentClass(OWLNamedClass hostClass)
    {
		/*
		// assume only one equivalent class
		Collection c = hostClass.getEquivalentClasses();
		Iterator it = c.iterator();
		while (it.hasNext())
		{
			RDFSClass definition = (RDFSClass) it.next();
			return definition;
		}
		return null;
		*/
		return hostClass.getDefinition();
	}



    public boolean containsDefinition(OWLNamedClass hostClass, RDFSClass aClass)
    {
		if (aClass == null) return false;

		RDFSClass definition = null;
		Collection c = hostClass.getEquivalentClasses();

		if (c.size() == 0) return false;

		Iterator it = c.iterator();
		while (it.hasNext())
		{
			definition = (RDFSClass) it.next();
			if (definition.getBrowserText().compareTo(aClass.getBrowserText()) == 0)
			{
				return true;
			}
		}

		if (definition instanceof OWLIntersectionClass)
		{
			OWLIntersectionClass intersectionCls = (OWLIntersectionClass) definition;
			if (intersectionCls.hasOperandWithBrowserText(aClass.getBrowserText()))
			    return true;
		}
		return false;
	}



    public Vector getDirectSuperclassItems(Cls hostClass, int set_type)
    {
		Vector v = new Vector();
		HashSet hset = new HashSet();
		if (set_type == NCIConditionsTableModel.SET_SUPERCLASS)
		{
			Slot superclassesSlot = owlModel.getSlot(Model.Slot.DIRECT_SUPERCLASSES);
			for (Iterator it = ((Cls) hostClass).getDirectOwnSlotValues(superclassesSlot).iterator(); it.hasNext();)
			{
				Cls superCls = (Cls) it.next();
				if (superCls instanceof RDFSNamedClass)
				{
					RDFSClass aClass = (RDFSClass) superCls;
                    if (aClass instanceof OWLNamedClass)
                    {
                        String value = SUBCLASSOF + ": " + getInternalName(superCls);
                        if (!hset.contains(value))
                        {
							TreeItem item = new TreeItem();
							item.setType(TreeItem.TYPE_PARENT);
							item.setName(SUBCLASSOF);

							item.setValue(getInternalName(superCls));
							item.setNameValue(value);
							item.setCls((RDFSClass) superCls);
							v.add(item);
							hset.add(value);
					    }
				    }

				}
			}
		}


		else if (set_type == NCIConditionsTableModel.SET_RESTRICTION)
		{
			//for (Iterator it = hostClass.getSuperclasses().iterator(); it.hasNext();)
			for (Iterator it = hostClass.getDirectSuperclasses().iterator(); it.hasNext();)
			{
				Cls superCls = (Cls) it.next();
				if (superCls instanceof OWLAnonymousClass )
				{
					RDFSClass r = (RDFSClass) superCls;

                    //if (aClass instanceof OWLAnonymousClass && !(aClass instanceof OWLIntersectionClass))
                    if (r instanceof OWLAnonymousClass)
                    {
						// need to exclude equivalent classes
                        boolean isDefinition = containsDefinition((OWLNamedClass) hostClass, r);
                        if (!isDefinition)
                        {
							String value = r.getBrowserText();
							if (!hset.contains(value))
							{
								TreeItem item = new TreeItem();
								item.setType(TreeItem.TYPE_RESTRICTION);
								item.setNameValue(r.getBrowserText());
								item.setCls((RDFSClass) r);
								v.add(item);
								hset.add(value);
						    }
					    }
				    }
				}
			}
		}
		return v;
    }


    public Vector getDefinitionItems(Cls hostClass, int set_type) {
		Vector v = new Vector();
		HashSet hset = new HashSet();

        Slot slot = owlModel.getSlot(Model.Slot.DIRECT_SUPERCLASSES);
        for (Iterator it = ((Cls) hostClass).getDirectOwnSlotValues(slot).iterator(); it.hasNext();) {
            Cls superCls = (Cls) it.next();

            if (superCls instanceof RDFSClass && superCls.getDirectOwnSlotValues(slot).contains(hostClass))
            {
                RDFSClass equivalentClass = (RDFSClass) superCls;
                if (equivalentClass instanceof OWLIntersectionClass)
                {
                    OWLIntersectionClass intersectionCls = (OWLIntersectionClass) equivalentClass;
                    //121406 KLO
                    if (set_type == NCIConditionsTableModel.SET_RESTRICTION && containsSimpleRestrictionsOnly(intersectionCls))
                    {
						String value = intersectionCls.getBrowserText();
						if (!hset.contains(value))
						{
							TreeItem item = new TreeItem();
							item.setType(TreeItem.TYPE_RESTRICTION);
							item.setNameValue(value);
							item.setCls((RDFSClass) equivalentClass);
							item.setIsDefining(true);
							v.add(item);
							hset.add(value);
						}
					}
					else
					{
 						Collection operands = ((OWLIntersectionClass) equivalentClass).getOperands();
						for (Iterator oit = operands.iterator(); oit.hasNext();) {
							RDFSClass operand = (RDFSClass) oit.next();

							if (set_type == NCIConditionsTableModel.SET_RESTRICTION && operand instanceof OWLAnonymousClass)   {
								String value = operand.getBrowserText();
								if (!hset.contains(value))
								{
									TreeItem item = new TreeItem();
									item.setType(TreeItem.TYPE_RESTRICTION);
									item.setNameValue(operand.getBrowserText());
									item.setCls((RDFSClass) operand);
									item.setIsDefining(true);
									v.add(item);
									hset.add(value);
								}

							}
							else if (set_type == NCIConditionsTableModel.SET_SUPERCLASS && operand instanceof OWLNamedClass)   {
								String value = SUBCLASSOF + ": " + getInternalName(operand);
								if (!hset.contains(value))
								{
									TreeItem item = new TreeItem();
									item.setType(TreeItem.TYPE_PARENT);
									item.setName(SUBCLASSOF);

									item.setValue(getInternalName(operand));
									item.setNameValue(value);
									item.setIsDefining(true);
									item.setCls((RDFSClass) operand);
									v.add(item);
									hset.add(value);
								}
							}
							//hset.add(edit_panel.getNCIEditTab().getBrowserText(operand));
					   }
			       }

                }


                else if (equivalentClass instanceof OWLUnionClass)
                {
					if (set_type == NCIConditionsTableModel.SET_RESTRICTION && equivalentClass instanceof OWLAnonymousClass)   {
                            String value = equivalentClass.getBrowserText();
                            if (!hset.contains(value))
                            {
								TreeItem item = new TreeItem();
								item.setType(TreeItem.TYPE_RESTRICTION);
								item.setNameValue(equivalentClass.getBrowserText());
								item.setCls((RDFSClass) equivalentClass);
								item.setIsDefining(true);
								v.add(item);
								hset.add(value);
						    }

					}
					else if (set_type == NCIConditionsTableModel.SET_SUPERCLASS && equivalentClass instanceof OWLNamedClass)   {
							String value = SUBCLASSOF + ": " + getInternalName(equivalentClass);
                            if (!hset.contains(value))
                            {
								TreeItem item = new TreeItem();
								item.setType(TreeItem.TYPE_PARENT);
								item.setName(SUBCLASSOF);

								item.setValue(getInternalName(equivalentClass));
								item.setNameValue(value);
								item.setIsDefining(true);
								item.setCls((RDFSClass) equivalentClass);
								v.add(item);
								hset.add(value);
						    }
					}
                }
                else // None of the above
                {
						if (set_type == NCIConditionsTableModel.SET_RESTRICTION && equivalentClass instanceof OWLAnonymousClass)   {
                            String value = equivalentClass.getBrowserText();
                            if (!hset.contains(value))
                            {
								TreeItem item = new TreeItem();
								item.setType(TreeItem.TYPE_RESTRICTION);
								item.setNameValue(equivalentClass.getBrowserText());
								item.setCls((RDFSClass) equivalentClass);
								item.setIsDefining(true);
								v.add(item);
								hset.add(value);
						    }

						}
						else if (set_type == NCIConditionsTableModel.SET_SUPERCLASS && equivalentClass instanceof OWLNamedClass)   {
							String value = SUBCLASSOF + ": " + getInternalName(equivalentClass);
                            if (!hset.contains(value))
                            {
								TreeItem item = new TreeItem();
								item.setType(TreeItem.TYPE_PARENT);
								item.setName(SUBCLASSOF);


								item.setValue(getInternalName(equivalentClass));
								item.setNameValue(value);
								item.setIsDefining(true);
								item.setCls((RDFSClass) equivalentClass);
								v.add(item);
								hset.add(value);
						    }
						}
				}
            }
        }
        return v;
    }

    public boolean removeEquivalentClass(OWLNamedClass hostClass, RDFSClass aClass)
    {
		try {
		   if (!containsDefinition(hostClass, aClass)) return true; // Nothing done

		   hostClass.removeEquivalentClass(aClass);

	 		// added by tt
			// hanldes the case that the class does not have any parent any more
		   if (hostClass.getNamedSuperclasses(false).size() == 0)
			   hostClass.addSuperclass(owlModel.getOWLThingClass());
		   //end added by tt

		   return true;
		} catch (Exception e) {
           return false;
		}
    }


    public boolean addEquivalentClass(OWLNamedClass hostClass, RDFSClass aClass)
    {

		try {
           if (containsDefinition(hostClass, aClass)) return true; // Nothing done
		   hostClass.addEquivalentClass(aClass);
		   return true;
		} catch (Exception e) {
           return false;
		}
	}

	public boolean addDirectSuperclass(Cls subCls, Cls supCls)
	{
		return addDirectSuperclass((RDFSClass) subCls, (RDFSClass) supCls);
	}


	public boolean removeDirectSuperclass(Cls sub, Cls sup)
	{
		String browserText = sup.getBrowserText();
		String owlThing = owlModel.getOWLThingClass().getBrowserText();

		if (browserText.compareTo(owlThing) == 0)
		{
			System.out.println("Cannot delete owlThing");
			return false;
		}

		OWLNamedClass subCls = (OWLNamedClass) sub;
		if (subCls == null)
		{
			System.out.println("Unable to find " + sub);
			return false;
		}
		OWLNamedClass supCls = (OWLNamedClass) sup;
		if (supCls == null)
		{
			System.out.println("Unable to find " + sup);
			return false;
		}

		subCls.removeDirectSuperclass(supCls);
/*
		// added by tt
		// hanldes the case that the class does not have any parent anymore
		if (sub.getDirectSuperclasses().size() == 0)
			sub.addDirectSuperclass(owlModel.getOWLThingClass());
		//end added by tt
*/
		return true;
	}



	public boolean addRestriction(RDFSClass hostClass, RDFSClass superCls){

		if (hostClass == null)
		{
			System.out.println("hostClass is null.");
			return false;
		}
		if (superCls == null)
		{
			System.out.println("superCls is null.");
			return false;
		}

		try {
			/*
			Collection sups = hostClass.getDirectSuperclasses();
			boolean toAdd = true;
			for(Iterator iterator = sups.iterator(); iterator.hasNext();){
				RDFSClass sup = (RDFSClass)iterator.next();

				if (sup.getBrowserText().compareTo(superCls.getBrowserText()) == 0)
				{
					toAdd = false;
					break;
				}

			}
			if (toAdd) hostClass.addDirectSuperclass(superCls);
			*/
			hostClass.addDirectSuperclass(superCls);
			return true;
	    } catch (Exception e) {
			return false;
		}

	}


    public boolean removeRestriction(OWLNamedClass hostClass, RDFSClass r)
    {
		if (hostClass == null) return false;
		try {
			hostClass.removeSuperclass(r);
			r.delete();

			return true;
	    } catch (Exception e) {
			//102206, KLO ???
			//Log.getLogger().log(Level.WARNING, "Exception caught", e);;
			System.out.println("RDFSClass delete method throws exception.");
		}
        return false;
    }

    public boolean canRemoveEquivalentClass(OWLNamedClass hostClass, RDFSClass aClass)
    {
		if (aClass instanceof OWLNamedClass) return true;
		RDFSClass definition = getEquivalentClass(hostClass);
		if (!(definition instanceof OWLIntersectionClass))
		{
			return true;
		}
		OWLIntersectionClass intersectionClass = (OWLIntersectionClass) definition;
		if (!(intersectionClass.hasOperandWithBrowserText(aClass.getBrowserText())))
		{
			return false;
		}

		return true;
	}


    public Object createNewValue(RDFProperty property, String text, String language) {
        Object newValue = null;
        if (language == null || language.trim().length() == 0) {
            newValue = text;
        }
        else {
            newValue = property.getOWLModel().createRDFSLiteralOrString(text, language);
        }
        return newValue;
    }


    public Object createNewValue(String text, String language) {
        Object newValue = null;
        if (language == null || language.trim().length() == 0) {
            newValue = text;
        }
        else {
            newValue = owlModel.createRDFSLiteralOrString(text, language);
        }
        return newValue;
    }


    private void dumpLanguageData(OWLNamedClass hostClass, String propertyName)
    {
		// Print out language again and see the changes:
		RDFProperty property = (RDFProperty) owlModel.getSlot(getSlotName(propertyName));
		Collection c = hostClass.getPropertyValues(property);
		for (Iterator it = c.iterator(); it.hasNext();) {
			Object value = it.next();

		    if (value instanceof RDFSLiteral)
		    {
				RDFSLiteral oldValue = (RDFSLiteral) value;
				String propertyvalue = oldValue.getString();
				System.out.println("Definition: " + propertyvalue);
				String language = oldValue.getLanguage();
				System.out.println("Language: " + language);
			}
		}
	}


	   public boolean resetLanguage(OWLNamedClass hostClass, String propertyName,
	                                 String propertyvalue, String newLanguage)
	   {
			if (hostClass == null) return false;
			if (propertyName == null) return false;

			try {
				RDFResource subject = (RDFResource) hostClass;
				RDFProperty property = (RDFProperty) owlModel.getSlot(getSlotName(propertyName));
				if (property == null) return false;

				Collection c = subject.getPropertyValues(property);
				if (c == null || c.size() == 0)
				{
					return false;
				}

				Iterator it = c.iterator();
				while (it.hasNext()) {
					Object value = it.next();
					if (value instanceof RDFSLiteral)
					{
						RDFSLiteral oldValue = (RDFSLiteral) value;
						if (oldValue.getString().compareTo(propertyvalue) == 0)
						{
							Object newValue = createNewValue(property, propertyvalue, newLanguage);
							subject.removePropertyValue(property, oldValue);

							//122006
							/*RDFSLiteral newLiteral = (RDFSLiteral) newValue;
							subject.addPropertyValue(property, newLiteral);*/
							subject.addPropertyValue(property, newValue);

							break;
						}
					} else { //String type
						String oldValue = (String) value;
						if (oldValue.compareTo(propertyvalue) == 0)
						{
							subject.removePropertyValue(property, oldValue);
							Object newValue = createNewValue(property, propertyvalue, newLanguage);

							subject.addPropertyValue(property, newValue);
							break;
						}
					}
				}

				return true;
		    } catch (Exception e) {
				return false;
			}
	   }


	   public void countProperties(OWLNamedClass subject, String propertyname, String label)
	   {
			if (subject == null) return;
			System.out.println("\n====" + label + " ====");
			System.out.println("propertyname " + propertyname);

			RDFProperty property = (RDFProperty) owlModel.getSlot(getSlotName(propertyname));
			Collection c = subject.getPropertyValues(property);
			System.out.println("Number of " + property.getBrowserText() + " : " + c.size() + "\n");
			System.out.println("========\n");
       }




/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Annotation Property



	private boolean createAnnotationProperty(OWLNamedClass cls, String propertyname, String propertyvalue)
	{
		if (cls == null) return false;
		Slot slot = owlModel.getSlot(propertyname);
		if (slot != null)
		{
			owlModel.addOwnSlotValue(cls, slot, propertyvalue);
		}
		return true;
	}


	public boolean addAnnotationProperty(String name, String slotname, String slotvalue)
	{
		boolean conceptexists = conceptExists(name);
		if (!conceptexists) return false;
		Cls cls = owlModel.getCls(name);
		Slot slot = owlModel.getSlot(slotname);
		if (slot == null) return false;

		owlModel.addOwnSlotValue(cls, slot, slotvalue);
		return true;
	}

	public boolean addAnnotationProperty(Cls cls, String slotname, String slotvalue)
	{
        if (cls == null) return false;
		Slot slot = owlModel.getSlot(slotname);
		if (slot == null) return false;

		owlModel.addOwnSlotValue(cls, slot, slotvalue);
		return true;
	}


	public boolean removeAnnotationProperty(OWLNamedClass cls, String propertyname, String propertyvalue)
	{
		if (cls == null) return false;

		Slot slot = owlModel.getSlot(getSlotName(propertyname));


		if (slot != null)
		{
			//101006
			//owlModel.removeOwnSlotValue(cls, slot, propertyvalue);
			RDFProperty prop = (RDFProperty) slot;
			Collection c = cls.getPropertyValues(prop);
			if (c == null || c.size() == 0) return true;

			Iterator it = cls.getPropertyValues(prop).iterator();
			while (it.hasNext())
			{
				//String val = (String) it.next();
				Object obj = (Object) it.next();
				String val = getObjectValue(obj);

				if (val.equals(propertyvalue))
				{
					cls.removePropertyValue(prop, obj);
					return true;
				}
			}
		}
		return false;
	}


	public boolean addObjectProperty(OWLNamedClass owl_cls,
	                                 String associationname,
	                                 String associationvalue)
	{
		Slot slot = (Slot) owlModel.getSlot(associationname);
		RDFSClass aClass = (RDFSClass) owlModel.getCls(associationvalue);

		try {
			if (slot != null)
			{
				owl_cls.addOwnSlotValue(slot, aClass);
				return true;
			}
     	} catch (Exception e) {
			System.out.println("Error addObjectProperty.");
			return false;
		}

		return false;
	}


	public boolean removeSuperclasses(OWLNamedClass cls, String browserText)
	{
		Collection parent_vec = cls.getDirectSuperclasses();//cls.getDirectSuperclasses();
		if (parent_vec == null || parent_vec.isEmpty())
		{
			return false;
		}

		if (parent_vec != null) {
			Iterator j = parent_vec.iterator();
			while(j.hasNext())
			{
				Object obj = j.next();
				RDFSClass r = (RDFSClass) obj;
				if (r != null)
				{
					if (r.getBrowserText().compareTo(browserText) == 0)
					{
                        try {
							cls.removeSuperclass(r);
						} catch (Exception e) {

						}
						return true;
					}
			    }
			}
		}

		return false;
	}

	/**
	 * Method for adding a new definition to a class. It handles all the cases: if there was no definition
	 * before; if there was just one restriction; or if the existing definition is an intersection.
	 * The new definition is added to the first defintion of the class. (Anyway in NCIEditTab the assumption
	 * is that each class has only one definition.
	 * @param cls - the class where the new definition is added
	 * @param newDefinition - the new definition to be added (can be a named class, a restriction, or a logical class)
	 * @return true - if adding was successful, false - otherwise
	 */
	public boolean addEquivalentDefinition(OWLNamedClass cls, RDFSClass newDefinition){
		if (cls == null || newDefinition == null)
			return false;

		RDFSClass definition = cls.getDefinition();

		try {
		//no defintion previously
		if (definition == null) {
			cls.addEquivalentClass(newDefinition);

			return true;
		}

		//if it is an intersection, add a new operand
		if (definition instanceof OWLIntersectionClass) {
			((OWLIntersectionClass)definition).addOperand(newDefinition);

			if (newDefinition instanceof RDFSNamedClass)
				cls.addDirectSuperclass(newDefinition);
			return true;
		}

		//else, if is only a restriction, which needs to be transformed into an intersection
        RDFSClass clonedOldEquivalentClass = definition;

        if (definition instanceof OWLAnonymousClass) {
        	Log.getLogger().warning("Creating clone of " + definition.getBrowserText());
        	clonedOldEquivalentClass = definition.createClone();
        }

        OWLIntersectionClass newClassDefinition = owlModel.createOWLIntersectionClass();
        newClassDefinition.addOperand(clonedOldEquivalentClass);
        newClassDefinition.addOperand(newDefinition);

        cls.addEquivalentClass(newClassDefinition);
        cls.removeEquivalentClass(definition);

        if (clonedOldEquivalentClass instanceof OWLNamedClass)
        	cls.addDirectSuperclass(clonedOldEquivalentClass);

		}catch (Exception e) {
			Log.getLogger().warning("Error at adding definition " + newDefinition.getBrowserText() + " to class " + cls.getBrowserText());
			return false;
		}

		return true;
	}



	/**
	 * Method for removing a definition from a defintion (block) of a class. It handles all the cases:
	 * if there is no definition; if the exisiting definition is a restriction; if the exisitng definition
	 * is an intersection.
	 * The method removes the defintionToDelete from the first definition of the class. (Anyway the NCIEditTab
	 * makes the assumption that each class has only one definition).
	 * @param cls - the class from which the defininion is to be deleted
	 * @param defintionToDelete - the definition to delete
	 * @return - true if deletion was successful, false - otherwise.
	 */

/*
	public boolean removeEquivalentDefinition(OWLNamedClass cls, RDFSClass defintionToDelete) {

System.out.println("########## removeEquivalentDefinition cls " + cls.getBrowserText());
System.out.println("########## removeEquivalentDefinition defintionToDelete " + defintionToDelete.getBrowserText());



		if (cls == null || defintionToDelete == null)
			return false;

		RDFSClass definition = cls.getDefinition();

		if (definition == null)
			return true;
		//there is only one definitition and it should be deleted
		if (definition.equals(defintionToDelete)) {


System.out.println("########## removeEquivalentDefinition definition.equals(defintionToDelete) -- there is only one definitition and it should be deleted" );


			// KLO, 040607
			if (definition instanceof OWLNamedClass)
			{
				cls.removeEquivalentClass(definition);
		    }
		    else
		    {
				definition.delete();
			}


			//if (!cls.hasNamedSuperclass()) {
			//	cls.addSuperclass(owlModel.getOWLThingClass());
			//}
			return true;
		}

System.out.println("########## removeEquivalentDefinition definition instanceof OWLIntersectionClass" );


		// the definition is an intersection
		if (definition instanceof OWLIntersectionClass) {
			Collection operands = new ArrayList(((OWLIntersectionClass) definition).getOperands());

			if (!hasOperand(((OWLIntersectionClass) definition), defintionToDelete)) {
				Log.getLogger().warning("Attempting to delete an unexisting operand from an intersection."
						+ " Operand: " + defintionToDelete + " Intersection: " + definition);


System.out.println("########## removeEquivalentDefinition !hasOperand -- RETURN FALSE" );


				return false;
			}


			if (operands.size() == 2) {


System.out.println("########## operands.size() == 2" );

				operands.remove(defintionToDelete);
				RDFSClass remainder = (RDFSClass) operands.iterator().next();

				Log.getLogger().warning("Creating clone of " + remainder.getBrowserText());
				RDFSClass copy = remainder.createClone();
				cls.addEquivalentClass(copy);
				if (definition instanceof OWLNamedClass)
				{
					cls.removeEquivalentClass(definition);
				}
				else
				{
					definition.delete();
				}

                //definition.delete();


			} else {


System.out.println("########## operands.size() > 2" );


System.out.println("########## operands.size() > 2 remove " + defintionToDelete.getBrowserText() );

				((OWLIntersectionClass) definition).removeOperand(defintionToDelete);
			}
		}
		else //just one restriction as defintion
		{
System.out.println("########## just one restriction as defintion  removeEquivalentClass" );


			cls.removeEquivalentClass(defintionToDelete);
		}


		if (cls.getNamedSuperclasses(false).size() == 0)
		{
			cls.addSuperclass(owlModel.getOWLThingClass());
		}


		// KLO, 040407
		if (defintionToDelete instanceof OWLNamedClass)
		{


System.out.println("########## defintionToDelete instanceof OWLNamedClass removeSuperclass" );

			cls.removeSuperclass(defintionToDelete);
		}

		return true;
	}
*/

	public boolean removeEquivalentDefinition(OWLNamedClass cls, RDFSClass defintionToDelete) {
		if (cls == null || defintionToDelete == null)
			return false;

		RDFSClass definition = cls.getDefinition();

		if (definition == null)
			return true;
		//there is only one definitition and it should be deleted
		if (definition.equals(defintionToDelete)) {
			if (definition instanceof OWLNamedClass)
			{
				cls.removeEquivalentClass(definition);
		    }
		    else
		    {
				definition.delete();
			}
			return true;
		}

		// the definition is an intersection
		boolean operand_found = false;
		if (definition instanceof OWLIntersectionClass) {

			if (hasOperand(((OWLIntersectionClass) definition), defintionToDelete)) {
				OWLIntersectionClass intersectionCls = (OWLIntersectionClass) definition;
				intersectionCls.removeOperand(defintionToDelete);
				operand_found = true;
				Collection operands = new ArrayList(intersectionCls.getOperands());
				if (operands.size() == 1)
				{
					RDFSClass remainder = (RDFSClass) operands.iterator().next();
					Log.getLogger().warning("Creating clone of " + remainder.getBrowserText());
					RDFSClass copy = remainder.createClone();
					cls.removeEquivalentClass(intersectionCls);
					cls.addEquivalentClass(copy);
				}
            }
            else
            {
				Collection operands = new ArrayList(((OWLIntersectionClass) definition).getOperands());
            	Iterator it = operands.iterator();
				while (it.hasNext())
				{
					RDFSClass next = (RDFSClass) it.next();
					if (next instanceof OWLIntersectionClass) {
						if (hasOperand(((OWLIntersectionClass) next), defintionToDelete)) {
							OWLIntersectionClass intersectionCls_2 = (OWLIntersectionClass) next;
							intersectionCls_2.removeOperand(defintionToDelete);
							operand_found = true;
							Collection operands_2 = new ArrayList(intersectionCls_2.getOperands());
							if (operands_2.size() == 1)
							{
								RDFSClass remainder = (RDFSClass) operands_2.iterator().next();
								Log.getLogger().warning("Creating clone of " + remainder.getBrowserText());
								RDFSClass copy = remainder.createClone();
								cls.removeEquivalentClass(intersectionCls_2);
								cls.addEquivalentClass(copy);
							}
							break;
						}
                    }
				}
			}

/*
			if (operands.size() == 2) {


System.out.println("########## operands.size() == 2" );

				operands.remove(defintionToDelete);
				RDFSClass remainder = (RDFSClass) operands.iterator().next();

				Log.getLogger().warning("Creating clone of " + remainder.getBrowserText());
				RDFSClass copy = remainder.createClone();
				cls.addEquivalentClass(copy);
				if (definition instanceof OWLNamedClass)
				{
					cls.removeEquivalentClass(definition);
				}

				else
				{
					definition.delete();
				}

                //definition.delete();


			} else {


System.out.println("########## operands.size() > 2" );


System.out.println("########## operands.size() > 2 remove " + defintionToDelete.getBrowserText() );

				((OWLIntersectionClass) definition).removeOperand(defintionToDelete);
			}
*/
		}

		/*
		else //just one restriction as defintion
		{
			cls.removeEquivalentClass(defintionToDelete);
		}
		*/


		if (cls.getNamedSuperclasses(false).size() == 0)
		{
			cls.addSuperclass(owlModel.getOWLThingClass());
		}


		// KLO, 040407
		if (defintionToDelete instanceof OWLNamedClass)
		{
			cls.removeSuperclass(defintionToDelete);
		}

		else
		{
			if (operand_found)
			{
				defintionToDelete.delete();
			}
		}

		return true;
	}



	private boolean hasOperand(OWLIntersectionClass intersection, RDFSClass operand) {
		for (Iterator iter = intersection.getOperands().iterator(); iter.hasNext();) {
			RDFSClass oper = (RDFSClass) iter.next();
			if (oper.equals(operand)) {
				return true;
			}
		}

		return false;
	}



	/*
	public boolean removeEquivalentDefinition(OWLNamedClass cls, RDFSClass defintionToDelete) {
		if (cls == null || defintionToDelete == null)
			return false;

		RDFSClass definition = cls.getDefinition();

		if (definition == null)
			return true;

		// the definition is an intersection
		if (definition instanceof OWLIntersectionClass) {
			Collection operands = new ArrayList(((OWLIntersectionClass) definition).getOperands());
			if (operands.size() == 2) {
				operands.remove(defintionToDelete);
				RDFSClass remainder = (RDFSClass) operands.iterator().next();

				RDFSClass copy = remainder.createClone();
				cls.addEquivalentClass(copy);
//try{
//				definition.delete();
//} catch (Exception e) {
//    Log.getLogger().log(Level.WARNING, "Exception caught", e);;
//}
                definition.delete();

			} else {
				((OWLIntersectionClass) definition)
						.removeOperand(defintionToDelete);
			}
		} else //just one restriction as defintion
			cls.removeEquivalentClass(defintionToDelete);

		if (!cls.hasNamedSuperclass()) {
			cls.addSuperclass(owlModel.getOWLThingClass());
		}
		return true;
	}
	*/




//=======================================================================================================
// Methods for validating batch editor input file:
//=======================================================================================================


    public Vector getAssociations(OWLNamedClass subject)
	{
		Vector w = new Vector();
 		Vector v = new Vector();
        Vector excludedProperties = new Vector();
        excludedProperties.add("ID");

		Collection allProperties = owlModel.getVisibleUserDefinedRDFProperties();
		for (Iterator it = allProperties.iterator(); it.hasNext();) {
			RDFProperty property = (RDFProperty) it.next();
			if (property.isAnnotationProperty()
				&& property.hasObjectRange()
				&& property instanceof DefaultOWLObjectProperty
				&& !excludedProperties.contains(property.getName()))
			{
				v.add(property);
			}
		}

		v.add(owlModel.getOWLDifferentFromProperty());
		v.add(owlModel.getOWLDisjointWithProperty());
		v.add(owlModel.getOWLEquivalentPropertyProperty());
		v.add(owlModel.getOWLSameAsProperty());
		v.add(owlModel.getRDFSIsDefinedByProperty());

		RDFProperty seeAlsoSlot = owlModel.getRDFProperty(RDFSNames.Slot.SEE_ALSO);
		v.add(seeAlsoSlot);

		for (int i=0; i<v.size(); i++)
		{
			RDFProperty property = (RDFProperty) v.elementAt(i);
			if(property.isVisible()) {
				Collection c = subject.getPropertyValues(property);
				for (Iterator it = c.iterator(); it.hasNext();) {
					Object value = it.next();
					OWLNamedClass target = (OWLNamedClass) value;
					String clsName = getInternalName(target);

					w.add(property.getName() + "|" + clsName);
				}
			}
		}
		v.clear();
		return w;
    }





    public Vector getSupportedAnnotationProperties()
    {
		Vector v = new Vector();
        Collection c = owlModel.getRDFProperties();
		Object[] propertiesArray = c.toArray();
		for (int i=0; i<propertiesArray.length; i++)
		{
			RDFProperty property = (RDFProperty) propertiesArray[i];
			if (property.isAnnotationProperty() && !property.hasObjectRange())
			{
				v.add(property.getName());
			}
		}

        return v;
    }


    public Vector getSupportedRoles()
    {
		Vector v = new Vector();
        Collection c = owlModel.getRDFProperties();
		Object[] propertiesArray = c.toArray();
		for (int i=0; i<propertiesArray.length; i++)
		{
			RDFProperty property = (RDFProperty) propertiesArray[i];
			if (!property.isAnnotationProperty() && property.hasObjectRange())
			{
				if (property.getName().indexOf(":") == -1)
				{
					v.add(property.getName());
			    }
			}
		}

        return v;
    }


    public Vector getSupportedAssociations()
    {
		Vector v = new Vector();
        Collection c = owlModel.getRDFProperties();
		Object[] propertiesArray = c.toArray();
		for (int i=0; i<propertiesArray.length; i++)
		{
			RDFProperty property = (RDFProperty) propertiesArray[i];
			if (property.isAnnotationProperty() && property.hasObjectRange())
			{
				if (property.getName().indexOf(":") == -1)
				{
					v.add(property.getName());
			    }
			}
		}

        return v;
    }



    public Vector getDirectSuperclassNames(Cls hostClass)
    {
		Vector v = new Vector();
		for (Iterator it = hostClass.getDirectSuperclasses().iterator(); it.hasNext();)
		{
			Cls superCls = (Cls) it.next();
			if (superCls instanceof OWLNamedClass)
			{
				v.add(superCls.getBrowserText());
			}
		}
	    return v;
    }


    public boolean hasAssociation(OWLNamedClass hostClass, String name, String value)
    {
		String t = name + "|" + value;
		Vector w = getAssociations(hostClass);
		for (int i=0; i<w.size(); i++)
		{
			String s = (String) w.elementAt(i);
			if (s.compareTo(t) == 0) return true;
		}
		return false;
	}

    public boolean hasProperty(OWLNamedClass cls, String name, String value)
    {
		RDFProperty property = owlModel.getRDFProperty(name);
		Collection c = cls.getPropertyValues(property);
		if (c.isEmpty())
		{
			return false;
		}

		Object[] valuesArray = c.toArray();
		for (int i=0; i<valuesArray.length; i++)
		{
			Object val = valuesArray[i];
			if (val.toString().compareTo(value) == 0) return true;
		}
		return false;
	}


    public boolean hasRole(OWLNamedClass hostClass, String name, String value)
    {
		for (Iterator it = hostClass.getDirectSuperclasses().iterator(); it.hasNext();)
		{
			Cls superCls = (Cls) it.next();
			if (superCls instanceof OWLSomeValuesFrom )
			{
				OWLSomeValuesFrom r = (OWLSomeValuesFrom) superCls;
				RDFProperty property = r.getOnProperty();
				if (property.getName().compareTo(name) == 0)
				{
					OWLNamedClass filler = (OWLNamedClass) r.getFiller();
					if (filler.getBrowserText().compareTo(value) == 0)
					{
						return true;
					}
			    }
			}
			else if (superCls instanceof OWLAllValuesFrom )
			{
				OWLAllValuesFrom r = (OWLAllValuesFrom) superCls;
				RDFProperty property = r.getOnProperty();
				if (property.getName().compareTo(name) == 0)
				{
					OWLNamedClass filler = (OWLNamedClass) r.getFiller();
					if (filler.getBrowserText().compareTo(value) == 0)
					{
						return true;
					}
			    }
			}
		}
		return false;
    }


    public static boolean containsSimpleRestrictionsOnly(RDFSClass equivalentClass)
    {
		if (equivalentClass instanceof OWLIntersectionClass)
		{
			Collection operands = ((OWLIntersectionClass) equivalentClass).getOperands();
			for (Iterator oit = operands.iterator(); oit.hasNext();) {
				RDFSClass operand = (RDFSClass) oit.next();
				if (operand instanceof OWLIntersectionClass) return false;
				else if (operand instanceof OWLUnionClass) return false;
				else if (operand instanceof OWLNamedClass) return false;
			}
			return true;
		}
		else if (equivalentClass instanceof OWLUnionClass)
		{
			return false;
		}
		else
		{
			return false;
		}
	}


    //===================================================================================================

    public static void outputDefinition(String label, OWLNamedClass cls)
    {
		if (cls == null)
		{
			return;
		}
	    RDFSClass definition = (RDFSClass) cls.getDefinition();
	    if (definition == null)
	    {
			System.out.println("NO definition");
			return;
		}
	    System.out.println(label + " " + " Definition: " + definition.getBrowserText());
    }

    public void removeDirectSuperclasses(RDFSClass cls)
    {
		Collection sups = cls.getDirectSuperclasses();
		if (sups == null) return;
		for(Iterator iterator = sups.iterator(); iterator.hasNext();){
			RDFSClass sup = (RDFSClass)iterator.next();
			if (sup instanceof OWLNamedClass)
			{
				if (getInternalName(sup).compareTo(getClsName(NCIEditTab.RETIRED_CONCEPTS)) != 0)
				{
					cls.removeDirectSuperclass(sup);
			    }
			}
			else
			{
				removeEquivalentDefinition((OWLNamedClass) cls, sup);
			}
		}
	}
}
